<?php
/**
 * Установка модуля
 * 
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    7.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2021 OOO «Диафан» (http://www.diafan.ru/)
 */

if (! defined('DIAFAN'))
{
	$path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}

class Bs_install extends Install
{
	/**
	 * @var string название
	 */
	public $title = "Баннеры";

	/**
	 * @var array таблицы в базе данных
	 */
	public $tables = array(
		array(
			"name" => "bs",
			"comment" => "Баннеры",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "название",
					"multilang" => true,
				),
				array(
					"name" => "text",
					"type" => "TEXT",
					"comment" => "описание",
					"multilang" => true,
				),
				array(
					"name" => "alt",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "атрибут alt для баннера-изображения",
					"multilang" => true,
				),
				array(
					"name" => "title",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "атрибут title для баннера-изображения",
					"multilang" => true,
				),
				array(
					"name" => "act",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "показывать на сайте: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "type",
					"type" => "ENUM('1','2') NOT NULL DEFAULT '1'",
					"comment" => "тип: 1 - файл, 2 - HTML-код",
				),
				array(
					"name" => "file",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "имя файла, загруженного в папку userfls/bs",
				),
				array(
					"name" => "html",
					"type" => "TEXT",
					"comment" => "HTML код баннера-блока",
				),
				array(
					"name" => "link",
					"type" => "TEXT",
					"comment" => "ссылка на баннер-изображение",
					"multilang" => true,
				),
				array(
					"name" => "cat_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор категории из таблицы {bs_category}",
				),
				array(
					"name" => "check_number",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "ограничить количество показов: 0 - нет, 1 - да",
				),
				array(
					"name" => "check_click",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "ограничить по количеству кликов: 0 - нет, 1 - да",
				),
				array(
					"name" => "show_click",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "ограничить по количеству кликов: осталось кликов",
				),
				array(
					"name" => "created",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата создания",
				),
				array(
					"name" => "date_start",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата начала показа",
				),
				array(
					"name" => "date_finish",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата окончания показа",
				),
				array(
					"name" => "show_number",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "ограничить количество показов: осталось показов",
				),
				array(
					"name" => "click",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "всего кликов",
				),
				array(
					"name" => "check_user",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "ограничить количество показов посетителю в сутки: 0 - нет, 1 - да",
				),
				array(
					"name" => "show_user",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "количество показов посетителю в сутки",
				),
				array(
					"name" => "count_view",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "всего показов",
				),
				array(
					"name" => "target_blank",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "открывать ссылку в новом окне",
				),
                array(
                    "name" => "sort",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "подрядковый номер для сортировки",
                ),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "bs_category",
			"comment" => "Категории баннеров",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "TEXT",
					"comment" => "название",
				),
				array(
					"name" => "act",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "показывать на сайте: 0 - нет, 1 - да",
				),
				array(
					"name" => "text",
					"type" => "TEXT",
					"comment" => "описание",
					"multilang" => true,
				),
                array(
                    "name" => "sort",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "подрядковый номер для сортировки",
                ),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "bs_site_rel",
			"comment" => "Данные о том, на каких страницах сайта выводятся баннеры",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор баннера из таблицы {bs_category}",
				),
				array(
					"name" => "site_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор страницы сайта из таблицы {site}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
	);

	/**
	 * @var array записи в таблице {modules}
	 */
	public $modules = array(
		array(
			"name" => "bs",
			"admin" => true,
			"site" => true,
		),
	);

	/**
	 * @var array меню административной части
	 */
	public $admin = array(
		array(
			"name" => "Баннеры",
			"rewrite" => "bs",
			"group_id" => 1,
			"sort" => 8,
			"act" => true,
			"docs" => "http://www.diafan.ru/moduli/bannery/",
			"children" => array(
				array(
					"name" => "Баннеры",
					"rewrite" => "bs",
					"act" => true,
				),
				array(
					"name" => "Категории",
					"rewrite" => "bs/category",
					"act" => true,
				),
				array(
					"name" => "Настройка",
					"rewrite" => "bs/config",
				),
			)
		),
	);

	/**
	 * @var array настройки
	 */
	public $config = array(
		array(
			"name" => "cat",
			"value" => 1,
		),		
	);

	/**
	 * @var array демо-данные
	 */
	public $demo = array(
		'bs_category' => array(
			array(
				'id' => 1,
				'name' => 'Слайдер на главной',
				'act' => 1,				
			),
			array(
				'id' => 2,
				'name' => 'Сквозной баннер внизу',
				'act' => 1,	
			),
		),
		'bs' => array(
			array(
				"name" => array("Распродажа в чёрную пятницу!", "Black Friday sale!"),
				"text" => array("<h2>Распродажа в чёрную пятницу!</h2>
<p>успей купить сотни товаров по привлекательной цене</p>", "<h2>Black Friday sale!</h2>
<p>Have time to buy hundreds of products at an attractive price</p>"),
				"type" => 1,
				"file" => "slide1.webp",
				"cat_id" => 1,
				"sort" => 3,
				"copy" => array("bs/slide1.webp"),
			),
			array(
				"name" => array("Женская и мужская одежда", "Women's and men's clothing"),
				"text" => array("<h2>Выбор стильных людей</h2>
<p>распродажа женской и мужской одежды</p>", "<h2>Selection of stylish people</h2>
<p>sale of women's and men's clothing</p>"),
				"type" => 1,
				"file" => "slide2.webp",
				"link" => array("BASE_PATHshop/odezhda-i-obuv/", "BASE_PATHeng/shop/odezhda-i-obuv/"),
				"cat_id" => 1,
				"sort" => 2,
				"copy" => array("bs/slide2.webp"),
			),
			array(
				"name" => array("Обувь", "Footwear"),
				"text" => array("<h2>Скидки до 50%!</h2>
<p>на всю коллекцию обуви</p>", "<h2>Discounts up to 50%!</h2>
<p>for the entire shoe collection</p>"),
				"type" => 1,
				"file" => "slide3.webp",
				"link" => array("BASE_PATHshop/odezhda-i-obuv/krosovki-i-kedy/", "BASE_PATHeng/shop/odezhda-i-obuv/krosovki-i-kedy/"),
				"cat_id" => 1,
				"sort" => 1,
				"copy" => array("bs/slide3.webp"),
			),
			array(
				"name" => array("Спецпредложение", "Special offer"),
				"type" => 1,
				"file" => "slide4.webp",
				"link" => array("BASE_PATHshop/odezhda-i-obuv/bryuki-i-dzhinsy/", "BASE_PATHeng/shop/odezhda-i-obuv/bryuki-i-dzhinsy/"),
				"cat_id" => 2,
				"sort" => 4,
				"copy" => array("bs/slide4.webp"),
			),
		),
	);
}