<?php
/**
 * Подключение для работы с капчей «Код на картинке»
 *
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    7.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2019 OOO «Диафан» (http://www.diafan.ru/)
 */

if (! defined('DIAFAN'))
{
	$path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}

/**
 * Captcha_kcaptcha_inc
 */
class Captcha_kcaptcha_inc extends Diafan
{
	/**
	 * Выводит капчу
	 *
	 * @param string $modules метка капчи
	 * @param string $error ошибка ввода кода, если запрос передан не через Ajax
	 * @param boolean $is_update капча генерируется для обновления
	 * @return string
	 */
	public function get($modules, $error, $is_update)
	{
		$result = array('modules' => $modules, 'error' => $error);

		$result['mode'] = $this->diafan->configmodules('kcaptcha_mode', 'captcha');

		$result['duration'] = $this->diafan->configmodules('kcaptcha_duration', 'captcha');
		if (empty($result['duration']))
        {
            $result['duration'] = 5;
        }

		$result['captchapin'] = time() - mktime(0,0,0);
		$result['captchaint'] = rand(1111, 9999);

		$result['form_key'] = $result['modules'].$result['captchaint'];

		if (! $is_update && $result['mode'] != 'nocode')
		{
			$this->diafan->_site->js_view[] = BASE_PATH.Custom::path('js/jquery.cookie.min.js');
		}

		ob_start();
		include(Custom::path('modules/captcha/backend/kcaptcha/captcha.kcaptcha.view'.($is_update ? '.update' : '').'.php'));
		$text = ob_get_contents();
		ob_end_clean();
		return $text;
	}

	/**
	 * Проверяет правильность ввода капчи
	 *
	 * @param string $modules метка капчи
	 * @return string|boolean false
	 */
	public function check($modules)
	{
		if (empty($_POST['captchaint']))
		{
			return $this->diafan->_('Введите защитный код', false);
		}
		if (! empty($_POST['cfio']))
		{
			return $this->diafan->_('Неправильно введен защитный код.', false);
		}
		$duration = $this->diafan->configmodules('kcaptcha_duration', 'captcha');
        if (empty($duration))
        {
            $duration = 5;
        }
		$c = time() - mktime(0,0,0);
		if(! isset($_POST['captchapin']) || $_POST['captchapin'] > $c || $c - $_POST['captchapin'] < $duration)
		{
			return $this->diafan->_('Повторите отправку формы через несколько секунд.', false);
		}
		//Форма с данным идентификатором captchaint не подтверждена, как «живая»
		if (empty($_SESSION['captcha_verify'][$modules][$_POST['captchaint']]))
		{
			return $this->diafan->_('Повторите отправку формы.', false);
		}
		unset($_SESSION['captcha_verify'][$modules][$_POST['captchaint']]);

		$mode = $this->diafan->configmodules('kcaptcha_mode', 'captcha');
		if ($mode != 'nocode')
		{
			//Защитный код не введен
			if (empty($_POST['captcha']))
			{
				return $this->diafan->_('Введите защитный код', false);
			}
			//В сессии не записан код с данным идентификатором captchaint или код из сессии не соответствует введенному. регистр не учитывается
			if (! isset($_SESSION['captcha'][$modules][$_POST['captchaint']]) || strtoupper($_SESSION['captcha'][$modules][$_POST['captchaint']]) != strtoupper($_POST['captcha']))
			{
				return $this->diafan->_('Неправильно введен защитный код.', false);
			}
			if (! isset($_COOKIE['captcha']) || strtoupper(urldecode($_COOKIE['captcha'])) != strtoupper($_POST['captcha']))
			{
				return $this->diafan->_('Неправильно введен защитный код.', false);
			}
			setcookie('captcha', '', 0, '/');
			//очищаем из сессии запись с данным идентификатором
			unset($_SESSION['captcha'][$modules][$_POST['captchaint']]);
		}
		return false;
	}
}
