<?php
/**
 * Подключение для работы с капчей «Yandex SmartCaptcha»
 * 
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    7.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2024 OOO «Диафан» (http://www.diafan.ru/)
 */

if (! defined('DIAFAN'))
{
	$path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}

/**
 * Captcha_yacaptcha_inc
 */
class Captcha_yacaptcha_inc extends Diafan
{
	/**
	 * Выводит капчу
	 * 
	 * @param string $modules метка капчи
	 * @param string $error ошибка ввода кода, если запрос передан не через Ajax
	 * @param boolean $is_update капча генерируется для обновления
	 * @return string
	 */
	public function get($modules, $error, $is_update)
	{
		if ($is_update)
		{
			return '';
		}

		$this->diafan->_site->js_view[] = 'https://smartcaptcha.yandexcloud.net/captcha.js?render=onload&onload=yacaptcha_init';

		$result['public_key'] = $this->diafan->configmodules('yacaptcha_public_key', 'captcha');
		$result['mode'] = $this->diafan->configmodules('yacaptcha_mode', 'captcha');
		$result['error'] = $error;
		$result['modules'] = $modules;

		ob_start();
		include(Custom::path('modules/captcha/backend/yacaptcha/captcha.yacaptcha.view.php'));
		$html = ob_get_contents();
		ob_end_clean();

		return $html;
	}

	/**
	 * Проверяет правильность ввода капчи
	 * 
	 * @return string|boolean false
	 */
	public function check($modules)
	{
		if (empty($_POST['smart-token'])) {
			return $this->diafan->_('Вы не прошли проверку.', false);
		}
		$result = json_decode(file_get_contents('https://smartcaptcha.yandexcloud.net/validate', false, stream_context_create(
			array(
				'http' => array(
					'method'  => 'POST',
					'header'  => 'Content-Type: application/x-www-form-urlencoded',
					'content' => http_build_query(array(
						'secret' => $this->diafan->configmodules('yacaptcha_private_key', 'captcha'),
						'token' => $_POST['smart-token'],
						'ip' => implode(',', $this->diafan->_users->get_IPs())
					)),
					'protocol_version' => 1.1,
					'timeout' => 1.0,
				),
				// 'ssl' => array(
				// 	'verify_peer' => false,
				// 	'verify_peer_name' => false,
				// ),
			)
		)), true);
	    if ($result && $result['status'] === 'ok') {
			return false;
	    } else {
			return $this->diafan->_('Вы не прошли проверку.', false);
		}
	}
}
