/**
 * JS-сценарий для Yandex SmartCaptcha
 * 
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    7.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2024 OOO «Диафан» (http://www.diafan.ru/)
 */



var yacaptcha = (function () {
	var initialized = false;
	var widgets = {};
	function init() {
		if (initialized || !window.smartCaptcha) {
			return;
		}
		$('.js_yacaptcha').each(function (i, widget) {
			render(widget);
		});
		$(document).off('submit', 'form.ajax');
		$(document).on('submit', 'form.ajax', function () {
			var form = this;
			var key = $('.js_yacaptcha', form).first().attr('id');
			if (key && widgets.hasOwnProperty(key)) {
				widgets[key].submit = true;
				if (widgets[key].invisible) {
					if (!widgets[key].unsubscribe.success) {
						widgets[key].unsubscribe.success = window.smartCaptcha.subscribe(widgets[key].id, 'success', function (token) {
							diafan_ajax.init(form);
							unsubscribe(key);
						});
						window.smartCaptcha.execute(widgets[key].id);
					}
					return false;
				}
			}
			return diafan_ajax.init(form);
		});
		initialized = true;
	}
	function render(widget) {
		var $widget = $(widget);
		var key = $widget.attr('id'),
			sitekey = $widget.data('sitekey'),
			invisible = $widget.hasClass('js_yacaptcha_invisible');
		var id = window.smartCaptcha.render(key, {
			sitekey: sitekey,
			invisible: invisible,
			/* первым вызывается callback, потом success */
			callback: function (token) {
				// $('#' + key + '_token').val(token);
				// т.к. генерирутся и записывается автоматом: <input type="hidden" name="smart-token">
			}
		});
		widgets[key] = {
			id: id,
			invisible: invisible,
			unsubscribe: {
				success: false,
			},
			submit: false,
		};
	}
	function reset() {
		if (!window.smartCaptcha) {
			return;
		}
		for (key in widgets) {
			if (widgets[key].submit) {
				unsubscribe(key);
				window.smartCaptcha.reset(widgets[key].id);
				widgets[key].submit = false;
			}
		}
	}
	function unsubscribe(key) {
		if (widgets[key].unsubscribe.success) {
			widgets[key].unsubscribe.success();
			widgets[key].unsubscribe.success = false;
		}
	}
	return {
		init: init,
		reset: reset,
	};
})();

function yacaptcha_init() {
	yacaptcha.init();
}

$(function () {
	yacaptcha_init();
});

/* вызывается автоматом */
function init_captcha_yacaptcha() {
	yacaptcha.reset();
}
