<?php
/**
 * Установка модуля
 *
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    7.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2021 OOO «Диафан» (http://www.diafan.ru/)
 */

if (! defined('DIAFAN'))
{
    $path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}

class Clauses_install extends Install
{
    /**
     * @var string название
     */
    public $title = "Статьи";

    /**
     * @var array таблицы в базе данных
     */
    public $tables = array(
        array(
            "name" => "clauses",
            "comment" => "Статьи",
            "fields" => array(
                array(
                    "name" => "id",
                    "type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
                    "comment" => "идентификатор",
                ),
                array(
                    "name" => "name",
                    "type" => "TEXT",
                    "comment" => "название",
                    "multilang" => true,
                ),
                array(
                    "name" => "act",
                    "type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
                    "comment" => "показывать на сайте: 0 - нет, 1 - да",
                    "multilang" => true,
                ),
                array(
                    "name" => "map_no_show",
                    "type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
                    "comment" => "не показывать на карте сайта: 0 - нет, 1 - да",
                ),
                array(
                    "name" => "changefreq",
                    "type" => "ENUM( 'always', 'hourly', 'daily', 'weekly', 'monthly', 'yearly', 'never' ) NOT NULL DEFAULT 'always'",
                    "comment" => "Changefreq для sitemap.xml",
                ),
                array(
                    "name" => "priority",
                    "type" => "VARCHAR(3) NOT NULL DEFAULT ''",
                    "comment" => "Priority для sitemap.xml",
                ),
                array(
                    "name" => "noindex",
                    "type" => "ENUM('0','1') NOT NULL DEFAULT '0'",
                    "comment" => "не индексировать: 0 - нет, 1 - да",
                ),
                array(
                    "name" => "created",
                    "type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "дата создания",
                ),
                array(
                    "name" => "date_start",
                    "type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "дата начала показа",
                ),
                array(
                    "name" => "date_finish",
                    "type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "дата окончания показа",
                ),
                array(
                    "name" => "cat_id",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "идентификатор основной категории из таблицы {clauses_category}",
                ),
                array(
                    "name" => "site_id",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "идентификатор страницы сайта из таблицы {site}",
                ),
                array(
                    "name" => "keywords",
                    "type" => "VARCHAR(250) NOT NULL DEFAULT ''",
                    "comment" => "ключевые слова, тег Keywords",
                    "multilang" => true,
                ),
                array(
                    "name" => "descr",
                    "type" => "TEXT",
                    "comment" => "описание, тэг Description",
                    "multilang" => true,
                ),
                array(
                    "name" => "canonical",
                    "type" => "VARCHAR(100) NOT NULL DEFAULT ''",
                    "comment" => "канонический тег",
                    "multilang" => true,
                ),
                array(
                    "name" => "title_meta",
                    "type" => "VARCHAR(250) NOT NULL DEFAULT ''",
                    "comment" => "заголовок окна в браузере, тег Title",
                    "multilang" => true,
                ),
                array(
                    "name" => "anons",
                    "type" => "TEXT",
                    "comment" => "анонс",
                    "multilang" => true,
                ),
                array(
                    "name" => "anons_plus",
                    "type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
                    "comment" => "добавлять анонс к описанию: 0 - нет, 1 - да",
                    "multilang" => true,
                ),
                array(
                    "name" => "text",
                    "type" => "LONGTEXT",
                    "comment" => "описание",
                    "multilang" => true,
                ),
                array(
                    "name" => "import",
                    "type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
                    "comment" => "элемент только что импортирован: 0 - нет, 1 - да",
                ),
                array(
                    "name" => "import_id",
                    "type" => "VARCHAR(100) NOT NULL DEFAULT ''",
                    "comment" => "собственный идентификатор элемента при импорте",
                ),
                array(
                    "name" => "sort",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "подрядковый номер для сортировки",
                ),
                array(
                    "name" => "timeedit",
                    "type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "время последнего изменения в формате UNIXTIME",
                ),
                array(
                    "name" => "admin_id",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "пользователь из таблицы {users}, добавивший или первый отредктировавший статью в административной части",
                ),
                array(
                    "name" => "access",
                    "type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
                    "comment" => "доступ ограничен: 0 - нет, 1 - да",
                ),
                array(
                    "name" => "theme",
                    "type" => "VARCHAR(50) NOT NULL DEFAULT ''",
                    "comment" => "шаблон страницы сайта",
                ),
                array(
                    "name" => "view",
                    "type" => "VARCHAR(50) NOT NULL DEFAULT ''",
                    "comment" => "шаблон модуля",
                ),
                array(
                    "name" => "trash",
                    "type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
                    "comment" => "запись удалена в корзину: 0 - нет, 1 - да",
                ),
            ),
            "keys" => array(
                "PRIMARY KEY (id)",
                "KEY site_id (site_id)",
				"KEY import_id (`import_id`)",
            ),
        ),
        array(
            "name" => "clauses_category",
            "comment" => "Категории статей",
            "fields" => array(
                array(
                    "name" => "id",
                    "type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
                    "comment" => "идентификатор",
                ),
                array(
                    "name" => "name",
                    "type" => "TEXT",
                    "comment" => "название",
                    "multilang" => true,
                ),
                array(
                    "name" => "act",
                    "type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
                    "comment" => "показывать на сайте: 0 - нет, 1 - да",
                    "multilang" => true,
                ),
                array(
                    "name" => "map_no_show",
                    "type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
                    "comment" => "не показывать на карте сайта: 0 - нет, 1 - да",
                ),
                array(
                    "name" => "changefreq",
                    "type" => "ENUM( 'always', 'hourly', 'daily', 'weekly', 'monthly', 'yearly', 'never' ) NOT NULL DEFAULT 'always'",
                    "comment" => "Changefreq для sitemap.xml",
                ),
                array(
                    "name" => "priority",
                    "type" => "VARCHAR(3) NOT NULL DEFAULT ''",
                    "comment" => "Priority для sitemap.xml",
                ),
                array(
                    "name" => "noindex",
                    "type" => "ENUM('0','1') NOT NULL DEFAULT '0'",
                    "comment" => "не индексировать: 0 - нет, 1 - да",
                ),
                array(
                    "name" => "parent_id",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "идентификатор родителя из таблицы {clauses_category}",
                ),
                array(
                    "name" => "count_children",
                    "type" => "SMALLINT(5) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "количество вложенных категорий",
                ),
                array(
                    "name" => "site_id",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "идентификатор страницы сайта из таблицы {site}",
                ),
                array(
                    "name" => "keywords",
                    "type" => "VARCHAR(250) NOT NULL DEFAULT ''",
                    "comment" => "ключевые слова, тег Keywords",
                    "multilang" => true,
                ),
                array(
                    "name" => "descr",
                    "type" => "TEXT",
                    "comment" => "описание, тэг Description",
                    "multilang" => true,
                ),
                array(
                    "name" => "canonical",
                    "type" => "VARCHAR(100) NOT NULL DEFAULT ''",
                    "comment" => "канонический тег",
                    "multilang" => true,
                ),
                array(
                    "name" => "title_meta",
                    "type" => "VARCHAR(250) NOT NULL DEFAULT ''",
                    "comment" => "заголовок окна в браузере, тег Title",
                    "multilang" => true,
                ),
                array(
                    "name" => "anons",
                    "type" => "TEXT",
                    "comment" => "анонс",
                    "multilang" => true,
                ),
                array(
                    "name" => "anons_plus",
                    "type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
                    "comment" => "добавлять анонс к описанию: 0 - нет, 1 - да",
                    "multilang" => true,
                ),
                array(
                    "name" => "text",
                    "type" => "TEXT",
                    "comment" => "описание",
                    "multilang" => true,
                ),
                array(
                    "name" => "import",
                    "type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
                    "comment" => "категория только что импортирован: 0 - нет, 1 - да",
                ),
                array(
                    "name" => "import_id",
                    "type" => "VARCHAR(100) NOT NULL DEFAULT ''",
                    "comment" => "собственный идентификатор категории при импорте",
                ),
                array(
                    "name" => "sort",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "подрядковый номер для сортировки",
                ),
                array(
                    "name" => "timeedit",
                    "type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "время последнего изменения в формате UNIXTIME",
                ),
                array(
                    "name" => "admin_id",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "пользователь из таблицы {users}, добавивший или первый отредктировавший категорию в административной части",
                ),
                array(
                    "name" => "access",
                    "type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
                    "comment" => "доступ ограничен: 0 - нет, 1 - да",
                ),
                array(
                    "name" => "theme",
                    "type" => "VARCHAR(50) NOT NULL DEFAULT ''",
                    "comment" => "шаблон страницы сайта",
                ),
                array(
                    "name" => "view",
                    "type" => "VARCHAR(50) NOT NULL DEFAULT ''",
                    "comment" => "шаблон модуля",
                ),
                array(
                    "name" => "view_rows",
                    "type" => "VARCHAR(50) NOT NULL DEFAULT ''",
                    "comment" => "шаблон модуля для элементов в списке категории",
                ),
                array(
                    "name" => "view_element",
                    "type" => "VARCHAR(50) NOT NULL DEFAULT ''",
                    "comment" => "шаблон модуля для элементов в категории",
                ),
                array(
                    "name" => "trash",
                    "type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
                    "comment" => "запись удалена в корзину: 0 - нет, 1 - да",
                ),
            ),
            "keys" => array(
                "PRIMARY KEY (id)",
                "KEY parent_id (parent_id)",
                "KEY site_id (site_id)",
				"KEY import_id (`import_id`)",
            ),
        ),
        array(
            "name" => "clauses_category_rel",
            "comment" => "Связи статей и категорий",
            "fields" => array(
                array(
                    "name" => "id",
                    "type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
                    "comment" => "идентификатор",
                ),
                array(
                    "name" => "element_id",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "идентификатор статьи из таблицы {clauses}",
                ),
                array(
                    "name" => "cat_id",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "идентификатор категории из таблицы {clauses_category}",
                ),
                array(
                    "name" => "trash",
                    "type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
                    "comment" => "запись удалена в корзину: 0 - нет, 1 - да",
                ),
            ),
            "keys" => array(
                "PRIMARY KEY (id)",
                "KEY cat_id (`cat_id`)",
            ),
        ),
        array(
            "name" => "clauses_category_parents",
            "comment" => "Родительские связи категорий статей",
            "fields" => array(
                array(
                    "name" => "id",
                    "type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
                    "comment" => "идентификатор",
                ),
                array(
                    "name" => "element_id",
                    "type" => "INT( 11 ) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "идентификатор категории из таблицы {clauses_category}",
                ),
                array(
                    "name" => "parent_id",
                    "type" => "INT( 11 ) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "идентификатор категории-родителя из таблицы {clauses_category}",
                ),
                array(
                    "name" => "trash",
                    "type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
                    "comment" => "запись удалена в корзину: 0 - нет, 1 - да",
                ),
            ),
            "keys" => array(
                "PRIMARY KEY (id)",
            ),
        ),
        array(
            "name" => "clauses_counter",
            "comment" => "Счетчик просмотров статей",
            "fields" => array(
                array(
                    "name" => "id",
                    "type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
                    "comment" => "идентификатор",
                ),
                array(
                    "name" => "element_id",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "идентификатор статьи из таблицы {clauses}",
                ),
                array(
                    "name" => "count_view",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "количество просмотров",
                ),
                array(
                    "name" => "trash",
                    "type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
                    "comment" => "запись удалена в корзину: 0 - нет, 1 - да",
                ),
            ),
            "keys" => array(
                "PRIMARY KEY (id)",
                "KEY element_id (`element_id`)",
            ),
        ),
        array(
            "name" => "clauses_rel",
            "comment" => "Связи похожих статей",
            "fields" => array(
                array(
                    "name" => "id",
                    "type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
                    "comment" => "идентификатор",
                ),
                array(
                    "name" => "element_id",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "идентификатор статьи из таблицы {clauses}",
                ),
                array(
                    "name" => "rel_element_id",
                    "type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
                    "comment" => "идентификатор связанной статьи из таблицы {clauses}",
                ),
                array(
                    "name" => "trash",
                    "type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
                    "comment" => "запись удалена в корзину: 0 - нет, 1 - да",
                ),
            ),
            "keys" => array(
                "PRIMARY KEY (id)",
            ),
        ),
    );

    /**
     * @var array записи в таблице {modules}
     */
    public $modules = array(
        array(
            "name" => "clauses",
            "admin" => true,
            "site" => true,
            "site_page" => true,
        ),
    );

    /**
     * @var array меню административной части
     */
    public $admin = array(
        array(
            "name" => "Статьи",
            "rewrite" => "clauses",
            "group_id" => 1,
            "sort" => 6,
            "act" => true,
            "docs" => "http://www.diafan.ru/moduli/stati/",
            "children" => array(
                array(
                    "name" => "Статьи",
                    "rewrite" => "clauses",
                    "act" => true,
                ),
                array(
                    "name" => "Категории",
                    "rewrite" => "clauses/category",
                    "act" => true,
                ),
                array (
                    'name' => 'Статистика',
                    'rewrite' => 'clauses/counter',
                    'act' => true,
                ),
                array(
                    "name" => "Настройки",
                    "rewrite" => "clauses/config",
                ),
            )
        ),
    );

    /**
     * @var array страницы сайта
     */
    public $site = array(
        array(
            "name" => array('Статьи', 'Articles'),
            "module_name" => "clauses",
            "rewrite" => "clauses",
            "menu" => 1,
            "parent_id" => 2,
        ),
    );

    /**
     * @var array настройки
     */
    public $config = array(
        array(
            "name" => "images_element",
            "value" => "1",
        ),
		array(
			"name" => "images_webp_element",
			"value" => "1",
		),
		array(
			"name" => "images_webp_cat",
			"value" => "1",
		),
        array(
            "name" => "list_img_element",
            "value" => "1",
        ),
        array(
            "name" => "use_animation",
            "value" => "1",
        ),
        array(
            "name" => "format_date",
            "value" => "2",
        ),
        array(
            "name" => "nastr",
            "value" => 12,
        ),
		array(
			"name" => "nastr_cat",
			"value" => 10,
		),
        array(
            "name" => "count_list",
            "value" => "4",
        ),
        array(
            "name" => "count_child_list",
            "value" => "4",
        ),
        array(
            "name" => "children_elements",
            "value" => "1",
        ),
        array(
            "name" => "counter",
            "value" => "1",
        ),
		array(
			"name" => "images_variations_element",
			"value" => "a:2:{s:4:\"webp\";i:1;s:2:\"vs\";a:2:{i:0;a:2:{s:4:\"name\";s:5:\"large\";s:2:\"id\";s:1:\"3\";}i:1;a:2:{s:4:\"name\";s:6:\"medium\";s:2:\"id\";s:1:\"1\";}}}",
		),
        array(
            "name" => "cat",
            "value" => "1",
        ),
		array(
			"name" => "images_variations_cat",
			"value" => "a:2:{i:0;a:2:{s:4:\"name\";s:5:\"large\";s:2:\"id\";s:1:\"5\";}i:1;a:2:{s:4:\"name\";s:6:\"medium\";s:2:\"id\";s:1:\"5\";}}",
		),
		array(
			"name" => "show_more",
			"value" => '1',
		),
    );

    /**
     * @var array демо-данные
     */
    public $demo = array(
		'config' => array(
			array(
				'name' => 'rating',
				'value' => 1,
			),
			array(
				'name' => 'comments',
				'value' => 1,
			),
			array(
				'name' => 'comments_cat',
				'value' => 1,
			),
			array(
				'name' => 'tags',
				'value' => 1,
			),
		),
        'clauses_category' => array(
			array(
				"id" => 1,
				"name" => array("Обзоры товаров", "Product Reviews"),
				"anons" => array("<p>В этой категории мы публикуем обзоры на самые популярные и новые товары из нашего ассортимента.</p>"),
				"text" => array("<p>Узнайте больше о характеристиках, преимуществах и недостатках различных моделей, чтобы сделать правильный выбор при покупке. Обзоры составлены на основе отзывов наших покупателей и экспертов, что поможет вам получить наиболее объективное представление о товаре.</p>"),
				"rewrite" => "clauses/obzory-tovarov",
				"sort" => 1,
			),
			array(
				"id" => 2,
				"name" => array("Советы по выбору товаров", "Tips on choosing products"),
				"anons" => array("<p>В этой категории статей мы делимся советами о том, как правильно выбирать товары в нашем интернет-магазине.</p>"),
				"text" => array("<p>Мы расскажем о различных критериях выбора, которые зависят от ваших потребностей, бюджета и предпочтений. Вы узнаете, на что стоит обратить внимание при выборе товара, какие функции могут быть полезны для вас, а также получите рекомендации по производителям и моделям.</p>"),
				"rewrite" => "clauses/sovety-po-vyboru-tovarov",
				"sort" => 2,
			),
			array(
				"id" => 3,
				"name" => array("Новинки ассортимента", "New products in the range"),
				"anons" => array("<p>В этой категории вы найдете статьи о новых товарах, которые недавно появились в нашем ассортименте.</p>"),
				"text" => array("<p>Мы расскажем о характеристиках и особенностях каждой модели, поможем вам оценить ее преимущества и принять решение о покупке. Также вы узнаете о тенденциях и перспективах развития рынка товаров, что позволит вам быть в курсе последних новинок и актуальных предложений.</p>"),
				"rewrite" => "clauses/instruktsii",
				"sort" => 3,
			),
        ),
        'clauses' => array(
            array(
				"id" => 1,
				"cat_id" => 1,
				"name" => array("Телевизор Ampera A55SV800: обзор и характеристики", "Ampera A55SV800 TV: overview and specifications"),
				"anons" => array("<p><span>Телевизор Ampera A55SV800 является отличным выбором для тех, кто ищет качественное и доступное устройство для просмотра телепередач, фильмов и игр. Этот телевизор оснащен 55-дюймовым экраном, который обеспечивает яркое и четкое изображение, благодаря разрешению 4K Ultra HD (3840x2160 пикселей).</span></p>", "<p>The Ampera A55SV800 TV is an excellent choice for those who are looking for a high-quality and affordable device for watching TV shows, movies and games. This TV is equipped with a 55-inch screen that provides a bright and clear image, thanks to the 4K Ultra HD resolution (3840x2160 pixels).</p>"),
				"text" => array("<p>Дизайн телевизора Ampera A55SV800 выполнен в современном стиле с тонкими рамками и металлической подставкой, которая обеспечивает устойчивость и удобство при установке. В комплекте с телевизором также идет пульт дистанционного управления, который позволяет управлять устройством на расстоянии.</p>
<p><a href=\"BASE_PATHuserfls/editor/medium/333_superskidki-na-televizory-.jpg\" data-fancybox=\"editor\"><img src=\"BASE_PATHuserfls/editor/medium/333_superskidki-na-televizory-.jpg\" alt=\"\" title=\"\" width=\"300\" height=\"300\" style=\"display: block; margin-left: auto; margin-right: auto;\"></a></p>
<p>Одной из главных особенностей телевизора Ampera является поддержка технологии Smart TV, которая позволяет смотреть онлайн-видео, играть в игры и пользоваться различными приложениями. Операционная система Android TV обеспечивает быстрый и удобный доступ к контенту, а встроенный Wi-Fi и Bluetooth позволяют подключать различные устройства без использования проводов.</p>
<p>Звуковая система телевизора Ampera состоит из двух динамиков общей мощностью 20 Вт. Они обеспечивают достаточно громкий и качественный звук, который подойдет для просмотра фильмов и телепередач.</p>", "<p>The design of the Ampera A55SV800 TV is made in a modern style with thin frames and a metal stand that provides stability and ease of installation. The TV also comes with a remote control that allows you to control the device from a distance.</p>
<p><a href=\"BASE_PATHuserfls/editor/medium/333_superskidki-na-televizory-.jpg\" data-fancybox=\"editor\"><img src=\"BASE_PATHuserfls/editor/medium/333_superskidki-na-televizory-.jpg\" alt=\"\" title=\"\" width=\"300\" height=\"300\" style=\"display: block; margin-left: auto; margin-right: auto;\"></a></p>
<p>One of the main features of the Ampera TV is the support of Smart TV technology, which allows you to watch online videos, play games and use various applications. The Android TV operating system provides fast and convenient access to content, and built-in Wi-Fi and Bluetooth allow you to connect various devices without using wires.</p>
<p>The sound system of the Ampera TV consists of two speakers with a total power of 20 watts. They provide loud enough and high-quality sound that is suitable for watching movies and TV shows.</p>"),
				"rewrite" => "clauses/obzory-tovarov/televizor-ampera-a55sv800-obzor-i-kharakteristiki",
				"sort" => 7,
				"images" => array("858_televizor-ampera-a55sv800-obzor-i-.webp"),
			),
			array(
				"id" => 2,
				"cat_id" => 1,
				"name" => array("Обзор игровой станции Vortex Gaming Station: идеальный выбор для геймеров"),
				"anons" => array("<p>Игровая станция Vortex Gaming Station &ndash; это идеальное сочетание стиля, функциональности и мощности. Эта модель стала настоящим хитом среди любителей компьютерных игр и геймеров, которые хотят получить максимальное удовольствие от игрового процесса.</p>
<p></p>"),
				"text" => array("<p>Первое, что бросается в глаза при взгляде на Vortex Gaming Station, это ее стильный и современный дизайн. Корпус станции выполнен в черном цвете с серебристыми вставками, что придает ей стильный и строгий вид. На передней панели расположены несколько разъемов для подключения различных устройств, а на задней &ndash; множество портов для подключения внешних устройств, таких как мониторы, клавиатуры, мыши и другие.</p>
<p>Внутри Vortex Gaming Station установлен мощный процессор Intel Core i7-9700K, который обеспечивает высокую производительность при запуске самых требовательных игр. В сочетании с видеокартой NVIDIA GeForce RTX 2080 Ti, эта игровая станция способна обрабатывать графику на высоком уровне, обеспечивая плавный и комфортный игровой процесс.</p>
<p>Помимо мощного железа, Vortex Gaming Station оснащена большим объемом оперативной памяти DDR4-3200 объемом 32 ГБ. Это позволяет запускать несколько приложений одновременно без снижения производительности, что особенно важно для многозадачных пользователей.</p>"),
				"rewrite" => "clauses/obzory-tovarov/obzor-igrovoy-stantsii-vortex-gaming-station-ideal",
				"sort" => 5,
				"images" => array("859_obzor-igrovoy-stantsii-vortex-ga.webp"),
			),
			array(
				"id" => 3,
				"cat_id" => 1,
				"name" => array("Изящество и стиль: обзор коктейльного платья с ассиметричным подолом"),
				"anons" => array("<p><span>На пороге важного мероприятия или долгожданного вечера, когда хочется выглядеть особенно элегантно и привлекательно, на помощь придет это изумительное коктейльное платье. Выполненное из мягкой и приятной на ощупь ткани, оно идеально подчеркнет вашу фигуру, придавая образу женственности и изящества.</span></p>"),
				"text" => array("<p>Отличительной особенностью этого платья является его ассиметричный подол. С одной стороны, он открывает стройные ножки, добавляя в образ нотку дерзости и сексуальности, а с другой &ndash; скрывает возможные недостатки, делая силуэт более гармоничным и пропорциональным.</p>
<p>Дополняет платье V-образный вырез, который визуально удлиняет шею и делает акцент на зоне декольте. Тонкие бретели и приталенный силуэт подчеркивают женственные формы, а длина миди придает образу изысканности и сдержанности.</p>
<p>Это платье станет идеальным выбором для любого мероприятия, будь то корпоративная вечеринка, светский прием или романтический ужин. Оно позволит вам чувствовать себя уверенно и комфортно, привлекая восхищенные взгляды окружающих.</p>
<p>Не упустите возможность приобрести это изумительное платье для своего гардероба и стать настоящей королевой вечера!</p>"),
				"rewrite" => "clauses/obzory-tovarov/izyaschestvo-i-stil-obzor-kokteylnogo-platya-s-ass",
				"sort" => 6,
				"images" => array("860_izyaschestvo-i-stil-obzor-ko.webp"),
			),
			array(
				"id" => 4,
				"cat_id" => 1,
				"name" => array("Укороченная рубашка - must-have женского гардероба"),
				"anons" => array("<p><span>Укороченные рубашки с воротником-стойкой стали настоящим хитом последних сезонов и уверенно держатся в топе модных трендов. Такая одежда идеально подходит для создания стильного и комфортного образа, подчеркивая индивидуальность и чувство стиля своей обладательницы.</span></p>"),
				"text" => array("<p>Рубашка с укороченным рукавом и воротником-стойкой станет отличным выбором для теплого времени года. Она прекрасно сочетается с различными видами низа, будь то узкие джинсы или классические брюки. Кроме того, такая модель рубашки идеально подойдет для создания многослойных образов, сочетаясь с легкими кардиганами, жилетами или кожаными куртками.</p>
<p>Для пошива таких рубашек используются различные материалы, от классического хлопка до более изысканных тканей, таких как шелк или шифон. Благодаря этому, каждая модница сможет подобрать модель, которая идеально впишется в ее гардероб и станет основой для множества стильных образов.</p>
<p>Воротник-стойка не только придает рубашке элегантный и изысканный вид, но и обеспечивает дополнительный комфорт при ношении. Он защищает от ветра и холода, а также скрывает мелкие недостатки кожи и несовершенства фигуры.</p>"),
				"rewrite" => "clauses/obzory-tovarov/ukorochennaya-rubashka---must-have-zhenskogo-garde",
				"sort" => 4,
				"images" => array("861_ukorochennaya-rubashka---must-have-.webp"),
			),
			array(
				"id" => 8,
				"cat_id" => 2,
				"name" => array("Советы по выбору товара: классические шторы “Изящество”", "Product selection tips: classic curtains “Grace”"),
				"anons" => array("<p><span>В данной статье мы рассмотрим некоторые советы, которые помогут вам выбрать идеальные классические шторы &ldquo;Изящество&rdquo;.</span></p>", "<p>In this article, we will look at some tips that will help you choose the perfect classic curtains &ldquo;Grace\".</p>"),
				"text" => array("<ol>
<li>
<p>Определите свой бюджет. Прежде всего, определитесь с тем, сколько вы готовы потратить на шторы. Это поможет вам сузить круг возможных вариантов.</p>
</li>
<li>
<p>Выберите ткань. Шторы &ldquo;Изящество&rdquo; могут быть выполнены из различных материалов, таких как хлопок, лен, шелк или вискоза. Учитывайте, что более тяжелые ткани могут лучше защитить от света и шума, в то время как более легкие ткани выглядят более воздушными и элегантными.</p>
</li>
<li>
<p>Выберите цвет. Цвет штор должен гармонировать с интерьером комнаты. Если вы хотите сделать акцент на шторах, выберите контрастный цвет. Если же вы хотите, чтобы шторы были менее заметными, выбирайте оттенок, близкий к цвету стен.</p>
</li>
<li>
<p>Учтите размер окна. Размер штор должен соответствовать размеру окна. Если окно большое, выбирайте шторы большего размера, чтобы они выглядели гармонично. Если окно маленькое, выбирайте шторы меньшего размера, чтобы не перегружать интерьер.</p>
</li>
<li>
<p>Учитывайте освещение. Если комната светлая, выбирайте темные шторы, чтобы защитить помещение от лишнего света.</p>
</li>
</ol>", "<p>Determine your budget. First of all, decide how much you are willing to spend on curtains. This will help you narrow down the range of possible options.</p>
<p>Choose a fabric. Grace curtains can be made of various materials such as cotton, linen, silk or viscose. Keep in mind that heavier fabrics can better protect against light and noise, while lighter fabrics look more airy and elegant.</p>
<p>Choose a color. The color of the curtains should be in harmony with the interior of the room. If you want to emphasize the curtains, choose a contrasting color. If you want the curtains to be less noticeable, choose a shade close to the color of the walls.</p>
<p>Consider the size of the window. The size of the curtains must match the size of the window. If the window is large, choose large curtains to make them look harmonious. If the window is small, choose smaller curtains so as not to overload the interior.</p>
<p>Consider the lighting. If the room is bright, choose dark curtains to protect the room from excess light.</p>"),
				"rewrite" => "clauses/sovety-po-vyboru-tovarov/sovety-po-vyboru-tovara-klassicheskie-shtory-izyas",
				"sort" => 10,
				"images" => array("854_sovety-po-vyboru-tovara-kl.webp"),
			),
			array(
				"id" => 9,
				"cat_id" => 2,
				"name" => array("Советы по выбору бензопилы для домашних нужд"),
				"anons" => array("<p><span>Бензопила &ndash; полезный инструмент для многих домашних задач, таких как распил дров, обрезка деревьев и даже строительство небольших сооружений. Но как выбрать подходящую бензопилу, учитывая множество моделей и характеристик на рынке?</span></p>"),
				"text" => array("<p>В этой статье мы дадим несколько советов, которые помогут вам сделать правильный выбор.</p>
<ol>
<li>
<p>Определитесь с задачами<br>Прежде всего, определите, для каких задач вам нужна бензопила. Если вам нужно распиливать дрова для камина, то вам подойдет стандартная модель с шиной длиной около 16 дюймов. Если же вам нужно обрезать деревья или выполнять строительные работы, то лучше выбрать более мощную модель с длинной шиной.</p>
</li>
<li>
<p>Мощность<br>Мощность бензопилы влияет на ее производительность и способность справляться с различными задачами. Для домашних нужд обычно достаточно бензопилы мощностью от 1,5 до 3,0 кВт. Однако если вы планируете использовать бензопилу для тяжелых работ, то стоит выбрать модель с большей мощностью.</p>
</li>
<li>
<p>Вес<br>Вес бензопилы также играет важную роль, особенно если вы планируете работать с ней на протяжении длительного времени. Чем меньше вес, тем меньше нагрузка на спину и руки.</p>
</li>
</ol>"),
				"rewrite" => "clauses/sovety-po-vyboru-tovarov/sovety-po-vyboru-benzopily-dlya-domashnikh-nuzhd",
				"sort" => 12,
				"images" => array("855_sovety-po-vyboru-benzopily-.webp"),
			),
			array(
				"id" => 10,
				"cat_id" => 2,
				"name" => array("Советы по выбору декора и интерьера для ремонта квартиры"),
				"anons" => array("<p><span>Ремонт квартиры &ndash; это процесс, который может быть как сложным, так и увлекательным. В этой статье мы поделимся некоторыми советами по выбору декора и интерьера, которые помогут сделать вашу квартиру уютной и стильной.</span></p>"),
				"text" => array("<p>Определите стиль интерьера Прежде всего, определитесь со стилем интерьера, который вам нравится. Это может быть классический, современный, скандинавский или любой другой стиль. Выбор стиля зависит от ваших предпочтений и характера.</p>
<p>Выберите цветовую гамму После того, как вы определились со стилем, выберите цветовую гамму для вашей квартиры. Цвета должны гармонировать друг с другом и создавать приятную атмосферу. Не забывайте о том, что светлые тона делают пространство более воздушным, а темные &ndash; более уютным.</p>
<p>Добавьте акценты Выберите акценты, которые будут подчеркивать ваш стиль интерьера. Это могут быть картины, вазы, подушки или другие предметы декора. Акценты должны быть яркими и привлекательными, но не перегружать пространство.</p>
<p>Используйте натуральные материалы Натуральные материалы, такие как дерево, камень и текстиль, добавят вашей квартире уюта и тепла. Они также могут быть использованы для создания акцентов или в качестве основных материалов для мебели и отделки.</p>"),
				"rewrite" => "clauses/sovety-po-vyboru-tovarov/sovety-po-vyboru-dekora-i-interera-dlya-remonta-kv",
				"sort" => 11,
				"images" => array("856_sovety-po-vyboru-dekora-i-i.webp"),
			),
			array(
				"id" => 11,
				"cat_id" => 2,
				"name" => array("Советы по выбору парфюма: как найти идеальный аромат"),
				"anons" => array("<p><span>Выбор парфюма &ndash; это всегда сложный и волнующий процесс. Ведь аромат &ndash; это то, что может рассказать о вас многое, дополнить ваш образ и подчеркнуть индивидуальность. В данной статье мы расскажем о том, как правильно выбрать парфюм, чтобы он стал вашим верным спутником и любимым аксессуаром.</span></p>"),
				"text" => array("<p>Определитесь с типом аромата Прежде всего, вам необходимо определиться с типом аромата, который вы хотите. Ароматы делятся на несколько категорий: свежие, цветочные, фруктовые, древесные, восточные и т.д. Выберите тот тип аромата, который вам больше всего нравится и подходит под ваш характер.</p>
<p>Обратите внимание на ноты аромата После того, как вы выбрали тип аромата, обратите внимание на его ноты. Большинство ароматов состоят из трех основных нот: верхних, средних и базовых. Верхние ноты раскрываются сразу после нанесения парфюма и длятся недолго. Средние ноты раскрываются через некоторое время после нанесения и являются основными нотами аромата. Базовые ноты раскрываются последними и остаются на коже дольше всего.</p>
<p>Протестируйте аромат на себе Перед тем, как купить парфюм, обязательно протестируйте его на себе. Нанесите несколько капель на запястье и подождите некоторое время, чтобы аромат раскрылся полностью.</p>"),
				"rewrite" => "clauses/sovety-po-vyboru-tovarov/sovety-po-vyboru-parfyuma-kak-nayti-idealnyy-aroma",
				"sort" => 9,
				"images" => array("857_sovety-po-vyboru-parfyuma-k.webp"),
			),
			array(
				"id" => 13,
				"cat_id" => 3,
				"name" => array("Ароматные новинки: обзор недавно появившихся товаров на сайте", "Fragrant novelties: an overview of recently appeared products on the site"),
				"anons" => array("<p><span>Привет, дорогие друзья! Сегодня мы рады представить вам обзор недавно появившихся товаров на нашем сайте. И первый из них &ndash; ароматические палочки.</span></p>", "<p>Hello, dear friends! Today we are pleased to present you an overview of the recently appeared products on our website. And the first of them is incense sticks.</p>"),
				"text" => array("<p>Ароматные палочки &ndash; это не просто способ наполнить ваш дом приятным запахом, но и возможность создать особую атмосферу, расслабиться и насладиться спокойствием. Они идеально подходят для медитаций, йоги и сеансов релаксации.</p>
<p>Мы предлагаем широкий ассортимент ароматических палочек, изготовленных из натуральных ингредиентов. В нашем каталоге вы найдете палочки с ароматами лаванды, жасмина, ванили, корицы и многих других.</p>
<p>Каждый аромат имеет свои особенности и свойства. Например, лаванда успокаивает и помогает снять стресс, жасмин улучшает настроение и способствует концентрации, ваниль создает атмосферу уюта и комфорта, а корица согревает и наполняет энергией.</p>
<p>Вы можете выбрать ароматические палочки как для себя, так и в подарок. Они станут отличным дополнением к интерьеру вашего дома или офиса, а также помогут создать неповторимую атмосферу на любом мероприятии.</p>
<p>Не забывайте, что выбор аромата &ndash; это индивидуальный процесс, и мы рекомендуем попробовать несколько вариантов, чтобы найти тот, который подходит именно вам.</p>
<p>Удачных покупок и приятных ароматов!</p>", "<p>Scented sticks are not just a way to fill your home with a pleasant smell, but also an opportunity to create a special atmosphere, relax and enjoy the tranquility. They are ideal for meditation, yoga and relaxation sessions.</p>
<p>We offer a wide range of incense sticks made from natural ingredients. In our catalog you will find sticks with aromas of lavender, jasmine, vanilla, cinnamon and many others.</p>
<p>Each fragrance has its own characteristics and properties. For example, lavender soothes and helps relieve stress, jasmine improves mood and promotes concentration, vanilla creates an atmosphere of coziness and comfort, and cinnamon warms and fills with energy.</p>
<p>You can choose incense sticks for yourself or as a gift. They will be a great addition to the interior of your home or office, and will also help create a unique atmosphere at any event.</p>
<p>Don't forget that choosing a fragrance is an individual process, and we recommend trying several options to find the one that suits you.</p>
<p>Have a good shopping and pleasant aromas!</p>"),
				"rewrite" => "clauses/instruktsii/aromatnye-novinki-obzor-nedavno-poyavivshikhsya-to",
				"sort" => 16,
				"images" => array("850_aromatnye-novinki-obzor-ne.webp"),
				"rel" => 16,
			),
			array(
				"id" => 14,
				"cat_id" => 3,
				"name" => array("Новый стиральный порошок на сайте: неизвестный, но эффективный!"),
				"anons" => array("<p><span>Если вы ищете новый стиральный порошок, который бы соответствовал вашим требованиям к качеству стирки и при этом был бы еще и экономичным, то у нас есть отличная новость! На сайте появился новый стиральный порошок от неизвестного ранее производителя, который уже успел завоевать доверие многих покупателей.</span></p>"),
				"text" => array("<p>Особенность этого стирального порошка заключается в его уникальной формуле, которая позволяет эффективно удалять даже самые сложные загрязнения. Кроме того, он обладает приятным ароматом, который сохраняется на белье после стирки.</p>
<p>Но самое главное &ndash; этот стиральный порошок очень экономичен в использовании. Одной упаковки хватает на длительное время, что делает его идеальным выбором для тех, кто ценит свое время и деньги.</p>
<p>Итак, если вы еще не пробовали этот новый стиральный порошок, то рекомендуем попробовать! Уверены, что он станет вашим верным помощником в борьбе за чистоту и свежесть вашего белья!</p>"),
				"rewrite" => "clauses/instruktsii/novyy-stiralnyy-poroshok-na-sayte-neizvestnyy-no-e",
				"sort" => 14,
				"images" => array("851_novyy-stiralnyy-poroshok-n.webp"),
				"rel" => 16,
			),
			array(
				"id" => 15,
				"cat_id" => 3,
				"name" => array("Клетка для домашней экзотической птицы: идеальное жилье для вашего питомца"),
				"anons" => array("<p><span>Клетка для домашней птицы &ndash; это не только место жительства вашего экзотического питомца, но и элемент декора, который станет украшением вашего дома. Наша компания предлагает широкий ассортимент клеток, которые подойдут для разных видов птиц и удовлетворят любые потребности владельцев.</span></p>"),
				"text" => array("<p>При выборе клетки для птицы следует учитывать несколько важных параметров. Во-первых, размеры клетки должны быть достаточными для комфортного проживания птицы. Во-вторых, материал клетки должен быть прочным и долговечным. В-третьих, клетка должна быть оборудована всем необходимым для комфортной жизни птицы: кормушкой, поилкой, жердочками и местом для отдыха.</p>
<p>Наши клетки изготовлены из качественных материалов и имеют стильный дизайн, который подойдет для любого интерьера. Мы предлагаем как стандартные клетки, так и клетки с дополнительными функциями, такими как автоматическая кормушка и система очистки.</p>
<p>Приобретая клетку для домашней птицы у нас, вы можете быть уверены в качестве продукции и профессионализме наших специалистов. Мы всегда рады помочь вам с выбором клетки, которая подойдет именно вашему питомцу и будет радовать вас своей красотой и функциональностью.</p>"),
				"rewrite" => "clauses/instruktsii/kletka-dlya-domashney-ekzoticheskoy-ptitsy-idealno",
				"sort" => 13,
				"images" => array("852_kletka-dlya-domashney-ekzoti.webp"),
				"rel" => array(13, 14, 15),
			),
			array(
				"id" => 16,
				"cat_id" => 3,
				"name" => array("Новомодная штукатурка: преображение вашего дома"),
				"anons" => array("<p><span>Хотите обновить интерьер своей квартиры без особых затрат? Тогда наш новый товар &ndash; декоративная штукатурка &ndash; именно то, что вам нужно! Этот уникальный материал позволяет создавать самые разнообразные фактуры и эффекты на стенах, имитируя различные поверхности &ndash; от дерева до мрамора.</span></p>"),
				"text" => array("<p>Кроме того, наша штукатурка обладает повышенной прочностью и долговечностью, что гарантирует ее долговечность и сохранение первоначального вида на долгие годы. А благодаря широкой цветовой гамме, вы сможете подобрать идеальный оттенок для своего интерьера.</p>
<p>Однако самое смешное в нашей штукатурке &ndash; это ее способность к самообновлению! Да-да, не удивляйтесь! Если вдруг на стене появится пятно или царапина, штукатурка сама обновит себя, превратив повреждение в оригинальное украшение.</p>
<p>Так что не упустите свой шанс преобразить свой дом с помощью нашей декоративной штукатурки! Смешные узоры и яркие цвета &ndash; гарантия того, что ваш интерьер будет неповторимым и запоминающимся!</p>"),
				"rewrite" => "clauses/instruktsii/novomodnaya-shtukaturka-preobrazhenie-vashego-doma",
				"sort" => 15,
				"images" => array("853_novomodnaya-shtukaturka-pre.webp"),
				"rel" => array(13, 14, 15),
			),
        ),
    );
}
