<?php
/**
 * Настройки модуля
 *
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    7.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2021 OOO «Диафан» (http://www.diafan.ru/)
 */

if (! defined('DIAFAN'))
{
	$path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}

/**
 * Images_admin_config
 */
class Images_admin_config extends Frame_admin
{
	/**
	 * @var array поля в базе данных для редактирования
	 */
	public $variables = array (
		'base' => array (
			'hr1' => array(
				'type' => 'title',
				'name' => 'Основные',
			),
			'hash_compare' => array(
				'type' => 'checkbox',
				'name' => 'Загружать только уникальные изображения',
				'help' => 'Если отмечено, то при загрузке изображений на сайт, будет проводится проверка на их уникальность. Вместо загрузки дублирующего изображения будет создана ссылка на оригинальное изображение.',
			),
			'hash_refresh' => array(
				'type' => 'function',
				'no_save' => true,
				'hide' => true,
				'depend' => 'hash_compare',
			),
			'max_image_size' => array(
				'type' => 'numtext',
				'name' => 'Максимальный размер загружаемого изображения (в пикселах)',
				'help' => 'Максимальный размер загружаемого изображения (в пикселах)',
			),
		),
	);

	/**
	 * @var array названия табов
	 */
	public $tabs_name = array(
		'base' => 'Основные настройки',
	);

	/**
	 * @var array настройки модуля
	 */
	public $config = array (
		//'tab_card', // использование вкладок
		'config', // файл настроек модуля
	);

	/**
	 * Редактирование поля "Генерация хэш значений имеющихся на сайте изображений"
	 *
	 * @return void
	 */
	public function edit_config_variable_hash_refresh()
	{
		echo '
		<div class="unit depend_field" id="hash_refresh" depend="hash_compare">';
		if(DB::query_result("SELECT COUNT(*) FROM {images} WHERE hash='' AND trash='0' LIMIT 1") > 0)
		{
			echo '
			<div class="inform_orange">
				'.$this->diafan->_('На сайте присутствуют изображения без соответствующих хэш-записей.%sРекомендуем для корректной работы модуля обновить хэш изображений.', '<br />').'
			</div>';
		}
		echo '
			<button id="hash_refresh_button" class="btn btn_blue btn_small"">'.$this->diafan->_("Обновить хэш изображений").'</button>
		</div>';
	}

	/**
	 * Редактирование поля "Максимальный размер загружаемого изображения"
	 *
	 * @return void
	 */
	public function edit_config_variable_max_image_size()
	{
		$max_image_size = $this->diafan->_images->get_max_image_size();
		echo '
		<div class="unit" id="'.$this->diafan->key.'">
			<div class="infofield">'.$this->diafan->variable_name().$this->diafan->help().'</div>
			<input type="text" class="number" name="'.$this->diafan->key.'" value="'.($this->diafan->value ? $this->diafan->value : $max_image_size).'"'.($max_image_size ? ' placeholder="'.$max_image_size.'"' : '').'>
		</div>';
	}
}
