<?php
/**
 * Установка модуля
 *
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    7.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2021 OOO «Диафан» (http://www.diafan.ru/)
 */

if (! defined('DIAFAN'))
{
	$path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}

class News_install extends Install
{
	/**
	 * @var string название
	 */
	public $title = "Новости";

	/**
	 * @var array таблицы в базе данных
	 */
	public $tables = array(
		array(
			"name" => "news",
			"comment" => "Новости",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "TEXT",
					"comment" => "название",
					"multilang" => true,
				),
				array(
					"name" => "act",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "показывать на сайте: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "map_no_show",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "не показывать на карте сайта: 0 - нет, 1 - да",
				),
				array(
					"name" => "changefreq",
					"type" => "ENUM( 'always', 'hourly', 'daily', 'weekly', 'monthly', 'yearly', 'never' ) NOT NULL DEFAULT 'always'",
					"comment" => "Changefreq для sitemap.xml",
				),
				array(
					"name" => "priority",
					"type" => "VARCHAR(3) NOT NULL DEFAULT ''",
					"comment" => "Priority для sitemap.xml",
				),
				array(
					"name" => "noindex",
					"type" => "ENUM('0','1') NOT NULL DEFAULT '0'",
					"comment" => "не индексировать: 0 - нет, 1 - да",
				),
				array(
					"name" => "created",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата создания",
				),
				array(
					"name" => "date_start",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата начала показа",
				),
				array(
					"name" => "date_finish",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата окончания показа",
				),
				array(
					"name" => "cat_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор основной категории из таблицы {news_category}",
				),
				array(
					"name" => "site_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор страницы сайта из таблицы {site}",
				),
				array(
					"name" => "keywords",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "ключевые слова, тег Keywords",
					"multilang" => true,
				),
				array(
					"name" => "descr",
					"type" => "TEXT",
					"comment" => "описание, тэг Description",
					"multilang" => true,
				),
				array(
					"name" => "canonical",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "канонический тег",
					"multilang" => true,
				),
				array(
					"name" => "title_meta",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "заголовок окна в браузере, тег Title",
					"multilang" => true,
				),
				array(
					"name" => "anons",
					"type" => "TEXT",
					"comment" => "анонс",
					"multilang" => true,
				),
				array(
					"name" => "anons_plus",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "добавлять анонс к описанию: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "text",
					"type" => "LONGTEXT",
					"comment" => "описание",
					"multilang" => true,
				),
				array(
					"name" => "prior",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "важно, всегда сверху: 0 - нет, 1 - да",
				),
				array(
					"name" => "import",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "элемент только что импортирован: 0 - нет, 1 - да",
				),
				array(
					"name" => "import_id",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "собственный идентификатор элемента при импорте",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "timeedit",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "время последнего изменения в формате UNIXTIME",
				),
				array(
					"name" => "admin_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "пользователь из таблицы {users}, добавивший или первый отредктировавший новость в административной части",
				),
				array(
					"name" => "access",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "доступ ограничен: 0 - нет, 1 - да",
				),
				array(
					"name" => "theme",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон страницы сайта",
				),
				array(
					"name" => "view",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон модуля",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY site_id (site_id)",
				"KEY import_id (`import_id`)",
			),
		),
		array(
			"name" => "news_rel",
			"comment" => "Связи похожих новостей",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор новости из таблицы {news}",
				),
				array(
					"name" => "rel_element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор связанной новости из таблицы {news}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "news_category",
			"comment" => "Категории новостей",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "TEXT",
					"comment" => "название",
					"multilang" => true,
				),
				array(
					"name" => "act",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "показывать на сайте: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "map_no_show",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "не показывать на карте сайта: 0 - нет, 1 - да",
				),
				array(
					"name" => "changefreq",
					"type" => "ENUM( 'always', 'hourly', 'daily', 'weekly', 'monthly', 'yearly', 'never' ) NOT NULL DEFAULT 'always'",
					"comment" => "Changefreq для sitemap.xml",
				),
				array(
					"name" => "priority",
					"type" => "VARCHAR(3) NOT NULL DEFAULT ''",
					"comment" => "Priority для sitemap.xml",
				),
				array(
					"name" => "noindex",
					"type" => "ENUM('0','1') NOT NULL DEFAULT '0'",
					"comment" => "не индексировать: 0 - нет, 1 - да",
				),
				array(
					"name" => "parent_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор родителя из таблицы {news_category}",
				),
				array(
					"name" => "count_children",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "количество вложенных категорий",
				),
				array(
					"name" => "site_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор страницы сайта из таблицы {site}",
				),
				array(
					"name" => "keywords",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "ключевые слова, тег Keywords",
					"multilang" => true,
				),
				array(
					"name" => "descr",
					"type" => "TEXT",
					"comment" => "описание, тэг Description",
					"multilang" => true,
				),
				array(
					"name" => "canonical",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "канонический тег",
					"multilang" => true,
				),
				array(
					"name" => "title_meta",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "заголовок окна в браузере, тег Title",
					"multilang" => true,
				),
				array(
					"name" => "anons",
					"type" => "TEXT",
					"comment" => "анонс",
					"multilang" => true,
				),
				array(
					"name" => "anons_plus",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "добавлять анонс к описанию: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "text",
					"type" => "TEXT",
					"comment" => "описание",
					"multilang" => true,
				),
				array(
					"name" => "import",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "категория только что импортирован: 0 - нет, 1 - да",
				),
				array(
					"name" => "import_id",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "собственный идентификатор категории при импорте",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "timeedit",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "время последнего изменения в формате UNIXTIME",
				),
				array(
					"name" => "admin_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "пользователь из таблицы {users}, добавивший или первый отредктировавший категорию в административной части",
				),
				array(
					"name" => "access",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "доступ ограничен: 0 - нет, 1 - да",
				),
				array(
					"name" => "theme",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон страницы сайта",
				),
				array(
					"name" => "view",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон модуля",
				),
				array(
					"name" => "view_rows",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон модуля для элементов в списке категории",
				),
				array(
					"name" => "view_element",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон модуля для элементов в категории",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY parent_id (parent_id)",
				"KEY site_id (site_id)",
				"KEY import_id (`import_id`)",
			),
		),
		array(
			"name" => "news_category_parents",
			"comment" => "Родительские связи категорий новостей",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT( 11 ) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT( 11 ) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор категории из таблицы {news_category}",
				),
				array(
					"name" => "parent_id",
					"type" => "INT( 11 ) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор категории-родителя из таблицы {news_category}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "news_category_rel",
			"comment" => "Связи новостей и категорий",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор новости из таблицы {news}",
				),
				array(
					"name" => "cat_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор категории из таблицы {news_category}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY cat_id (`cat_id`)",
			),
		),
		array(
			"name" => "news_counter",
			"comment" => "Счетчик просмотров новостей",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор новости из таблицы {news}",
				),
				array(
					"name" => "count_view",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "количество просмотров",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY element_id (`element_id`)",
			),
		),
	);

	/**
	 * @var array записи в таблице {modules}
	 */
	public $modules = array(
		array(
			"name" => "news",
			"admin" => true,
			"site" => true,
			"site_page" => true,
		),
	);

	/**
	 * @var array меню административной части
	 */
	public $admin = array(
		array(
			"name" => "Новости",
			"rewrite" => "news",
			"group_id" => 1,
			"sort" => 5,
			"act" => true,
			"docs" => "http://www.diafan.ru/moduli/novosti/",
			"children" => array(
				array(
					"name" => "Новости",
					"rewrite" => "news",
					"act" => true,
				),
				array(
					"name" => "Категории",
					"rewrite" => "news/category",
					"act" => true,
				),
				array (
					'name' => 'Статистика',
					'rewrite' => 'news/counter',
					'act' => true,
				),
				array(
					"name" => "Настройки",
					"rewrite" => "news/config",
				),
			)
		),
	);


	/**
	 * @var array страницы сайта
	 */
	public $site = array(
		array(
			"name" => array('Новости', 'News'),
			"module_name" => "news",
			"rewrite" => "news",
			"menu" => 1,
			"parent_id" => 2,
		),
	);

	/**
	 * @var array настройки
	 */
	public $config = array(
		array(
			"name" => "images_element",
			"value" => "1",
		),
		array(
			"name" => "images_webp_element",
			"value" => "1",
		),
		array(
			"name" => "images_webp_cat",
			"value" => "1",
		),
		array(
			"name" => "count_list",
			"value" => "4",
		),
		array(
			"name" => "count_child_list",
			"value" => "4",
		),
		array(
			"name" => "children_elements",
			"value" => "1",
		),
		array(
			"name" => "list_img_element",
			"value" => "1",
		),
		array(
			"name" => "use_animation",
			"value" => "1",
		),
		array(
			"name" => "format_date",
			"value" => "3",
		),
		array(
			"name" => "nastr",
			"value" => "12",
		),
		array(
			"name" => "nastr_cat",
			"value" => "10",
		),
		array(
			"name" => "rel_two_sided",
			"value" => "1",
		),
		array(
			"name" => "counter",
			"value" => "1",
		),
		array(
			"name" => "title_tpl",
			"value" => "%name, %category",
		),
		array(
			"name" => "keywords_tpl",
			"value" => "%name %category",
		),
		array(
			"name" => "images_variations_element",
			"value" => "a:2:{s:4:\"webp\";i:1;s:2:\"vs\";a:2:{i:0;a:2:{s:4:\"name\";s:5:\"large\";s:2:\"id\";s:1:\"3\";}i:1;a:2:{s:4:\"name\";s:6:\"medium\";s:2:\"id\";s:1:\"1\";}}}",
		),
		array(
			"name" => "cat",
			"value" => '1',
		),
		array(
			"name" => "show_more",
			"value" => '1',
		),
	);

	/**
	 * @var array демо-данные
	 */
	public $demo = array(
		'config' => array(
			array(
				'name' => 'rating',
				'value' => 1,
			),
			array(
				'name' => 'comments',
				'value' => 1,
			),
			array(
				'name' => 'tags',
				'value' => 1,
			),
		),
		'news_category' => array(
			array(
				"id" => 1,
				"name" => array("Новинки и тренды в мире товаров", "New products and trends in the world of goods"),
				"anons" => array("<p>Предоставляем актуальную информацию о последних достижениях и инновациях в области производства и продажи товаров.</p>", "<p>All the latest travel news in our news section.</p>"),
				"text" => array("<p>Здесь вы найдете обзоры новых продуктов, аналитические статьи о тенденциях развития рынка, а также новости о сотрудничестве с известными брендами и запуске новых партнерских программ. Следите за нашей лентой новостей, чтобы всегда быть в курсе последних событий и не пропустить интересные предложения и акции!</p>", "<p>Below is a list of current tourism news that our experts pick around InfoSpace and quickly spread to please all our customers. Welcome, read, rate, comment, share with friends! All for you!</p>"),
				"rewrite" => "news/novinki-i-trendy-v-mire-tovarov",
				"sort" => 1,
			),
			array(
				"id" => 2,
				"name" => array("Новости компании", "Company news"),
				"anons" => array("<p>Категория новостей &ldquo;Новости компании&rdquo; на нашем сайте содержит информацию о текущих событиях и мероприятиях, проводимых нашей компанией.</p>"),
				"text" => array("<p>Здесь вы можете узнать о новых проектах, успехах и достижениях нашего коллектива, а также о планах на будущее. Мы также публикуем интервью с сотрудниками, рассказываем о корпоративных мероприятиях и акциях, предоставляем отчеты о работе различных подразделений. Оставайтесь с нами, чтобы быть в курсе всех новостей и событий, связанных с нашей компанией!</p>"),
				"rewrite" => "news/novosti-kompanii",
				"sort" => 2,
			),
			array(
				"id" => 3,
				"name" => array("Скидки и акции", "Discounts and promotions"),
				"anons" => array("<p>Категория новостей &ldquo;Скидки и акции&rdquo; на нашем сайте поможет вам всегда быть в курсе самых выгодных предложений и сэкономить на покупках.</p>"),
				"text" => array("<p>Мы регулярно публикуем информацию о скидках, распродажах и специальных предложениях от нашего магазина, а также от наших партнеров. Следите за обновлениями, чтобы не упустить свою выгоду!</p>"),
				"rewrite" => "news/skidki-i-aktsii",
				"sort" => 3,
			),
		),
		'news' => array(
			array(
				"id" => 5,
				"cat_id" => 2,
				"name" => array("Новые поставщики бытовых товаров активно приходят на замену импорту"),
				"anons" => array("<p>В последние годы наблюдается значительный рост числа отечественных производителей бытовых товаров, которые активно замещают импортные аналоги. Новые игроки рынка предлагают продукцию высокого качества по доступным ценам, что способствует развитию импортозамещения в данной сфере.</p>"),
				"text" => array("<p>Среди наиболее успешных примеров можно отметить компанию &ldquo;Домашний комфорт&rdquo;, которая специализируется на производстве бытовой техники и электроники. Компания использует передовые технологии и инновационные материалы, благодаря чему ее продукция пользуется большим спросом у потребителей.</p>
<p>Другим важным фактором, способствующим развитию импортозамещения, является государственная поддержка отечественных производителей. В частности, правительством были приняты меры по снижению таможенных пошлин на ввоз сырья и комплектующих, что делает производство бытовых товаров более выгодным.</p>
<p>Таким образом, импортозамещение в сфере бытовых товаров является одним из ключевых направлений развития экономики России. С каждым годом все больше отечественных производителей выходят на рынок, предлагая качественные и доступные товары, способные заменить импортные аналоги.</p>"),
				"rewrite" => "news/novosti-kompanii/novye-postavschiki-bytovykh-tovarov-aktivno-prikho",
				"images" => array("874_novye-postavschiki-bytovykh-t.webp"),
			),
			array(
				"id" => 6,
				"cat_id" => 2,
				"name" => array("Интернет-магазину исполнилось 20 лет!"),
				"anons" => array("<p>Прошло уже 20 лет с тех пор, как мы начали свою работу в интернете. За это время мы стали одним из крупнейших интернет-магазинов в стране. Мы гордимся тем, что смогли помочь миллионам людей найти то, что им нужно, и получить это быстро и удобно.</p>"),
				"text" => array("<p>За эти годы мы прошли через множество изменений и инноваций, но одно осталось неизменным - наша любовь к клиентам и желание предоставлять им лучший сервис. Мы продолжаем развиваться и улучшать качество наших услуг, чтобы оставаться лидерами в своей области.</p>
<p>Поздравляем всех наших клиентов и партнеров с этим знаменательным событием! Мы ценим вашу поддержку и обещаем продолжать радовать вас лучшими товарами и услугами.</p>"),
				"rewrite" => "news/novosti-kompanii/internet-magazinu-ispolnilos-20-let",
				"images" => array("872_internet-magazinu-ispolnil.webp"),
			),
			array(
				"id" => 7,
				"cat_id" => 2,
				"name" => array("Мы открыли первый магазин в Казахстане!"),
				"anons" => array("<p>Дорогие друзья, мы рады сообщить вам замечательную новость! Мы открыли наш первый магазин в Казахстане! Это большое событие для нашей компании, и мы гордимся тем, что можем предложить нашим клиентам еще больше качественных товаров и услуг.</p>"),
				"text" => array("<p>Теперь у жителей Казахстана есть возможность познакомиться с нашими брендами, оценить качество нашей продукции и сервис, который мы предоставляем. В нашем магазине вы найдете широкий ассортимент товаров, от одежды и обуви до аксессуаров и косметики.</p>
<p>Открытие магазина в Казахстане стало возможным благодаря поддержке наших партнеров и местных властей. Мы благодарны за оказанное доверие и надеемся на долгосрочное и плодотворное сотрудничество.</p>
<p>Приглашаем всех посетить наш магазин и насладиться покупками! Мы уверены, что вам понравится наш ассортимент и уровень обслуживания. Не упустите шанс стать частью нашей дружной команды и быть в курсе новинок и акций!</p>"),
				"rewrite" => "news/novosti-kompanii/my-otkryli-pervyy-magazin-v-kazakhstane",
				"images" => array("871_my-otkryli-pervyy-magazin-v.webp"),
			),
			array(
				"id" => 8,
				"cat_id" => 2,
				"name" => array("Лотерея с ноутбуком в качестве главного приза!"),
				"anons" => array("<p>Внимание, дорогие покупатели! В нашем магазине проводится уникальная лотерея, главным призом которой является ноутбук!</p>"),
				"text" => array("<p>Для участия в лотерее необходимо совершить покупку на определенную сумму и получить лотерейный билет. Чем больше сумма покупки, тем больше шансов на победу.</p>
<p>Лотерея продлится до определенной даты, после чего будут определены победители. Главный приз - ноутбук - получит счастливчик, чей билет окажется выигрышным.</p>
<p>Кроме главного приза, будут разыграны и другие призы - смартфоны, планшеты и аксессуары.</p>
<p>Приходите в наш магазин, совершайте покупки и участвуйте в лотерее с главным призом - ноутбуком! Удачи всем участникам!</p>"),
				"rewrite" => "news/novosti-kompanii/lotereya-s-noutbukom-v-kachestve-glavnogo-priza",
				"images" => array("870_lotereya-s-noutbukom-v-kaches.webp"),
			),
			array(
				"id" => 9,
				"cat_id" => 2,
				"name" => array("Новые удобные кухонные ножи в продаже!", "New handy kitchen knives are on sale!"),
				"anons" => array("<p>Сегодня мы рады сообщить о поступлении в продажу новых удобных кухонных ножей. Наши эксперты тщательно отобрали лучшие модели от ведущих производителей, чтобы вы могли наслаждаться процессом приготовления любимых блюд.</p>", "<p>Today we are pleased to announce the launch of new convenient kitchen knives. Our experts have carefully selected the best models from leading manufacturers so that you can enjoy the process of cooking your favorite dishes.</p>"),
				"text" => array("<p>Новые кухонные ножи обладают рядом преимуществ, которые делают их идеальным выбором для любой кухни. Во-первых, они изготовлены из высококачественной нержавеющей стали, что обеспечивает их долговечность и надежность. Во-вторых, эргономичные ручки ножей обеспечивают удобный и безопасный хват, предотвращая скольжение и усталость рук.</p>
<p>Широкий ассортимент кухонных ножей включает в себя модели для различных целей - от универсальных ножей до специализированных, таких как нож для чистки овощей или нож для хлеба. Каждый нож имеет свой уникальный дизайн и характеристики, что позволяет выбрать наиболее подходящий вариант для ваших кулинарных нужд.</p>
<p>Не пропустите возможность обновить свою кухонную коллекцию с новыми удобными кухонными ножами! Они станут надежным помощником на вашей кухне и помогут приготовить самые изысканные блюда.</p>
<p></p>", "<p>The new kitchen knives have a number of advantages that make them an ideal choice for any kitchen. Firstly, they are made of high-quality stainless steel, which ensures their durability and reliability. Secondly, the ergonomic handles of the knives provide a comfortable and secure grip, preventing slipping and hand fatigue.</p>
<p>The wide range of kitchen knives includes models for various purposes - from universal knives to specialized ones, such as a paring knife or a bread knife. Each knife has its own unique design and characteristics, which allows you to choose the most suitable option for your culinary needs.</p>
<p>Don't miss the opportunity to update your kitchen collection with new handy kitchen knives! They will become a reliable assistant in your kitchen and will help you prepare the most delicious dishes.</p>"),
				"rewrite" => "news/novosti-kompanii/novye-udobnye-kukhonnye-nozhi-v-prodazhe",
				"images" => array("869_novye-udobnye-kukhonnye-nozh.webp"),
			),
			array(
				"id" => 15,
				"cat_id" => 1,
				"name" => array("Новинка в мире мебели: царская кровать - новый тренд в интерьере", ""),
				"anons" => array("<p>Здравствуйте, дорогие читатели! Сегодня мы хотим поделиться с вами новостью о новинке в мире мебели - царской кровати. Этот тренд в дизайне интерьера становится все более популярным, и на то есть веские причины.</p>", ""),
				"text" => array("<p>Царская кровать выглядит очень стильно и современно. Она создает ощущение легкости и воздушности, делая интерьер более светлым и воздушным. Кроме того, такая кровать идеально вписывается в концепцию минимализма, которая сейчас очень актуальна.</p>
<p>Секрет кровати заключается в том, что она как бы &ldquo;парит&rdquo; над полом, создавая иллюзию невесомости.</p>
<p>В нашем магазине вы можете найти кровати самых разных размеров и стилей. Мы предлагаем модели с различными вариантами отделки - от классических до самых смелых и ярких.</p>
<p>Покупая кровать, вы не только украсите свой интерьер, но и сделаете свой сон более комфортным. Благодаря своей уникальной конструкции, такая кровать обеспечивает лучшую вентиляцию и удобное положение тела во время сна.</p>
<p>Не упустите возможность стать обладателем этой модной и стильной новинки!</p>", ""),
				"rewrite" => "news/novinki-i-trendy-v-mire-tovarov/novinka-v-mire-mebeli-paryaschaya-krovat---novyy-t",
				"images" => array("868_novinka-v-mire-mebeli-tsars.webp"),
				"rel" => 20,
			),
			array(
				"id" => 16,
				"cat_id" => 1,
				"name" => array("Женская куртка – отличный товар для стильных и уверенных в себе!", ""),
				"anons" => array("<p>Приветствуем вас, дорогие посетители нашего сайта! Мы рады представить вам новинку, которая обязательно придется по вкусу всем модницам &ndash; женскую куртку! Эта стильная и практичная вещь станет незаменимым атрибутом вашего гардероба и поможет выделиться из толпы.</p>", ""),
				"text" => array("<p>Женская куртка выполнена из качественных материалов, которые обеспечивают долговечность и комфорт при носке. Благодаря универсальному дизайну, она отлично сочетается с любыми образами, будь то джинсы и кроссовки или же элегантное платье.</p>
<p>Наш товар представлен в различных размерах и цветах, поэтому каждая женщина сможет подобрать для себя идеальный вариант. Мы уверены, что женская куртка станет вашим любимым предметом одежды и поможет создать множество ярких образов.</p>
<p>Спешите приобрести этот модный и стильный товар по привлекательной цене! Будьте в тренде и наслаждайтесь комфортом вместе с нашей женской курткой!</p>", ""),
				"rewrite" => "news/novinki-i-trendy-v-mire-tovarov/zhenskaya-kurtka--otlichnyy-tovar-dlya-stilnykh-i-",
				"images" => array("867_zhenskaya-kurtka--otlichnyy-.webp"),
				"rel" => 20,
			),
			array(
				"id" => 17,
				"cat_id" => 1,
				"name" => array("Детская одежда - лучший выбор для вашего малыша!"),
				"anons" => array("<p>Дорогие родители, рады представить вам новинки нашего магазина - детскую одежду! Теперь вы сможете одеть своего малыша стильно и комфортно, а главное - безопасно для его здоровья.</p>"),
				"text" => array("<p>У нас вы найдете широкий ассортимент товаров для детей всех возрастов: от новорожденных до подростков. Мы предлагаем качественную одежду от ведущих производителей, которая отличается прочностью, долговечностью и привлекательным дизайном.</p>
<p>Наша детская одежда изготовлена из натуральных материалов, что обеспечивает оптимальный микроклимат и предотвращает возникновение аллергических реакций. Кроме того, мы следим за модными тенденциями и регулярно обновляем наш ассортимент, чтобы ваш ребенок выглядел модно и стильно.</p>
<p>Не откладывайте покупку - заходите к нам и выбирайте одежду для своего малыша уже сегодня! Мы уверены, что вы останетесь довольны качеством нашего товара и уровнем обслуживания.</p>
<p>.</p>"),
				"rewrite" => "news/novinki-i-trendy-v-mire-tovarov/detskaya-odezhda---luchshiy-vybor-dlya-vashego-mal",
				"images" => array("866_detskaya-odezhda---luchshiy-vyb.webp"),
			),
			array(
				"id" => 18,
				"cat_id" => 1,
				"name" => array("Новинки и тренды в мире техники: что ждет нас в будущем?"),
				"anons" => array("<p>Технологии постоянно развиваются и меняются, и мир техники не является исключением. В этой статье мы рассмотрим некоторые из наиболее интересных новинок и трендов, которые ожидаются в ближайшем будущем.</p>"),
				"text" => array("<p>Одним из главных трендов является развитие искусственного интеллекта (AI) и машинного обучения. Эти технологии уже сейчас активно используются в различных отраслях, таких как медицина, транспорт и финансы. В будущем AI станет еще более распространенным и будет использоваться в повседневной жизни.</p>
<p>Еще одним важным трендом является развитие виртуальной и дополненной реальности (VR/AR). Эти технологии уже сегодня используются в игровой индустрии и в некоторых сферах производства. Однако в будущем они могут стать еще более популярными и использоваться в образовании, медицине и многих других областях.</p>
<p>Также стоит отметить развитие беспроводных технологий, таких как 5G и Li-Fi. Эти технологии позволят передавать данные быстрее и эффективнее, что может привести к появлению новых возможностей для бизнеса и потребителей.</p>
<p>Наконец, нельзя не упомянуть о развитии экологичных технологий, таких как электромобили и солнечные панели. Эти технологии уже сегодня становятся все более популярными, и в будущем их использование может стать еще более распространенным.</p>"),
				"rewrite" => "news/novinki-i-trendy-v-mire-tovarov/novinki-i-trendy-v-mire-tekhniki-chto-zhdet-nas-v-",
				"images" => array("865_novinki-i-trendy-v-mire-tekh.webp"),
			),
			array(
				"id" => 19,
				"cat_id" => 1,
				"name" => array("Удобные кроссовки - идеальный выбор для активной жизни!"),
				"anons" => array("<p>Здравствуйте, уважаемые читатели! Сегодня мы рады представить вашему вниманию нашу новинку - удобные кроссовки! Эти кроссовки созданы специально для тех, кто ценит комфорт и стиль, а также ведет активный образ жизни.</p>"),
				"text" => array("<p>Удобные кроссовки выполнены из качественных материалов, обеспечивающих долговечность и износостойкость. Они имеют удобную посадку на ноге и отличную амортизацию, что позволяет чувствовать себя комфортно даже при длительной ходьбе или беге.</p>
<p>Дизайн кроссовок сочетает в себе стиль и функциональность. Они доступны в различных цветовых решениях, что позволяет каждому выбрать модель по своему вкусу. Кроссовки идеально сочетаются с любой одеждой и станут отличным дополнением к вашему гардеробу.</p>
<p>Мы уверены, что удобные кроссовки станут вашим верным спутником в повседневной жизни и помогут сохранить здоровье ваших ног. Не откладывайте покупку, приходите к нам и оцените все преимущества нашей новинки!</p>"),
				"rewrite" => "news/novinki-i-trendy-v-mire-tovarov/udobnye-krossovki---idealnyy-vybor-dlya-aktivnoy-z",
				"images" => array("864_udobnye-krossovki---idealn.webp"),
			),
			array(
				"id" => 20,
				"cat_id" => 1,
				"name" => array("Женские туфли на шпильке - новый сногшибательный товар на нашем сайте!", "Women's stiletto shoes are a stunning new product on our website!"),
				"anons" => array("<p>Уважаемые покупатели! Мы рады сообщить вам о появлении нового сногшибательного товара на нашем сайте - женских туфель на шпильке! Эти туфли станут идеальным дополнением к вашему гардеробу и помогут создать неповторимый образ.</p>", "<p>Dear customers! We are pleased to inform you about the appearance of a new stunning product on our website - women's stiletto shoes! These shoes will be the perfect addition to your wardrobe and will help you create a unique look.</p>"),
				"text" => array("<p>Изготовленные из высококачественных материалов, наши туфли обладают стильным дизайном и комфортом, который позволит вам чувствовать себя уверенно на протяжении всего дня. Благодаря разнообразию моделей и цветов, вы обязательно найдете пару, которая идеально подойдет именно вам.</p>
<p>Особое внимание стоит уделить нашим ценам - мы предлагаем самые доступные цены на рынке, что делает покупку еще более привлекательной. Не упустите возможность обновить свой гардероб и приобрести новую пару сногшибательных туфель на шпильке уже сегодня!</p>
<p>Следите за обновлениями на нашем сайте, чтобы не пропустить акции и специальные предложения, а также делитесь своими впечатлениями о покупке в комментариях. Мы всегда рады вашим отзывам и предложениям! Удачных покупок!</p>", "<p>Made of high-quality materials, our shoes have a stylish design and comfort that will allow you to feel confident all day long. Thanks to the variety of models and colors, you will definitely find a pair that is perfect for you.</p>
<p>Special attention should be paid to our prices - we offer the most affordable prices on the market, which makes the purchase even more attractive. Don't miss the opportunity to update your wardrobe and purchase a new pair of stunning stiletto heels today!</p>
<p>Stay tuned on our website so that you don't miss promotions and special offers, and also share your impressions about the purchase in the comments. We are always glad to receive your feedback and suggestions! Happy shopping!</p>"),
				"rewrite" => "news/novinki-i-trendy-v-mire-tovarov/zhenskie-tufli-na-shpilke---novyy-snogshibatelnyy-",
				"images" => array("863_zhenskie-tufli-na-shpilke---n.webp"),
				"rel" => array(15, 16),
			),
			array(
				"id" => 21,
				"cat_id" => 3,
				"name" => array("Суперскидки на телевизоры с искусственным интеллектом. Спешите!", "Super jumps on AI TVs!"),
				"anons" => array("<p>У нас замечательная новость для всех, кто мечтает о покупке современного телевизора с искусственным интеллектом по супервыгодной цене! Только сейчас у вас есть уникальная возможность приобрести самые передовые модели телевизоров со скидками до 50%!</p>", "<p>We have great news for everyone who dreams of buying a modern TV with artificial intelligence at a super-competitive price! Only now you have a unique opportunity to purchase the most advanced TV models with discounts up to 50%!</p>"),
				"text" => array("<p>Не упустите свой шанс стать обладателем умного телевизора, который изменит ваше представление о просмотре любимых фильмов, сериалов и телепередач. С нашими скидками вы сможете наслаждаться невероятным качеством изображения и звука, а также пользоваться всеми преимуществами ИИ.</p>
<p>Телевизоры с ИИ - это не просто техника, это ваш личный ассистент, который всегда готов помочь:</p>
<ul>
<li>поиск информации в интернете прямо с вашего телевизора;</li>
<li>рекомендации по контенту на основе ваших предпочтений;</li>
<li>управление другими устройствами умного дома;</li>
<li>голосовой ассистент, готовый ответить на все ваши вопросы.</li>
</ul>", "<p>Don't miss your chance to become the owner of a smart TV that will change your idea of watching your favorite movies, TV series and TV shows. With our discounts, you can enjoy incredible image and sound quality, as well as enjoy all the benefits of AI.</p>
<p>AI TVs are not just a technique, they are your personal assistant who is always ready to help:</p>
<p>Search for information on the Internet directly from your TV;<br>content recommendations based on your preferences;<br>managing other smart home devices;<br>a voice assistant ready to answer all your questions.</p>"),
				"rewrite" => "news/skidki-i-aktsii/superskidki-na-televizory-s-iskusstvennym-intellek",
				"images" => array("862_superskidki-na-televizory-.webp"),
			),
		),
	);
}
