<?php
/**
 * Формирует данные для формы платежной системы T-Bank
 *
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    7.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2021 OOO «Диафан» (http://www.diafan.ru/)
 */

if (! defined('DIAFAN'))
{
	$path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}

require_once('TinkoffMerchantAPI.php');

class Payment_tbank_model extends Diafan
{
    /**
     * Формирует данные для платежной формы Т-банк
     *
     * @param array $params настройки платежной системы
     * @param array $pay данные о платеже
     * @return array
     */
    public function get($params, $pay)
    {
        $arrFields = array(
            'OrderId' => $pay['element_id'],
            'Amount' => $pay['summ'] * 100,
            'DATA' => 'Email=' . $pay['details']['email'] . '|connection_type=diafan'
        );

        $Tinkoff = new TinkoffMerchantAPI($params['tbank_terminal'], $params['tbank_password'], $params['tbank_gateway']);
        $request = $Tinkoff->buildQuery('Init', $arrFields);
        $request = json_decode($request);
        $result = array();

        if ($request->PaymentURL) {
            $result['url'] = $request->PaymentURL;
            $result['text'] = $pay['text'];
        }

        return $result;
    }

    /**
     * Генерирует токен
     *
     * @param $password
     * @return string
     */
    public static function getToken($data, $password)
    {
        $data['Password'] = $password;
        ksort($data);

        if (isset($data['Token'])) {
            unset($data['Token']);
        }

        $values = implode('', array_values($data));

        return hash('sha256', $values);
    }

    /**
     * Получает данные о платеже и платежной системе
     *
     * @param $elementId
     * @return mixed
     */

    public function getPaymentByElementId($elementId)
    {
        return DB::query_fetch_array("SELECT ph.*, p.params FROM {payment_history} as ph 
            JOIN {payment} as p on ph.payment_id=p.id
            WHERE ph.element_id = %d and p.payment = 'tbank' LIMIT 1", $elementId);
    }

}