<?php
/**
 * Установка модуля
 * 
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    7.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2021 OOO «Диафан» (http://www.diafan.ru/)
 */

if (! defined('DIAFAN'))
{
	$path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}

class Rating_install extends Install
{
	/**
	 * @var string название
	 */
	public $title = "Рейтинг";

	/**
	 * @var array таблицы в базе данных
	 */
	public $tables = array(
		array(
			"name" => "rating",
			"comment" => "Рейтинг элементов модулей",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор элемента модуля",
				),
				array(
					"name" => "module_name",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "название модуля",
				),
				array(
					"name" => "element_type",
					"type" => "ENUM('element', 'cat') NOT NULL DEFAULT 'element'",
					"comment" => "тип элемента модуля",
				),
				array(
					"name" => "rating",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "средняя оценка",
				),
				array(
					"name" => "count_votes",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "количество оценок",
				),
				array(
					"name" => "created",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата последней оценки",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY element_id (element_id)",
				"KEY module_name (module_name(2))",
			),
		),
	);

	/**
	 * @var array записи в таблице {modules}
	 */
	public $modules = array(
		array(
			"name" => "rating",
			"admin" => true,
			"site" => true,
		),
	);

	/**
	 * @var array меню административной части
	 */
	public $admin = array(
		array(
			"name" => "Рейтинг",
			"rewrite" => "rating",
			"group_id" => 2,
			"sort" => 23,
			"act" => true,
			"docs" => "http://www.diafan.ru/moduli/rejtingi/",
			"children" => array(
				array(
					"name" => "Настройки",
					"rewrite" => "rating/config",
				),
			)
		),
	);

	/**
	 * @var array настройки
	 */
	public $config = array(
		array(
			"name" => "security",
			"value" => "4",
		),
		array(
			"name" => "rating",
			"module_name" => "ab",
			"value" => "1",
			"check_module" => true,
		),
		array(
			"name" => "rating",
			"module_name" => "clauses",
			"value" => "1",
			"check_module" => true,
		),
		array(
			"name" => "rating",
			"module_name" => "files",
			"value" => "1",
			"check_module" => true,
		),
		array(
			"name" => "rating",
			"module_name" => "news",
			"value" => "1",
			"check_module" => true,
		),
		array(
			"name" => "rating",
			"module_name" => "photo",
			"value" => "1",
			"check_module" => true,
		),
		array(
			"name" => "rating",
			"module_name" => "shop",
			"value" => "1",
			"check_module" => true,
		),
	);

	/**
	 * @var array демо-данные
	 */
	public $demo = array(
		'rating' => array(
			array(
				"rating" => 5,
				"module_name" => "shop",
				"element_id" => 36,
				"count_votes" => 1,
			),
			array(
				"rating" => 5,
				"module_name" => "news",
				"element_id" => 20,
				"count_votes" => 1,
			),
			array(
				"rating" => 5,
				"module_name" => "news",
				"element_id" => 19,
				"count_votes" => 1,
			),
			array(
				"rating" => 5,
				"module_name" => "news",
				"element_id" => 17,
				"count_votes" => 1,
			),
			array(
				"rating" => 4,
				"module_name" => "news",
				"element_id" => 18,
				"count_votes" => 1,
			),
			array(
				"rating" => 5,
				"module_name" => "news",
				"element_id" => 16,
				"count_votes" => 1,
			),
			array(
				"rating" => 5,
				"module_name" => "shop",
				"element_id" => 84,
				"count_votes" => 1,
			),
			array(
				"rating" => 5,
				"module_name" => "shop",
				"element_id" => 85,
				"count_votes" => 1,
			),
			array(
				"rating" => 5,
				"module_name" => "shop",
				"element_id" => 85,
				"count_votes" => 1,
			),
			array(
				"rating" => 3.5,
				"module_name" => "shop",
				"element_id" => 8,
				"count_votes" => 2,
			),
			array(
				"rating" => 5,
				"module_name" => "shop",
				"element_id" => 12,
				"count_votes" => 1,
			),
			array(
				"rating" => 4,
				"module_name" => "shop",
				"element_id" => 20,
				"count_votes" => 1,
			),
			array(
				"rating" => 5,
				"module_name" => "shop",
				"element_id" => 19,
				"count_votes" => 1,
			),
			array(
				"rating" => 5,
				"module_name" => "clauses",
				"element_id" => 11,
				"count_votes" => 1,
			),
			array(
				"rating" => 5,
				"module_name" => "clauses",
				"element_id" => 14,
				"count_votes" => 1,
			),
			array(
				"rating" => 5,
				"module_name" => "clauses",
				"element_id" => 1,
				"count_votes" => 1,
			),
			array(
				"rating" => 5,
				"module_name" => "news",
				"element_id" => 8,
				"count_votes" => 1,
			),
		),
	);
}