<?php
/**
 * Настройки модуля
 * 
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    7.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2021 OOO «Диафан» (http://www.diafan.ru/)
 */

if (! defined('DIAFAN'))
{
    $path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}

/**
 * Site_admin_config
 */
class Site_admin_config extends Frame_admin
{
	/**
	 * @var array поля в базе данных для редактирования
	 */
	public $variables = array (
		'config' => array (
			'hr1' => array(
				'type' => 'title',
				'name' => 'Основные',
			),
			'comments' => array(
				'type' => 'module',
				'name' => 'Прикрепить комментарии к страницам сайта',
				'help' => 'Если отмечена, пользователи сайта будут иметь возможность оставлять комментарии к каждой странице сайта. Комментарии для страницы сайта выводятся с помощью шаблонной функции show_comments. Комментарии выводятся только на страницах, к которым не подключен иной модуль со своими комментариями. Параметр не будет включен, если модуль «Комментарии» не установлен. Подробности см. в разделе [модуль «Комментарии»](http://www.diafan.ru/dokument/full-manual/upmodules/comments/).',
			),
			'tags' => array(
				'type' => 'module',
				'name' => 'Подключить теги',
				'help' => 'Если отмечена, к страницам сайта можно будет прикреплять теги. Теги для страницы сайта выводятся с помощью шаблонной функции show_tags. Параметр не будет включен, если модуль «Теги» не установлен. Подробности см. в разделе [модуль «Теги»](http://www.diafan.ru/dokument/full-manual/modules/tags/).',
			),
			'keywords' => array(
				'type' => 'module',
				'name' => 'Подключить перелинковку',
				'help' => 'Отображение перелинковки в модуле. Подробности см. в разделе [модуль «Перелинковка»](http://www.diafan.ru/dokument/full-manual/upmodules/keywords/).',
			),
			'hr2' => array(
				'type' => 'title',
				'name' => 'Изображения',
			),			
			'images' => array(
				'type' => 'module',
				'element_type' => array('element'),
				'hide' => true,
			),
			'images_element' => array(
				'type' => 'none',
				'name' => 'Использовать изображения',
				'help' => 'Если отмечена, к страницам сайта можно будет прикреплять дополнительные изображения. Изображения для страницы сайта выводятся с помощью шаблонной функции show_images.',
				'no_save' => true,
			),
			'images_variations_element' => array(
				'type' => 'none',
				'name' => 'Генерировать размеры изображений',
				'help' => 'Размеры изображений, заданные в модуле «Изображения» и тег латинскими буквами для подключения изображения на сайте. Параметр выводится, если отмечена опция «Использовать изображения».',
				'no_save' => true,
			),
			'use_animation' => array(
				'type' => 'none',
				'name' => 'Использовать анимацию при увеличении изображений',
				'help' => 'Параметр добавляет JavaScript код, позволяющий включить анимацию при увеличении изображений. Параметр выводится, если отмечена опция «Использовать изображения».',
				'no_save' => true,
			),
			'upload_max_filesize' => array(
				'type' => 'none',
				'name' => 'Максимальный размер загружаемых файлов',
				'help' => 'Параметр показывает максимально допустимый размер загружаемых файлов, установленный в настройках хостинга. Параметр выводится, если отмечена опция «Использовать изображения».',
				'no_save' => true,
			),
			'resize' => array(
				'type' => 'none',
				'name' => 'Применить настройки ко всем ранее загруженным изображениям',
				'help' => 'Позволяет переконвертировать размер уже загруженных изображений. Кнопка необходима, если изменены настройки размеров изображений. Параметр выводится, если отмечена опция «Использовать изображения».',
				'no_save' => true,
			),
			'hr3' => array(
				'type' => 'title',
				'name' => 'Автогенерация для SEO',
			),
			'rewrite_tpl' => array(
				'type' => 'text',
				'name' => 'Шаблон для автоматического генерирования ссылки у страниц',
				'help' => "Шаблон примеряется только для новых ссылок. Уже созданные не затрагиваются. В шаблон можно добавить:\n\n* %name – название,\n* %parent – страница-родитель,\n* %id – ID (SEO-специалисту)."
			),
			'title_tpl' => array(
				'type' => 'text',
				'name' => 'Шаблон для автоматического генерирования Title',
				'help' => "Если шаблон задан и для страницы не прописан заголовок *Title*, то заголовок автоматически генерируется по шаблону. В шаблон можно добавить:\n\n* %name – название,\n* %parent – название страницы верхнего уровня (SEO-специалисту).",
				'multilang' => true
			),
			'keywords_tpl' => array(
				'type' => 'text',
				'name' => 'Шаблон для автоматического генерирования Keywords',
				'help' => "Если шаблон задан и для страницы не заполнено поле *Keywords*, то поле *Keywords* автоматически генерируется по шаблону. В шаблон можно добавить:\n\n* %name – название,\n* %parent – название страницы верхнего уровня (SEO-специалисту).",
				'multilang' => true
			),
			'descr_tpl' => array(
				'type' => 'text',
				'name' => 'Шаблон для автоматического генерирования Description',
				'help' => "Если шаблон задан и для страницы не заполнено поле *Description*, то поле *Description* автоматически генерируется по шаблону. В шаблон можно добавить:\n\n* %name – название,\n* %parent – название страницы верхнего уровня (SEO-специалисту).",
				'multilang' => true
			),
			'where_access' => array(
				'type' => 'none',
				'hide' => true,
			),
		),
	);

	/**
	 * @var array настройки модуля
	 */
	public $config = array (
		'config', // файл настроек модуля
	);
}