<?php
/**
 * Редактирование настроек шаблона сайта
 *
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    7.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2020 OOO «Диафан» (http://www.diafan.ru/)
 */

if (! defined('DIAFAN'))
{
	$path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}

/**
 * Site_admin_theme
 */
class Site_admin_theme extends Frame_admin
{
	/**
	 * @var string таблица в базе данных
	 */
	public $table = 'site_theme';

	/**
	 * @var array поля  для редактирования
	 */
	public $variables = array (
		'base' => array (
			'edit' => array(
				'type' => 'function',
			),
		)
	);

	/**
	 * @var array настройки модуля
	 */
	public $config = array (
		'only_edit', // модуль состоит только из формы редактирования
	);

	/**
	 * Задает значения полей для формы
	 *
	 * @return array
	 */
	public function get_values()
	{
		return array('edit' => 'true');
	}


	/**
	 * Выводит контент модуля
	 * @return void
	 */
	public function edit_variable_edit()
	{
		// echo '<div class="inform_green">'.$this->diafan->_('Параметры настроек шаблона задаются веб-мастером при создании сайта');
		// echo ((defined("MOD_DEVELOPER") && MOD_DEVELOPER) ? $this->diafan->_(' в файле').' <b>'.BASE_PATH.Custom::path('modules/site/admin/site.admin.theme_custom.php').'</b>':'');
		// echo $this->diafan->_(' и выводятся с помощью шаблонного тега %s. <a href="https://www.diafan.ru/dokument/full-manual/sysmodules/site/#Nastroyki-shablona" target="_blank">Документация по  настройкам шаблона.</a>', '<code><span style="color: #000000"><span style="color: #0000BB">&lt;insert</span> <span style="color: #007700">name=</span><span style="color: #DD0000">&quot;show_theme&quot;</span> <span style="color: #007700">module=</span><span style="color: #DD0000">&quot;site&quot;</span> <span style="color: #007700">tag=</span><span style="color: #DD0000">&quot;...&quot;</span><span style="color: #0000BB">&gt;</span></span></code>').'</div>';

		$old = DB::query_fetch_key_array("SELECT * FROM {site_theme} ORDER BY id", 'name');

		Custom::inc('modules/site/admin/site.admin.theme_custom.php');
		$theme = new Site_admin_theme_custom($this->diafan);
		if (method_exists($theme, 'prepare_config'))
		{
			$theme->prepare_config();
		}
		if (! empty($theme->tabs_name))
		{
			$tabs_name = $theme->tabs_name;
		}
		elseif (! isset($theme->variables['base']))
		{
			$tabs_name = array(
				'base' => 'Основные настройки',
			);
			$theme->variables = array('base' => $theme->variables);
		}
		echo '<div id="tabs" index="'.$this->diafan->_admin->rewrite.'"><ul>';
		$i = 1;
		foreach ($theme->variables as $title => $variable_table)
		{
			echo '<li><a href="#tabs-'.( $i++ ).'">';
			if(! empty($tabs_name[$title]))
			{
				echo $this->diafan->_($tabs_name[$title]);
			}
			else
			{
				echo $title;
			}
			echo '</a></li>';
		}
		echo '</ul>';
		$i = 1;
		foreach ($theme->variables as $title => $variable_table)
		{
			echo '<div id="tabs-'.( $i++ ).'">';

			foreach ($variable_table as $name => $row)
			{
				$lang_id = ! empty($row['multilang']) ? _LANG : $this->diafan->_languages->site;
	
				$help = ! empty($row['help']) ? $this->diafan->help($row['help']) : '';

				if ($row['type'] != 'title')
				{
					if (defined('MOD_DEVELOPER') && MOD_DEVELOPER)
					{
						$help .= '<br><code><span style="color: #000000"><span style="color: #0000BB">&lt;insert</span> <span style="color: #007700">name=</span><span style="color: #DD0000">&quot;show_theme&quot;</span> <span style="color: #007700">module=</span><span style="color: #DD0000">&quot;site&quot;</span> <span style="color: #007700">tag=</span><span style="color: #DD0000">&quot;'.$name.'&quot;</span><span style="color: #0000BB">&gt;</span></span></code>';
					}
					if (! empty($row['tutorial']))
					{
						$help .=
						'<div class="tutorial">
							<div class="tutorial__head">
								<div class="tutorial__tab tutorial-tab">
									<div class="tutorial-tab__name">'.$this->diafan->_('Как это работает?').'</div>
								</div>
							</div>
							<div class="tutorial__body">
								<div class="tutorial__text">'.$row['tutorial'].'</div>
							</div>
						</div>';
					}
				}
	
				$value  = '';
				$values = array();
				if (! empty($old[$name]))
				{
					$value = $old[$name][0]['value'.$lang_id];
					foreach ($old[$name] as $k => $v)
					{
						if (! empty($v['value'.$lang_id]))
						{
							$values[] = $v['value'.$lang_id];
						}
					}
				}
	
				if (is_callable(array(&$theme, 'edit_variable_'.$name)))
				{
					call_user_func_array(array(&$theme, 'edit_variable_'.$name), array($values, $old));
				}
				else
				{
					$key = 'theme_'.$name;
	
					$attr = '';
					if (! empty($row['attr']))
					{
						$attr = $row['attr'];
					}
	
					$class = '';
					if (! empty($row['class']))
					{
						$class = $row['class'];
					}
					if (! empty($row['float']))
					{
						$class = ($class ? $class.' ' : '').'float';
					}
					if (! empty($row['short']))
					{
						$class = ($class ? $class.' ' : '').'short';
					}
	
					$depend    = ! empty($row['depend']) ? $row['depend'] : '';
					$disabled  = ! empty($row['disabled']);
					$maxlength = ! empty($row['maxlength']) ? $row['maxlength'] : 0;
					$select    = ! empty($row['select']) ? $row['select'] : array();
					$select_db = ! empty($row['select_db']) ? $row['select_db'] : array();
					$default   = isset($row['default']) ? $row['default'] : '';
	
					if (in_array($row['type'], array('url', 'radio', 'multiple', 'enum', 'list', 'image', 'favicon', 'color')))
					{
						if ($depend)
						{
							$attr .= ' depend="'.$depend.'"';
							$class = ($class ? $class.' ' : '').'depend_field';
						}
	
						switch ($row['type'])
						{
							case 'url':
								$this->diafan->show_table_tr_text($key, $row['name'], $value, $help, $disabled, $attr, $class, $maxlength);
								break;
	
							case 'radio':
								if (! $select && $select_db)
								{
									$select = $this->diafan->get_select_from_db($select_db);
								}
								$this->diafan->show_table_tr_radio($key, $row['name'], $value, $help, $disabled, $select, $attr, $class);
								break;
	
							case 'multiple':
								if (! $select && $select_db)
								{
									$select = $this->diafan->get_select_from_db($select_db);
								}
								$this->diafan->show_table_tr_multiple($key, $row['name'], $values, $help, $disabled, $select, $attr, $class);
								break;
	
							case 'enum':
								if (! $select && $select_db)
								{
									$select = $this->diafan->get_select_from_db($select_db);
								}
								echo '
								<div class="unit'.($class ? ' '.$class : '').'" id="'.$key.'"'.$attr.'>
									<div class="infofield">'.$row['name'].$help.'</div>';
									foreach ($select as $k => $option)
									{
										if (is_array($option))
										{
											$k = $option['id'];
											$option = $option['name'];
										}
										echo '<input name="'.$key.'[]"'.($disabled ? ' disabled' : '').' type="checkbox" value="'.$k.'"'.(in_array($k, $values) ? ' checked' : '').' id="input_'.$key.'_'.$k.'">
										<label for="input_'.$key.'_'.$k.'">'.$this->diafan->_($option).'</label>';
									}
									echo '
								</div>';
								break;
	
							case 'list':
								echo '
								<div class="unit'.($class ? ' '.$class : '').'" id="'.$key.'"'.$attr.'>
									<div class="infofield">'.$row['name'].$help.'</div>
									<div class="item-list">
										<div class="item-list__items">';
										$items = $values ? $values : array('');
										foreach ($items as $item)
										{
											echo '<div class="list-item"><input class="list-item__input" name="'.$key.'[]"'.($disabled ? ' disabled' : '').' type="text" value="'.$item.'">';
											if (! $disabled)
											{
												echo '<div class="list-item__actions">
													<button class="list-item__action list-item__action_delete" data-action="delete" title="'.$this->diafan->_('Удалить').'" type="button"><i class="fa fa-close"></i></button>
													<button class="list-item__action list-item__action_up" data-action="up" title="'.$this->diafan->_('Выше').'" type="button">↑</button>
													<button class="list-item__action list-item__action_down" data-action="down" title="'.$this->diafan->_('Ниже').'" type="button">↓</button>
												</div>';
											}
											echo '</div>';
										}
										echo '</div>';
										if (! $disabled)
										{
											echo '<div class="item-list__actions">
												<button class="item-list__action item-list__action_add" data-action="add" type="button">
													<i class="fa fa-plus-square"></i> '.$this->diafan->_('Добавить').'
												</button>
											</div>';
										}
										echo '
									</div>
								</div>';
								break;
	
							case 'image':
							case 'favicon':
								echo '
								<div class="unit'.($class ? ' '.$class : '').'" id="'.$key.'"'.$attr.'>
									<div class="infofield">'.$row['name'].$help.'</div>
									<div class="picture">';
										if ($row['type'] == 'favicon')
										{
											if (! $value)
											{
												$value = 'favicon.ico';
											}
											$dirname = '';
										}
										else
										{
											$dirname = USERFILES.'/site/theme/';
										}
										if ($value && file_exists(ABSOLUTE_PATH.$dirname.$value))
										{
											echo '
											<img src="'.BASE_PATH.$dirname.$value.'?'.rand(0, 999).'">
											<input type="hidden" name="delete_'.$name.'" value="">
											<label class="picture__delete" title="'.$this->diafan->_('Удалить текущее изображение').'"><i class="fa fa-close"></i></label>';
										}
										echo '<div class="picture__attach"><input class="file" type="file" name="'.$key.'"></div>
									</div>
								</div>';
								break;
	
							case 'color':
								echo
								'<div class="unit'.($class ? ' '.$class : '').'" id="'.$key.'"'.$attr.'>
									<div class="infofield">'.$row['name'].$help.'</div>
									<div class="colorpicker'.(! $value && ! $default ? ' colorpicker_blank' : '').'" data-format="hex" data-color="'.$default.'" data-alpha="100">
										<div class="colorpicker__pick">
											<input class="colorpicker__input" type="text" name="'.$key.'" placeholder="#000000" value="'.($value ? $value : $default).'">
											<label class="colorpicker__preview colorpicker-preview" style="background: '.($value ? $value : $default).';">
												<input class="colorpicker-preview__input" type="color" value="'.($value ? $value : $default).'">
												<span class="colorpicker-preview__icon fa fa-eyedropper"></span>
											</label>';
											// if ($default && $value != $default)
											// {
											// 	echo '<button class="colorpicker__reset" type="button" style="background: '.$default.';">'.$this->diafan->_('Оригинал').'</button>';
											// }
											// echo '<input class="colorpicker__alpha colorpicker-alpha number" type="text" name="'.$key.'_alpha" placeholder="100" value="100">
											// <input class="colorpicker-alpha__range" type="range" min="0" max="100" step="1" value="100">';
											echo '
										</div>';
										if (! empty($row['files']))
										{
											$paths = array();
											$files = explode(',', $row['files']);
											foreach ($files as $file)
											{
												$file = trim($file);
												if ($file && Custom::exists($file))
												{
													$paths[] = Custom::path($file);
												}
											}
											if ($paths)
											{
												echo '<div class="colorpicker__files">'.$this->diafan->_('Связанные файлы').': '.implode(', ', $paths).'</div>';
											}
										}
										echo
									'</div>
								</div>';
								break;
						}
					}
					else
					{
						if ($value)
						{
							switch ($row['type'])
							{
								case 'date':
									$value = $this->diafan->unixdate($this->diafan->formate_from_date($value));
									break;
								case 'datetime':
									$value = $this->diafan->unixdate($this->diafan->formate_from_datetime($value));
									break;
							}
						}
						$this->diafan->show_table_tr($row['type'], $key, $value, $row['name'], $help, $disabled, $maxlength, $select, $select_db, $depend, $attr, $class);
					}
				}
			}

			echo '</div>';
		}

		echo '</div>';
	}
	
	/**
	 * Сохраняет настройки шаблона
	 *
	 * @return boolean
	 */
	public function save()
	{
		// Прошел ли пользователь проверку идентификационного хэша
		if (! $this->diafan->_users->checked)
		{
			$this->diafan->redirect(URL);
			return false;
		}

		//проверка прав на сохранение
		if (! $this->diafan->_users->roles('edit', 'site/theme'))
		{
			$this->diafan->redirect(URL);
			return false;
		}
		
		$this->diafan->set_one_shot('<div class="ok">'.$this->diafan->_('Изменения сохранены!').'</div>');

		$old = DB::query_fetch_key_array("SELECT * FROM {site_theme} ORDER BY id", 'name');
		
		Custom::inc('modules/site/admin/site.admin.theme_custom.php');
		$theme = new Site_admin_theme_custom($this->diafan);
		if (method_exists($theme, 'prepare_config'))
		{
			$theme->prepare_config();
		}
		if (empty($theme->tabs_name) && ! isset($theme->variables['base']))
		{
			$theme->variables = array('base' => $theme->variables);
		}
		foreach ($theme->variables as $title => $variable_table)
		{
			foreach ($variable_table as $name => $row)
			{
				if (! empty($row['disabled']) || ! empty($row['no_save']))
				{
					continue;
				}
	
				$lang_id = ! empty($row['multilang']) ? _LANG : $this->diafan->_languages->site;
	
				if (is_callable(array(&$theme, 'save_variable_'.$name)))
				{
					$mask = '%s';
					$value = call_user_func_array(array(&$theme, 'save_variable_'.$name), array($old));
				}
				else
				{
					$mask = '%s';
					$value = ! empty($_POST['theme_'.$name]) ? $_POST['theme_'.$name] : false;
	
					switch ($row['type'])
					{
						case 'text':
						case 'url':
						case 'radio':
						case 'select':
						case 'multiple':
						case 'enum':
						case 'list':
						case 'email':
						case 'phone':
							$mask = '%h';
							break;
	
						case 'textarea':
							$mask = '%s';
							break;
	
						case 'editor':
							$mask = '%s';
							$value = $this->diafan->save_field_editor('theme_'.$name);
							break;
	
						case 'email':
							$mask = '%h';
							break;
	
						case 'phone':
							$mask = '%h';
							break;
	
						case 'numtext':
						case 'checkbox':
							$mask = '%d';
							break;
	
						case 'date':
						case 'datetime':
							$mask = '%d';
							$value = $this->diafan->unixdate($value);
							break;
	
						case 'floattext':
							$mask = '%f';
							$value = str_replace(',', '.', $value);
							break;
	
						case 'image':
						case 'favicon':
							$mask = '%h';
							if ($row['type'] == 'favicon')
							{
								$dirname = '';
								if (empty($old[$name][0]['value'.$lang_id]) && ! empty($_POST['delete_'.$name]))
								{
									File::delete_file('favicon.ico');
								}
							}
							else
							{
								$dirname = USERFILES.'/site/theme/';
							}
							if (! empty($old[$name]) && ! empty($_POST['delete_'.$name]))
							{
								foreach ($old[$name] as $v)
								{
									if ($v['value'.$lang_id])
									{
										File::delete_file($dirname.str_replace(array('\\','/'), '', $v['value'.$lang_id]));
									}
								}
							}
							$value = ! empty($old[$name][0]['value'.$lang_id]) && empty($_POST['delete_'.$name]) ? $old[$name][0]['value'.$lang_id] : false;
							if (isset($_FILES['theme_'.$name]) && is_array($_FILES['theme_'.$name]) && $_FILES['theme_'.$name]['name'] != '')
							{
								$tmp = 'tmp/theme'.rand(0, 99999);
								File::copy_file($_FILES['theme_'.$name]['tmp_name'], $tmp);
								$mimes = array(
									'image/gif' => 'gif',
									'image/jpeg' => 'jpg',
									'image/png' => 'png',
									'image/pjpeg' => 'jpg',
									'image/x-png'=> 'png',
									'image/svg' => 'svg',
									'image/svg+xml' => 'svg',
									'image/webp' => 'webp',
									'image/x-icon'=> 'ico',
									'image/vnd.microsoft.icon' => 'ico'
								);
								try
								{
									$mime = mime_content_type(ABSOLUTE_PATH.$tmp);
									if (empty($mime) || ! in_array($mime, array_keys($mimes)))
									{
										$this->diafan->set_one_shot('<div class="error">'.$this->diafan->_('Неверный формат файла. Изображения загружаются только в форматах GIF, JPEG, PNG, SVG, WEBP, ICO.').'</div>');
										throw new Exception('');
									}
									$extension = $mimes[$mime];
									if ($extension == 'svg')
									{
										Custom::inc('plugins/svg-sanitizer/svg-scanner.php');
										if ($msg = svg_scanner($tmp))
										{
											$this->diafan->set_one_shot('<div class="error">'.$msg.'</div>');
											throw new Exception('Ошибка загрузки SVG');
										}
									}
									$filename = $name;
									if (! empty($row['multilang']) && $row['type'] == 'image')
									{
										$filename .= $lang_id;
									}
									File::upload_file(ABSOLUTE_PATH.$tmp, $dirname.$filename.'.'.$extension);
									$value = $filename.'.'.$extension;
								}
								catch (Exception $e)
								{
									File::delete_file($tmp);
								}
							}
							break;
	
						case 'color':
							$mask = '%h';
							if (! preg_match('/^#([A-F0-9]{3}|[A-F0-9]{6})$/i', $value))
							{
								$value = ! empty($row['default']) ? $row['default'] : '';
							}
							$value_old = ! empty($old[$name][0]['value'.$lang_id]) ? $old[$name][0]['value'.$lang_id] : '';
							if (! empty($row['files']) && $value_old && $value && $value_old != $value)
							{
								File::create_dir('tmp/color', true);
								$files = explode(',', $row['files']);
								foreach ($files as $file)
								{
									$file = trim($file);
									if ($file && Custom::exists($file))
									{
										$path = Custom::path($file);
										$css = preg_replace('/'.$value_old.'/ui', $value, file_get_contents(ABSOLUTE_PATH.$path));
										$tmp = 'tmp/color/'.File::tempnam(basename($file), 'tmp/color');
										File::save_file($css, $tmp);
										$content = file_exists(ABSOLUTE_PATH.$tmp) ? file_get_contents(ABSOLUTE_PATH.$tmp) : false;
										if ($content === false || $css != $content)
										{
											if (file_exists(ABSOLUTE_PATH.$tmp)) File::delete_file($tmp);
											continue 3;
										}
										File::save_file($css, $path);
										$content = file_exists(ABSOLUTE_PATH.$path) ? file_get_contents(ABSOLUTE_PATH.$path) : false;
										if ($content === false || $css != $content) continue 3;
										if (file_exists(ABSOLUTE_PATH.$tmp)) File::delete_file($tmp);
									}
								}
								if (is_dir(ABSOLUTE_PATH.'tmp/color')) File::rm('tmp/color');
							}
							break;
					}
				}
				if ($value)
				{
					if (is_array($value))
					{
						if (! empty($old[$name]))
						{
							foreach ($old[$name] as $v)
							{
								if (! empty($v['value'.$lang_id]))
								{
									DB::query("DELETE FROM {site_theme} WHERE id=%d", $v['id']);
								}
							}
						}
						foreach ($value as $v)
						{
							if ($v)
							{
								DB::query("INSERT INTO {site_theme} (name, value".$lang_id.", type) VALUES ('%h', '".$mask."', '%h')", $name, $v, $row['type']);
							}
						}
					}
					else
					{
						if (! empty($old[$name]))
						{
							DB::query("UPDATE {site_theme} SET value".$lang_id."='".$mask."', type='%h' WHERE id=%d", $value, $row['type'], $old[$name][0]['id']);
							if (count($old[$name]) > 1)
							{
								DB::query("DELETE FROM {site_theme} WHERE id<>%d AND name='%h'", $old[$name][0]['id'], $name);
							}
						}
						else
						{
							DB::query("INSERT INTO {site_theme} (name, value".$lang_id.", type) VALUES ('%h', '".$mask."', '%h')", $name, $value, $row['type']);
						}
					}
				}
				elseif (! empty($old[$name]))
				{
					if (! empty($row['multilang']))
					{
						DB::query("UPDATE {site_theme} SET value".$lang_id."='' WHERE name='%h'", $name);
						$query_values = array();
						foreach ($this->diafan->_languages->all as $lang)
						{
							$query_values[] = "AND (value".$lang['id']."='' OR value".$lang['id']." IS NULL)";
						}
						DB::query("DELETE FROM {site_theme} WHERE name='%h' ".implode(' ', $query_values), $name);
					}
					else
					{
						DB::query("DELETE FROM {site_theme} WHERE name='%h'", $name);
					}
				}
			}
		}

		$this->diafan->_users->history('edit', 'site', 0, 'theme', 'site_theme');

		$this->diafan->redirect(URL);
		return true;
	}
}
