<?php
/**
 * Установка модуля
 *
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    7.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2021 OOO «Диафан» (http://www.diafan.ru/)
 */

if (! defined('DIAFAN'))
{
	$path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}

class Site_install extends Install
{
	/**
	 * @var boolean модуль является частью ядра
	 */
	public $is_core = true;

	/**
	 * @var string название
	 */
	public $title = "Страницы сайта";

	/**
	 * @var array таблицы в базе данных
	 */
	public $tables = array(
		array(
			"name" => "site",
			"comment" => "Страницы сайта",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "parent_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор родителя из таблицы {site}",
				),
				array(
					"name" => "count_children",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "количество вложенных страниц",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "название",
					"multilang" => true,
				),
				array(
					"name" => "title_meta",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "заголовок окна в браузере, тег Title",
					"multilang" => true,
				),
				array(
					"name" => "keywords",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "ключевые слова, тег Keywords",
					"multilang" => true,
				),
				array(
					"name" => "descr",
					"type" => "TEXT",
					"comment" => "описание, тэг Description",
					"multilang" => true,
				),
				array(
					"name" => "canonical",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "канонический тег",
					"multilang" => true,
				),
				array(
					"name" => "text",
					"type" => "LONGTEXT",
					"comment" => "контент",
					"multilang" => true,
				),
				array(
					"name" => "act",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "показывать на сайте: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "access",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "доступ ограничен: 0 - нет, 1 - да",
				),
				array(
					"name" => "date_start",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата начала показа",
				),
				array(
					"name" => "date_finish",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата окончания показа",
				),
				array(
					"name" => "admin_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "пользователь из таблицы {users}, добавивший или первый отредктировавший страницу в административной части",
				),
				array(
					"name" => "title_no_show",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "не копировать заголовок в H1: 0 - нет, 1 - да",
				),
				array(
					"name" => "map_no_show",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "не показывать на карте сайта: 0 - нет, 1 - да",
				),
				array(
					"name" => "changefreq",
					"type" => "ENUM( 'always', 'hourly', 'daily', 'weekly', 'monthly', 'yearly', 'never' ) NOT NULL DEFAULT 'always'",
					"comment" => "Changefreq для sitemap.xml",
				),
				array(
					"name" => "priority",
					"type" => "VARCHAR(3) NOT NULL DEFAULT ''",
					"comment" => "Priority для sitemap.xml",
				),
				array(
					"name" => "noindex",
					"type" => "ENUM('0','1') NOT NULL DEFAULT '0'",
					"comment" => "не индексировать: 0 - нет, 1 - да",
				),
				array(
					"name" => "search_no_show",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "не участвует в поисковой выдаче: 0 - нет, 1 - да",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "timeedit",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "время последнего изменения в формате UNIXTIME",
				),
				array(
					"name" => "theme",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон страницы сайта",
				),
				array(
					"name" => "module_name",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "прикрепленный модуль",
				),
				array(
					"name" => "js",
					"type" => "TEXT",
					"comment" => "JS-код",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY parent_id (parent_id)",
			),
		),
		array(
			"name" => "site_blocks",
			"comment" => "Блоки на сайте",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "название",
					"multilang" => true,
				),
				array(
					"name" => "text",
					"type" => "TEXT",
					"comment" => "описание",
					"multilang" => true,
				),
				array(
					"name" => "act",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "показывать на сайте: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "access",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "доступ ограничен: 0 - нет, 1 - да",
				),
				array(
					"name" => "title_no_show",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "не показывать заголовок: 0 - нет, 1 - да",
				),
				array(
					"name" => "date_start",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата начала показа",
				),
				array(
					"name" => "date_finish",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата окончания показа",
				),
				array(
					"name" => "admin_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "пользователь из таблицы {users}, добавивший или первый отредктировавший блок в административной части",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "timeedit",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "время последнего изменения в формате UNIXTIME",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "site_blocks_site_rel",
			"comment" => "Данные о том, на каких страницах выводятся блоки на сайте",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор блока из таблицы {site_blocks}",
				),
				array(
					"name" => "site_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор страницы сайта из таблицы {site}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY site_id (`site_id`)",
			),
		),
		array(
			"name" => "site_dynamic",
			"comment" => "Динамические блоки",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "название",
					"multilang" => true,
				),
				array(
					"name" => "text",
					"type" => "TEXT",
					"comment" => "описание",
				),
				array(
					"name" => "act",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "показывать на сайте: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "type",
					"type" => "VARCHAR(20) NOT NULL DEFAULT ''",
					"comment" => "тип",
				),
				array(
					"name" => "access",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "доступ ограничен: 0 - нет, 1 - да",
				),
				array(
					"name" => "title_no_show",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "не показывать заголовок: 0 - нет, 1 - да",
				),
				array(
					"name" => "date_start",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата начала показа",
				),
				array(
					"name" => "date_finish",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата окончания показа",
				),
				array(
					"name" => "admin_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "пользователь из таблицы {users}, добавивший или первый отредктировавший блок в административной части",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "timeedit",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "время последнего изменения в формате UNIXTIME",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "site_dynamic_element",
			"comment" => "Контент динамических блоков, заполенный в элементах модулей",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "dynamic_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор динамического блока из таблицы {site_dynamic}",
				),
				array(
					"name" => "module_name",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "название модуля",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор элемента модуля",
				),
				array(
					"name" => "element_type",
					"type" => "VARCHAR(20) NOT NULL DEFAULT ''",
					"comment" => "тип элемента",
				),
				array(
					"name" => "value",
					"type" => "TEXT",
					"comment" => "значение",
					"multilang" => true,
				),
				array(
					"name" => "parent",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "применить к элементам категории: 0 - нет, 1 - да",
				),
				array(
					"name" => "category",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "применить к вложенным элементам: 0 - нет, 1 - да",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY dynamic_id (`dynamic_id`)",
				"KEY element_id (`element_id`)",
				"KEY element_type (`element_type`)",
			),
		),
		array(
			"name" => "site_dynamic_module",
			"comment" => "Данные о том, в каких модулях заполняются динамические блоки",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "dynamic_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор динамического блока из таблицы {site_dynamic}",
				),
				array(
					"name" => "module_name",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "название модуля",
				),
				array(
					"name" => "element_type",
					"type" => "VARCHAR(20) NOT NULL DEFAULT ''",
					"comment" => "тип элемента",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY dynamic_id (`dynamic_id`)",
			),
		),
		array(
			"name" => "site_parents",
			"comment" => "Родительские связи страниц сайта",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор страницы из таблицы {site}",
				),
				array(
					"name" => "parent_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор страницы-родителя из таблицы {site}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "site_theme",
			"comment" => "Настройки шаблона",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "название",
				),
				array(
					"name" => "value",
					"type" => "TEXT",
					"comment" => "значение",
					"multilang" => true,
				),
				array(
					"name" => "type",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "тип",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
	);

	/**
	 * @var array записи в таблице {modules}
	 */
	public $modules = array(
		array(
			"name" => "site",
			"admin" => true,
			"site" => true,
		),
	);

	/**
	 * @var array меню административной части
	 */
	public $admin = array(
		array(
			"name" => "Страницы сайта",
			"rewrite" => "site",
			"group_id" => 1,
			"sort" => 3,
			"act" => true,
			"children" => array(
				array(
					"name" => "Страницы сайта",
					"rewrite" => "site",
					"act" => true,
				),
				array(
					"name" => "Настройки шаблона",
					"rewrite" => "site/theme",
					"act" => true,
				),
				array(
					"name" => "Блоки на сайте",
					"rewrite" => "site/blocks",
					"act" => true,
				),
				array(
					"name" => "Динамические блоки",
					"rewrite" => "site/dynamic",
					"act" => true,
				),
				array(
					"name" => "Настройки",
					"rewrite" => "site/config",
				),
			)
		),
		array(
			"name" => "Настройки шаблона",
			"rewrite" => "site/theme",
			"icon_name" => "cog",
			"group_id" => 5,
			"sort" => 34,
			"act" => true,
		),
	);

	/**
	 * @var array настройки
	 */
	public $config = array(
		array(
			"name" => "use_animation",
			"value" => "1",
		),
		array(
			"name" => "images_webp_element",
			"value" => "1",
		),
		array(
			"name" => "images_variations_element",
			"value" => "a:2:{s:4:\"webp\";i:1;s:2:\"vs\";a:2:{i:0;a:2:{s:4:\"name\";s:5:\"large\";s:2:\"id\";s:1:\"3\";}i:1;a:2:{s:4:\"name\";s:6:\"medium\";s:2:\"id\";s:1:\"2\";}}}",
		),
	);

	/**
	 * @var array страницы сайта
	 */
	public $site = array(
		array(
			"id" => 1,
			"name" => array('Главная страница', 'Home'),
			"text" => array('<p><strong>Добро пожаловать на наш новый сайт!</strong></p>
				<p>Здесь можно найти много чего интересного:</p>
				<ul>
				<li>каталог товаров (интернет-магазин)</li>
				<li>вопрос-ответ</li>
				<li>новости</li>
				<li>файловый архив</li>
				<li>фотогалерея</li>
				</ul>
				<p>и много-многое другое!</p>',

				'<p><strong>Welcome to our new site!</strong></p>
				<p>Here you can find more-more nice informations:</p>
				<ul>
				<li>catalog of goods (e-shop)</li>
				<li>FAQ</li>
				<li>news</li>
				<li>files</li>
				<li>photos</li>
				</ul>
				<p>and more-more else!</p>'),
			"rewrite" => "",
			"theme" => "site_start.php",
			"title_no_show" => true,
		),
		array(
			"id" => 2,
			'name' => array('Полезное', 'Useful'),
			'text' => array('<insert name="show_links" module="site">', '<insert name="show_links" module="site">'),
			"rewrite" => "useful",
			"sort" => 2,
			"menu" => 1,
		),
		array(
			"id" => 3,
			"sort" => 3,
		),
		array(
			"id" => 4,
			"name" => array('О компании', 'About the company'),
			"sort" => 4,
			"menu" => 1,
		),
	);

	/**
	 * @var array SQL-запросы
	 */
	public $sql = array(
		"site" => array(
			array(
				"id" => 150,
				"name" => array('Согласие на обработку персональных данных', 'Privacy Notice'),
				"text" => array('<p>Пользователь, оставляя заявку, принимает настоящее Согласие на обработку персональных данных (далее – Согласие). Действуя свободно, своей волей и в своем интересе, а также подтверждая свою дееспособность, Пользователь дает свое согласие на обработку своих персональных данных со следующими условиями:</p>
					<ol>
					<li>Данное Согласие дается на обработку персональных данных, как без использования средств автоматизации, так и с их использованием.</li>
					<li>Согласие дается на обработку следующих персональных данных: Персональные данные, не являющиеся специальными или биометрическими: номера контактных телефонов; адреса электронной почты; место работы и занимаемая должность; пользовательские данные (сведения о местоположении; тип и версия ОС; тип и версия Браузера; тип устройства и разрешение его экрана; источник откуда пришел на сайт пользователь; с какого сайта или по какой рекламе; язык ОС и Браузера; какие страницы открывает и на какие кнопки нажимает пользователь; ip-адрес.</li>
					<li>Персональные данные не являются общедоступными.</li>
					<li>Цель обработки персональных данных: обработка входящих запросов физических лиц с целью оказания консультирования; аналитики действий физического лица на веб-сайте и функционирования веб-сайта; проведение рекламных и новостных рассылок.</li>
					<li>Основанием для обработки персональных данных является: ст. 24 Конституции Российской Федерации; ст.6 Федерального закона №152-ФЗ «О персональных данных»; настоящее согласие на обработку персональных данных</li>
					<li>В ходе обработки с персональными данными будут совершены следующие действия: сбор; запись; систематизация; накопление; хранение; уточнение (обновление, изменение); извлечение; использование; передача (распространение, предоставление, доступ); блокирование; удаление; уничтожение.</li>
					<li>Персональные данные обрабатываются до отписки физического лица от рекламных и новостных рассылок. Также обработка персональных данных может быть прекращена по запросу субъекта персональных данных. Хранение персональных данных, зафиксированных на бумажных носителях осуществляется согласно Федеральномузакону №125-ФЗ «Об архивном деле в Российской Федерации» и иным нормативно правовым актам в области архивного дела и архивного хранения. </li>
					<li>Согласие может быть отозвано субъектом персональных данных или его представителем путем направления письменного заявления по адресу, указанному на главной странице настоящего сайта.</li>
					<li>В случае отзыва субъектом персональных данных или его представителем согласия на обработку персональных данных владельцы сайта вправе продолжить обработку персональных данных без согласия субъекта персональных данных при наличии оснований, указанных в пунктах 2 – 11 части 1 статьи 6, части 2 статьи 10 и части 2 статьи 11 Федерального закона №152-ФЗ «О персональных данных» от 27.07.2006 г.</li>
					<li>Настоящее согласие действует все время до момента прекращения обработки персональных данных, указанных в п.7 и п.8 данного Согласия.</li></ol></p>',
					'<p>On-line Privacy: we understands how important it is to keep all of our customers personal information private and safe. When you create an account at our site, we ask you for personal information. We use this information to process your orders and to communicate with you about the status of your orders. We also use your email address to send you newsletters IF you choose to subscribe yourself during registration. Should you no longer wish to receive our newsletter, simply request to be removed from our newsletter mailing list using one of the following methods: send to our email, reply to a newsletter with "Remove" in the subject line or you can unsubscribe or subscribe yourself using our edit account feature.</p>
					<p><b>We will not sell, rent or disclose your personally identifying information to third parties under any circumstances.</b></p>
					<p>We have implemented technology and security features, as well as strict internal guidelines, to safeguard the privacy of your personal information from unauthorized access or improper use. We will continue to enhance our security procedures as new technology becomes available.</p>
					<p>Order Privacy: All orders placed are handled in the strictest of confidence. Order information will only be disclosed to the person or persons named on the order.</p>
					<p>Packaging Privacy: All goods shipped by our site are shipped in plain brown boxes or similar discreet packaging. Our web-site address, our return address, and the name and address to which the package is being shipped are the only indications on the parcel.</p>
					<p>International Shipments do require the addition of a Customs Form. On these forms, we list item descriptions in the most general way possible.</p>
					<p>Credit Privacy: All credit information provided to us by our clientele is considered highly confidential and is thus kept private and protected. We do not distribute our customers charge card information to anyone.</p>'),
				"rewrite" => "privacy",
			),
		),
		"site_theme" => array(
			array(
				"name" => "logo_name",
				"value" => array("DIAFAN.CMS", "DIAFAN.CMS"),
				"type" => "textarea",
			),
			array(
				"name" => "logo_text",
				"value" => array("Демонстрационный сайт", "Demonstration site"),
				"type" => "textarea",
			),
			array(
				"name" => "favicon",
				"value" => array("favicon.ico"),
				"type" => "favicon",
			),
			array(
				"name" => "color",
				"value" => array("#c53c12"),
				"type" => "color",
			),
			array(
				"name" => "phone",
				"value" => array("999 999-99-99", "999 999-99-99"),
				"type" => "phone",
			),
			array(
				"name" => "email",
				"value" => array("cms@diafan.ru", "cms@diafancms.com"),
				"type" => "email",
			),
			array(
				"name" => "contacts",
				"value" => array("<p>г. Китеград, ул. Облачная,<br>д. 1, стр. 2, офис 345<br>Тел.: 999 999-99-99<br>E-mail: info@demosite.ru</p>", "<p>Kitezhgrad, Cloudy str.,<br>1, building 2, office 345<br>Tel.: 999 999-99-99<br>E-mail: info@demosite.ru</p>"),
				"type" => "editor",
			),
			array(
				"name" => "copyright",
				"value" => array("Demosite.ru", "Demosite.com"),
				"type" => "textarea",
			),
			array(
				"name" => "show_menu_top",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_langs",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_search",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_lk",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_compare",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_wishlist",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_cart",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_slider",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_catalog",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_content",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_banners",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_clauses",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_comments",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_faq",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_subscription",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_filter",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_news",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_photos",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_share",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_good_set",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_good_rel",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_good_order_rel",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "delivery",
				"value" => array("<p class=\"attribute_enum\"><a href=\"#\"><i class=\"fas fa-truck\"></i><span>Доставка бесплатная<br>от 3 000 ₽</span></a><a href=\"#\"><i class=\"fas fa-redo\"></i><span>14 дней<br>возврат/обмен</span></a><a href=\"#\"><i class=\"fas fa-umbrella\"></i><span>2 года<br>гарантии</span></a></p>", "<p class=\"attribute_enum\"><a href=\"#\"><i class=\"fas fa-truck\"></i><span>Free delivery<br>from 3 000 ₽</span></a><a href=\"#\"><i class=\"fas fa-redo\"></i><span>14 days<br>return/change</span></a><a href=\"#\"><i class=\"fas fa-umbrella\"></i><span>2 years<br>warranty</span></a></p>"),
				"type" => "editor",
			),
			array(
				"name" => "show_recomm",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_coupon",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_sharecart",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_menu_bottom",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_menu_footer",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_diafan",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_sitemap",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_privacy",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_upper",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_consultant",
				"value" => array("1"),
				"type" => "checkbox",
			),
			array(
				"name" => "show_payments",
				"value" => array("visa"),
				"type" => "enum",
			),
			array(
				"name" => "show_payments",
				"value" => array("", "mastercard"),
				"type" => "enum",
			),
			array(
				"name" => "show_payments",
				"value" => array("", "maestro"),
				"type" => "enum",
			),
			array(
				"name" => "show_payments",
				"value" => array("", "jcb"),
				"type" => "enum",
			),
			array(
				"name" => "show_payments",
				"value" => array("mir"),
				"type" => "enum",
			),
			array(
				"name" => "show_payments",
				"value" => array("", "paypal"),
				"type" => "enum",
			),
			array(
				"name" => "show_payments",
				"value" => array("unionpay"),
				"type" => "enum",
			),
			array(
				"name" => "show_payments",
				"value" => array("", "webmoney"),
				"type" => "enum",
			),
			array(
				"name" => "show_socnets",
				"value" => array("vk"),
				"type" => "enum",
			),
			array(
				"name" => "show_socnets",
				"value" => array("ok"),
				"type" => "enum",
			),
			array(
				"name" => "show_socnets",
				"value" => array("dzen"),
				"type" => "enum",
			),
			array(
				"name" => "show_socnets",
				"value" => array("youtube"),
				"type" => "enum",
			),
			array(
				"name" => "show_socnets",
				"value" => array("telegram"),
				"type" => "enum",
			),
			array(
				"name" => "show_socnets",
				"value" => array("", "youtube"),
				"type" => "enum",
			),
			array(
				"name" => "show_socnets",
				"value" => array("", "telegram"),
				"type" => "enum",
			),
			array(
				"name" => "show_socnets",
				"value" => array("", "instagram"),
				"type" => "enum",
			),
			array(
				"name" => "show_socnets",
				"value" => array("", "facebook"),
				"type" => "enum",
			),
			array(
				"name" => "show_socnets",
				"value" => array("", "x"),
				"type" => "enum",
			),
			array(
				"name" => "socnet_vk",
				"value" => array("#"),
				"type" => "text",
			),
			array(
				"name" => "socnet_ok",
				"value" => array("#"),
				"type" => "text",
			),
			array(
				"name" => "socnet_dzen",
				"value" => array("#"),
				"type" => "text",
			),
			array(
				"name" => "socnet_telegram",
				"value" => array("#", "#"),
				"type" => "text",
			),
			array(
				"name" => "socnet_youtube",
				"value" => array("#", "#"),
				"type" => "text",
			),
			array(
				"name" => "socnet_instagram",
				"value" => array("", "#"),
				"type" => "text",
			),
			array(
				"name" => "socnet_facebook",
				"value" => array("", "#"),
				"type" => "text",
			),
			array(
				"name" => "socnet_x",
				"value" => array("", "#"),
				"type" => "text",
			),
		),
	);

	/**
	 * @var array демо-данные
	 */
	public $demo = array(
		"site" => array(
			array(
				"id" => 1,
				"text" => array("<h1>О нашем магазине</h1><p>Здравствуйте, дорогие посетители! Мы рады представить вам наш демо-сайт, на котором вы можете ознакомиться с возможностями системы DIAFAN.CMS. Все изображения и тексты, которые вы видите на этом сайте, были разработаны совместно с ИИ.</p><p>Приятного просмотра!</p><p class=\"benefit_enum\"><a href=\"#\"><i class=\"fas fa-truck\"></i><span>Бесплатная<br>доставка</span></a><a href=\"#\"><i class=\"fas fa-umbrella\"></i><span>Гарантия<br>на все товары</span></a><a href=\"#\"><i class=\"fas fa-map-marker-alt\"></i><span>Магазины<br>в Москве</span></a><a href=\"#\"><i class=\"far fa-credit-card\"></i><span>Удобная<br>система оплаты</span></a></p>", "<h1>About our shop</h1><p>Welcome to our demo site, which is the result of collaboration with artificial intelligence (AI)! The site presents products, images and descriptions of which were created using the capabilities of AI. Even the text you are reading now was generated by our AI assistant.</p><p>The demo site is a convenient tool that allows you to familiarize yourself with the capabilities of DIAFAN.CMS.</p><p>Enjoy watching!</p><p class=\"benefit_enum\"><a href=\"#\"><i class=\"fas fa-truck\"></i><span>Free<br>shipping</span></a><a href=\"#\"><i class=\"fas fa-umbrella\"></i><span>Warranty<br>on all products</span></a><a href=\"#\"><i class=\"fas fa-map-marker-alt\"></i><span>Shops<br>in Moscow</span></a><a href=\"#\"><i class=\"far fa-credit-card\"></i><span>Convenient<br>system of payment</span></a></p>"),
			),
			array(
				"id" => 4,
				"text" => array("<p>Добро пожаловать на сайт нашей компании - мира безграничного выбора и товаров на любой вкус! Мы предлагаем вам окунуться в океан товаров, где каждый сможет найти что-то особенное для себя.</p>
<p>С самого начала мы ставили перед собой цель создать удобный и надежный сервис, который позволит нашим клиентам быстро и без лишних хлопот находить и приобретать товары по доступным ценам. Наша компания постоянно развивается и расширяет ассортимент, чтобы удовлетворить потребности самых требовательных покупателей.</p>
<p>Мы тщательно отбираем товары, чтобы предложить вам только лучшее. Наш ассортимент включает в себя товары различных категорий, таких как электроника, бытовая техника, одежда, обувь, аксессуары, товары для дома, спортивные товары и многое другое. Благодаря этому вы сможете найти все, что вам нужно, в одном месте.</p>
<p>Наша компания стремится к тому, чтобы каждый покупатель остался доволен своим выбором и полученным сервисом. Именно поэтому мы предлагаем быструю и надежную доставку, а также возможность возврата товара, если он не соответствует вашим ожиданиям.</p>
<p>Кроме того, мы постоянно работаем над улучшением нашего сай<span>та, чтобы сделать его более удобным и функциональным для наших клиентов. Вы можете легко найти нужный товар, используя удобный поиск, а также ознакомиться с отзывами других покупателей перед покупкой.</span></p>", "<p>Welcome to the website of our company - the world of unlimited choice and products for every taste! We invite you to plunge into the ocean of goods, where everyone can find something special for themselves.</p>
<p>From the very beginning, we set ourselves the goal of creating a convenient and reliable service that will allow our customers to quickly and hassle-free find and purchase goods at affordable prices. Our company is constantly developing and expanding its range to meet the needs of the most demanding customers.</p>
<p>We carefully select products to offer you only the best. Our range includes products of various categories such as electronics, household appliances, clothing, shoes, accessories, household goods, sporting goods and much more. Thanks to this, you will be able to find everything you need in one place.</p>
<p>Our company strives to ensure that every customer is satisfied with their choice and the service they receive. That is why we offer fast and reliable delivery, as well as the possibility of returning the product if it does not meet your expectations.</p>
<p>In addition, we are constantly working to improve our website to make it more convenient and functional for our customers. You can easily find the right product using a convenient search, as well as read reviews from other customers before buying.</p>"),
				"rewrite" => "about",
				"children" => array(
					array(
						"name" => array('Философия', 'Philosophy'),
						"text" => array("<h2>Мы работаем в России</h2>
<p>Наша компания уже несколько лет успешно работает на российском рынке, предоставляя широкий ассортимент товаров и услуг высокого качества. Мы стремимся быть лидерами в своей отрасли, постоянно совершенствуя свои продукты и услуги, а также предлагая нашим клиентам выгодные условия сотрудничества.</p>
<h2>Оптимальный выбор</h2>
<p>На нашем сайте представлен оптимальный выбор качественных товаров, которые смогут удовлетворить потребности любого покупателя. Мы предлагаем широкий ассортимент продукции от ведущих мировых производителей, что позволяет нашим клиентам выбирать из множества вариантов и находить именно то, что им нужно. Кроме того, наша команда профессионалов всегда готова помочь в выборе товара и ответить на все возникающие вопросы.</p>
<h2>Гарантии</h2>
<p>При покупке товаров на нашем сайте, вы можете быть уверены в гарантийных обязательствах нашей компании. Мы предоставляем гарантию на все товары, представленные на нашем сайте. Если товар не соответствует заявленным характеристикам или имеет дефекты, мы гарантируем возврат денежных средств или замену товара на аналогичный. Также мы предлагаем бесплатную доставку по всей территории России при заказе на сумму свыше 5000 рублей.</p>", "<h2>We work in Russia</h2>
<p>Our company has been successfully operating in the Russian market for several years, providing a wide range of high-quality goods and services. We strive to be leaders in our industry, constantly improving our products and services, as well as offering our customers favorable terms of cooperation.</p>
<h2>The optimal choice</h2>
<p>Our website offers an optimal selection of high-quality products that can meet the needs of any customer. We offer a wide range of products from the world's leading manufacturers, which allows our customers to choose from a variety of options and find exactly what they need. In addition, our team of professionals is always ready to help you choose a product and answer all your questions.</p>
<h2>Guarantees</h2>
<p>When purchasing products on our website, you can be sure of the warranty obligations of our company. We provide a guarantee for all products presented on our website. If the product does not meet the stated characteristics or has defects, we guarantee a refund or replacement of the product with a similar one. We also offer free shipping throughout Russia for orders over 5,000 rubles.</p>"),
						"rewrite" => "about/filosofiya",
						"menu" => 1,
					),
					array(
						"name" => array('История', 'History'),
						"text" => array("<p>Наша компания была основана в 2003 году и с тех пор мы прошли долгий путь развития и роста. Мы начинали как небольшая команда энтузиастов, работающих над созданием инновационных продуктов для улучшения жизни людей.</p>
<p>За прошедшие годы мы расширили нашу команду и географию присутствия, став одним из лидеров в своей отрасли. Мы продолжаем инвестировать в исследования и разработки, чтобы создавать еще более качественные и полезные продукты.</p>
<p>Одним из наших ключевых достижений является выход на международный рынок. Мы гордимся тем, что можем внести свой вклад в улучшение жизни людей по всему миру.</p>
<p>Но несмотря на все наши успехи, мы не забываем о наших корнях и продолжаем придерживаться принципов, которые помогли нам стать тем, кем мы являемся сегодня. Мы верим в силу инноваций, командной работы и стремления к лучшему.</p>
<p>В будущем мы планируем продолжать расти и развиваться, оставаясь верными нашим ценностям и принципам. Мы уверены, что впереди нас ждут новые возможности и достижения, которые позволят нам продолжать делать мир лучше с помощью наших товаров.</p>", "<p>Our company was founded in 2003 and since then we have gone a long way of development and growth. We started as a small team of enthusiasts working to create innovative products to improve people's lives.</p>
<p>Over the years, we have expanded our team and geography of presence, becoming one of the leaders in our industry. We continue to invest in research and development to create even better and more useful products.</p>
<p>One of our key achievements is entering the international market. We are proud to be able to contribute to improving the lives of people around the world.</p>
<p>But despite all our successes, we do not forget about our roots and continue to adhere to the principles that helped us become who we are today. We believe in the power of innovation, teamwork and striving for the best.</p>
<p>In the future, we plan to continue to grow and develop while remaining true to our values and principles. We are confident that there are new opportunities and achievements ahead of us that will allow us to continue to make the world a better place with our products.</p>"),
						"rewrite" => "about/istoriya",
						"menu" => 1,
					),
				),
			),
			array(
				"name" => array('Наши магазины', 'Our shops'),
				"text" => array("<p>Город: Китеград, улица: Облачная, дом: 10, название магазина: &ldquo;Волшебство под облаками&rdquo;.</p>
<p>Город: Летающий-лес, улица: Цветочная, дом: 45, название магазина: &ldquo;Сад чудес&rdquo;.</p>
<p>Город: Морской-прибой, улица: Береговая, дом: 33, название магазина: &ldquo;Подводный рай&rdquo;.</p>
<p>Город: Сказочная-страна, улица: Луговая, дом: 77, название магазина: &ldquo;Чудо-товары&rdquo;.</p>
<p>Город: Звездный-путь, улица: Космическая, дом: 22, название магазина: &ldquo;Галактические открытия&rdquo;.</p>
<p>Город: Солнечный-оазис, улица: Оазисная, дом: 55, название магазина: &ldquo;Фонтан желаний&rdquo;.</p>", "<p>City: Kitegrad, street: Cloudy, house: 10, store name: &ldquo;Magic under the clouds\".</p>
<p>City: Flying forest, street: Flower, house: 45, store name: &ldquo;Garden of Wonders\".</p>
<p>City: Morskoy-priboy, street: Beregovaya, house: 33, store name: &ldquo;Underwater paradise\".</p>
<p>City: Fairy-tale country, street: Lugovaya, house: 77, store name: &ldquo;Miracle goods\".</p>
<p>City: Star Trek, street: Kosmicheskaya, house: 22, store name: Galactic Discoveries.</p>
<p>City: Sunny oasis, street: Oazisnaya, house: 55, store name: &ldquo;Fountain of desires\".</p>"),
				"rewrite" => "shops",
				"menu" => 1,
				"children" => array(
					array(
						"name" => array('Интернет-магазины'),
						"text" => array("<p>&ldquo;Волшебный выбор&rdquo; -<span>&nbsp;</span>http://magic-choice.rup</p>
<p>&ldquo;Звездные товары&rdquo; -<span>&nbsp;</span>https://stellar-goods.comics</p>
<p>&ldquo;Летающие предметы&rdquo; -<span>&nbsp;</span>http://flying-objects.netur</p>
<p>&ldquo;Морские глубины&rdquo; -<span>&nbsp;</span>http://sea-depths.comics</p>
<p>&ldquo;Сказочные инструменты&rdquo; -<span>&nbsp;</span>http://fairytale-tools.orgen</p>
<p>&ldquo;Галактическая лавка&rdquo; -<span>&nbsp;</span>https://galactic-shop.clubnet</p>
<p>&ldquo;Лунные артефакты&rdquo; -<span>&nbsp;</span>http://lunar-artifacts.comics</p>
<p>&ldquo;Фонтан удачи&rdquo; -<span>&nbsp;</span>http://luck-fountain.comics</p>
<p>&ldquo;Подводное королевство&rdquo; -<span>&nbsp;</span>http://underwater-kingdom.netur</p>
<p>&ldquo;Волшебные предметы&rdquo; -<span>&nbsp;</span>http://magical-items.orgen</p>", "<p>The &ldquo;Magic Choice\" - http://magic-choice.rup</p>
<p>&ldquo;Star Products\" - https://stellar-goods.comics</p>
<p>&ldquo;Flying objects\" - http://flying-objects.netur</p>
<p>&ldquo;The depths of the sea\" - http://sea-depths.comics</p>
<p>&ldquo;Fabulous Tools\" - http://fairytale-tools.orgen</p>
<p>Galactic Shop - https://galactic-shop.clubnet</p>
<p>&ldquo;Lunar Artifacts\" - http://lunar-artifacts.comics</p>
<p>&ldquo;Fountain of Luck\" - http://luck-fountain.comics</p>
<p>&ldquo;Underwater Kingdom\" - http://underwater-kingdom.netur</p>
<p>&ldquo;Magic Items\" - http://magical-items.orgen</p>"),
						"rewrite" => "shops/on-line",
						"menu" => 1,
					),
					array(
						"name" => array("Оптовые склады", "Wholesale warehouses"),
						"text" => array("<p>Оптовый склад &ldquo;Волшебный мир&rdquo; - г. Китеград, ул. Облачная, д. 10</p>
<p>Оптовый склад &ldquo;Сказочная страна&rdquo; - г. Сказочная страна, ул. Луговая, д. 77</p>", "<p>Wholesale warehouse &ldquo;Magic world&rdquo; - Kitegrad, 10 Cloudy street</p>
<p>Wholesale warehouse &ldquo;Fairyland\" - Fairyland, Lugovaya str., 77</p>"),
						"rewrite" => "shops/optovye-sklady",
						"menu" => 1,
					),
				),
			),
		),
		"site_dynamic" => array(
			array(
				"id" => 1,
				"name" => array("Это актуально!", 'This is relevant!'),
				"type" => "editor",
				"module" => array(
					array(
						"element_type" => "element",
						"module_name" => "news",
					),
				),
				'element' => array(
					array(
						"module_name" => "news",
						"element_type" => "element",
						"element_id" => 20,
						"value" => array("<p>Вы можете найти актуальную информацию о модных тенденциях и новых коллекциях женских туфель на нашем сайте.</p>"),
					),
				),
			)
		),
		"site_theme" => array(
			array(
				"name" => "logo",
				"value" => array("logo1.svg", "logo2.svg"),
				"type" => "image",
				"copy" => array("site/theme/logo1.svg", "site/theme/logo2.svg"),
			),
		),
	);

	/**
	 * Выполняет действия при установке модуля
	 *
	 * @return void
	 */
	protected function action()
	{
		$this->config[] =
		array(
			"name" => "trial",
			"module_name" => "core",
			"value" => (time() + 86400 * 20),
		);
		if (! empty($_SESSION["install_name"]))
		{
			$this->site[0]["title_meta"] = $_SESSION["install_name"];
		}
	}
}
