<?php
/**
 * Модель данных виджета
 *
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    7.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2023 OOO «Диафан» (http://www.diafan.ru/)
 */
if ( ! defined('DIAFAN'))
{
	$path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}



class Users_admin_widget_notice_model extends Diafan
{
	/**
	 * @var array — данные виджета
	 */
	public $result;

	/**
	 * Получает данные виджета
	 * 
	 * @param string $rewrite — адрес страницы
	 * @param string $module_name — наименование модуля
	 * @param string $element_type — тип элемента
	 * @param integer $element_id — номер элемента
	 * @param integer $site_id — номер раздела
	 * 
	 * @return array
	 */
	public function fetch($rewrite, $module_name, $element_type, $element_id = 0, $site_id = 0)
	{
		$this->result['rewrite'] = $rewrite;
		$this->result['module_name'] = $module_name;
		$this->result['element_type'] = $element_type;
		$this->result['element_id'] = $element_id;
		$this->result['site_id'] = $site_id;

		$this->result['rows'] = DB::query_fetch_all("SELECT * FROM {users_notice} WHERE act='1' AND trash='0' AND (user_id=%d OR adresat_id=%d OR adresat_id=0) AND rewrite='%h' AND module_name='%h' AND element_type='%h' AND element_id=%d ORDER BY id",
		$this->diafan->_users->id, $this->diafan->_users->id, $rewrite, $module_name, $element_type, $element_id);

		$this->result['count'] = count($this->result['rows']);

		$users = array();
		foreach ($this->result['rows'] as $k => $row)
		{
			if ($row['user_id'])
			{
				$users[$row['user_id']] = $row['user_id'];

				$this->result['rows'][$k]['my'] = ($row['user_id'] == $this->diafan->_users->id);
			}
			if ($row['adresat_id'])
			{
				$users[$row['adresat_id']] = $row['adresat_id'];
			}
			$this->result['rows'][$k]['date'] = $this->format_date_5($row['created']);
		}

		$this->result['users'] = array();
		if ($users)
		{
			$this->result['users'] = DB::query_fetch_key("SELECT id, fio, name FROM {users} WHERE id IN (%s) AND trash='0'", implode(',', $users), 'id');
		}

		$this->result['adresats'] = array();
		if ($roles = DB::query_fetch_value("SELECT DISTINCT(role_id) FROM {users_role_perm} WHERE type='admin'", 'role_id'))
		{
			$this->result['adresats'] = DB::query_fetch_key("SELECT id, fio, name FROM {users} WHERE act='1' AND trash='0' AND role_id IN (%s) AND id<>%d", implode(',', $roles), $this->diafan->_users->id, 'id');
		}
	}

	/**
	 * Осуществляет умное форматирование даты
	 *
	 * @param integer $date дата в формет UNIX
	 * 
	 * @return string
	 */
	private function format_date_5($date)
	{
		if (! $date)
			return '';

		$months_array = array(
			'01' => $this->diafan->_('января'),
			'02' => $this->diafan->_('февраля'),
			'03' => $this->diafan->_('марта'),
			'04' => $this->diafan->_('апреля'),
			'05' => $this->diafan->_('мая'),
			'06' => $this->diafan->_('июня'),
			'07' => $this->diafan->_('июля'),
			'08' => $this->diafan->_('августа'),
			'09' => $this->diafan->_('сентября'),
			'10' => $this->diafan->_('октября'),
			'11' => $this->diafan->_('ноября'),
			'12' => $this->diafan->_('декабря')
		);
		$week_array = array(
			'1' => $this->diafan->_('понедельник'),
			'2' => $this->diafan->_('вторник'),
			'3' => $this->diafan->_('среда'),
			'4' => $this->diafan->_('четверг'),
			'5' => $this->diafan->_('пятница'),
			'6' => $this->diafan->_('суббота'),
			'0' => $this->diafan->_('воскресенье')
		);
		if (time() - $date < 3600)
		{
			$min = round((time() - $date) / 60);
			if ($min < 2)
			{
			return $this->diafan->_('1 минуту назад');
			}
			if ($min % 10 == 1 && $min > 20)
			{
			return $this->diafan->_('%s минуту назад', $min);
			}
			if ($min % 10 < 5 && ($min > 20 || $min < 10))
			{
			return $this->diafan->_('%s минуты назад', $min);
			}
			return $this->diafan->_('%s минут назад', $min);
		}

		if ($date >= mktime(0, 0, 0, date("m"), date("d"), date("Y")))
		{
			return  $this->diafan->_('Сегодня').', '.date("H:i", $date);
		}

		if ($date >= mktime(0, 0, 0, date("m"), date("d"), date("Y")) - 86400)
		{
			return $this->diafan->_('Вчера').', '.date("H:i", $date);
		}

		if ($date >= time() - 86400 * 30)
		{
			return date("d ", $date).$months_array[date("m", $date)].', '.$week_array[date("w", $date)];
		}

		if (date("Y", $date) == date("Y"))
		{
			return date("d ", $date).$months_array[date("m", $date)];
		}
		return date("d ", $date).$months_array[date("m", $date)].date(" Y ", $date).' '.$this->diafan->_('г.');
	}
}
