<?php
/**
 * Контроллер виджета
 *
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    7.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2023 OOO «Диафан» (http://www.diafan.ru/)
 */
if ( ! defined('DIAFAN'))
{
	$path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}



class Users_admin_widget_notice extends Diafan
{
	/**
	 * @var array — пусковая панель виджета
	 */
	public $launchbar = array(
		'main' => array(
			'home' => array(
				'icon' => array(
					'fa' => 'sticky-note',
					'close' => array(
						'fa' => 'times',
					),
				),
				'name' => 'Заметки',
				'color' => '',
				'num' => 0,
			),
		),
	);

	/**
	 * @var array — панель инструментов виджета
	 */
	public $toolbar = array(
		'main' => array(
			// 'home' => array(
			// 	'icon' => array(
			// 		'fa' => 'home',
			// 	),
			// 	'name' => 'Заметки',
			// 	'color' => '',
			// 	'num' => 0,
			// ),
			'hide' => array(
				'icon' => array(
					'fa' => 'eye-slash',
				),
				'name' => 'Больше не показывать',
				'color' => '',
				'num' => 0,
			),
		),
	);

	/**
	 * @var array — данные виджета из таблицы {admin_widget}
	 */
	private $row;

	/**
	 * @var array — настройки виджета
	 */
	private $config;

	/**
	 * @var boolean — флаг активности виджета
	 */
	public $active;

	/**
	 * @var array — стили виджета
	 */
	public $css;

	/**
	 * @var array — скрипты виджета
	 */
	public $js;

	/**
	 * Конструктор класса
	 *
	 * @param object $diafan — контейнер приложения
	 * @param array — данные виджета из таблицы {admin_widget}
	 * 
	 * @return void
	 */
	public function __construct($diafan, $row)
	{
		parent::__construct($diafan);
		$this->row = $row;
		$this->config = $this->params ? unserialize($this->params) : array();
		$this->active = false;
		$this->css = array();
		$this->js = array();
	}

	/**
	 * Получает значение внутренней переменной
	 *
	 * @param string $name — название внутренней переменной
	 * 
	 * @return mixed
	 */
	public function __get($name)
	{
		if (isset($this->row[$name]))
		{
			return $this->row[$name];
		}
		elseif(! isset($this->cache['var'][$name]))
		{
			switch ($name)
			{
				case 'model':
					$class_name = 'Users_admin_widget_notice_'.$name;
					Custom::inc('modules/users/admin/widget/notice/users.admin.widget.notice.'.$name.'.php');
					$this->cache['var'][$name] = new $class_name($this->diafan);
					break;

				default:
					$this->cache['var'][$name] = null;
			}
		}
		return $this->cache['var'][$name];
	}

	/**
	 * Получает настройку/и виджета
	 *
	 * @param string $name — наименование настройки
	 * @param mixed $default — значение по умолчанию
	 * 
	 * @return mixed
	 */
	public function config($name = '', $default = null)
	{
		return $name ? (isset($this->config[$name]) ? $this->config[$name] : $default) : $this->config;
	}

	/**
	 * Получает представление виджета
	 * 
	 * @return string
	 */
	public function init()
	{
		$this->model->fetch($this->diafan->_admin->rewrite, $this->diafan->_admin->module, $this->diafan->config('config') ? 'config' : $this->diafan->element_type(), $this->diafan->id, $this->diafan->config('element_site') ? $this->diafan->_route->site : 0);
		
		$this->active = ! empty($this->model->result['count']);

		foreach ($this->launchbar as $group => $items) foreach ($items as $key => $item)
		{
			if ($key == 'home')
			{
				$this->launchbar[$group][$key]['active'] = $this->active;
				$this->launchbar[$group][$key]['num'] = $this->model->result['count'];
				break 2;
			}
		}
		foreach ($this->toolbar as $group => $items) foreach ($items as $key => $item)
		{
			if ($key == 'hide')
			{
				$this->toolbar[$group][$key]['hide'] = ! $this->active;
				break 2;
			}
		}
		return $this->view('show', $this->model->result);
	}

	/**
	 * Получает представление панели запуска
	 * 
	 * @return string
	 */
	public function launchbar()
	{
		return $this->view('launchbar', $this->launchbar);
	}

	/**
	 * Получает представление панели инструментов
	 * 
	 * @return string
	 */
	public function toolbar()
	{
		return $this->view('toolbar', $this->toolbar);
	}

	/**
	 * Получает данные виджета
	 * 
	 * @param string $rewrite — адрес страницы
	 * @param string $module_name — наименование модуля
	 * @param string $element_type — тип элемента
	 * @param integer $element_id — номер элемента
	 * @param integer $site_id — номер раздела
	 * 
	 * @return array
	 */
	public function fetch($rewrite, $module_name, $element_type, $element_id = 0, $site_id = 0)
	{
		$this->model->fetch($rewrite, $module_name, $element_type, $element_id, $site_id);
		return $this->model->result;
	}

	/**
	 * Получает представление данных
	 * 
	 * @param string $name — название шаблона
	 * @param array $result — данные для представления
	 * @param string $template — модификатор шаблона
	 * 
	 * @return string
	 */
	public function view($name, $result = array(), $template = '')
	{
		$view = '';
		if ($name)
		{
			$file = '';
			$path = 'modules/users/admin/widget/notice/views/users.admin.widget.notice.view.'.$name;
			if ($template)
			{
				if (Custom::exists($path.'_'.$template.'.php'))
				{
					$file = $path.'_'.$template.'.php';
					if (Custom::exists('modules/users/admin/widget/notice/js/users.admin.widget.notice.'.$name.'_'.$template.'.js'))
					{
						$this->js[] = 'modules/users/admin/widget/notice/js/users.admin.widget.notice.'.$name.'_'.$template.'.js';
					}
					if (Custom::exists('modules/users/admin/widget/notice/css/users.admin.widget.notice.'.$name.'_'.$template.'.css'))
					{
						$this->css[] = 'modules/users/admin/widget/notice/css/users.admin.widget.notice.'.$name.'_'.$template.'.css';
					}
				}
			}
			elseif (Custom::exists($path.'.php'))
			{
				$file = $path.'.php';
				if (Custom::exists('modules/users/admin/widget/notice/js/users.admin.widget.notice.'.$name.'.js'))
				{
					$this->js[] = 'modules/users/admin/widget/notice/js/users.admin.widget.notice.'.$name.'.js';
				}
				if (Custom::exists('modules/users/admin/widget/notice/css/users.admin.widget.notice.'.$name.'.css'))
				{
					$this->css[] = 'modules/users/admin/widget/notice/css/users.admin.widget.notice.'.$name.'.css';
				}
			}
			if ($file)
			{
				ob_start();
				include(ABSOLUTE_PATH.Custom::path($file));
				$view = ob_get_contents();
				ob_end_clean();
			}
		}
		return $view;
	}

	/**
	 * Подключает скрипты
	 *
	 * @return void
	 */
	public function show_js()
	{
		$this->diafan->_admin->js_view[] = 'modules/users/admin/widget/notice/js/users.admin.widget.notice.js';
		foreach($this->js as $file)
		{
			if (in_array($file, $this->diafan->_admin->js_view))
			{
				continue;
			}
			$this->diafan->_admin->js_view[] = $file;
		}
	}

	/**
	 * Подключает стили
	 *
	 * @return void
	 */
	public function show_css()
	{
		// $this->diafan->_admin->css_view[] = 'modules/users/admin/widget/notice/css/users.admin.widget.notice.css';

		echo '<link rel="stylesheet" href="'.BASE_PATH.File::compress(Custom::path('modules/users/admin/widget/notice/css/users.admin.widget.notice.css'), 'css').'" type="text/css">';
		foreach($this->css as $file)
		{
			if (in_array($file, $this->css))
			{
				continue;
			}
			if (substr($file, 0, 4) != 'http')
			{
				$file = BASE_PATH.File::compress(Custom::path($file), 'css');
			}
			echo '<link rel="stylesheet" href="'.$file.'" type="text/css">';
		}
	}
}

/**
 * Users_admin_widget_notice_exception
 *
 * Исключение для контроллера
 */
class Users_admin_widget_notice_exception extends Exception
{
	public function __construct($message, $code = 0, $previous = null)
	{
        parent::__construct('Users_admin_widget_notice: '.$message, $code, $previous);
    }
}
