<?php
/**
 * Установка модуля
 *
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    7.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2019 OOO «Диафан» (http://www.diafan.ru/)
 */

if (! defined('DIAFAN'))
{
	$path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}

class Shop_install extends Install
{
	/**
	 * @var string название
	 */
	public $title = "Интернет-магазин";

	/**
	 * @var array таблицы в базе данных
	 */
	public $tables = array(
		array(
			"name" => "shop",
			"comment" => "Товары",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "TEXT",
					"comment" => "название",
					"multilang" => true,
				),
				array(
					"name" => "act",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "показывать на сайте: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "date_start",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата начала показа",
				),
				array(
					"name" => "date_finish",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата окончания показа",
				),
				array(
					"name" => "article",
					"type" => "VARCHAR(30) NOT NULL DEFAULT ''",
					"comment" => "артикул",
				),
				array(
					"name" => "measure_unit",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "единица измерения",
					"multilang" => true,
				),
				array(
					"name" => "weight",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "вес",
				),
				array(
					"name" => "length",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "длина",
				),
				array(
					"name" => "width",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "ширина",
				),
				array(
					"name" => "height",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "высота",
				),
				array(
					"name" => "map_no_show",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "не показывать на карте сайта: 0 - нет, 1 - да",
				),
				array(
					"name" => "changefreq",
					"type" => "ENUM( 'always', 'hourly', 'daily', 'weekly', 'monthly', 'yearly', 'never' ) NOT NULL DEFAULT 'always'",
					"comment" => "Changefreq для sitemap.xml",
				),
				array(
					"name" => "priority",
					"type" => "VARCHAR(3) NOT NULL DEFAULT ''",
					"comment" => "Priority для sitemap.xml",
				),
				array(
					"name" => "noindex",
					"type" => "ENUM('0','1') NOT NULL DEFAULT '0'",
					"comment" => "не индексировать: 0 - нет, 1 - да",
				),
				array(
					"name" => "cat_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор основной категории из таблицы {shop_category}",
				),
				array(
					"name" => "site_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор страницы сайта из таблицы {site}",
				),
				array(
					"name" => "brand_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор производителя из таблицы {shop_brand}",
				),
				array(
					"name" => "keywords",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "ключевые слова, тег Keywords",
					"multilang" => true,
				),
				array(
					"name" => "descr",
					"type" => "TEXT",
					"comment" => "описание, тэг Description",
					"multilang" => true,
				),
				array(
					"name" => "canonical",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "канонический тег",
					"multilang" => true,
				),
				array(
					"name" => "title_meta",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "заголовок окна в браузере, тег Title",
					"multilang" => true,
				),
				array(
					"name" => "anons",
					"type" => "TEXT",
					"comment" => "анонс",
					"multilang" => true,
				),
				array(
					"name" => "anons_plus",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "добавлять анонс к описанию: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "text",
					"type" => "TEXT",
					"comment" => "описание",
					"multilang" => true,
				),
				array(
					"name" => "yandex",
					"type" => "TEXT",
					"comment" => "данные для выгрузки в Яндекс.Маркет",
				),
				array(
					"name" => "show_yandex",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "выгружать в Яндекс.Маркет: 0 - нет, 1 - да (если в настройках выбрана выгрузка только выбранных товаров)",
				),
				array(
					"name" => "google",
					"type" => "TEXT",
					"comment" => "данные для выгрузки в Google Merchant",
				),
				array(
					"name" => "show_google",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "выгружать в Google Merchant: 0 - нет, 1 - да (если в настройках выбрана выгрузка только выбранных товаров)",
				),
				array(
					"name" => "no_buy",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "товар временно отсутствует: 0 - нет, 1 - да",
				),
				array(
					"name" => "import",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "элемент только что импортирован: 0 - нет, 1 - да",
				),
				array(
					"name" => "import_id",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "собственный идентификатор элемента при импорте",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "timeedit",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "время последнего изменения в формате UNIXTIME",
				),
				array(
					"name" => "counter_buy",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "количество покупок",
				),
				array(
					"name" => "hit",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "хит: 0 - нет, 1 - да",
				),
				array(
					"name" => "new",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "новинка: 0 - нет, 1 - да",
				),
				array(
					"name" => "action",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "акция: 0 - нет, 1 - да",
				),
				array(
					"name" => "is_file",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "это товар-файл: 0 - нет, 1 - да",
				),
				array(
					"name" => "good_set",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "это набор товаров: 0 - нет, 1 - да",
				),
				array(
					"name" => "access",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "доступ ограничен: 0 - нет, 1 - да",
				),
				array(
					"name" => "admin_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "пользователь из таблицы {users}, добавивший или первый отредктировавший товар в административной части",
				),
				array(
					"name" => "theme",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон страницы сайта",
				),
				array(
					"name" => "view",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон модуля",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY site_id (`site_id`)",
				"KEY brand_id (`brand_id`)",
				"KEY import_id (`import_id`)",
				"KEY cat_id (`cat_id`)",
			),
		),
		array(
			"name" => "shop_additional_cost",
			"comment" => "Сопутствующие услуги",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "название",
					"multilang" => true,
				),
				array(
					"name" => "text",
					"type" => "TEXT",
					"comment" => "описание",
					"multilang" => true,
				),
				array(
					"name" => "act",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "показывать на сайте: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "shop_rel",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "связана с товаром: 0 - нет, 1 - да",
				),
				array(
					"name" => "percent",
					"type" => "DOUBLE NOT NULL default '0'",
					"comment" => "процент от стоимости товаров в корзине",
				),
				array(
					"name" => 'price',
					"type" => "DOUBLE NOT NULL default '0'",
					"comment" => "цена",
				),
				array(
					"name" => "amount",
					"type" => "DOUBLE NOT NULL default '0'",
					"comment" => "бесплатно от",
				),
				array(
					"name" => "required",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "всегда включено в цену: 0 - нет, 1 - да",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_additional_cost_rel",
			"comment" => "Связь сопутствующих услуг и товаров",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор товара из таблицы {shop}",
				),
				array(
					"name" => "additional_cost_id",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор сопутствующей услуги из таблицы {shop_additional_cost}",
				),
				array(
					"name" => "summ",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "сумма",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY element_id (`element_id`)",
			),
		),
		array(
			"name" => "shop_additional_cost_category_rel",
			"comment" => "Связь сопутствующих услуг и категорий",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "cat_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор категории из таблицы {shop_category}",
				),
				array(
					"name" => "element_id",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор сопутствующей услуги из таблицы {shop_additional_cost}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY cat_id (`cat_id`)",
			),
		),
		array(
			"name" => "shop_brand",
			"comment" => "Бренды",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "TEXT",
					"comment" => "название",
					"multilang" => true,
				),
				array(
					"name" => "act",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "показывать на сайте: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "map_no_show",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "не показывать на карте сайта: 0 - нет, 1 - да",
				),
				array(
					"name" => "changefreq",
					"type" => "ENUM( 'always', 'hourly', 'daily', 'weekly', 'monthly', 'yearly', 'never' ) NOT NULL DEFAULT 'always'",
					"comment" => "Changefreq для sitemap.xml",
				),
				array(
					"name" => "priority",
					"type" => "VARCHAR(3) NOT NULL DEFAULT ''",
					"comment" => "Priority для sitemap.xml",
				),
				array(
					"name" => "noindex",
					"type" => "ENUM('0','1') NOT NULL DEFAULT '0'",
					"comment" => "не индексировать: 0 - нет, 1 - да",
				),
				array(
					"name" => "site_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор страницы сайта из таблицы {site}",
				),
				array(
					"name" => "keywords",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "ключевые слова, тег Keywords",
					"multilang" => true,
				),
				array(
					"name" => "descr",
					"type" => "TEXT",
					"comment" => "описание, тэг Description",
					"multilang" => true,
				),
				array(
					"name" => "canonical",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "канонический тег",
					"multilang" => true,
				),
				array(
					"name" => "title_meta",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "заголовок окна в браузере, тег Title",
					"multilang" => true,
				),
				array(
					"name" => "text",
					"type" => "TEXT",
					"comment" => "описание",
					"multilang" => true,
				),
				array(
					"name" => "import",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "производитель только что импортирован: 0 - нет, 1 - да",
				),
				array(
					"name" => "import_id",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "собственный идентификатор производителя при импорте",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "timeedit",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "время последнего изменения в формате UNIXTIME",
				),
				array(
					"name" => "admin_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "пользователь из таблицы {users}, добавивший или первый отредктировавший производителя в административной части",
				),
				array(
					"name" => "theme",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон страницы сайта",
				),
				array(
					"name" => "view",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон модуля",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY site_id (`site_id`)",
				"KEY import_id (`import_id`)",
			),
		),
		array(
			"name" => "shop_brand_category_rel",
			"comment" => "Связи производителей и категорий",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор производителя из таблицы {shop_brand}",
				),
				array(
					"name" => "cat_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор категории из таблицы {shop_category}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY cat_id (`cat_id`)",
				"KEY element_id (`element_id`)",
			),
		),
		array(
			"name" => "shop_cart",
			"comment" => "Товары в корзине",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "user_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор пользователя из таблицы {users}",
				),
				array(
					"name" => "session_id",
					"type" => "VARCHAR(64) NOT NULL DEFAULT ''",
					"comment" => "номер сессии пользователя из таблицы {sessions}, если не заполнен user_id",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "имя",
				),
				array(
					"name" => "mail",
					"type" => "VARCHAR(64) NOT NULL DEFAULT ''",
					"comment" => "e-mail",
				),
				array(
					"name" => "summ",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "сумма заказа",
				),
				array(
					"name" => "count_goods",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "количество товарных позиций",
				),
				array(
					"name" => "delivery_id",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор службы доставки из таблицы {shop_delivery}",
				),
				array(
					"name" => 'additional_cost',
					"type" => "TEXT",
					"comment" => "идентификаторы сопутствующих услугах, разделенные запятой",
				),
				array(
					"name" => 'orders',
					"type" => "TEXT",
					"comment" => "идентификаторы совершенных заказов, разделенные запятой",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY user_id (`user_id`)",
			),
		),
		array(
			"name" => "shop_cart_goods",
			"comment" => "Товары в корзине",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "cart_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор корзины из таблицы {shop_cart}",
				),
				array(
					"name" => "created",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата создания",
				),
				array(
					"name" => "good_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор товара из таблицы {shop}",
				),
				array(
					"name" => "price_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор цены товара - поле price_id из таблицы {shop_price}",
				),
				array(
					"name" => "count",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "количество товара",
				),
				array(
					"name" => 'param',
					"type" => "TEXT",
					"comment" => "идентификаторы характеристик товара из таблицы {shop_parm_select}, разделенные запятой",
				),
				array(
					"name" => 'additional_cost',
					"type" => "TEXT",
					"comment" => "идентификаторы сопутствующих услугах, разделенные запятой",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY cart_id (`cart_id`)",
			),
		),
		array(
			"name" => "shop_cart_log_mail",
			"comment" => "Лог отправлений писем",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "created",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата отправления",
				),
				array(
					"name" => "cart_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор корзины из таблицы {cart_id}",
				),
				array(
					"name" => "order_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор заказа из таблицы {shop_order}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY cart_id (`cart_id`)",
			),
		),
		array(
			"name" => "shop_cashregister",
			"comment" => "Чеки для онлайн кассы",
			"fields" => array(
				array(
					"name" => "master_id",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "главный идентификатор в формате UNIXTIME",
				),
				array(
					"name" => "slave_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "вторичный идентификатор в интервале master_id",
				),
				array(
					"name" => "id",
					"type" => "VARCHAR(21) NOT NULL DEFAULT ''",
					"comment" => "объединенный идентификатор",
				),
				array(
					"name" => "type",
					"type" => "ENUM('presell', 'sell', 'refund') NOT NULL DEFAULT 'sell'",
					"comment" => "тип чека: presell - предоплата, sell - полная оплата, refund - возврат",
				),
				array(
					"name" => "important",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "защита от повторого чека: 0 - нет, 1 - да",
				),
				array(
					"name" => "order_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор заказа из таблицы {shop_orders}",
				),
				array(
					"name" => "external_id",
					"type" => "VARCHAR(255) NOT NULL DEFAULT ''",
					"comment" => "идентификатор чека, полученный от сервиса",
				),
				array(
					"name" => "auto",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '1'",
					"comment" => "метод отправки чека: 0 - ручной, 1 - автоматический",
				),
				array(
					"name" => "timeedit",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "время последнего изменения в формате UNIXTIME",
				),
				array(
					"name" => "timesent",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "время отправки чека в формате UNIXTIME",
				),
				array(
					"name" => "status",
					"type" => "ENUM('0', '1', '2') NOT NULL DEFAULT '0'",
					"comment" => "отчет об отправки чека: 0 - не отправлено, 1 - отправлено, 2 - ошибка отправления",
				),
				array(
					"name" => "payment",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "касса из платежного метода",
				),
				array(
					"name" => "error",
					"type" => "TEXT",
					"comment" => "отчет об ошибке отправления",
				),
				array(
					"name" => "trace",
					"type" => "TEXT",
					"comment" => "трассировка отправления",
				),
			),
			"keys" => array(
				"PRIMARY KEY (`master_id`, `slave_id`)",
			),
		),
		array(
			"name" => "shop_category",
			"comment" => "Категории товаров",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "TEXT",
					"comment" => "название",
					"multilang" => true,
				),
				array(
					"name" => "act",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "показывать на сайте: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "map_no_show",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "не показывать на карте сайта: 0 - нет, 1 - да",
				),
				array(
					"name" => "changefreq",
					"type" => "ENUM( 'always', 'hourly', 'daily', 'weekly', 'monthly', 'yearly', 'never' ) NOT NULL DEFAULT 'always'",
					"comment" => "Changefreq для sitemap.xml",
				),
				array(
					"name" => "priority",
					"type" => "VARCHAR(3) NOT NULL DEFAULT ''",
					"comment" => "Priority для sitemap.xml",
				),
				array(
					"name" => "noindex",
					"type" => "ENUM('0','1') NOT NULL DEFAULT '0'",
					"comment" => "не индексировать: 0 - нет, 1 - да",
				),
				array(
					"name" => "parent_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор родителя из таблицы {shop_category}",
				),
				array(
					"name" => "count_children",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "количество вложенных категорий",
				),
				array(
					"name" => "site_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор страницы сайта из таблицы {site}",
				),
				array(
					"name" => "keywords",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "ключевые слова, тег Keywords",
					"multilang" => true,
				),
				array(
					"name" => "descr",
					"type" => "TEXT",
					"comment" => "описание, тэг Description",
					"multilang" => true,
				),
				array(
					"name" => "canonical",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "канонический тег",
					"multilang" => true,
				),
				array(
					"name" => "title_meta",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "заголовок окна в браузере, тег Title",
					"multilang" => true,
				),
				array(
					"name" => "anons",
					"type" => "TEXT",
					"comment" => "анонс",
					"multilang" => true,
				),
				array(
					"name" => "anons_plus",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "добавлять анонс к описанию: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "text",
					"type" => "TEXT",
					"comment" => "описание",
					"multilang" => true,
				),
				array(
					"name" => "show_yandex",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "выгружать в Яндекс.Маркет: 0 - нет, 1 - да (если в настройках выбрана выгрузка только выбранных категорий)",
				),
				array(
					"name" => "show_google",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "выгружать в Google Merchant: 0 - нет, 1 - да (если в настройках выбрана выгрузка только выбранных товаров)",
				),
				array(
					"name" => "import",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "категория только что импортирован: 0 - нет, 1 - да",
				),
				array(
					"name" => "import_id",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "собственный идентификатор категории при импорте",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "timeedit",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "время последнего изменения в формате UNIXTIME",
				),
				array(
					"name" => "access",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "доступ ограничен: 0 - нет, 1 - да",
				),
				array(
					"name" => "admin_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "пользователь из таблицы {users}, добавивший или первый отредктировавший категорию в административной части",
				),
				array(
					"name" => "theme",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон страницы сайта",
				),
				array(
					"name" => "view",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон модуля",
				),
				array(
					"name" => "view_rows",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон модуля для элементов в списке категории",
				),
				array(
					"name" => "view_element",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон модуля для элементов в категории",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY parent_id (`parent_id`)",
				"KEY site_id (`site_id`)",
				"KEY import_id (`import_id`)",
			),
		),
		array(
			"name" => "shop_category_rel",
			"comment" => "Связи товаров и категорий",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор товара из таблицы {shop}",
				),
				array(
					"name" => "cat_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор категории из таблицы {shop_category}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY cat_id (`cat_id`)",
				"KEY element_id (`element_id`)",
			),
		),
		array(
			"name" => "shop_category_parents",
			"comment" => "Родительские связи категорий товаров",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор категории из таблицы {shop_category}",
				),
				array(
					"name" => "parent_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор категории-родителя из таблицы {shop_category}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_counter",
			"comment" => "Счетчик просмотров товаров",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор товара из таблицы {shop}",
				),
				array(
					"name" => "count_view",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "количество просмотров",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY element_id (`element_id`)",
			),
		),
		array(
			"name" => "shop_currency",
			"comment" => "Дополнительные валюты магазина",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(20) NOT NULL DEFAULT ''",
					"comment" => "название",
				),
				array(
					"name" => "exchange_rate",
					"type" => "DOUBLE NOT NULL default '0'",
					"comment" => "курс к основной валюте",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_delivery",
			"comment" => "Способы доставки",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "название",
					"multilang" => true,
				),
				array(
					"name" => "text",
					"type" => "TEXT",
					"comment" => "описание",
					"multilang" => true,
				),
				array(
					"name" => "service",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "служба доставки",
				),
				array(
					"name" => "params",
					"type" => "TEXT",
					"comment" => "серилизованные настройки службы доставки",
				),
				array(
					"name" => "act",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "показывать на сайте: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_delivery_history",
			"comment" => "История заказов на доставку",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "order_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "номер заказа",
				),
				array(
					"name" => "status",
					"type" => "ENUM('init', 'complete', 'error') NOT NULL DEFAULT 'init'",
					"comment" => "статус заказа на доставку: init - создан, complete - оформление завершено, error - ошибка",
				),
				array(
					"name" => "created",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата создания",
				),
				array(
					"name" => "summ",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "стоимость доставки",
				),
				array(
					"name" => "service",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "служба доставки",
				),
				array(
					"name" => "service_id",
					"type" => "VARCHAR(50) NOT NULL DEFAULT '0'",
					"comment" => "идентификатор заказа в системе службы доставки",
				),
				array(
					"name" => "tracknumber",
					"type" => "VARCHAR(50) NOT NULL DEFAULT '0'",
					"comment" => "трек-номер заказа",
				),
				array(
					"name" => "data",
					"type" => "TEXT",
					"comment" => "данные о заказе, используемые бэкендом",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_delivery_thresholds",
			"comment" => "Стоимость способов доставки",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "delivery_id",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор способа доставки из таблицы {shop_delivery}",
				),
				array(
					"name" => 'price',
					"type" => "DOUBLE NOT NULL default '0'",
					"comment" => "стоимость",
				),
				array(
					"name" => "amount",
					"type" => "DOUBLE NOT NULL default '0'",
					"comment" => "сумма, от которой действует стоимость",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_discount",
			"comment" => "Скидки",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "variable",
					"type" => "ENUM( 'goods', 'users', 'order' ) NOT NULL DEFAULT 'goods'",
					"comment" => "тип скидки: goods - товары, users - пользователи, order - заказы",
				),
				array(
					"name" => "date_start",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата начала действия",
				),
				array(
					"name" => "date_finish",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата окночания действия",
				),
				array(
					"name" => "discount",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "процент скидки",
				),
				array(
					"name" => "amount",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "действует от цены товара",
				),
				array(
					"name" => "deduction",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "фиксированная сумма скидки",
				),
				array(
					"name" => "threshold",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "действует от общей суммы заказа",
				),
				array(
					"name" => "threshold_cumulative",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "действует от общей оплаченных заказов",
				),
				array(
					"name" => "threshold_goods",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "действует от общего количества позиций в заказе",
				),
				array(
					"name" => "threshold_count",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "действует от общего количества товаров в заказе",
				),
				array(
					"name" => "delivery_id",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор службы доставки из таблицы {shop_delivery}",
				),
				array(
					"name" => "payment_id",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "метод оплаты из таблицы {payment}",
				),
				array(
					"name" => "threshold_combine",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '1'",
					"comment" => "комбинировать с другими скидками: 0 - нет, 1 - да",
				),
				array(
					"name" => "no_auth",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "скидка действует только для неавторизованных пользователей: 0 - нет, 1 - да",
				),
				array(
					"name" => "role_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "тип пользователя из таблицы {users_role}, для которого установлена скидка",
				),
				array(
					"name" => "act",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "показывать на сайте: 0 - нет, 1 - да",
				),
				array(
					"name" => "person",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "скидка действует только для определенных пользователей: 0 - нет, 1 - да",
				),
				array(
					"name" => "text",
					"type" => "TEXT",
					"comment" => "описание скидки для администратора",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_discount_coupon",
			"comment" => "Купоны на скидку",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "discount_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор скидки из таблицы {shop_discount}",
				),
				array(
					"name" => "coupon",
					"type" => "VARCHAR( 10 ) NOT NULL DEFAULT ''",
					"comment" => "код купона",
				),
				array(
					"name" => "count_use",
					"type" => "TINYINT(3) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "cколько раз можно использовать купон",
				),
				array(
					"name" => "used",
					"type" => "TINYINT(3) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "сколько раз купон использован при оформлении заказа",
				),
				array(
					"name" => "trash",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY discount_id (`discount_id`)",
			),
		),
		array(
			"name" => "shop_discount_object",
			"comment" => "Товары и категории, на которые действуют скидки",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "discount_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор скидки из таблицы {shop_discount}",
				),
				array(
					"name" => "brand_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор производителя из таблицы {shop_brand}",
				),
				array(
					"name" => "cat_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор категории из таблицы {shop_category}",
				),
				array(
					"name" => "good_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор товара из таблицы {shop}",
				),
				array(
					"name" => "param_value",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор характеристики из таблицы {shop_param_select}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY discount_id (`discount_id`)",
			),
		),
		array(
			"name" => "shop_discount_periodicity",
			"comment" => "Периодичность применения скидки",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "discount_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор скидки из таблицы {shop_discount}",
				),
				array(
					"name" => "type",
					"type" => "ENUM('week', 'time') NOT NULL DEFAULT 'week'",
					"comment" => "тип: week - день недели, time - время",
				),
				array(
					"name" => "day_week",
					"type" => "ENUM('0','1','2','3','4','5','6','7') NOT NULL DEFAULT '0'",
					"comment" => "день недели",
				),
				array(
					"name" => "time_start",
					"type" => "INT(6) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата начала действия",
				),
				array(
					"name" => "time_finish",
					"type" => "INT(6) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата окночания действия",
				),
				array(
					"name" => "trash",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY discount_id (`discount_id`)",
			),
		),
		array(
			"name" => "shop_discount_person",
			"comment" => "Пользователи, для которых действуют скидки",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "discount_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор скидки из таблицы {shop_discount}",
				),
				array(
					"name" => "user_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор пользователя из таблицы {users}",
				),
				array(
					"name" => "session_id",
					"type" => "VARCHAR(64) NOT NULL DEFAULT ''",
					"comment" => "номер сессии пользователя из таблицы {sessions}",
				),
				array(
					"name" => "coupon_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор купона из таблицы {shop_discount_coupon}",
				),
				array(
					"name" => "used",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "скидка уже использована: 0 - нет, 1 - да",
				),
				array(
					"name" => "trash",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY discount_id (`discount_id`)",
			),
		),
		array(
			"name" => "shop_files_codes",
			"comment" => "Коды для скачивания товаров-нематериальных активов",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "shop_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор товара из таблицы {shop}",
				),
				array(
					"name" => "code",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "код для скачивания товара-файла",
				),
				array(
					"name" => "date_finish",
					"comment" => "дата и время окончания действия кода",
					// TO_DO: такой вариант записи подходит для MySQL 5.6 и выше
					// "type" => "DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP",
					"type" => "TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_import",
			"comment" => "Описание полей файлов импорта",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "название",
				),
				array(
					"name" => "cat_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор описания файла из таблицы {shop_import}",
				),
				array(
					"name" => "type",
					"type" => "VARCHAR(20) NOT NULL DEFAULT ''",
					"comment" => "тип",
				),
				array(
					"name" => "params",
					"type" => "TEXT",
					"comment" => "серилизованные данные о поле",
				),
				array(
					"name" => "required",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "серилизованные данные о поле",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "выдавать ошибку, если поле не заполнено: 0 - нет, 1 - да",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_import_category",
			"comment" => "Описание файлов импорта",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "название",
				),
				array(
					"name" => "format",
					"type" => "ENUM('csv', 'xls') NOT NULL DEFAULT 'csv'",
					"comment" => "формат файла",
				),
				array(
					"name" => "type",
					"type" => "ENUM('good', 'category', 'brand') NOT NULL DEFAULT 'good'",
					"comment" => "тип данных: good - товары, category - категории товаров",
				),
				array(
					"name" => "delete_items",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "удалять не описанные в файле данные: 0 - нет, 1 - да",
				),
				array(
					"name" => "site_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор страницы сайта из таблицы {site}",
				),
				array(
					"name" => "cat_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор категории из таблицы {shop_category}",
				),
				array(
					"name" => "count_part",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "количество строк, выгружаемых за один проход скрипта",
				),
				array(
					"name" => "delimiter",
					"type" => "VARCHAR(20) NOT NULL DEFAULT ''",
					"comment" => "разделитель данных в строке",
				),
				array(
					"name" => "end_string",
					"type" => "VARCHAR(20) NOT NULL DEFAULT ''",
					"comment" => "обозначать конец строки символом",
				),
				array(
					"name" => "encoding",
					"type" => "VARCHAR(20) NOT NULL DEFAULT ''",
					"comment" => "кодировка",
				),
				array(
					"name" => "sub_delimiter",
					"type" => "VARCHAR(20) NOT NULL DEFAULT ''",
					"comment" => "разделитель данных внутри поля",
				),
				array(
					"name" => "header",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "первая строка - названия столбцов: 0 - нет, 1 - да",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "trash",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_rel",
			"comment" => "Связи похожих товаров",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор товара из таблицы {shop}",
				),
				array(
					"name" => "rel_element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор похожего товара из таблицы {shop}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_order",
			"comment" => "Заказы",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "user_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор пользователя из таблицы {users}",
				),
				array(
					"name" => "created",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата создания",
				),
				array(
					"name" => "status",
					"type" => "ENUM('0', '1', '2', '3', '4') NOT NULL DEFAULT '0'",
					"comment" => "действие статуса заказа",
				),
				array(
					"name" => "status_id",
					"type" => "TINYINT(3) NOT NULL DEFAULT '0'",
					"comment" => "идентификатор статуса заказа из таблицы {shop_order_status}",
				),
				array(
					"name" => "lang_id",
					"type" => "TINYINT(2) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор языковой версии сайта, с которой был сделан заказ, из таблицы {languages}",
				),
				array(
					"name" => "summ",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "общая сумма заказа",
				),
				array(
					"name" => 'cost_summ',
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "затраты на заказ",
				),
				array(
					"name" => "delivery_id",
					"type" => "VARCHAR( 10 ) NOT NULL DEFAULT '0'",
					"comment" => "способ доставки из таблицы {shop_delivery}",
				),
				array(
					"name" => "delivery_summ",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "стоимость доставки",
				),
				array(
					"name" => "delivery_info",
					"type" => "TEXT",
					"comment" => "данные службы доставки",
				),
				array(
					"name" => "discount_id",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор скидки из таблицы {shop_discount}",
				),
				array(
					"name" => "discount_summ",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "сумма скидки",
				),
				array(
					"name" => "custom_discount_summ",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "сумма измененной скидки",
				),
				array(
					"name" => "coupon",
					"type" => "VARCHAR(255) NOT NULL DEFAULT ''",
					"comment" => "код купона",
				),
				array(
					"name" => "count_minus",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "товары списаны: 0 - нет, 1 - да",
				),
				array(
					"name" => "referer",
					"type" => "TEXT",
					"comment" => "рефферер",
				),
				array(
					"name" => "text",
					"type" => "TEXT",
					"comment" => "комментарии админа",
				),
				array(
					"name" => "text_client",
					"type" => "TEXT",
					"comment" => "комментарий для покупателя",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_order_additional_cost",
			"comment" => "Сопутствующие услуги, включенные в заказ",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "order_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор заказа из таблицы {shop_order}",
				),
				array(
					"name" => "order_goods_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор записи о купленном товаре из таблицы {shop_order_goods}",
				),
				array(
					"name" => "additional_cost_id",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор сопутствующей услуги из таблицы {shop_additional_cost}",
				),
				array(
					"name" => "summ",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "сумма",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_order_backend",
			"comment" => "Расширения заказа",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "название",
				),
				array(
					"name" => "backend",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "бэкенд",
				),
				array(
					"name" => "params",
					"type" => "TEXT",
					"comment" => "серилизованные настройки",
				),
				array(
					"name" => "act",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "показывать на сайте: 0 - нет, 1 - да",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "list",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "выводить в списке: 0 - нет, 1 - да",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_order_backend_element",
			"comment" => "Данные о заказе для бэкенда",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "order_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор заказа из таблицы {shop_order}",
				),
				array(
					"name" => "backend",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор расширения из таблицы {shop_order_backend}",
				),
				array(
					"name" => "type",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "тип - для разделения данных внутри бэкенда",
				),
				array(
					"name" => "value",
					"type" => "TEXT",
					"comment" => "данные",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_order_goods",
			"comment" => "Товары в заказе",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "order_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор заказа из таблицы {shop_order}",
				),
				array(
					"name" => "good_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор товара из таблицы {shop}",
				),
				array(
					"name" => "discount_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор скидки из таблицы {shop_discount}",
				),
				array(
					"name" => "count_goods",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "количество товаров",
				),
				array(
					"name" => 'price',
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "цена",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY good_id (`good_id`)",
				"KEY order_id (`order_id`)",
			),
		),
		array(
			"name" => "shop_order_goods_param",
			"comment" => "Дополнительных характеристики товаров в заказе",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "value",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "значение характеристики",
				),
				array(
					"name" => "param_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор характеристики из таблицы {shop_param}",
				),
				array(
					"name" => "order_goods_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор записи о купленном товаре из таблицы {shop_order_goods}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY order_goods_id (`order_goods_id`)",
			),
		),
		array(
			"name" => "shop_order_param",
			"comment" => "Поля конструктора формы оформления заказа",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "название",
					"multilang" => true,
				),
				array(
					"name" => "text",
					"type" => "TEXT",
					"comment" => "описание",
					"multilang" => true,
				),
				array(
					"name" => "type",
					"type" => "VARCHAR(20) NOT NULL DEFAULT ''",
					"comment" => "тип",
				),
				array(
					"name" => "info",
					"type" => "VARCHAR(30) NOT NULL DEFAULT ''",
					"comment" => "смысловая нагрузка",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "required",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "обязательно для заполнения: 0 - нет, 1 - да",
				),
				array(
					"name" => "show_in_form",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "использовать в стандатной форме оформления заказа: 0 - нет, 1 - да",
				),
				array(
					"name" => "show_in_form_register",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "позволять редактировать из личного кабинета: 0 - нет, 1 - да",
				),
				array(
					"name" => "show_in_form_one_click",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "использовать в форме быстрого заказа: 0 - нет, 1 - да",
				),
				array(
					"name" => "config",
					"type" => "TEXT",
					"comment" => "дополнительные настройки поля",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_order_param_element",
			"comment" => "Значения полей конструктора оформления заказа",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "value",
					"type" => "TEXT",
					"comment" => "значение",
				),
				array(
					"name" => "param_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор поля из таблицы {shop_order_param}",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор заказа из таблицы {shop_order}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_order_param_select",
			"comment" => "Варианты значений полей конструктора оформления заказа типа список",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "param_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор поля из таблицы {shop_order_param}",
				),
				array(
					"name" => "value",
					"type" => "TINYINT(2) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "значение для типа характеристики «галочка»: 0 - нет, 1 - да",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "значение",
					"multilang" => true,
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY param_id (`param_id`)",
			),
		),
		array(
			"name" => "shop_order_param_user",
			"comment" => "Значения полей конструктора оформления заказа, предзаполненные пользователями",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "value",
					"type" => "TEXT",
					"comment" => "значение",
				),
				array(
					"name" => "param_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор поля из таблицы {shop_order_param}",
				),
				array(
					"name" => "user_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор пользователя из таблицы {users}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_order_status",
			"comment" => "Статусы заказов",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "TINYINT(3) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "название",
					"multilang" => true,
				),
				array(
					"name" => "color",
					"type" => "VARCHAR(20) NOT NULL DEFAULT ''",
					"comment" => "цвет",
				),
				array(
					"name" => "status",
					"type" => "ENUM('0', '1', '2', '3', '4') NOT NULL DEFAULT '0'",
					"comment" => "действие статуса заказа",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "count_minus",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "списывать товары: 0 - нет, 1 - да",
				),
				array(
					"name" => "send_mail",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "отправлять уведомление пользователю о смене статуса: 0 - нет, 1 - да",
				),
				array(
					"name" => "custom_mail",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "уведомление при установке статуса: 0 - нет, 1 - да",
				),
				array(
					"name" => "subject_mail",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "тема письма",
					"multilang" => true,
				),
				array(
					"name" => "text_mail",
					"type" => "TEXT",
					"comment" => "текст письма",
					"multilang" => true,
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_param",
			"comment" => "Дополнительные характеристики товаров",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "название",
					"multilang" => true,
				),
				array(
					"name" => "type",
					"type" => "VARCHAR(20) NOT NULL DEFAULT ''",
					"comment" => "тип",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "search",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "характеристика выводится в форме поиска: 0 - нет, 1 - да",
				),
				array(
					"name" => "list",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "характеристика выводится в списке товаров: 0 - нет, 1 - да",
				),
				array(
					"name" => "block",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "характеристика выводится в блоке товаров: 0 - нет, 1 - да",
				),
				array(
					"name" => "id_page",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "характеристика выводится на странице товара: 0 - нет, 1 - да",
				),
				array(
					"name" => "required",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "доступен к выбору при заказе: 0 - нет, 1 - да",
				),
				array(
					"name" => "text",
					"type" => "TEXT",
					"comment" => "описание",
					"multilang" => true,
				),
				array(
					"name" => "config",
					"type" => "TEXT",
					"comment" => "дополнительные настройки поля",
				),
				array(
					"name" => "display_in_sort",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "выводится в блоке для сортировки: 0 - нет, 1 - да",
				),
				array(
					"name" => "yandex_use",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "выгружается в файл YML: 0 - нет, 1 - да",
				),
				array(
					"name" => "yandex_name",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "название в файле YML",
				),
				array(
					"name" => "yandex_unit",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "единица измерения в файле YML",
				),
				array(
					"name" => "measure_unit",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "единица измерения",
					"multilang" => true,
				),
				array(
					"name" => "site_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор страницы сайта из таблицы {site}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_param_category_rel",
			"comment" => "Связи дополнительных характеристик товаров и категорий",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор характеристики из таблицы {shop_param}",
				),
				array(
					"name" => "cat_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор категории из таблицы {shop_category}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY cat_id (`cat_id`)",
			),
		),
		array(
			"name" => "shop_param_element",
			"comment" => "Значения дополнительных характеристик товаров",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "value",
					"type" => "TEXT",
					"comment" => "значение",
					"multilang" => true,
				),
				array(
					"name" => "param_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор характеристики из таблицы {shop_param}",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор товара из таблицы {shop}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY element_id (`element_id`)",
				"KEY param_id (`param_id`)",
			),
		),
		array(
			"name" => "shop_param_select",
			"comment" => "Варианты значений дополнительных характеристик товаров типа список",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "param_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор характеристики из таблицы {shop_param}",
				),
				array(
					"name" => "value",
					"type" => "TINYINT(2) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "значение для типа характеристики «галочка»: 0 - нет, 1 - да",
				),
				array(
					"name" => "name",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "значение",
					"multilang" => true,
				),
				array(
					"name" => "h1",
					"type" => "TEXT",
					"comment" => "название страницы",
					"multilang" => true,
				),
				array(
					"name" => "act",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "показывать на сайте: 0 - нет, 1 - да",
					"multilang" => true,
				),
				array(
					"name" => "date_start",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата начала показа",
				),
				array(
					"name" => "date_finish",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата окончания показа",
				),
				array(
					"name" => "map_no_show",
					"type" => "ENUM( '0', '1' ) NOT NULL DEFAULT '0'",
					"comment" => "не показывать на карте сайта: 0 - нет, 1 - да",
				),
				array(
					"name" => "changefreq",
					"type" => "ENUM( 'always', 'hourly', 'daily', 'weekly', 'monthly', 'yearly', 'never' ) NOT NULL DEFAULT 'always'",
					"comment" => "Changefreq для sitemap.xml",
				),
				array(
					"name" => "priority",
					"type" => "VARCHAR(3) NOT NULL DEFAULT ''",
					"comment" => "Priority для sitemap.xml",
				),
				array(
					"name" => "noindex",
					"type" => "ENUM('0','1') NOT NULL DEFAULT '0'",
					"comment" => "не индексировать: 0 - нет, 1 - да",
				),
				array(
					"name" => "keywords",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "ключевые слова, тег Keywords",
					"multilang" => true,
				),
				array(
					"name" => "descr",
					"type" => "TEXT",
					"comment" => "описание, тэг Description",
					"multilang" => true,
				),
				array(
					"name" => "canonical",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "канонический тег",
					"multilang" => true,
				),
				array(
					"name" => "title_meta",
					"type" => "VARCHAR(250) NOT NULL DEFAULT ''",
					"comment" => "заголовок окна в браузере, тег Title",
					"multilang" => true,
				),
				array(
					"name" => "text",
					"type" => "TEXT",
					"comment" => "описание",
					"multilang" => true,
				),
				array(
					"name" => "timeedit",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "время последнего изменения в формате UNIXTIME",
				),
				array(
					"name" => "admin_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "пользователь из таблицы {users}, добавивший или первый отредктировавший страницу в административной части",
				),
				array(
					"name" => "theme",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон страницы сайта",
				),
				array(
					"name" => "view",
					"type" => "VARCHAR(50) NOT NULL DEFAULT ''",
					"comment" => "шаблон модуля",
				),
				array(
					"name" => "sort",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "подрядковый номер для сортировки",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY param_id (`param_id`)",
			),
		),
		array(
			"name" => "shop_price",
			"comment" => "Цены товаров",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "good_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор товара из таблицы {shop}",
				),
				array(
					"name" => 'price',
					"type" => "DOUBLE UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "цена",
				),
				array(
					"name" => "old_price",
					"type" => "DOUBLE UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "исходная цена",
				),
				array(
					"name" => "cost_price",
					"type" => "DOUBLE UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "закупочная цена",
				),
				array(
					"name" => "count_goods",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "количество товара",
				),
				array(
					"name" => "price_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор исходной цены из таблицы {shop_price}",
				),
				array(
					"name" => "date_start",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата начала действия",
				),
				array(
					"name" => "date_finish",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата окончания действия",
				),
				array(
					"name" => "discount",
					"type" => "DOUBLE NOT NULL DEFAULT '0'",
					"comment" => "скидка",
				),
				array(
					"name" => "discount_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор скидки из таблицы {shop_discount}",
				),
				array(
					"name" => "no_auth",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "цена действует только для неавторизованных пользователей: 0 - нет, 1 - да",
				),
				array(
					"name" => "person",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "цена действует только для определенных пользователей: 0 - нет, 1 - да",
				),
				array(
					"name" => "role_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор типа пользователя из таблицы {users_role}, для которого действует скидка",
				),
				array(
					"name" => "currency_id",
					"type" => "SMALLINT(5) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор валюты из таблицы {shop_currency}",
				),
				array(
					"name" => "import_id",
					"type" => "VARCHAR(100) NOT NULL DEFAULT ''",
					"comment" => "собственный идентификатор при импорте",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY good_id (`good_id`)",
				"KEY price_id (`price_id`)",
				"KEY import_id (`import_id`)",
			),
		),
		array(
			"name" => "shop_price_image_rel",
			"comment" => "Изображения товаров, прикрепленные к цене",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "price_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор исходной цены из таблицы {shop_price}",
				),
				array(
					"name" => "image_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор изображения из таблицы {images}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY price_id (`price_id`)",
			),
		),
		array(
			"name" => "shop_price_param",
			"comment" => "Дополнительные характеристики, учитываемые в цене",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "price_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор исходной цены из таблицы {shop_price}",
				),
				array(
					"name" => "param_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор характеристики из таблицы {shop_param}",
				),
				array(
					"name" => "param_value",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "значение характеристики - идентификатор из таблицы {shop_param_select}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY price_id (`price_id`)",
				"KEY param_id (`param_id`)",
				"KEY param_value (`param_value`)",
			),
		),
		array(
			"name" => "shop_set",
			"comment" => "Набор товаров",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор товара из таблицы {shop}",
				),
				array(
					"name" => "set_element_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор включенного в набор товара из таблицы {shop}",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY element_id (`element_id`)",
			),
		),
		array(
			"name" => "shop_waitlist",
			"comment" => "Товары в списке ожидания",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "good_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор товара из таблицы {shop}",
				),
				array(
					"name" => "user_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор пользователя из таблицы {users}",
				),
				array(
					"name" => "lang_id",
					"type" => "TINYINT(2) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор языка сайта из таблицы {languages}",
				),
				array(
					"name" => "mail",
					"type" => "VARCHAR(64) NOT NULL DEFAULT ''",
					"comment" => "e-mail пользователя",
				),
				array(
					"name" => "created",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата создания",
				),
				array(
					"name" => 'param',
					"type" => "TEXT",
					"comment" => "серилизованные данные о характеристиках товара (доступных к выбору при заказе)",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
			),
		),
		array(
			"name" => "shop_wishlist",
			"comment" => "Товары в списке пожеланий",
			"fields" => array(
				array(
					"name" => "id",
					"type" => "INT(11) UNSIGNED NOT NULL AUTO_INCREMENT",
					"comment" => "идентификатор",
				),
				array(
					"name" => "good_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор товара из таблицы {shop}",
				),
				array(
					"name" => "user_id",
					"type" => "INT(11) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "идентификатор пользователя из таблицы {users}",
				),
				array(
					"name" => "session_id",
					"type" => "VARCHAR(64) NOT NULL DEFAULT ''",
					"comment" => "номер сессии пользователя из таблицы {sessions}",
				),
				array(
					"name" => "created",
					"type" => "INT(10) UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "дата создания",
				),
				array(
					"name" => "count",
					"type" => "DOUBLE UNSIGNED NOT NULL DEFAULT '0'",
					"comment" => "количество товара",
				),
				array(
					"name" => 'param',
					"type" => "TEXT",
					"comment" => "серилизованные данные о характеристиках товара (доступных к выбору при заказе)",
				),
				array(
					"name" => 'additional_cost',
					"type" => "TEXT",
					"comment" => "идентификаторы сопутствующих услугах, разделенные запятой",
				),
				array(
					"name" => "is_file",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "товар-файл: 0 - нет, 1 - да",
				),
				array(
					"name" => "trash",
					"type" => "ENUM('0', '1') NOT NULL DEFAULT '0'",
					"comment" => "запись удалена в корзину: 0 - нет, 1 - да",
				),
			),
			"keys" => array(
				"PRIMARY KEY (id)",
				"KEY user_id (`user_id`)",
			),
		),
	);

	/**
	 * @var array записи в таблице {modules}
	 */
	public $modules = array(
		array(
			"name" => "shop",
			"admin" => true,
			"site" => true,
			"site_page" => true,
		),
		array(
			"name" => "cart",
			"site" => true,
			"site_page" => true,
			"admin" => true,
			"title" => "Корзина",
		),
		array(
			"name" => "cashregister",
			"site" => true,
			"admin" => true,
			"title" => "Онлайн касса",
		),
		array(
			"name" => "delivery",
			"site" => true,
			"admin" => true,
			"title" => "Служба доставки",
		),
		array(
			"name" => "order",
			"site" => true,
			"admin" => true,
			"title" => "Заказы",
		),
		array(
			"name" => "wishlist",
			"site" => true,
			"site_page" => true,
			"title" => "Отложенные",
		),
	);

    /**
     * @var array страницы сайта
     */
    public $site = array(
        array(
			"sort" => 4,
            "name" => array('Интернет-магазин', 'Catalog'),
            "act" => true,
            "module_name" => "shop",
            "rewrite" => "shop",
            "children" => array(
                array(
                    "name" => array('Корзина', 'Cart'),
                    "act" => true,
                    "module_name" => "cart",
                    "rewrite" => "shop/cart",
                    "map_no_show" => true,
                    "noindex" => true,
                    "search_no_show" => true,
                ),
                array(
                    "name" => array('Заказ оформлен', 'Order complete'),
                    "act" => true,
                    "rewrite" => "shop/cart/done",
                    "text" => array('<p>Спасибо за Ваш заказ! В ближайшее время мы свяжемся с Вами!</p><insert name="show_last_order" module="cart">', '<p>Thank you for your order! In the near future we will contact you!</p><insert name="show_last_order" module="cart">'),
                    "map_no_show" => true,
                    "noindex" => true,
                    "search_no_show" => true,
                ),
                array(
                    "name" => array('Отложенные', 'Wishlist'),
                    "act" => true,
                    "module_name" => "wishlist",
                    "rewrite" => "shop/wishlist",
                    "map_no_show" => true,
                    "noindex" => true,
                    "search_no_show" => true,
                ),
            ),
        ),
    );

	/**
	 * @var array меню административной части
	 */
	public $admin = array(
		array(
			"name" => "Каталог",
			"rewrite" => "shop",
			"group_id" => 4,
			"sort" => 11,
			"act" => true,
			"docs" => "http://www.diafan.ru/moduli/magazin/",
			"children" => array(
				array(
					"name" => "Товары",
					"rewrite" => "shop",
					"act" => true,
				),
				array(
					"name" => "Скидки",
					"rewrite" => "shop/discount",
					"act" => true,
				),
				array(
					"name" => "Скидки для пользователей",
					"rewrite" => "shop/discount/users",
					"act" => false,
				),
				array(
					"name" => "Скидки на заказы",
					"rewrite" => "shop/discount/order",
					"act" => false,
				),
				array(
					"name" => "Производители",
					"rewrite" => "shop/brand",
					"act" => true,
				),
				array(
					"name" => "Категории",
					"rewrite" => "shop/category",
					"act" => true,
				),
				array(
					"name" => "Характеристики",
					"rewrite" => "shop/param",
					"act" => true,
				),
				array(
					"name" => "Страницы характеристик",
					"rewrite" => "shop/parampage",
					"act" => true,
				),
				array(
					"name" => "Импорт/экспорт",
					"rewrite" => "shop/importexport",
					"act" => true,
				),
				array(
					"name" => "Настройки",
					"rewrite" => "shop/config",
				),
			)
		),
		array(
			"name" => "Заказы",
			"rewrite" => "order",
			"group_id" => 4,
			"sort" => 12,
			"act" => true,
			"docs" => "http://www.diafan.ru/moduli/magazin/",
			"children" => array(
				array(
					"name" => "Заказы",
					"rewrite" => "order",
					"act" => true,
				),
				array(
					"name" => "Расширения для заказа",
					"rewrite" => "order/backend",
					"act" => true,
				),
				array(
					"name" => "Настройки",
					"rewrite" => "order/config",
				),
			)
		),
		array(
			"name" => "Оплата",
			"rewrite" => "payment",
			"group_id" => "4",
			"sort" => 13,
			"act" => true,
			"children" => array(
				array(
					"name" => "Он-лайн касса",
					"rewrite" => "cashregister",
					"sort" => 4,
					"act" => true,
				),
				array(
					"name" => "Настройки",
					"rewrite" => "cashregister/config",
				),
			)
		),
		array(
			"name" => "Доставка",
			"rewrite" => "delivery",
			"group_id" => 4,
			"sort" => 14,
			"act" => true,
			"docs" => "http://www.diafan.ru/moduli/magazin/",
			"children" => array(
				array(
					"name" => "Способы доставки",
					"rewrite" => "delivery",
					"act" => true,
				),
				array(
					"name" => "История доставок",
					"rewrite" => "delivery/history",
					"act" => true,
				),
			)
		),
		array(
			"name" => "Формы и справочники",
			"rewrite" => "shop/additionalcost",
			"group_id" => 4,
			"sort" => 15,
			"act" => true,
			"docs" => "http://www.diafan.ru/moduli/magazin/",
			"children" => array(
				array(
					"name" => "Сопутствующие услуги",
					"rewrite" => "shop/additionalcost",
					"act" => true,
				),
				array(
					"name" => "Форма оформления заказа",
					"rewrite" => "shop/orderparam",
					"act" => true,
				),
				array(
					"name" => "Статусы заказа",
					"rewrite" => "shop/orderstatus",
					"act" => true,
				),
				array(
					"name" => "Валюты",
					"rewrite" => "shop/currency",
					"act" => true,
				),
			)
		),
		array(
			"name" => "Статистика",
			"rewrite" => "shop/ordercount",
			"group_id" => 4,
			"sort" => 16,
			"act" => true,
			"docs" => "http://www.diafan.ru/moduli/magazin/",
			"children" => array(
				array(
					"name" => "Отчеты о заказах",
					"rewrite" => "shop/ordercount",
					"act" => true,
				),
				array(
					"name" => "Отложенные товары",
					"rewrite" => "shop/wishlist",
					"act" => true,
				),
				array(
					"name" => "Список ожиданий",
					"rewrite" => "shop/waitlist",
					"act" => true,
				),
				array(
					"name" => "Брошенные корзины",
					"rewrite" => "cart",
					"act" => true,
				),
				array(
					"name" => "Статистика товаров",
					"rewrite" => "shop/counter",
					"act" => true,
				),
			)
		),
	);

	/**
	 * @var array настройки
	 */
	public $config = array(
		array(
			"name" => "cat",
			"value" => "1",
		),
		array(
			"name" => "images_element",
			"value" => "1",
		),
		array(
			"name" => "images_webp_element",
			"value" => "1",
		),
		array(
			"name" => "images_webp_cat",
			"value" => "1",
		),
		array(
			"name" => "images_brand",
			"value" => "1",
		),
		array(
			"name" => "images_webp_brand",
			"value" => "1",
		),
		array(
			"name" => "use_animation",
			"value" => "1",
		),
		array(
			"name" => "list_img_element",
			"value" => "1",
		),
		array(
			"name" => "counter",
			"value" => 0,
		),
		array(
			"name" => "nastr",
			"value" => "12",
		),
		array(
			"name" => "nastr_cat",
			"value" => "12",
		),
		array(
			"name" => "search_price",
			"value" => "1",
		),
		array(
			"name" => "search_article",
			"value" => "1",
		),
		array(
			"name" => "format_price_3",
			"value" => " ",
		),
		array(
			"name" => "tax_name",
			"value" => array("НДС", "VAT"),
		),
		array(
			"name" => "tax",
			"value" => 20,
		),
		array(
			"name" => "currency",
			"value" => array("₽", "₽"),
		),
		array(
			"name" => "yandex",
			"value" => "1",
		),
		array(
			"name" => "yandex_import",
			"value" => "1",
		),
		array(
			"name" => "currencyyandex",
			"value" => "RUR",
		),
		array(
			"name" => "bid",
			"value" => "15",
		),
		array(
			"name" => "order_redirect",
			"value" => "shop/cart/done",
		),
		array(
			"name" => "mes",
			"value" => array(
				"Оплата заказа №%id на сумму %summ руб.",
				"Payment order #%id in the amount of %summ RUB.",
			),
		),
		array(
			"name" => "subject",
			"value" => array(
				"Ваш заказ %id на сайте %url",
				"Your order %id on %url",
			),
		),
		array(
			"name" => "message",
			"value" => array(
				"<p>Здравствуйте!</p><p>Вы оформили заказ #%id на сайте %url</p>%order<p>Способ оплаты: %payment</p><p>%message</p><p>Спасибо за Ваш заказ! В ближайшее время мы с Вами свяжемся для подтверждения заказа</p>",
				"<p>Dear %fio!</p><p>You maked an order #%id on web site %url</p>%order<p>Payment system: %payment</p><p>%message</p><p>Thank you for your order! We will connect with you in near time to confirm order.</p>",
			),
		),
		array(
			"name" => "subject_admin",
			"value" => "Новый заказ %id на %url",
		),
		array(
			"name" => "message_admin",
			"value" => "Здравствуйте!<br>На сайте %url появился новый заказ номер %id: %order<br>Способ оплаты: %payment<br><br>%message",
		),
		array(
			"name" => "subject_change_status",
			"value" => array(
				"Статус заказа изменен",
				"Order status changed",
			),
		),
		array(
			"name" => "message_change_status",
			"value" => array(
				"Здравствуйте!<br>Статус заказ №%order изменен на «%status».",
				'Hello!<br>Status order #%order is changed to "%status".',
			),
		),
		array(
			"name" => "desc_payment",
			"value" => array(
				"Заказ #%id оплачен",
				"Payment order #%id",
			),
		),
		array(
			"name" => "payment_success_text",
			"value" => array(
				"<p>Спасибо, платеж успешно принят. В ближайшее время мы с Вами свяжемся для уточнения деталей заказа.</p>",
				"<p>Thank you for the payment was successful. In the near future we will contact you to clarify details of your order.</p>",
			),
		),
		array(
			"name" => "payment_fail_text",
			"value" => array(
				"<p>Извините, платеж не прошел.</p>",
				"<p>Sorry, the payment failed.</p>",
			),
		),
		array(
			"name" => "subject_file_sale_message",
			"value" => array(
				"Вы оформили заказ %id на сайте %url",
				"You placed an order %id on web site %url",
			),
		),
		array(
			"name" => "file_sale_message",
			"value" => array(
				"Здравствуйте!<br>Вы оформили заказ на сайте %url:<br><br>Номер заказа: %id<br>Файлы можно скачать по ссылкам в течение часа: %files<br><br>Спасибо за Ваш заказ!",
				"Hello!<br>You place your order online %url:<br><br>Order number: %id. Files can be downloaded from the links at the hour: %files.<br><br>Thank you for your order!",
			),
		),
		array(
			"name" => "subject_share_cart",
			"value" => array(
				"Товары в корзине %url",
				"Your goods on %url",
			),
		),
		array(
			"name" => "message_share_cart",
			"value" => array(
				"Здравствуйте!<br>Вы положили товары в корзину на сайте %url<br><br>Корзина всегда будет доступна по ссылке: <a href=\"%link\">%link</a>.",
				"Hello! <br> You filled the cart on the site %url:<br><br>The cart will all be available at the link:  <a href=\"%link\">%link</a>.",
			),
		),
		array(
			"name" => "subject_abandonmented_cart",
			"value" => array(
				"Товары в корзине %url",
				"Your goods on %url",
			),
		),
		array(
			"name" => "message_abandonmented_cart",
			"value" => array(
				"Здравствуйте!<br>Вы положили товары в корзину на сайте %url<br><br>Мы сохраним Вашу корзину и Вы в любой момент можете продолжить оформление заказа пройдя по ссылке: <a href=\"%link\">%link</a>. Но все товары не всегда будут в наличии. Пожалуйста, поторопитесь оформить заказ пока товары не раскупили.<br>Вот что Вы отложили:<br>%goods",
				"Hello! <br> You filled the cart on the site %url:<br><br>We save your cart and you can continue to process the order at any time by following the link:  <a href=\"%link\">%link</a>. But we do not promise that all goods will be available. Please hurry to place an order until the goods are sold.<br>Here is your order:<br>%goods",
			),
		),
		array(
			"name" => "attachment_extensions",
			"value" => "zip, rar",
		),
		array(
			"name" => "rel_two_sided",
			"value" => 1,
		),
		array(
			"name" => "subject_waitlist",
			"value" => array(
				"Товар поступил на склад",
				"Product will be available",
			),
		),
		array(
			"name" => "message_waitlist",
			"value" => array(
				"Здравствуйте!<br>Товар <a href=\"%link\">%good</a> поступил на склад.",
				"Hello! <br>Product <a href=\"%link\">%good</a> entered the warehouse.",
			),
		),
		array(
			"name" => "images_variations_element",
			"value" => "a:2:{s:4:\"webp\";i:1;s:2:\"vs\";a:3:{i:0;a:2:{s:4:\"name\";s:5:\"large\";s:2:\"id\";s:1:\"3\";}i:1;a:2:{s:4:\"name\";s:6:\"medium\";s:2:\"id\";s:1:\"2\";}i:2;a:2:{s:4:\"name\";s:7:\"preview\";s:2:\"id\";s:1:\"4\";}}}",
		),
		array(
			"name" => "images_variations_brand",
			"value" => "a:2:{s:4:\"webp\";i:1;s:2:\"vs\";a:3:{i:0;a:2:{s:4:\"name\";s:5:\"large\";s:2:\"id\";s:1:\"3\";}i:1;a:2:{s:4:\"name\";s:6:\"medium\";s:2:\"id\";s:1:\"2\";}i:2;a:2:{s:4:\"name\";s:7:\"preview\";s:2:\"id\";s:1:\"4\";}}}",
		),
		array(
			"name" => "show_more",
			"value" => '1',
		),
		array(
			"name" => "format_price_1",
			"value" => "2",
		),
		array(
			"name" => "format_price_2",
			"value" => ".",
		),
		array(
			"name" => "format_price_3",
			"value" => "",
		),
		array(
			"name" => "weight_unit",
			"value" => "g",
		),
		array(
			"name" => "auto_send",
			"module_name" => "cashregister",
			"value" => "1",
		),
		array(
			"name" => "defer",
			"module_name" => "cashregister",
			"value" => "1",
		),
		array(
			"name" => "status_presell",
			"module_name" => "cashregister",
			"value" => "2",
		),
		array(
			"name" => "status_sell",
			"module_name" => "cashregister",
			"value" => "4",
		),
		array(
			"name" => "status_refund",
			"module_name" => "cashregister",
			"value" => "3",
		),
		array(
			"name" => "images_cat",
			"value" => "1",
		),
		array(
			"name" => "images_variations_cat",
			"value" => "a:2:{s:4:\"webp\";i:1;s:2:\"vs\";a:2:{i:0;a:2:{s:4:\"name\";s:5:\"large\";s:2:\"id\";s:1:\"3\";}i:1;a:2:{s:4:\"name\";s:6:\"medium\";s:2:\"id\";s:1:\"2\";}}}",
		),
		array(
			"name" => "list_img_cat",
			"value" => 1,
		),
		array(
			"name" => "order_min_summ",
			"value" => 1000,
		),
		array(
			"name" => "title_tpl",
			"value" => array(
				"%name %article купить, %category %parent_category",
				"Buy %name %article, %category %parent_category",
			),
		),
		array(
			"name" => "keywords_tpl",
			"value" => array(
				"%name купить, %name заказать, %name недорого, %name %category",
				"%name buy, %name order, %name low price, %name %category",
			),
		),
		array(
			"name" => "descr_tpl",
			"value" => array(
				"%name %article, %category, %parent_category, %anons",
				"%name %article, %category, %parent_category, %anons",
			),
		),
		array(
			"name" => "title_tpl_cat",
			"value" => array(
				"%name",
				"%name",
			),
		),
		array(
			"name" => "descr_tpl_cat",
			"value" => array(
				"%name %anons",
				"%name %anons",
			),
		),
		array(
			"name" => "rewrite_tpl",
			"value" => "%category/%name-%article-%id",
		),
		array(
			"name" => "rewrite_tpl_cat",
			"value" => "%name",
		),
		array(
			"name" => "rewrite_tpl_brand",
			"value" => "%name",
		),
	);

	/**
	 * @var array SQL-запросы
	 */
	public $sql = array(
		"shop_category" => array(
			array(
				"id" => 1,
				"name" => array('Наши товары', 'Our goods'),
				"menu" => 2,
			),
		),
		"shop_brand" => array(
			array(
				"id" => 1,
				"name" => array('Фирма А', 'Firm A'),
			),
		),
		"shop_param" => array(
			array(
				"id" => 1,
				"name" => array('Размер', 'Size'),
				"type" => "text",
				"id_page" => 1,
			),
		),
		"shop_discount" => array(
			array(
				"id" => 1,
				"discount" => 5,
				"text" => array('Глобальная'),
			),
		),
		"shop_import_category" => array(
			array(
				"id" => 1,
				"name" => 'Импорт товаров',
				"format" => 'csv',
				"type" => 'good',
				"count_part" => 20,
				"delimiter" => ';',
				"encoding" => 'cp1251',
				"sub_delimiter" => '|',
			),
			array(
				"id" => 2,
				"name" => 'Импорт категорий',
				"format" => 'csv',
				"type" => 'category',
				"count_part" => 20,
				"delimiter" => ';',
				"encoding" => 'cp1251',
				"sub_delimiter" => '|',
			),
			array(
				"id" => 3,
				"name" => 'Импорт производителей',
				"format" => 'csv',
				"type" => 'brand',
				"count_part" => 20,
				"delimiter" => ';',
				"encoding" => 'cp1251',
				"sub_delimiter" => '|',
			),
		),
		"shop_import" => array(
			array(
				"name" => 'Идентификатор',
				"type" => 'id',
				"params" => 'a:1:{s:4:"type";s:4:"site";}',
				"cat_id" => 1,
			),
			array(
				"name" => 'Артикул товара',
				"type" => 'article',
				"cat_id" => 1,
			),
			array(
				"name" => 'Название товара',
				"type" => 'name',
				"cat_id" => 1,
			),
			array(
				"name" => 'Краткое описание',
				"type" => 'anons',
				"cat_id" => 1,
			),
			array(
				"name" => 'Полное описание товара',
				"type" => 'text',
				"cat_id" => 1,
			),
			array(
				"name" => 'Цена',
				"type" => 'price',
				"params" => 'a:5:{s:9:"delimitor";s:1:"&";s:11:"select_type";s:3:"key";s:5:"count";i:0;s:8:"currency";i:0;s:15:"select_currency";s:3:"key";}',
				"cat_id" => 1,
			),
			array(
				"name" => 'Количество',
				"type" => 'count',
				"params" => 'a:2:{s:9:"delimitor";s:1:"&";s:11:"select_type";s:3:"key";}',
				"cat_id" => 1,
			),
			array(
				"name" => 'Хит (1/0)',
				"type" => 'hit',
				"cat_id" => 1,
			),
			array(
				"name" => 'Новинка (1/0)',
				"type" => 'new',
				"cat_id" => 1,
			),
			array(
				"name" => 'Акция (1/0)',
				"type" => 'action',
				"cat_id" => 1,
			),
			array(
				"name" => 'Идентификатор',
				"type" => 'id',
				"params" => 'a:1:{s:4:"type";s:4:"site";}',
				"cat_id" => 2,
			),
			array(
				"name" => 'Название категории',
				"type" => 'name',
				"cat_id" => 2,
			),
			array(
				"name" => 'Краткое описание категории',
				"type" => 'anons',
				"cat_id" => 2,
			),
			array(
				"name" => 'Полное описание категории',
				"type" => 'text',
				"cat_id" => 2,
			),
			array(
				"name" => 'Идентификатор',
				"type" => 'id',
				"params" => 'a:1:{s:4:"type";s:4:"site";}',
				"cat_id" => 3,
			),
			array(
				"name" => 'Название производителя',
				"type" => 'name',
				"cat_id" => 3,
			),
			array(
				"name" => 'Описание категории',
				"type" => 'text',
				"cat_id" => 3,
			),
		),
		"shop_order_status" => array(
			array(
				"id" => 1,
				"name" => array('Новый', 'New'),
				"status" => 0,
				"color" => '#D54640',
			),
			array(
				"id" => 2,
				"name" => array('В обработке', 'In processing'),
				"status" => 1,
				'color' => '#F49D10',
				"count_minus" => true,
				"send_mail" => true,
			),
			array(
				"id" => 3,
				"name" => array('Отменен', 'Canceled'),
				"status" => 2,
				'color' => '#A6AEB0',
				"send_mail" => true,
			),
			array(
				"id" => 4,
				"name" => array('Выполнен', 'Completed'),
				"status" => 3,
				'color' => '#8AC73C',
				"count_minus" => true,
				"send_mail" => true,
			),
		),
		"shop_currency" => array(
			array(
				"name" => 'Euro',
				"exchange_rate" => 100,
			),
		),
		"shop_order_backend" => array(
			array(
				"name" => "Товарный чек",
				"backend" => "packing_list",
				"act" => 1,
				"sort" => 1,
			),
		),
		"shop_order_param" => array(
			array(
				"id" => 1,
				"name" => array("Имя", "Name&amp;Surname"),
				"type" => "text",
				"info" => "name",
				"required" => 1,
				"show_in_form" => 1,
				"show_in_form_one_click" => 1,
			),
			array(
				"id" => 2,
				"name" => array('Телефон', 'Phone'),
				"type" => "phone",
				"info" => "phone",
				"required" => 1,
				"show_in_form" => 1,
				"show_in_form_one_click" => 1,
			),
			array(
				"id" => 3,
				"name" => array('E-mail', 'E-mail'),
				"type" => "email",
				"info" => "email",
				"required" => 1,
				"show_in_form" => 1,
			),
			array(
				"id" => 4,
				"name" => array("Адрес", "Address"),
				"type" => "textarea",
				"info" => "address",
				"show_in_form" => 1,
			),
			array(
				"id" => 5,
				"name" => array("Комментарий", "Comments"),
				"type" => "textarea",
				"info" => "comment",
				"show_in_form" => 1,
			),
			array(
				"id" => 6,
				"name" => array("Я не робот", "I'm not a robot"),
				"type" => "checkbox",
				"required" => 1,
				"show_in_form" => 1,
			),
		),
		"shop_delivery" => array(
			array(
				"id" => 1,
				"name" => array('Курьер', 'Сourier'),
				"text" => array('Товар доставляется курьером до двери Вашего дома.', 'Goods are delivered to the door of your house.'),
				"status" => 0,
				"thresholds" => array(
					array(
						'price' => 500,
					),
					array(
						"amount" => 6000,
					),
				)
			),
			array(
				"id" => 2,
				"name" => array('Saferoute', 'Saferoute'),
				"text" => array('Агрегатор доставки для интернет-магазинов.', 'Delivery aggregator for online stores.'),
				"status" => 0,
				"service" => 'saferoute',
			),
		),
	);

	/**
	 * @var array демо-данные
	 */
	public $demo = array(
		'config' => array(
			array(
				'name' => 'rating',
				'value' => 1,
			),
			array(
				'name' => 'comments',
				'value' => 1,
			),
			array(
				'name' => 'tags',
				'value' => 1,
			),
		),
		'site' => array(
			array(
				'id' => 'shop',
				'text' => array(
					'<p>Чтобы положить товар в корзину, перейдите в интернет-магазин.</p>',
					'<p>To put the goods in a basket, pass in the catalogue of the goods.</p>'
				),
			),
		),
		'shop_category' => array(
			array(
				"id" => 1,
				"name" => array("Одежда и обувь", "Clothes and shoes"),
				"text" => array("<p>В нашем ассортименте представлена разнообразная одежда и обувь для мужчин, женщин и детей. Мы предлагаем качественные товары от известных брендов, а также стильные новинки сезона. У нас вы найдете все, что нужно для создания модного образа - от базовых вещей до ярких аксессуаров. Наш ассортимент постоянно обновляется, чтобы соответствовать актуальным тенденциям моды.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
				"rewrite" => "shop/odezhda-i-obuv",
				"sort" => 1,
				"menu" => array("2", "3"),
				'import_id' => '1',
				"children" => array(
					array(
						"id" => 6,
						"name" => array("Верхняя одежда", "Outerwear"),
						"text" => array("<p>В категории верхней одежды вы найдете куртки, пальто, пуховики и другую одежду, которая защитит вас от холода и ветра. Мы предлагаем модели разных стилей и размеров, так что каждый сможет найти что-то подходящее для себя. Верхняя одежда из нашего ассортимента отличается высоким качеством и долговечностью, поэтому вы сможете носить ее не один сезон.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/odezhda-i-obuv/verkhnyaya-odezhda",
						"sort" => 6,
						"images" => array("867_zhenskaya-kurtka--otlichnyy-.webp"),
						"menu" => array("2", "3"),
						'import_id' => '6',
						"children" => array(
							array(
								"id" => 44,
								"name" => array("Спортивный стиль", "Sporty style"),
								"text" => array("<p>В категории верхней одежды вы найдете куртки, пальто, пуховики и другую одежду, которая защитит вас от холода и ветра. Мы предлагаем модели разных стилей и размеров, так что каждый сможет найти что-то подходящее для себя. Верхняя одежда из нашего ассортимента отличается высоким качеством и долговечностью, поэтому вы сможете носить ее не один сезон.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
								"rewrite" => "shop/odezhda-i-obuv/verkhnyaya-odezhda/sportivnyy-stil",
								"sort" => 44,
								"menu" => "2",
								'import_id' => '44',
							),
							array(
								"id" => 45,
								"name" => array("Натуральный мех", "Natural fur"),
								"text" => array("<p>В категории верхней одежды вы найдете куртки, пальто, пуховики и другую одежду, которая защитит вас от холода и ветра. Мы предлагаем модели разных стилей и размеров, так что каждый сможет найти что-то подходящее для себя. Верхняя одежда из нашего ассортимента отличается высоким качеством и долговечностью, поэтому вы сможете носить ее не один сезон.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
								"rewrite" => "shop/odezhda-i-obuv/verkhnyaya-odezhda/naturalnyy-mekh",
								"sort" => 45,
								"menu" => "2",
								'import_id' => '45',
							),
							array(
								"id" => 46,
								"name" => array("Эко мех", "Eco fur"),
								"text" => array("<p>В категории верхней одежды вы найдете куртки, пальто, пуховики и другую одежду, которая защитит вас от холода и ветра. Мы предлагаем модели разных стилей и размеров, так что каждый сможет найти что-то подходящее для себя. Верхняя одежда из нашего ассортимента отличается высоким качеством и долговечностью, поэтому вы сможете носить ее не один сезон.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
								"rewrite" => "shop/odezhda-i-obuv/verkhnyaya-odezhda/eko-mekh",
								"sort" => 46,
								"menu" => "2",
								'import_id' => '46',
							),
						),
					),
					array(
						"id" => 7,
						"name" => array("Платья", "Dresses"),
						"text" => array("<p>В категории платьев вы найдете большой выбор моделей разных стилей и фасонов. Мы предлагаем платья для любых случаев - от повседневных до вечерних. У нас есть платья как для женщин, так и для девочек. Все наши платья изготовлены из качественных материалов и имеют стильный дизайн.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/odezhda-i-obuv/platya",
						"sort" => 7,
						"images" => array("860_izyaschestvo-i-stil-obzor-ko.webp"),
						"menu" => array("2", "3"),
						'import_id' => '7',
					),
					array(
						"id" => 8,
						"name" => array("Рубашки", "Shirts"),
						"text" => array("<p>В категории рубашек вы найдете модели разных цветов, размеров и стилей. Мы предлагаем как классические рубашки, так и более оригинальные варианты. Все наши рубашки изготовлены из качественных материалов и хорошо сидят на фигуре.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/odezhda-i-obuv/rubashki",
						"sort" => 8,
						"images" => array("861_ukorochennaya-rubashka---must-have-.webp"),
						"menu" => array("2", "3"),
						'import_id' => '8',
					),
					array(
						"id" => 9,
						"name" => array("Брюки и джинсы", "Trousers and jeans"),
						"text" => array("<p>В категории брюк и джинсов вы найдете мужские, женские и детские модели. Мы предлагаем классические брюки, а также более свободные и удобные модели. Все наши брюки изготовлены из прочных материалов и имеют современный дизайн.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/odezhda-i-obuv/bryuki-i-dzhinsy",
						"sort" => 9,
						"images" => array("986_klassicheskaya-rubashka-s-dli.webp"),
						"menu" => array("2", "3"),
						'import_id' => '9',
					),
					array(
						"id" => 40,
						"name" => array("Детям", "For children"),
						"text" => array("<p>В категории детской одежды вы найдете все необходимое для малышей и детей постарше. Мы предлагаем яркую, стильную и удобную одежду для разных случаев. У нас есть одежда для мальчиков и девочек, а также для новорожденных. Все наши товары изготовлены из безопасных материалов и соответствуют высоким стандартам качества.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/odezhda-i-obuv/detyam",
						"sort" => 40,
						"images" => array("866_detskaya-odezhda---luchshiy-vyb.webp"),
						"menu" => array("2", "3"),
						'import_id' => '40',
					),
					array(
						"id" => 41,
						"name" => array("Обувь", "Footwear"),
						"text" => array("<p>В категории обуви вы найдете широкий ассортимент моделей для мужчин, женщин и детей. У нас есть обувь для различных видов деятельности, включая повседневную носку, спорт и выход в свет. Мы предлагаем различные стили, размеры и цвета, чтобы каждый мог найти идеальную пару обуви для себя.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/odezhda-i-obuv/krosovki-i-kedy",
						"sort" => 41,
						"images" => array("864_udobnye-krossovki---idealn.webp"),
						"menu" => "2",
						'import_id' => '41',
					),
				),
			),
			array(
				"id" => 2,
				"name" => array("Электроника", "Electronics"),
				"text" => array("<p>Предлагаем широкий ассортимент продукции, включающий в себя бытовую технику, компьютеры, смартфоны, аксессуары, товары для дома и многое другое. Здесь вы найдете последние новинки и проверенные временем модели, обеспечивающие комфорт и удобство в повседневной жизни. Мы предлагаем продукцию от ведущих мировых производителей, гарантирующих качество и надежность своих устройств. С нашей помощью вы сможете обновить свою технику, сделать жизнь более технологичной и приятной.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
				"rewrite" => "shop/elektronika",
				"sort" => 2,
				"menu" => array("2", "3"),
				'import_id' => '2',
				"children" => array(
					array(
						"id" => 10,
						"name" => array("Телевизоры", "Televisions"),
						"text" => array("<p>Категория товаров &ldquo;Телевизоры&rdquo; предлагает широкий выбор моделей с различными диагоналями и функционалом. Здесь вы найдете как бюджетные варианты для кухни или спальни, так и премиум-телевизоры для гостиной или домашнего кинотеатра. В нашем ассортименте представлены модели от ведущих мировых брендов, таких как Samsung, LG, Sony, Philips и других производителей.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/elektronika/televizory",
						"sort" => 10,
						"images" => array("862_superskidki-na-televizory-.webp"),
						"menu" => array("2", "3"),
						'import_id' => '10',
						"children" => array(
							array(
								"id" => 18,
								"name" => array("По размеру экрана", "By screen size"),
								"text" => array("<p>Категория товаров &ldquo;Телевизоры&rdquo; предлагает широкий выбор моделей с различными диагоналями и функционалом. Здесь вы найдете как бюджетные варианты для кухни или спальни, так и премиум-телевизоры для гостиной или домашнего кинотеатра. В нашем ассортименте представлены модели от ведущих мировых брендов, таких как Samsung, LG, Sony, Philips и других производителей.</p>"),
								"rewrite" => "shop/elektronika/televizory/po-razmeru-ekrana",
								"sort" => 18,
								"menu" => "2",
								'import_id' => '18',
								"children" => array(
									array(
										"id" => 20,
										"name" => array("Телевизоры 19&quot;-20&quot;", "TVs 19&quot;-20&quot;"),
										"rewrite" => "shop/elektronika/televizory/po-razmeru-ekrana/televizory-19-20",
										"sort" => 20,
										"menu" => "2",
										'import_id' => '20',
									),
									array(
										"id" => 21,
										"name" => array("Телевизоры 40&quot;-50&quot;", "TVs 40&quot;-50&quot;"),
										"rewrite" => "shop/elektronika/televizory/po-razmeru-ekrana/televizory-40-50",
										"sort" => 21,
										"menu" => "2",
										'import_id' => '21',
									),
									array(
										"id" => 22,
										"name" => array("Телевизоры 55&quot;-65&quot;", "TVs 55&quot;-65&quot;"),
										"rewrite" => "shop/elektronika/televizory/po-razmeru-ekrana/televizory-55-65",
										"sort" => 22,
										"menu" => "2",
										'import_id' => '22',
									),
								),
							),
							array(
								"id" => 19,
								"name" => array("По типу", "By type"),
								"text" => array("<p>Категория товаров &ldquo;Телевизоры&rdquo; предлагает широкий выбор моделей с различными диагоналями и функционалом. Здесь вы найдете как бюджетные варианты для кухни или спальни, так и премиум-телевизоры для гостиной или домашнего кинотеатра. В нашем ассортименте представлены модели от ведущих мировых брендов, таких как Samsung, LG, Sony, Philips и других производителей.</p>"),
								"rewrite" => "shop/elektronika/televizory/po-tipu",
								"sort" => 19,
								"menu" => "2",
								'import_id' => '19',
								"children" => array(
									array(
										"id" => 24,
										"name" => array("LED телевизоры", "LED TVs"),
										"rewrite" => "shop/elektronika/televizory/po-tipu/led-televizory",
										"sort" => 24,
										"menu" => "2",
										'import_id' => '24',
									),
									array(
										"id" => 25,
										"name" => array("OLED телевизоры", "OLED TVs"),
										"rewrite" => "shop/elektronika/televizory/po-tipu/oled-televizory",
										"sort" => 25,
										"menu" => "2",
										'import_id' => '25',
									),
									array(
										"id" => 26,
										"name" => array("ЖК-телевизоры", "LCD TVs"),
										"rewrite" => "shop/elektronika/televizory/po-tipu/zhk-televizory",
										"sort" => 26,
										"menu" => "2",
										'import_id' => '26',
									),
									array(
										"id" => 27,
										"name" => array("Телевизоры со SMART TV", "TVs with SMART TV"),
										"rewrite" => "shop/elektronika/televizory/po-tipu/televizory-so-smart-tv",
										"sort" => 27,
										"menu" => "2",
										'import_id' => '27',
									),
								),
							),
							array(
								"id" => 23,
								"name" => array("По разрешению", "By screen resolution"),
								"text" => array("<p>Категория товаров &ldquo;Телевизоры&rdquo; предлагает широкий выбор моделей с различными диагоналями и функционалом. Здесь вы найдете как бюджетные варианты для кухни или спальни, так и премиум-телевизоры для гостиной или домашнего кинотеатра. В нашем ассортименте представлены модели от ведущих мировых брендов, таких как Samsung, LG, Sony, Philips и других производителей.</p>"),
								"rewrite" => "shop/elektronika/televizory/po-razresheniyu",
								"sort" => 23,
								"menu" => "2",
								'import_id' => '23',
								"children" => array(
									array(
										"id" => 28,
										"name" => array("HD", "HD"),
										"rewrite" => "shop/elektronika/televizory/po-razresheniyu/hd",
										"sort" => 28,
										"menu" => "2",
										'import_id' => '28',
									),
									array(
										"id" => 29,
										"name" => array("Full HD", "Full HD"),
										"rewrite" => "shop/elektronika/televizory/po-razresheniyu/full-hd",
										"sort" => 29,
										"menu" => "2",
										'import_id' => '29',
									),
									array(
										"id" => 30,
										"name" => array("4K", "4K"),
										"rewrite" => "shop/elektronika/televizory/po-razresheniyu/4k",
										"sort" => 30,
										"menu" => "2",
										'import_id' => '30',
									),
									array(
										"id" => 31,
										"name" => array("8K", "8K"),
										"rewrite" => "shop/elektronika/televizory/po-razresheniyu/8k",
										"sort" => 31,
										"menu" => "2",
										'import_id' => '31',
									),
								),
							),
						),
					),
					array(
						"id" => 14,
						"name" => array("Смартфоны", "Smartphones"),
						"text" => array("<p>В категории &ldquo;Смартфоны&rdquo; вы найдете широкий выбор устройств от ведущих производителей. Наши смартфоны обладают высокой производительностью, качественными экранами и мощными камерами. Вы сможете выбрать модель, которая идеально подойдет под ваши потребности и бюджет.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/elektronika/smartfony",
						"sort" => 14,
						"images" => array("902_galaxytouch.webp"),
						"menu" => array("2", "3"),
						'import_id' => '14',
						"children" => array(
							array(
								"id" => 32,
								"name" => array("Стоимость до 10000 руб.", "Price is up to 10,000 rubles."),
								"text" => array("<p>В категории &ldquo;Смартфоны&rdquo; вы найдете широкий выбор устройств от ведущих производителей. Наши смартфоны обладают высокой производительностью, качественными экранами и мощными камерами. Вы сможете выбрать модель, которая идеально подойдет под ваши потребности и бюджет.</p>"),
								"rewrite" => "shop/elektronika/smartfony/stoimost-do-10000-rub",
								"sort" => 32,
								"menu" => "2",
								'import_id' => '32',
							),
							array(
								"id" => 33,
								"name" => array("Стоимость от 15000 до 30000 руб.", "Price is up to 30,000 rubles."),
								"text" => array("<p>В категории &ldquo;Смартфоны&rdquo; вы найдете широкий выбор устройств от ведущих производителей. Наши смартфоны обладают высокой производительностью, качественными экранами и мощными камерами. Вы сможете выбрать модель, которая идеально подойдет под ваши потребности и бюджет.</p>"),
								"rewrite" => "shop/elektronika/smartfony/stoimost-ot-15000-do-30000-rub",
								"sort" => 33,
								"menu" => "2",
								'import_id' => '33',
							),
							array(
								"id" => 34,
								"name" => array("Стоимость выше 30000 руб.", "Price is above 30,000 rubles."),
								"text" => array("<p>В категории &ldquo;Смартфоны&rdquo; вы найдете широкий выбор устройств от ведущих производителей. Наши смартфоны обладают высокой производительностью, качественными экранами и мощными камерами. Вы сможете выбрать модель, которая идеально подойдет под ваши потребности и бюджет.</p>"),
								"rewrite" => "shop/elektronika/smartfony/stoimost-vyshe-30000-rub",
								"sort" => 34,
								"menu" => "2",
								'import_id' => '34',
							),
							array(
								"id" => 35,
								"name" => array("Аксессуары для смартфонов", "Smartphone Accessories"),
								"rewrite" => "shop/elektronika/smartfony/aksessuary-dlya-smartfonov",
								"sort" => 35,
								"menu" => "2",
								'import_id' => '35',
								"children" => array(
									array(
										"id" => 36,
										"name" => array("Чехлы для смартфонов", "Smartphone Cases"),
										"rewrite" => "shop/elektronika/smartfony/aksessuary-dlya-smartfonov/chekhly-dlya-smartfonov",
										"sort" => 36,
										"menu" => "2",
										'import_id' => '36',
									),
									array(
										"id" => 37,
										"name" => array("Кабели для смартфонов", "Cables for smartphones"),
										"rewrite" => "shop/elektronika/smartfony/aksessuary-dlya-smartfonov/kabeli-dlya-smartfonov",
										"sort" => 37,
										"menu" => "2",
										'import_id' => '37',
									),
									array(
										"id" => 38,
										"name" => array("Защитные стекла и плёнки", "Protective glasses and films"),
										"rewrite" => "shop/elektronika/smartfony/aksessuary-dlya-smartfonov/zaschitnye-stekla-i-plyonki",
										"sort" => 38,
										"menu" => "2",
										'import_id' => '38',
									),
								),
							),
						),
					),
					array(
						"id" => 15,
						"name" => array("Ноутбуки", "Laptops"),
						"text" => array("<p>Ноутбуки - это портативные компьютеры, которые идеально подходят для работы в дороге, учебы и развлечений. В нашей категории товаров вы найдете ноутбуки различных производителей, с различными характеристиками и ценовыми категориями. Мы предлагаем ноутбуки с экранами разных размеров, процессорами разных мощностей и различным объемом памяти. Выберите ноутбук, который подходит именно вам!</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/elektronika/noutbuki",
						"sort" => 15,
						"images" => array("1126_noutbuki.webp"),
						"menu" => array("2", "3"),
						'import_id' => '15',
					),
					array(
						"id" => 16,
						"name" => array("Компьютеры", "Computers"),
						"text" => array("<p>Компьютеры - это универсальные устройства, которые могут использоваться для различных задач, таких как работа, учеба, игры и многое другое. В категории товаров &ldquo;Компьютеры&rdquo; вы найдете различные модели, начиная от бюджетных и заканчивая высокопроизводительными. Мы предлагаем компьютеры с различными конфигурациями, такими как различные процессоры, объемы оперативной памяти и жесткие диски разных размеров. Также у нас есть широкий выбор аксессуаров для компьютеров, таких как клавиатуры, мыши и мониторы.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/elektronika/kompyutery",
						"sort" => 16,
						"images" => array("939_matrix-pc.webp"),
						"menu" => array("2", "3"),
						'import_id' => '16',
					),
					array(
						"id" => 17,
						"name" => array("Игры и консоли", "Games and consoles"),
						"text" => array("<p>Игры и консоли - это категория товаров, в которой вы найдете широкий ассортимент игр для различных игровых платформ, а также сами игровые консоли. У нас есть игры для PlayStation, Xbox, Nintendo и других платформ. Кроме того, мы предлагаем различные аксессуары для игровых консолей, такие как контроллеры, зарядные устройства и кабели.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/elektronika/igry-i-konsoli",
						"sort" => 17,
						"images" => array("1128_igry-i-konsoli.webp"),
						"menu" => array("2", "3"),
						'import_id' => '17',
					),
				),
			),
			array(
				"id" => 3,
				"name" => array("Дом и сад", "House and garden"),
				"text" => array("<p>В категории товаров для дома и сада вы найдете все необходимое для обустройства вашего участка и создания уюта в доме. Мы предлагаем широкий ассортимент товаров от ведущих производителей, включая садовую мебель, инструменты, товары для ландшафтного дизайна, бытовую технику и многое другое. Наша продукция отличается высоким качеством, долговечностью и стильным дизайном.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
				"rewrite" => "shop/dom-i-sad",
				"sort" => 3,
				"menu" => array("2", "3"),
				'import_id' => '3',
				"children" => array(
					array(
						"id" => 47,
						"name" => array("Посуда", "Tableware"),
						"text" => array("<p>В категории посуды вы найдете все необходимые предметы для приготовления пищи и сервировки стола. Мы предлагаем посуду из различных материалов, таких как керамика, стекло, металл и пластик. У нас есть посуда для различных кухонь мира, а также разнообразные формы и размеры. Наша посуда поможет вам приготовить вкусные блюда и украсить ваш стол.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/dom-i-sad/posuda",
						"sort" => 47,
						"images" => array("1029_tarelka-obedennaya-iz-belog.webp"),
						"menu" => array("2", "3"),
						'import_id' => '47',
						"children" => array(
							array(
								"id" => 51,
								"name" => array("Для приготовления", "For cooking"),
								"text" => array("<p>В категории посуды вы найдете все необходимые предметы для приготовления пищи и сервировки стола. Мы предлагаем посуду из различных материалов, таких как керамика, стекло, металл и пластик. У нас есть посуда для различных кухонь мира, а также разнообразные формы и размеры. Наша посуда поможет вам приготовить вкусные блюда и украсить ваш стол.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
								"rewrite" => "shop/dom-i-sad/posuda/dlya-prigotovleniya",
								"sort" => 51,
								"menu" => "2",
								'import_id' => '51',
							),
							array(
								"id" => 52,
								"name" => array("Кухонные принадлежности", "Kitchen utensils"),
								"text" => array("<p>В категории кухонных принадлежностей вы найдете все необходимые инструменты для приготовления пищи. Мы предлагаем широкий ассортимент товаров, включая ножи, разделочные доски, ложки, вилки, кастрюли, сковородки и многое другое. Наши кухонные принадлежности изготовлены из высококачественных материалов и прослужат вам долгое время.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
								"rewrite" => "shop/dom-i-sad/posuda/kukhonnye-prinadlezhnosti",
								"sort" => 52,
								"menu" => "2",
								'import_id' => '52',
							),
							array(
								"id" => 53,
								"name" => array("Ножи и разделочные доски", "Knives and cutting boards"),
								"text" => array("<p>В категории ножей и разделочных досок вы найдете широкий ассортимент этих необходимых кухонных принадлежностей. Мы предлагаем ножи различных размеров и назначений, а также разделочные доски из различных материалов. Наши ножи изготовлены из высококачественной стали и имеют острое лезвие, которое долго не тупится. Разделочные доски изготовлены из прочных материалов, которые легко моются и не впитывают запахи.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
								"rewrite" => "shop/dom-i-sad/posuda/nozhi-i-razdelochnye-doski",
								"sort" => 53,
								"menu" => "2",
								'import_id' => '53',
							),
						),
					),
					array(
						"id" => 48,
						"name" => array("Текстиль", "Textile"),
						"text" => array("<p>В категории текстиля вы найдете широкий ассортимент товаров для дома, таких как постельное белье, полотенца, скатерти, салфетки и многое другое. Мы предлагаем текстиль из различных материалов, включая хлопок, лен, шелк и синтетические ткани. Наши товары отличаются высоким качеством, стильным дизайном и доступными ценами.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/dom-i-sad/tekstil",
						"sort" => 48,
						"images" => array("1056_myagkie-polotentsa-komfort.webp"),
						"menu" => array("2", "3"),
						'import_id' => '48',
					),
					array(
						"id" => 49,
						"name" => array("Декор и интерьер", "Decor and interior"),
						"text" => array("<p>В категории декора и интерьера вы найдете широкий ассортимент товаров для украшения вашего дома. Мы предлагаем предметы интерьера, такие как вазы, статуэтки, картины и зеркала, а также декоративные элементы, такие как свечи, гирлянды и наклейки. Наши товары помогут создать уютную атмосферу в вашем доме и подчеркнуть вашу индивидуальность.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/dom-i-sad/dekor-i-interer",
						"sort" => 49,
						"images" => array("874_novye-postavschiki-bytovykh-t.webp"),
						"menu" => array("2", "3"),
						'import_id' => '49',
					),
					array(
						"id" => 50,
						"name" => array("Хозяйственные товары", "Household goods"),
						"text" => array("<p>В категории хозяйственных товаров вы найдете всё необходимое для поддержания порядка и чистоты в доме. Мы предлагаем товары для уборки, такие как швабры, тряпки, щетки и средства для мытья посуды, а также бытовые принадлежности, такие как мешки для мусора, перчатки и гладильные доски. Наши хозяйственные товары помогут вам сэкономить время и силы на уборке, а также обеспечат высокое качество уборки.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/dom-i-sad/khozyaystvennye-tovary",
						"sort" => 50,
						"images" => array("1069_sadovaya-tachka-pomoschnik.webp"),
						"menu" => array("2", "3"),
						'import_id' => '50',
					),
				),
			),
			array(
				"id" => 4,
				"name" => array("Мебель", "Furniture"),
				"text" => array("<p>В категории мебели вы найдете широкий ассортимент предметов для дома. Мы предлагаем мебель для гостиной, спальни, детской комнаты, прихожей и других помещений. У нас есть мебель различных стилей и ценовых категорий, чтобы каждый смог подобрать подходящий вариант.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
				"rewrite" => "shop/mebel",
				"sort" => 4,
				"menu" => array("2", "3"),
				'import_id' => '4',
				"children" => array(
					array(
						"id" => 54,
						"name" => array("Мебель для хранения", "Storage Furniture"),
						"text" => array("<p>В категории мебели вы найдете широкий ассортимент предметов для дома. Мы предлагаем мебель для гостиной, спальни, детской комнаты, прихожей и других помещений. У нас есть мебель различных стилей и ценовых категорий, чтобы каждый смог подобрать подходящий вариант.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/mebel/mebel-dlya-khraneniya",
						"sort" => 54,
						"images" => array("1073_komod-elegant.webp"),
						"menu" => array("2", "3"),
						'import_id' => '54',
					),
					array(
						"id" => 55,
						"name" => array("Мебель для спальни", "Bedroom Furniture"),
						"text" => array("<p>В категории мебели вы найдете широкий ассортимент предметов для дома. Мы предлагаем мебель для гостиной, спальни, детской комнаты, прихожей и других помещений. У нас есть мебель различных стилей и ценовых категорий, чтобы каждый смог подобрать подходящий вариант.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/mebel/mebel-dlya-spalni",
						"sort" => 55,
						"images" => array("1077_paryaschaya-krovat-nevesomos.webp"),
						"menu" => array("2", "3"),
						'import_id' => '55',
					),
					array(
						"id" => 56,
						"name" => array("Мебель для гостиной", "Living room furniture"),
						"text" => array("<p>В категории мебели вы найдете широкий ассортимент предметов для дома. Мы предлагаем мебель для гостиной, спальни, детской комнаты, прихожей и других помещений. У нас есть мебель различных стилей и ценовых категорий, чтобы каждый смог подобрать подходящий вариант.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/mebel/mebel-dlya-gostinoy",
						"sort" => 56,
						"images" => array("1081_stilnyy-udobnyy-divan-ri.webp"),
						"menu" => array("2", "3"),
						'import_id' => '56',
					),
					array(
						"id" => 57,
						"name" => array("Мягкая мебель", "Upholstered furniture"),
						"text" => array("<p>В категории мебели вы найдете широкий ассортимент предметов для дома. Мы предлагаем мебель для гостиной, спальни, детской комнаты, прихожей и других помещений. У нас есть мебель различных стилей и ценовых категорий, чтобы каждый смог подобрать подходящий вариант.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/mebel/myagkaya-mebel",
						"sort" => 57,
						"images" => array("1080_stilnyy-udobnyy-divan-ri.webp"),
						"menu" => array("2", "3"),
						'import_id' => '57',
					),
				),
			),
			array(
				"id" => 5,
				"name" => array("Красота и здоровье", "Beauty and health"),
				"text" => array("<p>В категории &ldquo;Красота и здоровье&rdquo; вы найдете товары для ухода за телом, лицом и волосами. Мы предлагаем косметику от ведущих брендов, средства для ухода за кожей, волосами и ногтями, а также аксессуары для макияжа и укладки волос. Кроме того, в этой категории вы найдете товары для здоровья, такие как витамины, БАДы, спортивное питание и товары для контроля веса.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
				"rewrite" => "shop/krasota-i-zdorove",
				"sort" => 5,
				"menu" => array("2", "3"),
				'import_id' => '5',
				"children" => array(
					array(
						"id" => 58,
						"name" => array("Уход за лицом", "Facial care"),
						"text" => array("<p>В категории &ldquo;Красота и здоровье&rdquo; вы найдете товары для ухода за телом, лицом и волосами. Мы предлагаем косметику от ведущих брендов, средства для ухода за кожей, волосами и ногтями, а также аксессуары для макияжа и укладки волос. Кроме того, в этой категории вы найдете товары для здоровья, такие как витамины, БАДы, спортивное питание и товары для контроля веса.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/krasota-i-zdorove/ukhod-za-litsom",
						"sort" => 58,
						"images" => array("1089_krem-dlya-litsa-beauty-cream.webp"),
						"menu" => array("2", "3"),
						'import_id' => '58',
					),
					array(
						"id" => 59,
						"name" => array("Уход за телом", "Body care"),
						"text" => array("<p>В категории &ldquo;Красота и здоровье&rdquo; вы найдете товары для ухода за телом, лицом и волосами. Мы предлагаем косметику от ведущих брендов, средства для ухода за кожей, волосами и ногтями, а также аксессуары для макияжа и укладки волос. Кроме того, в этой категории вы найдете товары для здоровья, такие как витамины, БАДы, спортивное питание и товары для контроля веса.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/krasota-i-zdorove/ukhod-za-telom",
						"sort" => 59,
						"images" => array("1087_krem-dlya-litsa-beauty-cream.webp"),
						"menu" => array("2", "3"),
						'import_id' => '59',
					),
					array(
						"id" => 60,
						"name" => array("Косметика", "Cosmetics"),
						"text" => array("<p>В категории &ldquo;Красота и здоровье&rdquo; вы найдете товары для ухода за телом, лицом и волосами. Мы предлагаем косметику от ведущих брендов, средства для ухода за кожей, волосами и ногтями, а также аксессуары для макияжа и укладки волос. Кроме того, в этой категории вы найдете товары для здоровья, такие как витамины, БАДы, спортивное питание и товары для контроля веса.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/krasota-i-zdorove/kosmetika",
						"sort" => 60,
						"images" => array("1084_krasnaya-gubnaya-pomada-velvet.webp"),
						"menu" => array("2", "3"),
						'import_id' => '60',
					),
					array(
						"id" => 61,
						"name" => array("Порфюмерия", "Perfumery"),
						"text" => array("<p>В категории &ldquo;Красота и здоровье&rdquo; вы найдете товары для ухода за телом, лицом и волосами. Мы предлагаем косметику от ведущих брендов, средства для ухода за кожей, волосами и ногтями, а также аксессуары для макияжа и укладки волос. Кроме того, в этой категории вы найдете товары для здоровья, такие как витамины, БАДы, спортивное питание и товары для контроля веса.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/krasota-i-zdorove/porfyumeriya",
						"sort" => 61,
						"images" => array("857_sovety-po-vyboru-parfyuma-k.webp"),
						"menu" => array("2", "3"),
						'import_id' => '61',
					),
				),
			),
			array(
				"id" => 11,
				"name" => array("Строительство и ремонт", "Construction and renovation"),
				"text" => array("<p>В категории &ldquo;Строительство и ремонт&rdquo; вы найдете все необходимые товары для проведения ремонтных работ в доме или квартире. Мы предлагаем строительные материалы, инструменты и оборудование от ведущих производителей, а также профессиональные советы по ремонту и дизайну интерьера. Наша цель - помочь вам создать уютное и комфортное пространство в вашем доме.</p>"),
				"rewrite" => "shop/stroitelstvo-i-remont",
				"sort" => 11,
				"menu" => array(
					array(
						"cat_id" => 2,
						"act" => array(1),
					),
				),
				'import_id' => '11',
				"children" => array(
					array(
						"id" => 62,
						"name" => array("Инструменты", "Tools"),
						"text" => array("<p>В категории инструментов вы найдете широкий ассортимент оборудования и аксессуаров для ремонта, строительства и различных видов работ. Мы предлагаем ручные инструменты, электроинструменты, строительные материалы и многое другое. Все наши продукты отличаются высоким качеством и надежностью.</p>"),
						"rewrite" => "shop/stroitelstvo-i-remont/instrumenty",
						"sort" => 62,
						"images" => array("855_sovety-po-vyboru-benzopily-.webp"),
						"menu" => array(
							array(
								"cat_id" => 2,
								"act" => array(1),
							),
						),
						'import_id' => '62',
					),
					array(
						"id" => 63,
						"name" => array("Сантехника", "Plumbing"),
						"text" => array("<p>В категории товаров &laquo;Сантехника&raquo; вы найдете все необходимое для оборудования ванной комнаты и туалета. Мы предлагаем сантехническое оборудование, такое как унитазы, раковины, ванны и душевые кабины, а также сантехнические аксессуары, такие как смесители, полотенцесушители и держатели для полотенец. Все наши товары имеют высокое качество и надежность.</p>"),
						"rewrite" => "shop/stroitelstvo-i-remont/santekhnika",
						"sort" => 63,
						"images" => array("1112_rakovina-lodochka-dlya-ku.webp"),
						"menu" => array(
							array(
								"cat_id" => 2,
								"act" => array(1),
							),
						),
						'import_id' => '63',
					),
					array(
						"id" => 64,
						"name" => array("Строительные материалы", "Building materials"),
						"text" => array("<p>В категории &ldquo;Строительство и ремонт&rdquo; вы найдете все необходимые товары для проведения ремонтных работ в доме или квартире. Мы предлагаем строительные материалы, инструменты и оборудование от ведущих производителей, а также профессиональные советы по ремонту и дизайну интерьера. Наша цель - помочь вам создать уютное и комфортное пространство в вашем доме.</p>"),
						"rewrite" => "shop/stroitelstvo-i-remont/stroitelnye-materialy",
						"sort" => 64,
						"images" => array("853_novomodnaya-shtukaturka-pre.webp"),
						"menu" => array(
							array(
								"cat_id" => 2,
								"act" => array(1),
							),
						),
						'import_id' => '64',
					),
					array(
						"id" => 65,
						"name" => array("Отделочные материалы", "Finishing materials"),
						"text" => array("<p>В категории &ldquo;Строительство и ремонт&rdquo; вы найдете все необходимые товары для проведения ремонтных работ в доме или квартире. Мы предлагаем строительные материалы, инструменты и оборудование от ведущих производителей, а также профессиональные советы по ремонту и дизайну интерьера. Наша цель - помочь вам создать уютное и комфортное пространство в вашем доме.</p>"),
						"rewrite" => "shop/stroitelstvo-i-remont/otdelochnye-materialy",
						"sort" => 65,
						"images" => array("1113_shtukaturka-stroyplast.webp"),
						"menu" => array(
							array(
								"cat_id" => 2,
								"act" => array(1),
							),
						),
						'import_id' => '65',
					),
				),
			),
			array(
				"id" => 12,
				"name" => array("Бытовая химия", "Household chemicals"),
				"text" => array("<p>В категории бытовой химии вы найдете средства для стирки, уборки, мытья посуды и другие товары для поддержания чистоты и порядка в доме. Мы предлагаем продукцию от ведущих производителей, которая отличается высоким качеством и эффективностью. Наши товары помогут вам справиться с любыми загрязнениями и сделают ваш дом еще более уютным и комфортным.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
				"rewrite" => "shop/bytovaya-khimiya",
				"sort" => 12,
				"menu" => array("2", "3"),
				'import_id' => '12',
				"children" => array(
					array(
						"id" => 66,
						"name" => array("Средства для стирки", "Laundry products"),
						"text" => array("<p>В категории бытовой химии вы найдете средства для стирки, уборки, мытья посуды и другие товары для поддержания чистоты и порядка в доме. Мы предлагаем продукцию от ведущих производителей, которая отличается высоким качеством и эффективностью. Наши товары помогут вам справиться с любыми загрязнениями и сделают ваш дом еще более уютным и комфортным.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/bytovaya-khimiya/sredstva-dlya-stirki",
						"sort" => 66,
						"images" => array("851_novyy-stiralnyy-poroshok-n.webp"),
						"menu" => array("2", "3"),
						'import_id' => '66',
					),
					array(
						"id" => 67,
						"name" => array("Чистящие средства", "Cleaning products"),
						"text" => array("<p>В категории бытовой химии вы найдете средства для стирки, уборки, мытья посуды и другие товары для поддержания чистоты и порядка в доме. Мы предлагаем продукцию от ведущих производителей, которая отличается высоким качеством и эффективностью. Наши товары помогут вам справиться с любыми загрязнениями и сделают ваш дом еще более уютным и комфортным.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/bytovaya-khimiya/chistyaschie-sredstva",
						"sort" => 67,
						"images" => array("1117_universalnoe-chistyaschee-sre.webp"),
						"menu" => array("2", "3"),
						'import_id' => '67',
					),
					array(
						"id" => 68,
						"name" => array("Освежители и ароматизаторы", "Fresheners and flavors"),
						"text" => array("<p>В категории бытовой химии вы найдете средства для стирки, уборки, мытья посуды и другие товары для поддержания чистоты и порядка в доме. Мы предлагаем продукцию от ведущих производителей, которая отличается высоким качеством и эффективностью. Наши товары помогут вам справиться с любыми загрязнениями и сделают ваш дом еще более уютным и комфортным.</p>", "<p>Our online store offers a wide range of products for men, women and children. We carry top quality items in categories such as clothing, footwear, accessories, beauty &amp; health, home &amp; kitchen, garden &amp; tools, and more. Our products are carefully selected to meet the needs of our customers and we offer competitive prices, fast shipping and excellent customer service. Browse our site to find the perfect items for your home or wardrobe!</p>"),
						"rewrite" => "shop/bytovaya-khimiya/osvezhiteli-i-aromatizatory",
						"sort" => 68,
						"images" => array("850_aromatnye-novinki-obzor-ne.webp"),
						"menu" => array("2", "3"),
						'import_id' => '68',
					),
				),
			),
		),
		'shop_param' => array(
			array(
				"id" => 1,
				"name" => array("Диагональ экрана", "Screen diagonal"),
				"type" => "select",
				"cat_id" => array(34, 33, 32, 14),
				"sort" => 1,
				"search" => "1",
				"id_page" => "1",
				"select" => array(
					array(
						"id" => "84",
						"name" => array("5.1 дюйма", "5.1 inches")
					),
					array(
						"id" => "85",
						"name" => array("6 дюймов", "5.5 inches")
					),
					array(
						"id" => "86",
						"name" => array("6.1 дюйма", "6 inches")
					),
					array(
						"id" => "87",
						"name" => array("6.5 дюймов", "6.1 inches")
					)
				),
			),
			array(
				"id" => 2,
				"name" => array("Цвет", "Color"),
				"type" => "multiple",
				"cat_id" => array(57, 56, 55, 54, 52, 51, 50, 49, 48, 47, 46, 45, 44, 41, 40, 36, 4, 3, 9, 8, 7, 6, 1),
				"sort" => 2,
				"required" => "1",
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "1", "name" => array("Хаки/Бежевый", "Light haki")), array("id" => "2", "name" => array("Синий/Голубой", "Light blue")), array("id" => "3", "name" => array("Зеленый", "Green")), array("id" => "10", "name" => array("Синий", "Blue")), array("id" => "11", "name" => array("Нави", "Navi")), array("id" => "12", "name" => array("Фисташковый", "Pistascoe")), array("id" => "16", "name" => array("Хаки", "Haki")), array("id" => "19", "name" => array("Желтый", "Yellow")), array("id" => "38", "name" => array("Бордовый", "Dark red")), array("id" => "42", "name" => array("Коричневый", "Brown")), array("id" => "47", "name" => array("Серый/красный", "Gray/red")), array("id" => "48", "name" => array("Серый/электрик", "Gray")), array("id" => "131", "name" => array("Оранжевый", "Orange")), array("id" => "132", "name" => array("Черный", "Black")), array("id" => "133", "name" => array("Разноцветный", "Multicolor")), array("id" => "151", "name" => array("Белый", "White"))),
			),
			array(
				"id" => 3,
				"name" => array("Процессор", "Processor"),
				"type" => "select",
				"measure_unit" => array("человек", "man"),
				"cat_id" => array(34, 33, 32, 31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 10),
				"sort" => 3,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "88", "name" => array("Qualcomm Snapdragon 888", "Qualcomm Snapdragon 888")), array("id" => "89", "name" => array("Apple A14 Bionic", "Apple A14 Bionic")), array("id" => "90", "name" => array("MediaTek Dimensity 1200", "MediaTek Dimensity 1200")), array("id" => "91", "name" => array("Samsung Exynos 2200", "Samsung Exynos 2200")), array("id" => "92", "name" => array("Huawei Kirin 9000", "Huawei Kirin 9000")), array("id" => "106", "name" => array("Intel Core i7", "Intel Core i7")), array("id" => "107", "name" => array("AMD Ryzen 5", "AMD Ryzen 5")), array("id" => "108", "name" => array("Intel Core i5", "Intel Core i5"))),
			),
			array(
				"id" => 4,
				"name" => array("Оперативная память", "Random access memory"),
				"type" => "select",
				"cat_id" => array(34, 33, 32, 31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 10),
				"sort" => 4,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "4", "name" => array("2 ГБ", "2 Gb")), array("id" => "5", "name" => array("4 ГБ", "4 Gb")), array("id" => "20", "name" => array("8 ГБ", "8 Gb")), array("id" => "109", "name" => array("16 ГБ", "16 Gb"))),
			),
			array(
				"id" => 5,
				"name" => array("Встроенная память", "Built-in memory"),
				"type" => "select",
				"cat_id" => array(34, 33, 32, 31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 17, 16, 15, 14, 10),
				"sort" => 5,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "93", "name" => array("32 ГБ", "32 Gb")), array("id" => "94", "name" => array("64 ГБ", "64 Gb")), array("id" => "95", "name" => array("128 ГБ", "128 Gb")), array("id" => "96", "name" => array("256 ГБ", "256 Gb")), array("id" => "97", "name" => array("512 ГБ", "512 Gb"))),
			),
			array(
				"id" => 6,
				"name" => array("Материал", "Material"),
				"type" => "select",
				"cat_id" => array(38, 37, 36, 35),
				"sort" => 6,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "98", "name" => array("Пластик", "Plastic")), array("id" => "99", "name" => array("Кожа", "Skin")), array("id" => "100", "name" => array("Эко кожа", "Eco-leather")), array("id" => "101", "name" => array("Силикон", "Selicon"))),
			),
			array(
				"id" => 7,
				"name" => array("Подходит для моделей", "Suitable for models"),
				"type" => "multiple",
				"measure_unit" => array("кг.", "kg"),
				"cat_id" => array(38, 37, 36, 35),
				"sort" => 7,
				"id_page" => "1",
				"select" => array(array("id" => "102", "name" => array("AuraPhone", "AuraPhone")), array("id" => "103", "name" => array("CosmosSmart", "CosmosSmart")), array("id" => "104", "name" => array("ElectraNote", "ElectraNote")), array("id" => "105", "name" => array("GalaxyTouch", "GalaxyTouch"))),
			),
			array(
				"id" => 8,
				"name" => array("Размер", "Size"),
				"type" => "multiple",
				"cat_id" => array(46, 45, 44, 40, 9, 8, 7, 6),
				"sort" => 8,
				"required" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "114", "name" => array("S", "S")), array("id" => "115", "name" => array("M", "M")), array("id" => "116", "name" => array("L", "L")), array("id" => "117", "name" => array("XL", "XL")), array("id" => "118", "name" => array("XXL", "XXL"))),
			),
			array(
				"id" => 10,
				"name" => array("Материал", "Structural material"),
				"type" => "select",
				"cat_id" => array(46, 45, 44, 41, 40, 9, 8, 7, 6),
				"sort" => 9,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "8", "name" => array("Шерсть", "Wool")), array("id" => "9", "name" => array("Полиэстер", "Polyester")), array("id" => "15", "name" => array("Нейлон", "Nylon")), array("id" => "119", "name" => array("Хлопок", "Cotton")), array("id" => "120", "name" => array("Комбинированный", "Combined")), array("id" => "121", "name" => array("Кожа", "Skin")), array("id" => "122", "name" => array("Эко мех", "Eco-friendly fur")), array("id" => "123", "name" => array("Натуральный мех", "Natural")), array("id" => "124", "name" => array("Замша", "Suede fur")), array("id" => "163", "name" => array("Джинс", "Jeans"))),
			),
			array(
				"id" => 11,
				"name" => array("Стиль", "Style"),
				"type" => "select",
				"cat_id" => array(46, 45, 44, 41, 40, 9, 8, 7, 6),
				"sort" => 10,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "125", "name" => array("Классика", "Classic")), array("id" => "126", "name" => array("Спортивный", "Sporting")), array("id" => "127", "name" => array("Парка", "The Park")), array("id" => "128", "name" => array("Пуховик", "Down jacket")), array("id" => "134", "name" => array("Вечернее", "Evening")), array("id" => "135", "name" => array("Коктейльное", "Cocktail menu")), array("id" => "136", "name" => array("Молодежный", "Youth"))),
			),
			array(
				"id" => 12,
				"name" => array("Материал", "Material"),
				"type" => "select",
				"cat_id" => array(53, 52, 51, 47),
				"sort" => 11,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "17", "name" => array("Керамика", "Ceramics")), array("id" => "18", "name" => array("Сталь", "Steel")), array("id" => "129", "name" => array("Пластик", "Plastic")), array("id" => "130", "name" => array("Стекло", "Glass")), array("id" => "193", "name" => array("Дерево", "Tree")), array("id" => "194", "name" => array("Бамбук", "Bamboo"))),
			),
			array(
				"id" => 14,
				"name" => array("Страна изготовитель", "Manufacturer's country"),
				"type" => "select",
				"measure_unit" => array("кг.", "kg"),
				"cat_id" => array(53, 52, 51, 47),
				"sort" => 12,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "137", "name" => array("Китай", "China")), array("id" => "138", "name" => array("Россия", "Russia")), array("id" => "139", "name" => array("Индия", "India")), array("id" => "140", "name" => array("Италия", "Italy")), array("id" => "202", "name" => array("Чехия", "Czech")), array("id" => "203", "name" => array("Германия", "Germany"))),
			),
			array(
				"id" => 15,
				"name" => array("Возрастное назначение", "Age assignment"),
				"type" => "select",
				"measure_unit" => array("л.", "l"),
				"cat_id" => array(53, 52, 51, 47),
				"sort" => 13,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "143", "name" => array("Взрослая", "Adult")), array("id" => "144", "name" => array("Детская", "Childish"))),
			),
			array(
				"id" => 16,
				"name" => array("Материал", "Material"),
				"type" => "select",
				"cat_id" => array(48, 3),
				"sort" => 14,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "145", "name" => array("Хлопок", "Cotton")), array("id" => "146", "name" => array("Лен", "Flax")), array("id" => "147", "name" => array("Шерсть", "Wool")), array("id" => "148", "name" => array("Шелк", "Silk")), array("id" => "149", "name" => array("Полиэстр", "Polyester")), array("id" => "150", "name" => array("Нейлон", "Nylon"))),
			),
			array(
				"id" => 17,
				"name" => array("Размер шторы", "Curtain size"),
				"type" => "multiple",
				"cat_id" => array(48, 3),
				"sort" => 15,
				"required" => "1",
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "152", "name" => array("200х270", "200X270")), array("id" => "153", "name" => array("300х270", "300X270")), array("id" => "154", "name" => array("400х270", "400X270"))),
			),
			array(
				"id" => 19,
				"name" => array("Форма", "Form"),
				"type" => "select",
				"cat_id" => array(49, 3),
				"sort" => 17,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "158", "name" => array("Круглая", "Round")), array("id" => "159", "name" => array("Овальная", "Oval")), array("id" => "160", "name" => array("Квадратная", "Square")), array("id" => "161", "name" => array("Прямоугольная", "Rectangular"))),
			),
			array(
				"id" => 20,
				"name" => array("Стиль", "Style"),
				"type" => "select",
				"cat_id" => array(57, 56, 55, 54, 53, 52, 51, 49, 48, 47, 4, 3),
				"sort" => 18,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "23", "name" => array("Классика", "Classic")), array("id" => "24", "name" => array("Современный", "Modern")), array("id" => "25", "name" => array("Скандинавский", "Scandinavian")), array("id" => "26", "name" => array("Лофт", "Loft")), array("id" => "27", "name" => array("Бохо", "Boho")), array("id" => "166", "name" => array("Модерн", "Neo-Modern")), array("id" => "165", "name" => array("Арт-деко", "Art Deco")), array("id" => "164", "name" => array("Прованс", "Provence"))),
			),
			array(
				"id" => 21,
				"name" => array("Ткань", "Cloth"),
				"type" => "select",
				"cat_id" => array(57, 56, 55, 54, 4),
				"sort" => 19,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "36", "name" => array("Велюр", "Velour")), array("id" => "40", "name" => array("Шенилл", "Chenille")), array("id" => "41", "name" => array("Рогожка", "Gunny")), array("id" => "175", "name" => array("Жаккард", "Jacquard")), array("id" => "174", "name" => array("Флок", "Flock")), array("id" => "176", "name" => array("Гобелен", "Tapestry"))),
			),
			array(
				"id" => 22,
				"name" => array("Тип кожи", "Skin Type"),
				"type" => "multiple",
				"cat_id" => array(61, 60, 59, 58, 5),
				"sort" => 20,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "177", "name" => array("Нормальная", "Normal")), array("id" => "178", "name" => array("Сухая", "Dry")), array("id" => "179", "name" => array("Жирная", "Fat")), array("id" => "180", "name" => array("Комбинированная", "Combined"))),
			),
			array(
				"id" => 23,
				"name" => array("Возраст", "Age"),
				"type" => "select",
				"cat_id" => array(61, 60, 59, 58, 5),
				"sort" => 21,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "181", "name" => array("Подростковый", "Teenage")), array("id" => "182", "name" => array("Молодой", "Young")), array("id" => "183", "name" => array("Средний", "Average")), array("id" => "184", "name" => array("Пожилой", "Elderly"))),
			),
			array(
				"id" => 24,
				"name" => array("Текстура", "Texture"),
				"type" => "select",
				"measure_unit" => array("кг.", "kg"),
				"cat_id" => array(61, 60, 59, 58, 5),
				"sort" => 22,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "185", "name" => array("Матовая", "Matte")), array("id" => "186", "name" => array("Сияющая", "Shining")), array("id" => "187", "name" => array("Перламутровая", "Mother of pearl")), array("id" => "188", "name" => array("Глянцевая", "Glossy"))),
			),
			array(
				"id" => 25,
				"name" => array("Семейство ароматов", "The fragrance family"),
				"type" => "multiple",
				"cat_id" => array(61, 60, 59, 58, 5),
				"sort" => 23,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "171", "name" => array("Древесные", "Woody")), array("id" => "170", "name" => array("Зеленые", "Green")), array("id" => "169", "name" => array("Восточные", "Eastern")), array("id" => "168", "name" => array("Фруктовые", "Fruit")), array("id" => "167", "name" => array("Цветочные", "Floral")), array("id" => "172", "name" => array("Морские", "Marine")), array("id" => "173", "name" => array("Табачные", "Tobacco"))),
			),
			array(
				"id" => 26,
				"name" => array("Материал", "Material"),
				"type" => "select",
				"cat_id" => array(65, 64, 63, 62, 57, 56, 55, 54, 11, 4),
				"sort" => 24,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "189", "name" => array("ДСП", "Chipboard")), array("id" => "190", "name" => array("ДВП", "FIBERBOARD")), array("id" => "191", "name" => array("МДФ", "MDF")), array("id" => "192", "name" => array("ОСБ", "OSB")), array("id" => "198", "name" => array("Алюминий", "Aluminum")), array("id" => "199", "name" => array("Пластик", "Plastic")), array("id" => "200", "name" => array("Сталь", "Steel")), array("id" => "201", "name" => array("Керамика", "Ceramics"))),
			),
			array(
				"id" => 27,
				"name" => array("Прочность", "Endurance"),
				"type" => "multiple",
				"measure_unit" => array("кг.", "kg"),
				"cat_id" => array(65, 64, 63, 62, 11),
				"sort" => 25,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "195", "name" => array("Мягкая", "Soft")), array("id" => "196", "name" => array("Твердая", "Solid")), array("id" => "197", "name" => array("Очень твердая", "Very hard"))),
			),
			array(
				"id" => 28,
				"name" => array("Мощность", "Power"),
				"type" => "select",
				"cat_id" => array(65, 64, 63, 62, 11),
				"sort" => 26,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "49", "name" => array("780 Вт", "780 Vt")), array("id" => "50", "name" => array("790 Вт", "790 Vt")), array("id" => "51", "name" => array("800 Вт", "800 Vt")), array("id" => "52", "name" => array("1120 Вт", "820 Vt"))),
			),
			array(
				"id" => 29,
				"name" => array("Управление", "Management"),
				"type" => "select",
				"cat_id" => array(65, 64, 63, 62, 11),
				"sort" => 27,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "44", "name" => array("Рычажное", "Lever")), array("id" => "45", "name" => array("Вентильное", "The valve")), array("id" => "46", "name" => array("Нажимное", "Pressure")), array("id" => "82", "name" => array("Термостат", "Thermostat")), array("id" => "83", "name" => array("Порционное", "Portioned"))),
			),
			array(
				"id" => 30,
				"name" => array("Тип продукта", "Product Type"),
				"type" => "select",
				"measure_unit" => array("л.", "l"),
				"cat_id" => array(68, 67, 66, 12),
				"sort" => 28,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "78", "name" => array("Порошок", "Powder")), array("id" => "79", "name" => array("Жидкость", "Liquid")), array("id" => "80", "name" => array("Гель", "Gel")), array("id" => "81", "name" => array("Спрей", "Spray")), array("id" => "110", "name" => array("Таблетки", "Pills"))),
			),
			array(
				"id" => 31,
				"name" => array("Объем, л", "Volume, l"),
				"type" => "select",
				"cat_id" => array(68, 67, 66, 12),
				"sort" => 31,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "53", "name" => array("1", "1")), array("id" => "54", "name" => array("1,5", "1,5")), array("id" => "55", "name" => array("2", "2")), array("id" => "56", "name" => array("2,5", "2,5")), array("id" => "57", "name" => array("5", "5"))),
			),
			array(
				"id" => 32,
				"name" => array("Назначение", "Appointment"),
				"type" => "select",
				"cat_id" => array(68, 67, 66, 12),
				"sort" => 32,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "58", "name" => array("Кухня", "Kitchen")), array("id" => "59", "name" => array("Ванная", "Bathroom")), array("id" => "60", "name" => array("Туалет", "Toilet")), array("id" => "61", "name" => array("Пол", "Paul")), array("id" => "205", "name" => array("Зеркало", "Mirror")), array("id" => "204", "name" => array("Стекло", "Glass"))),
			),
			array(
				"id" => 33,
				"name" => array("Тип матрицы", "Type of matrix"),
				"type" => "select",
				"cat_id" => array(31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 10),
				"sort" => 33,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "62", "name" => array("IPS", "IPS")), array("id" => "63", "name" => array("VA", "VA")), array("id" => "64", "name" => array("TN", "VGA")), array("id" => "65", "name" => array("OLED", "OLED"))),
			),
			array(
				"id" => 34,
				"name" => array("Частота обновления экрана", "Screen refresh rate"),
				"type" => "select",
				"cat_id" => array(31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 10),
				"sort" => 34,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "66", "name" => array("50", "50")), array("id" => "67", "name" => array("60", "60")), array("id" => "68", "name" => array("100", "100")), array("id" => "69", "name" => array("120", "120"))),
			),
			array(
				"id" => 35,
				"name" => array("Время отклика", "Response time"),
				"type" => "select",
				"cat_id" => array(31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 10),
				"sort" => 35,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "70", "name" => array("1 миллисекунда", "1 millisecond")), array("id" => "71", "name" => array("2 миллисекунды", "2 millisecond")), array("id" => "72", "name" => array("3 миллисекунды", "4 millisecond")), array("id" => "73", "name" => array("5 миллисекунд", "6 millisecond"))),
			),
			array(
				"id" => 36,
				"name" => array("Яркость", "Brightness"),
				"type" => "select",
				"cat_id" => array(31, 30, 29, 28, 27, 26, 25, 24, 23, 22, 21, 20, 19, 18, 10),
				"sort" => 36,
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "74", "name" => array("250 кд/м²", "250 cd/m2")), array("id" => "75", "name" => array("300 кд/м²", "300 cd/m2")), array("id" => "76", "name" => array("400 кд/м²", "350 cd/m2")), array("id" => "77", "name" => array("500 кд/м²", "500 cd/m2"))),
			),
			array(
				"id" => 37,
				"name" => array("Размер", "Size"),
				"type" => "multiple",
				"cat_id" => 41,
				"sort" => 37,
				"required" => "1",
				"search" => "1",
				"id_page" => "1",
				"select" => array(array("id" => "206", "name" => array("37", "37")), array("id" => "207", "name" => array("38", "38")), array("id" => "208", "name" => array("39", "39")), array("id" => "209", "name" => array("40", "40")), array("id" => "210", "name" => array("41", "41")), array("id" => "211", "name" => array("42", "42")), array("id" => "212", "name" => array("43", "43"))),
			),
		),
		'shop' => array(
			array(
				"id" => 1,
				"cat_id" => array(40, 1),
				"brand_id" => 10,
				"article" => "24068",
				"name" => array(
					"Осенняя куртка для мальчика “Маленький путешественник”",
					"Autumn jacket for a boy “Little traveler”"
				),
				"anons" => array(
					"<p>Стильная и удобная осенняя куртка &ldquo;Маленький путешественник&rdquo; создана специально для маленьких искателей приключений.</p>",
					"<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"
				),
				"text" => array(
					"<p>Куртка изготовлена из водоотталкивающей ткани, которая защитит вашего малыша от дождя и ветра. Капюшон с утяжкой и эластичные манжеты на рукавах обеспечивают дополнительную защиту от непогоды. Дизайн куртки украшен яркими вставками и логотипом &ldquo;Маленький путешественник&rdquo;, что делает ее особенно привлекательной для мальчиков.</p>",
					"<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"
				),
				"rewrite" => "shop/odezhda-i-obuv/detyam/osennyaya-kurtka-dlya-malchika-malenkiy-puteshestv",
				"images" => array(
					"1008_osennyaya-kurtka-dlya-malchik.webp",
					"1009_osennyaya-kurtka-dlya-malchik.webp"
				),
				"param" => array(
					"2" => "42",
					"8" => array("117", "116", "115", "114"),
					"10" => "121",
					"11" => "125"
				),
				"price" => array(
					array(
						"price" => "3990",
						"param" => array("8" => "0", "2" => "42")
					)
				),
				"sort" => 1,
				'import_id' => '1',
			),
			array(
				"id" => 2,
				"cat_id" => array(40, 1),
				"brand_id" => 10,
				"article" => "95322",
				"name" => array("Детский спортивный костюм для девочки “Бабочка”", "Children's tracksuit for a girl “Butterfly”"),
				"anons" => array("<p>Яркий и стильный детский спортивный костюм для девочек &ldquo;Бабочка&rdquo; станет отличным выбором для маленьких спортсменок.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Костюм состоит из удобных леггинсов и топа с длинными рукавами, выполненных из эластичного материала. Дизайн костюма украшен принтом с изображением красивых бабочек, что делает его особенно привлекательным для девочек. Этот костюм идеально подходит для занятий танцами, гимнастикой или просто для повседневной носки дома.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/detyam/detskiy-sportivnyy-kostyum-dlya-devochki-babochka",
				"images" => array("1005_detskiy-sportivnyy-kostyum-.webp"),
				"param" => array("8" => array("116", "115", "114"), "2" => "151", "10" => "9", "11" => "126"),
				"price" => array(array("price" => "6490", "param" => array("8" => "0", "2" => "151"))),
				"sort" => 2,
				'import_id' => '2',
			),
			array(
				"id" => 3,
				"cat_id" => array(12, 68),
				"brand_id" => 8,
				"article" => "25373",
				"name" => array("Автоматический освежитель воздуха “AirRefresh”", "Automatic air freshener “AirRefresh”"),
				"anons" => array("<p>Автоматический освежитель воздуха &ldquo;AirRefresh&rdquo; - это отличное решение для создания приятной атмосферы в вашем доме.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Он работает по таймеру, поэтому вы можете настроить его на автоматическое включение и отключение в нужное время. &ldquo;AirRefresh&rdquo; эффективно устраняет неприятные запахи, наполняя помещение свежим и приятным ароматом. Он абсолютно безопасен для здоровья, не вызывает аллергических реакций и не содержит вредных веществ.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/bytovaya-khimiya/avtomaticheskiy-osvezhitel-vozdukha-airrefresh",
				"images" => array("1118_avtomaticheskiy-osvezhitel-.webp"),
				"param" => array("32" => "59", "31" => "54", "30" => "79"),
				"price" => array(array("price" => "900")),
				"sort" => 3,
				'import_id' => '3',
			),
			array(
				"id" => 4,
				"cat_id" => array(65, 11),
				"brand_id" => 2,
				"article" => "25593",
				"name" => array("Декоративные стильные обои “ArtDeco”", "Decorative stylish wallpaper “ArtDeco”"),
				"anons" => array("<p>Декоративные стильные обои &ldquo;ArtDeco&rdquo; - это идеальное решение для тех, кто хочет создать неповторимый интерьер своего дома.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Они обладают высоким качеством и прочностью, что гарантирует долговечность отделки. Широкий выбор дизайнов и фактур позволяет подобрать обои под любой стиль интерьера. &ldquo;ArtDeco&rdquo; легко клеятся на стены и не требуют особых навыков для монтажа. С нашими декоративными стильными обоями ваш дом станет еще уютнее и привлекательнее!</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/stroitelstvo-i-remont/otdelochnye-materialy/dekorativnye-stilnye-oboi-artdeco",
				"images" => array("1114_dekorativnye-stilnye-oboi.webp"),
				"param" => array("27" => "196"),
				"price" => array(array("price" => "10490")),
				"action" => "1",
				"sort" => 4,
				'import_id' => '4',
			),
			array(
				"id" => 5,
				"cat_id" => array(64, 11),
				"brand_id" => 4,
				"article" => "25573",
				"name" => array("Штукатурка “СтройПласт”", "StroyPlast plaster"),
				"anons" => array("<p>Штукатурка &ldquo;СтройПласт&rdquo; - это высококачественный материал для отделки стен и потолков.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Она обладает высокой прочностью и долговечностью, а также обеспечивает отличную звуко- и теплоизоляцию. Штукатурка легко наносится на поверхность и быстро сохнет, что позволяет сэкономить время на отделочных работах. &ldquo;СтройПласт&rdquo; доступна в различных цветовых решениях, что позволяет выбрать наиболее подходящий вариант для вашего интерьера.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/stroitelstvo-i-remont/stroitelnye-materialy/shtukaturka-stroyplast",
				"images" => array("1113_shtukaturka-stroyplast.webp"),
				"param" => array("27" => "195"),
				"price" => array(array("price" => "640")),
				"sort" => 5,
				'import_id' => '5',
			),
			array(
				"id" => 6,
				"cat_id" => array(63, 11),
				"brand_id" => 12,
				"article" => "25563",
				"name" => array("Раковина “Лодочка” для кухни", "The “Boat” sink for the kitchen"),
				"anons" => array("<p>Раковина для кухни &ldquo;Лодочка&rdquo; - это стильное и практичное решение для вашей кухни.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Она выполнена из высококачественной нержавеющей стали, что обеспечивает ее прочность и долговечность. Раковина имеет удобную форму, которая позволяет легко мыть посуду и продукты. В верхней части раковины имеется отверстие для установки смесителя. Раковина &ldquo;Лодочка&rdquo; прекрасно сочетается с любым интерьером кухни и станет настоящим украшением вашего дома.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/stroitelstvo-i-remont/santekhnika/rakovina-lodochka-dlya-kukhni",
				"images" => array("1112_rakovina-lodochka-dlya-ku.webp"),
				"param" => array("27" => "196", "26" => "201"),
				"price" => array(array("price" => "7990")),
				"sort" => 6,
				'import_id' => '6',
			),
			array(
				"id" => 7,
				"cat_id" => array(63, 11),
				"brand_id" => 4,
				"article" => "21139",
				"name" => array("Смеситель для кухни “AquaStar”", "Kitchen faucet “AquaStar”"),
				"anons" => array("<p>Смеситель для кухни &ldquo;AquaStar&rdquo; - это стильный и функциональный аксессуар для вашей кухни.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Он выполнен из высококачественной латуни, что гарантирует его прочность и долговечность. Смеситель имеет однорычажный механизм управления, что позволяет легко регулировать температуру и напор воды. Особенностью данного смесителя является высокий поворотный излив, который обеспечивает удобство при мытье посуды и наполнении больших емкостей. В комплекте со смесителем идут гибкая подводка и крепеж, что позволяет быстро и легко установить его на вашу кухню.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/stroitelstvo-i-remont/santekhnika/smesitel-dlya-kukhni-aquastar",
				"images" => array("1109_kukhonnyy-kran-aquatech.webp", "1108_kukhonnyy-kran-aquatech.webp"),
				"param" => array("29" => "46", "27" => "196", "26" => "200"),
				"price" => array(array("price" => "3990")),
				"hit" => "1",
				"sort" => 7,
				'import_id' => '7',
			),
			array(
				"id" => 8,
				"cat_id" => array(63, 11),
				"brand_id" => 11,
				"article" => "23038",
				"name" => array("Кухонный кран “AquaTech”", "AquaTech kitchen faucet"),
				"anons" => array("<p>Кухонный кран &ldquo;AquaTech&rdquo; - идеальный выбор для вашей кухни.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Он имеет современный дизайн и прочную конструкцию, которая прослужит вам долгие годы. Кран оснащен удобным поворотным изливом, который позволяет легко регулировать направление воды. Функция мягкого открывания и закрывания делает использование крана еще более комфортным. В комплекте с краном идут аэратор и гибкая подводка, что упрощает процесс установки.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/stroitelstvo-i-remont/santekhnika/kukhonnyy-kran-aquatech",
				"images" => array("1108_kukhonnyy-kran-aquatech.webp", "1109_kukhonnyy-kran-aquatech.webp"),
				"param" => array("29" => "45", "27" => "196", "26" => "198"),
				"price" => array(array("price" => "11990")),
				"new" => "1",
				"sort" => 8,
				'import_id' => '8',
			),
			array(
				"id" => 9,
				"cat_id" => array(62, 11),
				"brand_id" => 1,
				"article" => "23098",
				"name" => array("Универсальный помощник-инструмент для мужчин “MasterMan”", "Universal assistant-tool for men “MasterMan”"),
				"anons" => array("<p>Универсальный помощник для мужчин &ldquo;MasterMan&rdquo; - это незаменимый набор инструментов для домашних дел и мелкого ремонта.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>В комплект входят различные отвертки, плоскогубцы, молоток, рулетка, уровень, разводной ключ и другие полезные инструменты. Все они аккуратно уложены в компактный кейс, который легко взять с собой на дачу или в поездку. С универсальным помощником-инструментом &ldquo;MasterMan&rdquo; вы всегда будете готовы к любым бытовым задачам и мелким ремонтам.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/stroitelstvo-i-remont/instrumenty/universalnyy-pomoschnik-instrument-dlya-muzhchin-m",
				"images" => array("1107_universalnyy-pomoschnik-ins.webp"),
				"param" => array("29" => "44", "27" => "197", "26" => "200"),
				"price" => array(array("price" => "6090")),
				"sort" => 9,
				'import_id' => '9',
			),
			array(
				"id" => 10,
				"cat_id" => array(62, 11),
				"brand_id" => 1,
				"article" => "25553",
				"name" => array("Бензопила “PowerCut”", "Chainsaw “PowerCut”"),
				"anons" => array("<p>Бензопила &ldquo;PowerCut&rdquo; - это надежный и эффективный инструмент для распиливания древесины.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Она оснащена мощным двигателем, который обеспечивает высокую скорость пиления, а система автоматической смазки цепи обеспечивает долгий срок службы пилы. Благодаря эргономичному дизайну, пила удобно лежит в руке и позволяет работать с комфортом. &ldquo;PowerCut&rdquo; идеально подойдет как для профессиональных лесозаготовителей, так и для владельцев загородных домов.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/stroitelstvo-i-remont/instrumenty/benzopila-powercut",
				"images" => array("1106_benzopila-powercut.webp"),
				"param" => array("29" => "46", "28" => "51", "27" => "197", "26" => "200"),
				"price" => array(array("price" => "14490")),
				"sort" => 10,
				'import_id' => '10',
			),
			array(
				"id" => 11,
				"cat_id" => array(62, 11),
				"brand_id" => 14,
				"article" => "95286",
				"name" => array("Дрель электрическая “MasterDrill”", "Electric drill “MasterDrill”"),
				"anons" => array("<p>Дрель электрическая &ldquo;MasterDrill&rdquo; - это идеальный инструмент для сверления отверстий в различных материалах.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Благодаря мощному двигателю, дрель обеспечивает высокую производительность и скорость сверления. Эргономичный дизайн и удобные рукоятки позволяют работать с комфортом и точностью. Дрель оснащена быстрозажимным патроном, что позволяет быстро менять сверла и адаптировать дрель для разных задач. &ldquo;MasterDrill&rdquo; подойдет как для домашнего использования, так и для профессионального применения.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/stroitelstvo-i-remont/instrumenty/drel-elektricheskaya-masterdrill",
				"images" => array("1105_drel-elektricheskaya-masterdri.webp"),
				"param" => array("28" => "52", "27" => "197", "26" => "200", "29" => "46"),
				"price" => array(array("price" => "9990")),
				"sort" => 11,
				'import_id' => '11',
			),
			array(
				"id" => 12,
				"cat_id" => array(5, 61),
				"brand_id" => 11,
				"article" => "23118",
				"name" => array("Мужской парфюм “Hero”", "Men's perfume “Hero”"),
				"anons" => array("<p>&ldquo;Hero&rdquo; - это мужественный и энергичный аромат, который подчеркнет вашу индивидуальность и силу характера.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Верхние ноты бергамота и зеленого яблока придают аромату свежесть и легкость, в то время как средние ноты кедра и пачули придают ему глубину и чувственность. Базовые ноты амбры и мускуса создают теплый и притягательный шлейф, который будет следовать за вами на протяжении всего дня.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/krasota-i-zdorove/muzhskoy-parfyum-hero",
				"images" => array("1096_parfyumernyy-nabor-romantic-duo.webp", "1097_parfyumernyy-nabor-romantic-duo.webp", "1098_parfyumernyy-nabor-romantic-duo.webp"),
				"param" => array("25" => "173", "23" => "182"),
				"price" => array(array("price" => "5990")),
				"new" => "1",
				"sort" => 12,
				'import_id' => '12',
			),
			array(
				"id" => 13,
				"cat_id" => array(8, 1),
				"brand_id" => 7,
				"article" => "21078",
				"name" => array("Укороченная рубашка с воротником-стойкой", "Cropped shirt with a stand-up collar"),
				"anons" => array("<p>Эта укороченная рубашка с воротником-стойкой &ndash; стильный и удобный выбор для летнего гардероба.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Выполненная из легкого материала, она обеспечивает комфорт в жаркую погоду. Воротник-стойка добавляет образу элегантности и утонченности, а укороченный крой подчеркивает талию и создает женственный силуэт. С такой рубашкой вы сможете создавать разнообразные образы, сочетая ее с джинсами, брюками или юбкой. Идеально подойдет для прогулок, встреч с друзьями или работы в офисе.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/rubashki/ukorochennaya-rubashka-s-vorotnikom-stoykoy",
				"images" => array("985_klassicheskaya-rubashka-s-dli.webp"),
				"param" => array("2" => "42", "10" => "120", "8" => array("117", "116", "115", "114"), "11" => "126"),
				"price" => array(array("price" => "3990", "param" => array("8" => "0", "2" => "42"))),
				"hit" => "1",
				"sort" => 13,
				'import_id' => '13',
			),
			array(
				"id" => 14,
				"cat_id" => array(8, 1),
				"brand_id" => 6,
				"article" => "21051",
				"name" => array("Рубашка в клетку с манжетами на пуговицах", "Plaid shirt with button cuffs"),
				"anons" => array("<p>Эта рубашка в клетку &ndash; отличный выбор для тех, кто любит классический стиль с изюминкой.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Она выполнена из качественной ткани в клетку, что делает ее стильной и универсальной. Манжеты на пуговицах добавляют удобства и позволяют регулировать длину рукавов. Два накладных кармана на груди добавляют практичности, а воротник на стойке выглядит элегантно и стильно. Такая рубашка идеально подойдет для создания повседневных образов, а также для работы или учебы. Сочетайте ее с джинсами или брюками, и вы всегда будете выглядеть модно и уверенно.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/rubashki/rubashka-v-kletku-s-manzhetami-na-pugovitsakh",
				"images" => array("986_klassicheskaya-rubashka-s-dli.webp", "987_klassicheskaya-rubashka-s-dli.webp", "988_klassicheskaya-rubashka-s-dli.webp"),
				"param" => array("2" => "48", "10" => "9", "8" => array("117", "116", "115", "114"), "11" => "126"),
				"price" => array(array("price" => "7290", "param" => array("8" => "0", "2" => "48"))),
				"sort" => 14,
				'import_id' => '14',
			),
			array(
				"id" => 15,
				"cat_id" => array(8, 1),
				"brand_id" => 4,
				"article" => "21149",
				"name" => array("Спортивная рубашка с карманами", "Sports shirt with pockets"),
				"anons" => array("<p>Эта спортивная рубашка с капюшоном и карманами &ndash; идеальный выбор для активного образа жизни.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Она выполнена из качественного материала, который обеспечивает комфорт и свободу движений. Капюшон на рубашке добавляет защиту от солнца и дождя, а карманы по бокам служат для хранения мелочей. Эта модель подойдет для занятий спортом, прогулок на свежем воздухе и даже для повседневной носки. Она легко сочетается с джинсами, шортами и спортивными брюками, позволяя создавать разнообразные образы.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/rubashki/sportivnaya-rubashka-s-karmanami",
				"images" => array("987_klassicheskaya-rubashka-s-dli.webp", "988_klassicheskaya-rubashka-s-dli.webp", "986_klassicheskaya-rubashka-s-dli.webp"),
				"param" => array("8" => array("118", "117", "116", "115", "114"), "11" => "136", "2" => "1", "10" => "15"),
				"price" => array(array("price" => "4490", "param" => array("8" => "0", "2" => "1"))),
				"hit" => "1",
				"sort" => 15,
				'import_id' => '15',
			),
			array(
				"id" => 16,
				"cat_id" => array(8, 1),
				"brand_id" => 2,
				"article" => "21191",
				"name" => array("Классическая рубашка с длинным рукавом", "Classic Long Sleeve Shirt"),
				"anons" => array("<p>Эта классическая рубашка с длинным рукавом &ndash; идеальный выбор для тех, кто ценит стиль и комфорт.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Она выполнена из высококачественного хлопка, что обеспечивает мягкость и долговечность изделия. Классический воротник и прямой крой делают эту рубашку универсальной, подходящей для любого типа фигуры. Длинные рукава с манжетами на пуговицах позволяют регулировать длину и создавать различные образы с этой рубашкой. Она отлично подойдет для работы, учебы или повседневной носки и станет незаменимой частью вашего гардероба.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/rubashki/klassicheskaya-rubashka-s-dlinnym-rukavom",
				"images" => array("988_klassicheskaya-rubashka-s-dli.webp", "985_klassicheskaya-rubashka-s-dli.webp", "986_klassicheskaya-rubashka-s-dli.webp", "987_klassicheskaya-rubashka-s-dli.webp"),
				"rel" => array(13, 14, 15),
				"param" => array("11" => "125", "8" => array("117", "116", "115", "114"), "2" => "151", "10" => "119"),
				"price" => array(array("price" => "6990", "param" => array("8" => "0", "2" => "151"))),
				"hit" => "1",
				"sort" => 16,
				'import_id' => '16',
			),
			array(
				"id" => 17,
				"cat_id" => array(7, 1),
				"brand_id" => 2,
				"article" => "21041",
				"name" => array("Изящное коктейльное платье с ассиметричным подолом", "Elegant cocktail dress with an asymmetrical hem"),
				"anons" => array("<p>Это изящное коктейльное платье - идеальное решение для создания неповторимого образа на вечеринке, свидании или особом мероприятии.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Выполнено из высококачественных материалов, оно обеспечивает комфорт при носке и сохраняет форму даже после многочисленных стирок. Платье имеет асимметричный подол, который придает образу легкость и оригинальность. Глубокий вырез на груди и открытая спина подчеркивают женственность и сексуальность обладательницы платья. Длинные рукава и длина чуть выше колена делают платье удобным для носки в любое время года. Это коктейльное платье станет незаменимым элементом вашего гардероба и позволит вам выглядеть стильно и элегантно на любом мероприятии.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/platya/izyaschnoe-kokteylnoe-plate-s-assimetrichnym-podol",
				"images" => array("975_elegantnoe-plate-s-tsvetoch.webp", "974_elegantnoe-plate-s-tsvetoch.webp", "973_elegantnoe-plate-s-tsvetoch.webp"),
				"param" => array("2" => array("48", "132"), "8" => array("117", "114"), "10" => "121", "11" => "136"),
				"price" => array(array("price" => "6490", "param" => array("8" => "0", "2" => "48"), "image_rel" => "975_elegantnoe-plate-s-tsvetoch.webp"), array("price" => "6290", "param" => array("8" => "0", "2" => "132"), "image_rel" => "974_elegantnoe-plate-s-tsvetoch.webp")),
				"action" => "1",
				"sort" => 17,
				'import_id' => '17',
			),
			array(
				"id" => 18,
				"cat_id" => array(7, 1),
				"brand_id" => 6,
				"article" => "21068",
				"name" => array("Женственное платье макси с широкими рукавами", "Feminine maxi dress with wide sleeves"),
				"anons" => array("<p>Это женственное платье макси - идеальный выбор для создания элегантного и стильного образа.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Изготовленное из высококачественной ткани, оно обеспечивает комфорт и долговечность, а изысканный дизайн подчеркивает вашу фигуру. Отличительной особенностью этого платья являются широкие рукава, которые придают ему легкость и воздушность. Вы можете выбрать платье с длинным рукавом или коротким, в зависимости от ваших предпочтений и сезона. Это макси-платье идеально подходит для вечерних прогулок, свиданий и особых случаев. Не упустите возможность добавить в свой гардероб это стильное и модное платье!</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/platya/zhenstvennoe-plate-maksi-s-shirokimi-rukavami",
				"images" => array("973_elegantnoe-plate-s-tsvetoch.webp", "975_elegantnoe-plate-s-tsvetoch.webp", "976_elegantnoe-plate-s-tsvetoch.webp"),
				"param" => array("11" => "135", "2" => array("133", "131"), "10" => "15", "8" => array("118", "117", "116", "115", "114")),
				"price" => array(array("price" => "3590", "param" => array("8" => "0", "2" => "133"), "image_rel" => "976_elegantnoe-plate-s-tsvetoch.webp"), array("price" => "3590", "param" => array("8" => "0", "2" => "131"), "image_rel" => "973_elegantnoe-plate-s-tsvetoch.webp")),
				"sort" => 18,
				'import_id' => '18',
			),
			array(
				"id" => 19,
				"cat_id" => array(7, 1),
				"brand_id" => 7,
				"article" => "21181",
				"name" => array("Облегающее платье с вырезом на спине", "Fitted dress with a cutout on the back"),
				"anons" => array("<p>Это облегающее платье - идеальный выбор для тех, кто хочет подчеркнуть свою фигуру.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Благодаря изысканному материалу и уникальному крою, это платье обеспечит вам комфорт и стиль на протяжении всего дня. Главной особенностью этого платья является вырез на спине, который добавляет в образ нотку сексуальности и изысканности. Платье доступно в различных цветах, поэтому вы сможете выбрать тот, который лучше всего подходит вашему тону кожи и предпочтениям. Идеально подходит для вечеринок, свиданий или особых случаев, это облегающее платье с вырезом на спине станет вашим любимым нарядом в гардеробе!</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/platya/oblegayuschee-plate-s-vyrezom-na-spine",
				"images" => array("974_elegantnoe-plate-s-tsvetoch.webp", "975_elegantnoe-plate-s-tsvetoch.webp"),
				"param" => array("8" => array("118", "117", "116", "115", "114"), "11" => "134", "2" => array("132", "42"), "10" => "120"),
				"price" => array(array("price" => "6490", "param" => array("8" => "0", "2" => "132"), "image_rel" => "974_elegantnoe-plate-s-tsvetoch.webp"), array("price" => "5490", "param" => array("8" => "0", "2" => "42"), "image_rel" => "975_elegantnoe-plate-s-tsvetoch.webp")),
				"new" => "1",
				"sort" => 19,
				'import_id' => '19',
			),
			array(
				"id" => 20,
				"cat_id" => array(7, 1),
				"brand_id" => 13,
				"article" => "21161",
				"name" => array("Элегантное платье с цветочным принтом", "Elegant Floral Print Dress"),
				"anons" => array("<p>Это потрясающее элегантное платье - идеальный выбор для создания нежного и женственного образа.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Выполненное из высококачественного материала, оно обеспечивает максимальный комфорт и долговечность. Платье украшено изящным цветочным принтом, который придает ему особый шарм и неповторимость. Соблазнительный V-образный вырез и расклешенная юбка подчеркивают достоинства фигуры, а длинные рукава создают дополнительную утонченность. Это платье идеально подойдет как для повседневной носки, так и для особых случаев, таких как свидания, вечеринки или особые мероприятия. Не упустите возможность приобрести это стильное и элегантное платье, которое станет неотъемлемой частью вашего гардероба!</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/platya/elegantnoe-plate-s-tsvetochnym-printom",
				"images" => array("976_elegantnoe-plate-s-tsvetoch.webp", "973_elegantnoe-plate-s-tsvetoch.webp", "974_elegantnoe-plate-s-tsvetoch.webp", "975_elegantnoe-plate-s-tsvetoch.webp"),
				"rel" => array(19, 18, 17),
				"param" => array("11" => "125", "8" => array("117", "116", "115", "114"), "2" => array("19", "131"), "10" => "119"),
				"price" => array(array("price" => "7990", "param" => array("8" => "0", "2" => "19"), "image_rel" => "976_elegantnoe-plate-s-tsvetoch.webp"), array("price" => "7990", "param" => array("8" => "0", "2" => "131"), "image_rel" => "973_elegantnoe-plate-s-tsvetoch.webp")),
				"hit" => "1",
				"sort" => 20,
				'import_id' => '20',
			),
			array(
				"id" => 21,
				"cat_id" => array(41, 1),
				"brand_id" => 2,
				"article" => "21181",
				"name" => array("Мужские туфли “Бизнесмен”", "Men's shoes “Businessman”"),
				"anons" => array("<p>Мужские кожаные туфли классического дизайна, идеально подходят для деловых встреч и официальных мероприятий.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Имеют слегка закругленный носок и сдержанный дизайн, без лишних деталей. Внутри мягкая подкладка и кожаная стелька для дополнительного комфорта. Устойчивая подошва из полиуретана обеспечивает отличное сцепление и долговечность.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/krosovki-i-kedy/muzhskie-tufli-biznesmen",
				"images" => array("1011_botinki-na-platforme-usto.webp", "1012_botinki-na-platforme-usto.webp"),
				"param" => array("2" => array("42", "132"), "10" => "120", "37" => array("212", "208", "209", "210", "211", "207")),
				"price" => array(array("price" => "1490", "param" => array("37" => "0", "2" => "42"), "image_rel" => "1012_botinki-na-platforme-usto.webp"), array("price" => "1190", "param" => array("37" => "0", "2" => "132"), "image_rel" => "1011_botinki-na-platforme-usto.webp")),
				"new" => "1",
				"sort" => 21,
				'import_id' => '21',
			),
			array(
				"id" => 22,
				"cat_id" => array(41, 1),
				"brand_id" => 4,
				"article" => "24090",
				"name" => array("Лоферы “Классический стиль”", "Classic Style loafers"),
				"anons" => array("<p>Элегантные лоферы из мягкой замши станут отличным дополнением к деловому образу.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Благодаря своей универсальности, они подойдут как для офиса, так и для повседневной носки.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/krosovki-i-kedy/lofery-klassicheskiy-stil",
				"images" => array("1012_botinki-na-platforme-usto.webp", "1010_botinki-na-platforme-usto.webp"),
				"param" => array("2" => array("42", "132"), "10" => "124", "37" => array("212", "211", "210", "209")),
				"price" => array(array("price" => "2590", "param" => array("37" => "0", "2" => "42"), "image_rel" => "1012_botinki-na-platforme-usto.webp"), array("price" => "2590", "param" => array("37" => "0", "2" => "132"), "image_rel" => "1010_botinki-na-platforme-usto.webp")),
				"sort" => 22,
				'import_id' => '22',
			),
			array(
				"id" => 23,
				"cat_id" => array(41, 1),
				"brand_id" => 6,
				"article" => "25633",
				"name" => array("Туристические кроссовки “Горный путь”", "Hiking sneakers “Mountain way”"),
				"anons" => array("<p>Для любителей активного отдыха и туризма, эти туристические кроссовки станут незаменимым помощником.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Прочная подошва с глубоким протектором обеспечивает отличное сцепление на различных поверхностях, а водоотталкивающая мембрана защищает ноги от промокания. Усиленный носок и пятка защищают от ударов и обеспечивают долговечность модели.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/krosovki-i-kedy/turisticheskie-krossovki-gornyy-put",
				"images" => array("1018_begovye-krossovki-lyogkiy-.webp", "1019_begovye-krossovki-lyogkiy-.webp"),
				"param" => array("37" => array("212", "211", "210", "209", "208", "207"), "2" => "42", "11" => "126", "10" => "121"),
				"price" => array(array("price" => "10990", "param" => array("37" => "0", "2" => "42"))),
				"sort" => 23,
				'import_id' => '23',
			),
			array(
				"id" => 24,
				"cat_id" => array(41, 1),
				"brand_id" => 2,
				"article" => "25623",
				"name" => array("Кроссовки для фитнеса “Активная тренировка”", "Sneakers for fitness “Active training”"),
				"anons" => array("<p>Если вы ищете кроссовки для эффективных тренировок, то &ldquo;Активная тренировка&rdquo; - идеальный выбор.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Прочная резиновая подошва обеспечивает отличное сцепление с поверхностью, а мягкий внутренний материал гарантирует комфорт даже при интенсивной тренировке. Стильный дизайн в спортивном стиле подойдет как для мужчин, так и для женщин.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/krosovki-i-kedy/krossovki-dlya-fitnesa-aktivnaya-trenirovka",
				"images" => array("1021_begovye-krossovki-lyogkiy-.webp", "1020_begovye-krossovki-lyogkiy-.webp", "1019_begovye-krossovki-lyogkiy-.webp"),
				"param" => array("11" => "125", "10" => "124", "37" => array("210", "209", "208", "207", "206"), "2" => "11"),
				"price" => array(array("price" => "5900", "param" => array("37" => "0", "2" => "11"))),
				"sort" => 24,
				'import_id' => '24',
			),
			array(
				"id" => 25,
				"cat_id" => array(41, 1),
				"brand_id" => 9,
				"article" => "95285",
				"name" => array("Беговые кроссовки “Лёгкий шаг”", "Running shoes “Easy step”"),
				"anons" => array("<p>Эти беговые кроссовки созданы для комфорта и скорости.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Подошва из амортизирующего материала эффективно снижает ударную нагрузку на суставы, а дышащий материал верха обеспечивает хорошую вентиляцию и комфорт даже во время длительных пробежек. Стильный дизайн и яркие цвета делают эту модель идеальным выбором для любителей бега.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/krosovki-i-kedy/begovye-krossovki-lyogkiy-shag",
				"images" => array("1017_begovye-krossovki-lyogkiy-.webp", "1018_begovye-krossovki-lyogkiy-.webp", "1019_begovye-krossovki-lyogkiy-.webp", "1020_begovye-krossovki-lyogkiy-.webp", "1021_begovye-krossovki-lyogkiy-.webp"),
				"param" => array("10" => "121", "11" => "126", "2" => "133", "37" => array("212", "211", "210", "209", "208")),
				"price" => array(array("price" => "8490", "param" => array("37" => "0", "2" => "133"))),
				"new" => "1",
				"sort" => 25,
				'import_id' => '25',
			),
			array(
				"id" => 26,
				"cat_id" => array(41, 1),
				"brand_id" => 10,
				"article" => "95284",
				"name" => array("Ботинки на платформе “Устойчивый шаг”", "Platform shoes “Steady step”"),
				"anons" => array("<p>Ботинки на массивной платформе - идеальный выбор для холодного времени года.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Прочная кожа и меховая подкладка обеспечат комфорт и защиту от холода. Универсальный дизайн подойдет к любой одежде.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/krosovki-i-kedy/botinki-na-platforme-ustoychivyy-shag",
				"images" => array("1010_botinki-na-platforme-usto.webp", "1011_botinki-na-platforme-usto.webp", "1012_botinki-na-platforme-usto.webp"),
				"rel" => array(23, 22, 24, 21, 25),
				"param" => array("37" => array("211", "210", "209", "208", "207", "206"), "2" => "132", "11" => "125", "10" => "121"),
				"price" => array(array("price" => "6990", "param" => array("37" => "0", "2" => "132"))),
				"new" => "1",
				"sort" => 26,
				'import_id' => '26',
			),
			array(
				"id" => 27,
				"cat_id" => array(9, 1),
				"brand_id" => 11,
				"article" => "25653",
				"name" => array("Джинсы с высокой талией", "High-waisted jeans"),
				"anons" => array("<p>Женские джинсы с высокой талией &ndash; это модный и удобный элемент гардероба, который поможет создать множество стильных образов. Высокая посадка на талии визуально удлиняет ноги и подчеркивает талию, делая фигуру более стройной. Джинсы выполнены из качественного денима, который сохраняет свой цвет и форму даже после множества стирок. Они идеально сочетаются с различной обувью &ndash; от каблуков до кроссовок, и подойдут для создания как повседневных, так и вечерних образов.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Высокая посадка на талии визуально удлиняет ноги и подчеркивает талию, делая фигуру более стройной. Джинсы выполнены из качественного денима, который сохраняет свой цвет и форму даже после множества стирок. Они идеально сочетаются с различной обувью &ndash; от каблуков до кроссовок, и подойдут для создания как повседневных, так и вечерних образов.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/bryuki-i-dzhinsy/dzhinsy-s-vysokoy-taliey",
				"images" => array("1003_dzhinsy-s-vysokoy-taliey.webp"),
				"param" => array("8" => array("116", "114"), "2" => "10", "10" => "163", "11" => "126"),
				"price" => array(array("price" => "7990", "param" => array("8" => "0", "2" => "10"))),
				"sort" => 27,
				'import_id' => '27',
			),
			array(
				"id" => 28,
				"cat_id" => array(9, 1),
				"brand_id" => 2,
				"article" => "25643",
				"name" => array("Брюки карго с множеством карманов", "Cargo pants with multiple pockets"),
				"anons" => array("<p>Мужские брюки карго &ndash; это стильный и функциональный элемент гардероба, который идеально подойдет для активных людей, ценящих комфорт и удобство.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Эти брюки выполнены из прочного материала, который не только долговечен, но и обеспечивает отличную вентиляцию. Множество карманов на брюках позволяют разместить все необходимые мелочи, такие как телефон, ключи и кошелек. Дизайн брюк карго универсален и позволяет сочетать их с различными видами обуви &ndash; от кроссовок до ботинок. Идеальны для повседневной носки, активного отдыха и путешествий.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/bryuki-i-dzhinsy/bryuki-kargo-s-mnozhestvom-karmanov",
				"images" => array("986_klassicheskaya-rubashka-s-dli.webp"),
				"param" => array("11" => "125", "8" => array("117", "116", "115", "114"), "10" => "124", "2" => "10"),
				"price" => array(array("price" => "6990", "param" => array("8" => "0", "2" => "10"))),
				"sort" => 28,
				'import_id' => '28',
			),
			array(
				"id" => 29,
				"cat_id" => array(9, 1),
				"brand_id" => 6,
				"article" => "25613",
				"name" => array("Узкие брюки-слим", "Slim Skinny Trousers"),
				"anons" => array("<p>Мужские узкие брюки-слим &ndash; это стильная и функциональная одежда, которая идеально подойдет для тех, кто следит за модными тенденциями и ценит комфорт.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Эти брюки изготовлены из высококачественного материала, который обеспечивает отличную посадку по фигуре и долговечность. Благодаря своему узкому крою, брюки-слим подчеркивают достоинства фигуры и скрывают недостатки. Они прекрасно сочетаются с различными видами верха, такими как рубашки, свитера или пиджаки, и подходят для создания как деловых, так и повседневных образов.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/bryuki-i-dzhinsy/uzkie-bryuki-slim",
				"images" => array("987_klassicheskaya-rubashka-s-dli.webp", "986_klassicheskaya-rubashka-s-dli.webp", "997_muzhskie-bryuki-chinos.webp"),
				"param" => array("8" => array("118", "117", "116", "115"), "2" => "132", "10" => "119", "11" => "125"),
				"price" => array(array("price" => "9990", "param" => array("8" => "0", "2" => "132"))),
				"sort" => 29,
				'import_id' => '29',
			),
			array(
				"id" => 30,
				"cat_id" => array(9, 1),
				"brand_id" => 2,
				"article" => "25603",
				"name" => array("Мужские брюки-чинос", "Men's Chinos Trousers"),
				"anons" => array("<p>Мужские брюки-чинос &ndash; это универсальный и практичный элемент гардероба, который подойдет как для работы, так и для отдыха.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Эти брюки имеют свободный крой, который обеспечивает комфортную посадку и не сковывает движения. Изготовлены из прочного и износостойкого материала, они прослужат вам долгое время. Благодаря разнообразной цветовой гамме, вы сможете подобрать чиносы под любой образ и настроение. Идеально сочетаются с рубашками, футболками и пиджаками, создавая стильный и элегантный вид.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/bryuki-i-dzhinsy/muzhskie-bryuki-chinos",
				"images" => array("997_muzhskie-bryuki-chinos.webp", "987_klassicheskaya-rubashka-s-dli.webp", "986_klassicheskaya-rubashka-s-dli.webp"),
				"rel" => array(27, 28, 29),
				"param" => array("2" => "132", "8" => array("117", "116", "115"), "10" => "121", "11" => "126"),
				"price" => array(array("price" => "9490", "param" => array("8" => "0", "2" => "132"))),
				"hit" => "1",
				"sort" => 30,
				'import_id' => '30',
			),
			array(
				"id" => 31,
				"cat_id" => array(40, 1),
				"brand_id" => 10,
				"article" => "95281",
				"name" => array("Детский спортивный костюм “Быстрый спринт”", "Children's tracksuit “Fast Sprint”"),
				"anons" => array("<p>Этот детский спортивный костюм идеально подойдет для занятий различными видами спорта.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Он изготовлен из эластичного и быстросохнущего материала, обеспечивающего комфорт и свободу движения. На куртке и брюках имеются удобные карманы, а на рукавах и штанинах - манжеты для дополнительной защиты от ветра. Дизайн костюма выполнен в ярких цветах и украшен логотипом &ldquo;Быстрый спринт&rdquo;, что делает его стильным и привлекательным для маленьких спортсменов.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/detyam/detskiy-sportivnyy-kostyum-bystryy-sprint",
				"images" => array("1006_detskiy-sportivnyy-kostyum-.webp", "1005_detskiy-sportivnyy-kostyum-.webp"),
				"param" => array("11" => "126", "10" => "8", "8" => array("116", "115", "114"), "2" => "2"),
				"price" => array(array("price" => "3990", "param" => array("8" => "0", "2" => "2"))),
				"sort" => 31,
				'import_id' => '31',
			),
			array(
				"id" => 32,
				"cat_id" => array(5, 61),
				"brand_id" => 9,
				"name" => array("Женский парфюм “Elegance”", "Women's perfume “Elegance”"),
				"anons" => array("<p>&ldquo;Elegance&rdquo; - это изысканный и утонченный аромат, который идеально подойдет для уверенных в себе женщин.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Верхние ноты сочного граната и розового перца придают аромату пикантность и свежесть, в то время как ноты сердца жасмина и пиона создают нежный и женственный аккорд. Базовые ноты сандалового дерева и ванили создают теплый и стойкий шлейф, который будет сопровождать вас в течение всего дня.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/krasota-i-zdorove/zhenskiy-parfyum-elegance",
				"images" => array("1097_parfyumernyy-nabor-romantic-duo.webp", "1096_parfyumernyy-nabor-romantic-duo.webp", "1098_parfyumernyy-nabor-romantic-duo.webp"),
				"param" => array("25" => "169", "23" => "183"),
				"price" => array(array("price" => "8990")),
				"new" => "1",
				"sort" => 32,
				'import_id' => '32',
			),
			array(
				"id" => 33,
				"cat_id" => array(5, 61),
				"brand_id" => 6,
				"article" => "95147",
				"name" => array("Парфюмерный набор “Romantic Duo”", "Perfume set “Romantic Duo”"),
				"anons" => array("<p>Набор &ldquo;Romantic Duo&rdquo; - идеальный подарок для тех, кто хочет подчеркнуть свою индивидуальность и выразить чувства.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>В набор входят два аромата: мужской парфюм &ldquo;Hero&rdquo; и женский парфюм &ldquo;Elegance&rdquo;.</p><p>&ldquo;Hero&rdquo; - мужественный и энергичный парфюм, который подчеркивает силу характера и индивидуальность. Верхние ноты бергамота и зеленого яблока создают свежесть и легкость, средние ноты кедра и пачули добавляют глубину и чувственность, а базовые ноты амбры и мускуса оставляют теплый и притягательный шлейф.</p><p>&ldquo;Elegance&rdquo; - изысканный и тонкий аромат для уверенных и стильных женщин. Пикантные верхние ноты сочного граната и розового перца дополняются нежным сердцем жасмина и пиона, создавая женственный аккорд, а теплые базовые ноты сандалового дерева и ванили оставляют стойкий и притягательный след.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/krasota-i-zdorove/parfyumernyy-nabor-romantic-duo",
				"images" => array("1098_parfyumernyy-nabor-romantic-duo.webp", "1096_parfyumernyy-nabor-romantic-duo.webp", "1097_parfyumernyy-nabor-romantic-duo.webp"),
				"rel" => array(32, 12),
				"param" => array("25" => "171", "23" => "182"),
				"price" => array(array("price" => "4590")),
				"sort" => 33,
				'import_id' => '33',
			),
			array(
				"id" => 34,
				"cat_id" => array(5, 59),
				"brand_id" => 7,
				"article" => "95141",
				"name" => array("Увлажняющий крем для тела “Body Cream”", "Body moisturizer “Body Cream”"),
				"anons" => array("<p>Увлажняющий крем для тела &ldquo;Body Cream&rdquo; от известного бренда - это идеальное средство для ежедневного ухода за вашей кожей.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Крем содержит натуральные масла и экстракты, которые питают и увлажняют кожу, придавая ей мягкость и гладкость. Благодаря легкой текстуре, крем быстро впитывается и не оставляет жирной пленки на коже. &ldquo;Body Cream&rdquo; подходит для всех типов кожи и может использоваться как утром, так и вечером. Регулярное применение крема поможет сохранить кожу здоровой и красивой.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/krasota-i-zdorove/uvlazhnyayuschiy-krem-dlya-tela-body-cream",
				"images" => array("1089_krem-dlya-litsa-beauty-cream.webp", "1087_krem-dlya-litsa-beauty-cream.webp", "1088_krem-dlya-litsa-beauty-cream.webp"),
				"param" => array("24" => "187", "23" => "183", "22" => array("180", "178", "177"), "25" => array("169", "167")),
				"price" => array(array("price" => "2790")),
				"new" => "1",
				"sort" => 34,
				'import_id' => '34',
			),
			array(
				"id" => 35,
				"cat_id" => array(5, 59),
				"brand_id" => 9,
				"article" => "95140",
				"name" => array("Питательный крем для тела “Nourishing Body Cream”", "Nourishing body cream “Nourishing Body Cream”"),
				"anons" => array("<p>Питательный крем для тела &ldquo;Nourishing Body Cream&rdquo; предназначен для глубокого увлажнения и питания кожи.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Он содержит комплекс натуральных масел и экстрактов, которые смягчают и успокаивают кожу, делая её мягкой и шелковистой. Крем легко впитывается, не оставляя жирной плёнки на коже, и подходит для использования в любое время суток.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/krasota-i-zdorove/pitatelnyy-krem-dlya-tela-nourishing-body-cream",
				"images" => array("1087_krem-dlya-litsa-beauty-cream.webp", "1088_krem-dlya-litsa-beauty-cream.webp", "1089_krem-dlya-litsa-beauty-cream.webp"),
				"param" => array("24" => "185", "23" => "184", "22" => "179", "25" => "172"),
				"price" => array(array("price" => "2290")),
				"sort" => 35,
				'import_id' => '35',
			),
			array(
				"id" => 36,
				"cat_id" => array(5, 58),
				"brand_id" => 13,
				"article" => "13643",
				"name" => array("Крем для лица “Beauty Cream”", "Face cream “Beauty Cream”"),
				"anons" => array("<p>Увлажняющий крем для лица &ldquo;Beauty Cream&rdquo; - идеальное средство для ухода за вашей кожей.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Он содержит натуральные ингредиенты, которые питают и увлажняют кожу, делая ее мягкой и гладкой. Крем подходит для всех типов кожи, включая чувствительную. Регулярное использование крема поможет сохранить молодость и красоту вашей кожи.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/krasota-i-zdorove/krem-dlya-litsa-beauty-cream",
				"images" => array("1088_krem-dlya-litsa-beauty-cream.webp", "1087_krem-dlya-litsa-beauty-cream.webp", "1089_krem-dlya-litsa-beauty-cream.webp"),
				"param" => array("24" => "188", "23" => "183", "22" => array("178", "177"), "25" => "171"),
				"price" => array(array("price" => "1590")),
				"hit" => "1",
				"sort" => 36,
				'import_id' => '36',
			),
			array(
				"id" => 38,
				"cat_id" => array(5, 60, 58),
				"brand_id" => 9,
				"article" => "13573",
				"name" => array("Набор косметики “Glam Look”", "A set of cosmetics “Glam Look”"),
				"anons" => array("<p>Подарите себе возможность выглядеть безупречно с набором косметики &ldquo;Glam Look&rdquo;!</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>В этом наборе вы найдете всё необходимое для создания яркого и выразительного макияжа. В состав набора входят:</p><ul><li>Тушь для ресниц &ldquo;Black Star&rdquo;, которая придаст вашему взгляду глубину и объем.</li><li>Карандаш для глаз &ldquo;Smokey Eyes&rdquo;, который поможет создать дымчатый эффект и подчеркнуть красоту ваших глаз.</li><li>Тени для век &ldquo;Soft Shades&rdquo;, которые обеспечат мягкое и легкое нанесение, а также позволят создать разнообразные образы.</li><li>Румяна &ldquo;Glowing Skin&rdquo;, которые придадут лицу свежий и здоровый вид.</li><li>Хайлайтер &ldquo;Highlighter Pearl&rdquo;, который придаст вашему лицу сияние и подчеркнет красоту кожи.</li><li>Помада для губ &ldquo;Matte Lipstick&rdquo;, которая обеспечит матовый финиш и стойкость цвета.<br>Набор косметики &ldquo;Glam Look&rdquo; станет отличным подарком для любой женщины, которая ценит качество и разнообразие в своей косметичке.</li></ul>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/krasota-i-zdorove/nabor-kosmetiki-glam-look",
				"images" => array("1085_nabor-kosmetiki-glam-look.webp", "1084_krasnaya-gubnaya-pomada-velvet.webp"),
				"param" => array("25" => array("168", "167"), "23" => "183", "22" => array("180", "179", "178", "177"), "24" => "185"),
				"price" => array(array("price" => "8390")),
				"new" => "1",
				"sort" => 38,
				'import_id' => '38',
			),
			array(
				"id" => 39,
				"cat_id" => array(5, 60, 58),
				"brand_id" => 6,
				"article" => "13452",
				"name" => array("Красная губная помада “Velvet Passion”", "Red lipstick “Velvet Passion”"),
				"anons" => array("<p>Покорите всех своим ярким образом с красной губной помадой &ldquo;Velvet Passion&rdquo;!</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Этот роскошный оттенок подойдет для создания вечернего макияжа или для акцентирования внимания на губах. Помада обладает бархатистой текстурой, которая обеспечивает гладкое и равномерное нанесение. Благодаря стойким пигментам, цвет ваших губ останется насыщенным на протяжении всего дня. Помада содержит витамин Е, который увлажняет и питает губы, делая их мягкими и гладкими.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/krasota-i-zdorove/krasnaya-gubnaya-pomada-velvet-passion",
				"images" => array("1084_krasnaya-gubnaya-pomada-velvet.webp"),
				"rel" => array(38, 36),
				"param" => array("22" => array("180", "179", "178", "177"), "25" => array("169", "168", "167"), "24" => "187", "23" => "182"),
				"price" => array(array("price" => "2490")),
				"hit" => "1",
				"sort" => 39,
				'import_id' => '39',
			),
			array(
				"id" => 40,
				"cat_id" => array(4, 57, 56),
				"brand_id" => 11,
				"article" => "13523",
				"name" => array("Яркий удобный диван “Софи”", "Bright comfortable sofa “Sophie”"),
				"anons" => array("<p>Яркий удобный диван &ldquo;Софи&rdquo; станет прекрасным дополнением для вашего интерьера.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Этот диван не только удобен для сидения, но и идеально подходит для расслабления после долгого дня. Материалы высокого качества и стильный дизайн делают его отличным выбором для любого дома.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/mebel/yarkiy-udobnyy-divan-sofi",
				"images" => array("1080_stilnyy-udobnyy-divan-ri.webp", "1081_stilnyy-udobnyy-divan-ri.webp"),
				"param" => array("20" => "165", "2" => "42", "26" => "189", "21" => "175"),
				"price" => array(array("price" => "69990", "param" => array("2" => "0"))),
				"new" => "1",
				"sort" => 40,
				'import_id' => '40',
			),
			array(
				"id" => 41,
				"cat_id" => array(4, 57, 56),
				"brand_id" => 13,
				"article" => "95128",
				"name" => array("Стильный удобный диван “Рио”", "Stylish comfortable sofa “Rio”"),
				"anons" => array("<p>Стильный удобный диван &ldquo;Рио&rdquo; - идеальный выбор для вашего дома!</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Этот диван не только красив, но и очень удобен. Он изготовлен из качественных материалов, которые обеспечивают долговечность и прочность. Кроме того, диван имеет стильный дизайн, который подойдет для любого интерьера.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/mebel/stilnyy-udobnyy-divan-rio",
				"images" => array("1080_stilnyy-udobnyy-divan-ri.webp", "1081_stilnyy-udobnyy-divan-ri.webp"),
				"param" => array("2" => array("131", "42"), "20" => "166", "26" => "191", "21" => "36"),
				"price" => array(array("price" => "159990", "param" => array("2" => "131"), "image_rel" => "1080_stilnyy-udobnyy-divan-ri.webp"), array("price" => "129990", "param" => array("2" => "42"), "image_rel" => "1081_stilnyy-udobnyy-divan-ri.webp")),
				"new" => "1",
				"sort" => 41,
				'import_id' => '41',
			),
			array(
				"id" => 42,
				"cat_id" => array(4, 57, 56),
				"brand_id" => 12,
				"article" => "95148",
				"name" => array("Кресло “Уют”", "The Comfort chair"),
				"anons" => array("<p>Кресло &ldquo;Уют&rdquo; - это идеальное сочетание комфорта и стиля. Изготовленное из высококачественных материалов, оно обеспечит Вам максимальный комфорт во время отдыха или работы. Кресло оснащено механизмом качания, который позволит Вам расслабиться после долгого дня. Благодаря своему универсальному дизайну, кресло &ldquo;Уют&rdquo; прекрасно впишется в любой интерьер. Сделайте свой дом еще уютнее с креслом &ldquo;Уют&rdquo;!</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Изготовленное из высококачественных материалов, оно обеспечит Вам максимальный комфорт во время отдыха или работы. Кресло оснащено механизмом качания, который позволит Вам расслабиться после долгого дня. Благодаря своему универсальному дизайну, кресло &ldquo;Уют&rdquo; прекрасно впишется в любой интерьер. Сделайте свой дом еще уютнее с креслом &ldquo;Уют&rdquo;!</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/mebel/kreslo-uyut",
				"images" => array("1079_kreslo-uyut.webp"),
				"param" => array("26" => "189", "21" => "41", "2" => "1", "20" => "23"),
				"price" => array(array("price" => "9890", "param" => array("2" => "0"))),
				"action" => "1",
				"sort" => 42,
				'import_id' => '42',
			),
			array(
				"id" => 43,
				"cat_id" => array(4, 55),
				"brand_id" => 11,
				"article" => "12453",
				"name" => array("Кровать “Комфорт”", "Comfort bed"),
				"anons" => array("<p>Кровать &ldquo;Комфорт&rdquo; - идеальное решение для тех, кто ценит уют и комфорт.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Изготовленная из качественных материалов, она обеспечит вам крепкий и здоровый сон. Кровать имеет удобное изголовье, на котором можно отдохнуть после трудного дня. Стильный дизайн кровати подойдет для любого интерьера. Выбрав кровать &ldquo;Комфорт&rdquo;, вы сделаете правильный выбор!</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/mebel/krovat-komfort",
				"images" => array("1078_krovat-komfort.webp"),
				"param" => array("20" => "165", "2" => "42", "21" => "41", "26" => "191"),
				"price" => array(array("price" => "65890", "param" => array("2" => "0"))),
				"new" => "1",
				"sort" => 43,
				'import_id' => '43',
			),
			array(
				"id" => 44,
				"cat_id" => array(4, 55),
				"brand_id" => 12,
				"article" => "95311",
				"name" => array("Парящая кровать &quot;Невесомость&quot;", "Floating bed &quot;Weightlessness&quot;"),
				"anons" => array("<p>Парящая кровать &ldquo;невесомость&rdquo; - это уникальное решение для вашего интерьера.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Кровать будто парит в воздухе, создавая ощущение невесомости и свободы. Благодаря специальной технологии, кровать выглядит легкой и изящной, но при этом она очень прочная и надежная.</p><p>Кровать изготовлена из высококачественных материалов и имеет ортопедическое основание для комфортного сна. Также она оборудована удобным механизмом подъема, который позволяет легко получить доступ к вместительному ящику для хранения постельного белья и других вещей.</p><p>Дизайн кровати &ldquo;невесомость&rdquo; подойдет для любого стиля интерьера, от современного до классического. Она станет центром внимания в вашей спальне и подарит вам незабываемые сны.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/mebel/paryaschaya-krovat-nevesomost",
				"images" => array("1077_paryaschaya-krovat-nevesomos.webp"),
				"param" => array("21" => "36", "2" => "133", "20" => "24", "26" => "191"),
				"price" => array(array("price" => "35990", "param" => array("2" => "0"))),
				"action" => "1",
				"sort" => 44,
				'import_id' => '44',
			),
			array(
				"id" => 45,
				"cat_id" => array(4, 54),
				"brand_id" => 1,
				"article" => "11203",
				"name" => array("Прикроватная тумбочка “Комфорт”", "Comfort bedside table"),
				"anons" => array("<p>Прикроватная тумбочка &ldquo;Комфорт&rdquo; станет незаменимым элементом вашего интерьера.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Она сочетает в себе стильный дизайн, функциональность и удобство использования. Тумбочка имеет два вместительных ящика для хранения личных вещей и аксессуаров, а также удобную полочку для книг или журналов. Корпус тумбочки выполнен из качественного древесного материала, который обеспечивает прочность и долговечность изделия.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/mebel/prikrovatnaya-tumbochka-komfort",
				"images" => array("1072_komod-elegant.webp"),
				"param" => array("26" => "190", "20" => "166", "2" => "42"),
				"price" => array(array("price" => "4490", "param" => array("2" => "0"))),
				"sort" => 45,
				'import_id' => '45',
			),
			array(
				"id" => 46,
				"cat_id" => array(4, 56, 54),
				"brand_id" => 11,
				"article" => "11223",
				"name" => array("Комод “Уют”", "Chest of drawers “Comfort”"),
				"anons" => array("<p>Комод &ldquo;Уют&rdquo; - это незаменимый элемент мебели для вашего дома.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Он выполнен из качественных материалов, что гарантирует его долговечность и надежность. Комод имеет три больших ящика, которые идеально подойдут для хранения белья, одежды и других вещей. На верхней части комода имеется удобная поверхность, которую можно использовать для размещения различных предметов декора. Стильный дизайн и универсальный цвет комода позволят ему гармонично вписаться в интерьер любой комнаты.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/mebel/komod-uyut",
				"images" => array("1073_komod-elegant.webp", "1071_komod-elegant.webp"),
				"rel" => array(42, 40, 41, 43, 45),
				"param" => array("26" => "191", "20" => "27", "2" => "151"),
				"price" => array(array("price" => "5090", "param" => array("2" => "0"))),
				"action" => "1",
				"sort" => 46,
				'import_id' => '46',
			),
			array(
				"id" => 47,
				"cat_id" => array(50, 3),
				"brand_id" => 1,
				"article" => "11213",
				"name" => array("Лопата с укороченной ручкой “Универсал”", "Shovel with a shortened Universal handle"),
				"anons" => array("<p>Лопата &ldquo;Универсал&rdquo; с укороченной ручкой предназначена для выполнения различных задач на садовом участке или во время ремонта.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Она имеет прочную стальную рабочую часть с тефлоновым покрытием, которое предотвращает налипание грязи и облегчает очистку инструмента. Укороченная ручка позволяет работать в труднодоступных местах и снижает нагрузку на спину. Эргономичная форма ручки обеспечивает удобный хват и комфорт во время работы.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/khozyaystvennye-tovary/lopata-s-ukorochennoy-ruchkoy-universal",
				"images" => array("1070_lopata-s-ukorochennoy-ruchko.webp"),
				"param" => array("2" => array("132", "48", "42")),
				"price" => array(array("price" => "1390", "param" => array("2" => "0"))),
				"new" => "1",
				"sort" => 47,
				'import_id' => '47',
			),
			array(
				"id" => 48,
				"cat_id" => array(50, 3),
				"brand_id" => 3,
				"article" => "14157",
				"name" => array("Садовая тачка “Помощник”", "Garden wheelbarrow “Helper”"),
				"anons" => array("<p>Садовая тачка &ldquo;Помощник&rdquo; - незаменимый инструмент для работы в саду и на приусадебном участке.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>С ее помощью вы сможете перевозить землю, удобрения, мусор и другие материалы. Тачка оснащена двумя колесами для большей устойчивости и легкости перемещения грузов. Прочная стальная рама и вместительное корыто из пластика обеспечивают долговечность и надежность конструкции. Тачка имеет эргономичные ручки для удобного хвата и маневренности.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/khozyaystvennye-tovary/sadovaya-tachka-pomoschnik",
				"images" => array("1069_sadovaya-tachka-pomoschnik.webp"),
				"param" => array("2" => "1"),
				"price" => array(array("price" => "3200", "param" => array("2" => "0"))),
				"hit" => "1",
				"sort" => 48,
				'import_id' => '48',
			),
			array(
				"id" => 49,
				"cat_id" => array(49, 3),
				"brand_id" => 8,
				"article" => "14383",
				"name" => array("Картина “Городской пейзаж”", "The painting “Urban Landscape”"),
				"anons" => array("<p>На картине &ldquo;Городской пейзаж&rdquo; изображена захватывающая дух панорама современного города.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Яркие огни небоскребов, потоки машин и людей создают ощущение динамики и энергии городской жизни.</p><p>Картина выполнена на высококачественном холсте с использованием современных материалов, что обеспечивает ее прочность и долговечность. Она станет идеальным дополнением к интерьеру вашей гостиной, спальни или рабочего кабинета.</p><p>Размер картины &ldquo;Городской пейзаж&rdquo; составляет 60х40 см, что позволяет разместить ее на стене без особых проблем. Благодаря своему стильному и яркому дизайну, она станет настоящим украшением вашего дома и подчеркнет ваш вкус.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/dekor-i-interer/kartina-gorodskoy-peyzazh",
				"images" => array("1062_kartina-gorodskoy-peyzazh.webp", "1063_kartina-gorodskoy-peyzazh.webp"),
				"param" => array("19" => "161", "20" => "165"),
				"price" => array(array("price" => "12990", "param" => array("2" => "0"))),
				"action" => "1",
				"sort" => 49,
				'import_id' => '49',
			),
			array(
				"id" => 50,
				"cat_id" => array(49, 3),
				"brand_id" => 3,
				"article" => "14177",
				"name" => array("“Девушка среди ветвей”", "“The girl among the branches”"),
				"anons" => array("<p>На картине изображена юная девушка, сидящая среди ветвей большого дерева.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Её лицо выражает спокойствие и безмятежность, а взгляд устремлен куда-то вдаль. Ветви дерева создают красивый узор вокруг нее, словно защищая и обнимая ее.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/dekor-i-interer/devushka-sredi-vetvey",
				"images" => array("1066_devushka-sredi-vetvey.webp"),
				"param" => array("19" => "161", "20" => "164"),
				"price" => array(array("price" => "6900", "param" => array("2" => "0"))),
				"no_buy" => "1",
				"sort" => 50,
				'import_id' => '50',
			),
			array(
				"id" => 51,
				"cat_id" => array(49, 3),
				"brand_id" => 11,
				"article" => "31080",
				"name" => array("Искусственная трава “Зеленый рай”", "Artificial grass “Green Paradise”"),
				"anons" => array("<p>Искусственная трава &ldquo;Зеленый рай&rdquo; - идеальное решение для тех, кто хочет создать уютный уголок природы в своем доме или офисе.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Она прекрасно имитирует настоящую траву, передавая все детали и оттенки зелени.</p><p>Искусственная трава &ldquo;Зеленый рай&rdquo; изготовлена из высококачественных материалов, что обеспечивает ее долговечность и стойкость к износу. Она не требует специального ухода и не боится влаги, поэтому ее можно использовать как внутри, так и снаружи помещений.</p><p>Вы можете выбрать искусственную траву &ldquo;Зеленый рай&rdquo; разных размеров и форм, чтобы создать свой уникальный уголок природы. Она легко устанавливается и не занимает много места, что делает ее идеальным решением для небольших помещений.</p><p>С искусственной травой &ldquo;Зеленый рай&rdquo; вы сможете наслаждаться красотой природы, не заботясь о ее уходе и сохранении. Создайте свой собственный зеленый рай в вашем доме с помощью нашей искусственной травы!</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/dekor-i-interer/iskusstvennaya-trava-zelenyy-ray",
				"images" => array("1058_zelenyy-oazis.webp"),
				"param" => array("2" => "3", "20" => "25", "19" => "161"),
				"price" => array(array("price" => "6990", "param" => array("2" => "3"))),
				"action" => "1",
				"sort" => 51,
				'import_id' => '51',
			),
			array(
				"id" => 52,
				"cat_id" => array(49, 3),
				"brand_id" => 3,
				"article" => "95126",
				"name" => array("“Абстрактный экспрессионизм”", "“Abstract Expressionism”"),
				"anons" => array("<p>Эта картина станет ярким акцентом в вашем интерьере благодаря абстрактному экспрессионистскому стилю.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Она выполнена в ярких цветах и имеет размеры 60x80 см, что позволяет использовать ее как в больших, так и в маленьких помещениях.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/dekor-i-interer/abstraktnyy-ekspressionizm",
				"images" => array("1063_kartina-gorodskoy-peyzazh.webp", "1062_kartina-gorodskoy-peyzazh.webp"),
				"param" => array("20" => "166", "19" => "161"),
				"price" => array(array("price" => "14100", "param" => array("2" => "0"))),
				"new" => "1",
				"sort" => 52,
				'import_id' => '52',
			),
			array(
				"id" => 53,
				"cat_id" => array(4, 54),
				"brand_id" => 6,
				"article" => "95133",
				"name" => array("Комод “Элегант”", "Chest of drawers “Elegant”"),
				"anons" => array("<p>Комод &ldquo;Элегант&rdquo; - это стильный и функциональный предмет мебели, который станет украшением вашего интерьера.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Он имеет три ящика для хранения одежды и аксессуаров, а также широкую столешницу, на которой можно разместить зеркало или вазу с цветами. Корпус комода выполнен из высококачественного древесного материала, а ручки на ящиках придают ему особый шарм. Комод &ldquo;Элегант&rdquo; станет отличным выбором для тех, кто ценит комфорт и элегантность в своем доме.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/mebel/komod-elegant",
				"images" => array("1071_komod-elegant.webp", "1072_komod-elegant.webp", "1073_komod-elegant.webp"),
				"param" => array("26" => "189", "20" => "24", "2" => "48"),
				"price" => array(array("price" => "6890", "param" => array("2" => "0"))),
				"sort" => 53,
				'import_id' => '53',
			),
			array(
				"id" => 54,
				"cat_id" => array(14, 32, 2),
				"brand_id" => 5,
				"article" => "27104",
				"name" => array("AuraPhone", "AuraPhone"),
				"anons" => array("<p>AuraPhone - это стильный и мощный смартфон, который станет незаменимым помощником в повседневной жизни.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Он обладает ярким и четким дисплеем, быстрым процессором и достаточным объемом памяти для хранения всех ваших данных. С AuraPhone вы сможете наслаждаться качественными фотографиями, быстрой работой приложений и долгой работой без подзарядки.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/smartfony/auraphone",
				"images" => array("904_galaxytouch.webp", "902_galaxytouch.webp", "903_galaxytouch.webp", "905_galaxytouch.webp"),
				"rel" => array(96, 94, 95),
				"param" => array("5" => "93", "4" => "4", "3" => "92", "1" => "84"),
				"price" => array(array("price" => "7999")),
				"hit" => "1",
				"sort" => 54,
				'import_id' => '54',
			),
			array(
				"id" => 55,
				"cat_id" => array(47, 51, 3),
				"brand_id" => 12,
				"article" => "95214",
				"name" => array("Набор посуды из нержавеющей стали для варки и приготовления пищи", "Stainless Steel Cookware Set for cooking and cooking"),
				"anons" => array("<p>В набор входит все необходимое для приготовления разнообразных блюд.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Кастрюли различного объема позволят приготовить как суп, так и кашу. Многослойное дно обеспечивает быстрый и равномерный нагрев. Крышки с отверстиями для пара сохранят тепло и защитят от выкипания. Эргономичные ручки обеспечат удобство использования. Подходит для любых типов плит, в том числе индукционных. Можно мыть в посудомоечной машине.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/posuda/nabor-posudy-iz-nerzhaveyuschey-stali-dlya-varki-i",
				"images" => array("1031_nabor-posudy-dlya-varki-iz-n.webp", "1030_nabor-posudy-dlya-varki-iz-n.webp"),
				"param" => array("20" => "164", "15" => "143", "14" => "138", "12" => "18", "2" => array("11", "132")),
				"price" => array(array("price" => "1990", "param" => array("2" => "132")), array("price" => "1990", "param" => array("2" => "11"))),
				"sort" => 55,
				'import_id' => '55',
			),
			array(
				"id" => 56,
				"cat_id" => array(47, 51, 3),
				"brand_id" => 4,
				"article" => "95217",
				"name" => array("Набор посуды для варки из нержавеющей стали", "A set of stainless steel cooking utensils"),
				"anons" => array("<p>Этот набор посуды изготовлен из высококачественной нержавеющей стали, обеспечивающей прочность, долговечность и простоту ухода.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>В набор входят кастрюля, сотейник и сковорода различного диаметра, что позволяет готовить различные блюда с минимальным количеством посуды. Многослойное дно посуды быстро и равномерно нагревается, обеспечивая сохранение вкуса и аромата продуктов. Крышки из жаропрочного стекла со стальными ручками позволяют контролировать процесс приготовления, а также сохранять тепло после окончания варки. Эргономичные ручки из бакелита предотвращают ожоги и обеспечивают удобный хват. Набор подходит для всех типов плит и может использоваться в духовке.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/posuda/nabor-posudy-dlya-varki-iz-nerzhaveyuschey-stali",
				"images" => array("1030_nabor-posudy-dlya-varki-iz-n.webp", "1031_nabor-posudy-dlya-varki-iz-n.webp"),
				"param" => array("2" => array("133", "132"), "12" => "18", "20" => "166", "15" => "143", "14" => "137"),
				"price" => array(array("price" => "14990", "param" => array("2" => "0"))),
				"new" => "1",
				"sort" => 56,
				'import_id' => '56',
			),
			array(
				"id" => 57,
				"cat_id" => array(53, 52, 3),
				"brand_id" => 12,
				"article" => "33120",
				"name" => array("Разделочная доска с антибактериальным покрытием", "Cutting board with antibacterial coating"),
				"anons" => array("<p>Эта разделочная доска отличается высоким качеством и прочностью.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Она изготовлена из экологически чистых материалов и имеет специальное антибактериальное покрытие, которое предотвращает размножение бактерий и микробов на поверхности доски. Доска имеет удобные ручки по бокам, которые позволяют легко переносить ее и убирать после использования. Благодаря компактным размерам, она не занимает много места на кухне и подходит для разделывания различных продуктов.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/posuda/nozhi-i-razdelochnye-doski/razdelochnaya-doska-s-antibakterialnym-pokrytiem",
				"images" => array("1037_razdelochnaya-doska-iz-bambu.webp", "1036_razdelochnaya-doska-iz-bambu.webp"),
				"param" => array("14" => "139", "12" => "193", "20" => "165", "15" => "143"),
				"price" => array(array("price" => "2580", "param" => array("2" => "0"))),
				"sort" => 57,
				'import_id' => '57',
			),
			array(
				"id" => 58,
				"cat_id" => array(53, 52, 3),
				"brand_id" => 8,
				"article" => "95211",
				"name" => array("Разделочная доска из бамбука с ручкой", "Bamboo cutting board with handle"),
				"anons" => array("<p>Представляем вам прочную и удобную разделочную доску из бамбука.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Бамбук - это экологически чистый и прочный материал, который идеально подходит для изготовления разделочных досок. Наша доска имеет удобную ручку для переноски и оптимальный размер для комфортной работы на кухне. Бамбуковая поверхность доски обладает антибактериальными свойствами и не впитывает запахи, что делает ее идеальным выбором для нарезки различных продуктов. Эта доска станет незаменимым помощником на вашей кухне и поможет сохранить свежесть и качество продуктов.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/posuda/nozhi-i-razdelochnye-doski/razdelochnaya-doska-iz-bambuka-s-ruchkoy",
				"images" => array("1036_razdelochnaya-doska-iz-bambu.webp", "1037_razdelochnaya-doska-iz-bambu.webp"),
				"param" => array("12" => "194", "20" => "24", "15" => "143", "14" => "139"),
				"price" => array(array("price" => "990", "param" => array("2" => "0"))),
				"new" => "1",
				"sort" => 58,
				'import_id' => '58',
			),
			array(
				"id" => 59,
				"cat_id" => 53,
				"brand_id" => 11,
				"article" => "95213",
				"name" => array("Нож с расписанным клинком ручной работы", "A knife with a hand-painted blade"),
				"anons" => array("<p>Мы рады предложить вам уникальный нож с расписанным вручную клинком.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Этот нож не только станет незаменимым инструментом на вашей кухне, но и украсит любую коллекцию ножей благодаря своему неповторимому дизайну. Лезвие ножа изготовлено из высококачественной стали и расписано вручную опытными мастерами, что придает ножу особую индивидуальность и шарм. Рукоять ножа выполнена из дерева и украшена резьбой, что обеспечивает удобный и надежный хват во время использования. Данный нож прекрасно подойдет как для профессиональных поваров, так и для любителей кулинарии, которые ценят качественные и красивые кухонные принадлежности.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/posuda/nozhi-i-razdelochnye-doski/nozh-s-raspisannym-klinkom-ruchnoy-raboty",
				"images" => array("1035_stolovyy-nozh-iz-vysokokache.webp", "1034_stolovyy-nozh-iz-vysokokache.webp"),
				"param" => array("15" => "143", "14" => "140", "20" => "25", "12" => "18"),
				"price" => array(array("price" => "4890")),
				"new" => "1",
				"sort" => 59,
				'import_id' => '59',
			),
			array(
				"id" => 60,
				"cat_id" => array(53, 52, 3),
				"brand_id" => 9,
				"article" => "34043",
				"name" => array("Столовый нож из высококачественной нержавеющей стали", "Table knife made of high-quality stainless steel"),
				"anons" => array("<p>Столовый нож - незаменимый элемент сервировки стола.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Он выполнен из высококачественной нержавеющей стали, что обеспечивает его прочность и долговечность. Острое лезвие позволяет с легкостью нарезать даже самые твердые продукты, а эргономичная ручка обеспечивает комфорт при использовании. Этот столовый нож прекрасно подойдет для сервировки как повседневного, так и праздничного стола.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/posuda/nozhi-i-razdelochnye-doski/stolovyy-nozh-iz-vysokokachestvennoy-nerzhaveyusch",
				"images" => array("1034_stolovyy-nozh-iz-vysokokache.webp", "1035_stolovyy-nozh-iz-vysokokache.webp"),
				"param" => array("20" => "23", "15" => "143", "14" => "138", "12" => "18"),
				"price" => array(array("price" => "1900", "param" => array("2" => "0"))),
				"new" => "1",
				"sort" => 60,
				'import_id' => '60',
			),
			array(
				"id" => 61,
				"cat_id" => array(47, 52, 3),
				"brand_id" => 9,
				"article" => "34033",
				"name" => array("Тарелка обеденная из белого фарфора с рельефным краем", "Dining plate made of white porcelain with a raised edge"),
				"anons" => array("<p>Описание товара: Предлагаем вам приобрести обеденные тарелки из высококачественного белого фарфора.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Рельефный край тарелок придает им особый шарм и изящество. Тарелки идеально подходят для сервировки обеденного стола и станут настоящим украшением вашего дома. Благодаря своей прочности и долговечности, эти тарелки прослужат вам долгие годы, сохраняя свой первоначальный вид.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/posuda/tarelka-obedennaya-iz-belogo-farfora-s-relefnym-kr",
				"images" => array("1029_tarelka-obedennaya-iz-belog.webp"),
				"param" => array("15" => "143", "14" => "140", "12" => "17", "20" => "27", "2" => "151"),
				"price" => array(array("price" => "1590", "param" => array("2" => "151"))),
				"sort" => 61,
				'import_id' => '61',
			),
			array(
				"id" => 62,
				"cat_id" => array(47, 52, 51, 3),
				"brand_id" => 8,
				"article" => "34023",
				"name" => array("Антипригарная сковорода из литого алюминия с каменным покрытием и индукционным дном", "Non-stick cast aluminum frying pan with stone coating and induction bottom"),
				"anons" => array("<p>Сковорода с инновационным каменным покрытием, которое обеспечивает легкую очистку и удобство в использовании.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Изготовлена из литого алюминия, что гарантирует быстрый нагрев и равномерное распределение тепла по всей поверхности.</p><p>Особенности сковороды:</p><ol><li>Каменное покрытие - обеспечивает антипригарные свойства, предотвращает прилипание и пригорание пищи.</li><li>Индукционное дно - подходит для использования на любых видах плит, включая индукционные плиты.</li><li>Эргономичная ручка - изготовлена из бакелита, не нагревается и обеспечивает безопасный хват.</li><li>Совместимость с духовкой - благодаря каменному покрытию сковороду можно использовать для запекания блюд в духовке.</li><li>Можно мыть в посудомоечной машине - облегчает уход за сковородой.</li><li>В комплекте идет крышка из термостойкого стекла с отверстием для выпуска пара.</li><li>Подходит для всех типов плит, включая индукционные, газовые и электрические.</li><li>Размеры: диаметр 24, 26, 28 см.</li></ol>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/posuda/antiprigarnaya-skovoroda-iz-litogo-alyuminiya-s-ka",
				"images" => array("1028_antiprigarnaya-skovoroda-iz.webp"),
				"param" => array("20" => "23", "15" => "143", "14" => "138", "12" => "18", "2" => "48"),
				"price" => array(array("price" => "2190", "param" => array("2" => "48"))),
				"new" => "1",
				"sort" => 62,
				'import_id' => '62',
			),
			array(
				"id" => 63,
				"cat_id" => array(47, 51, 3),
				"brand_id" => 4,
				"article" => "34013",
				"name" => array("Кастрюля из нержавеющей стали с многослойным дном, индукционным покрытием и эргономичными ручками", "Stainless steel pan with multi-layer bottom, induction coating and ergonomic handles"),
				"anons" => array("<p>Представляем вашему вниманию кастрюлю из высококачественной нержавеющей стали, которая станет незаменимым помощником на вашей кухне!</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Эта кастрюля обладает многослойным дном с алюминиевой сердцевиной, что обеспечивает быстрый и равномерный нагрев, а также сохранение тепла на длительное время. Благодаря индукционному покрытию, кастрюля подходит для всех видов плит, включая индукционные.</p><p>Эргономичные ручки с силиконовыми вставками обеспечивают безопасный и удобный хват при использовании, а также предотвращают соскальзывание кастрюли с плиты. Крышка из жаропрочного стекла с отверстием для выхода пара позволяет контролировать процесс приготовления и предотвращает выкипание.</p><p>В этой кастрюле вы сможете готовить самые разнообразные блюда - от супов и тушеных блюд до каш и гарниров. Внутренняя поверхность кастрюли легко моется, а благодаря компактному размеру она не займет много места на вашей кухне.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/posuda/kastryulya-iz-nerzhaveyuschey-stali-s-mnogosloynym",
				"images" => array("1027_kastryulya-iz-nerzhaveyuschey-st.webp"),
				"param" => array("20" => "23", "15" => "143", "14" => "137", "12" => "18", "2" => "132"),
				"price" => array(array("price" => "1650", "param" => array("2" => "132"))),
				"sort" => 63,
				'import_id' => '63',
			),
			array(
				"id" => 64,
				"cat_id" => array(49, 3),
				"brand_id" => 6,
				"article" => "95278",
				"name" => array("Элегантный флорариум", "Elegant florarium"),
				"anons" => array("<p>Ищете стильное и неприхотливое растение для своего интерьера? Тогда наш элегантный флорариум &ndash; идеальный выбор!</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Этот компактный стеклянный сосуд сочетает в себе красоту природы и минималистичный дизайн, который легко впишется в любой интерьер.</p><p>Внутри флорариума находится небольшое растение, которое не требует сложного ухода и полива. Оно будет радовать вас своим пышным цветением и зеленью, создавая уютную атмосферу в вашем доме или офисе.</p><p>Элегантный флорариум станет отличным подарком для друзей и близких, которые ценят красоту и гармонию в своем окружении. Выберите свой любимый вид флорариума и наслаждайтесь его изяществом каждый день!</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/dekor-i-interer/elegantnyy-florarium",
				"images" => array("1059_zelenyy-oazis.webp", "1058_zelenyy-oazis.webp"),
				"param" => array("20" => "26", "19" => "160", "2" => "3"),
				"price" => array(array("price" => "4490", "param" => array("2" => "3"))),
				"action" => "1",
				"sort" => 64,
				'import_id' => '64',
			),
			array(
				"id" => 65,
				"cat_id" => array(49, 3),
				"brand_id" => 11,
				"article" => "71161",
				"name" => array("Зеленый оазис", "The Green Oasis"),
				"anons" => array("<p>Создайте уютный уголок природы прямо у себя дома с помощью нашего зеленого растения в горшке &ldquo;Зеленый оазис&rdquo;.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Этот пышный и яркий цветок станет прекрасным дополнением к вашему интерьеру и наполнит помещение свежестью и чистотой.</p><p>Наш &ldquo;Зеленый оазис&rdquo; не только красив, но и прост в уходе. Он не требует частого полива и может расти при различных уровнях освещенности. Благодаря своей компактной форме, он идеально подходит для небольших помещений и может быть размещен на подоконнике или столе.</p><p>Выберите свой любимый вид растения из нашего ассортимента и наслаждайтесь его красотой каждый день!</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/dekor-i-interer/zelenyy-oazis",
				"images" => array("1067_zelenyy-oazis.webp", "1058_zelenyy-oazis.webp", "1059_zelenyy-oazis.webp"),
				"rel" => array(51, 49, 52, 64),
				"param" => array("19" => "158", "20" => "24", "2" => "3"),
				"price" => array(array("price" => "3290", "param" => array("2" => "3"))),
				"sort" => 65,
				'import_id' => '65',
			),
			array(
				"id" => 66,
				"cat_id" => array(48, 3),
				"brand_id" => 2,
				"article" => "95164",
				"name" => array("Мягкие полотенца “Комфорт”", "Soft towels “Comfort”"),
				"anons" => array("<p>Мягкие и пушистые полотенца &ldquo;Комфорт&rdquo; - идеальный выбор для вашей ванной комнаты.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Изготовленные из 100% хлопка, они обладают высокой впитывающей способностью и быстро высушивают тело после душа или ванны.</p><p>Разнообразие размеров и цветов позволит подобрать полотенца &ldquo;Комфорт&rdquo;, которые идеально впишутся в ваш интерьер. Вы можете выбрать однотонные полотенца или с яркими узорами, чтобы создать настроение в ванной комнате.</p><p>Полотенца &ldquo;Комфорт&rdquo; приятны на ощупь и не вызывают раздражения кожи. Они отлично стираются и долго сохраняют свою форму и цвет, обеспечивая вам комфорт и уют каждый день.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/tekstil/myagkie-polotentsa-komfort",
				"images" => array("1056_myagkie-polotentsa-komfort.webp", "1057_myagkie-polotentsa-komfort.webp"),
				"param" => array("20" => "165", "16" => "145", "2" => "133"),
				"price" => array(array("price" => "2690", "param" => array("17" => "0", "2" => "0"))),
				"sort" => 66,
				'import_id' => '66',
			),
			array(
				"id" => 67,
				"cat_id" => array(48, 3),
				"brand_id" => 2,
				"article" => "95179",
				"name" => array("Универсальные салфетки “Домашний комфорт”", "Universal napkins “Home comfort”"),
				"anons" => array("<p>Универсальные салфетки &laquo;Домашний комфорт&raquo; - это идеальное решение для уборки вашего дома.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Изготовленные из мягкого и прочного материала, они прекрасно подходят для очистки различных поверхностей, таких как мебель, полы, окна и многое другое.</p><p>Салфетки &laquo;Домашний комфорт&raquo; обладают высокой абсорбирующей способностью, что позволяет быстро и эффективно удалять грязь и пыль. Они легко моются и быстро высыхают, что делает их идеальным выбором для занятых домохозяек.</p><p>Большой выбор размеров и цветов позволяет подобрать салфетки, которые идеально впишутся в интерьер вашего дома. Вы можете выбрать как однотонные модели, так и салфетки с яркими узорами, чтобы добавить немного цвета в вашу уборку.</p><p>Универсальные салфетки &laquo;Домашний комфорт&raquo; не только удобны в использовании, но и экономичны. Они прослужат вам долго и не потребуют замены, что делает их отличным выбором для дома.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/tekstil/universalnye-salfetki-domashniy-komfort",
				"images" => array("1055_universalnye-salfetki-d.webp"),
				"param" => array("20" => "23", "16" => "146", "2" => "133"),
				"price" => array(array("price" => "980", "param" => array("17" => "0", "2" => "0"))),
				"new" => "1",
				"sort" => 67,
				'import_id' => '67',
			),
			array(
				"id" => 68,
				"cat_id" => array(48, 3),
				"brand_id" => 2,
				"article" => "95223",
				"name" => array("Нитяные шторы “Паутинка”", "Gossamer thread curtains"),
				"anons" => array("<p>Нитяные шторы &ldquo;Паутинка&rdquo; - это прекрасное решение для тех, кто хочет добавить в свой интерьер немного легкости и воздушности.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Они идеально подойдут для создания расслабляющей атмосферы в спальне или гостиной, а также могут использоваться для зонирования пространства.</p><p>Нитяные шторы изготавливаются из качественных материалов, таких как полиэстер или вискоза, что обеспечивает их прочность и долговечность. Они не требуют особого ухода и легко стираются в стиральной машине.</p><p>Широкий выбор цветов и размеров позволяет подобрать нитяные шторы &ldquo;Паутинка&rdquo; для любого интерьера. Вы можете выбрать однотонную модель или с узором, а также подобрать шторы, которые будут сочетаться с другими элементами декора в вашей комнате.</p><p>Эти шторы станут отличным дополнением к вашему интерьеру и помогут создать уютную атмосферу в доме.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/tekstil/nityanye-shtory-pautinka",
				"images" => array("1048_shtory-elegant.webp", "1042_shtory-garmoniya-uyuta.webp", "1043_shtory-garmoniya-uyuta.webp"),
				"param" => array("2" => array("133", "48", "38"), "20" => "27", "16" => "150", "17" => array("154", "152")),
				"price" => array(array("price" => "5290", "param" => array("17" => "154", "2" => "0"), "image_rel" => "1043_shtory-garmoniya-uyuta.webp"), array("price" => "3290", "param" => array("17" => "152", "2" => "0"), "image_rel" => "1048_shtory-elegant.webp")),
				"new" => "1",
				"sort" => 68,
				'import_id' => '68',
			),
			array(
				"id" => 69,
				"cat_id" => array(48, 3),
				"brand_id" => 2,
				"article" => "95219",
				"name" => array("Классические шторы “Изящество”", "Classic curtains “Grace”"),
				"anons" => array("<p>Классические шторы &ldquo;Изящество&rdquo; - это идеальный выбор для тех, кто ценит элегантность и комфорт в интерьере своего дома.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Выполненные в традиционном стиле, эти шторы станут прекрасным дополнением к любому дизайну комнаты.</p><p>Благодаря использованию высококачественных материалов, шторы &ldquo;Изящество&rdquo; обладают прочностью и долговечностью. Они также обеспечивают хорошую защиту от света и посторонних взглядов, позволяя создать уютную атмосферу в помещении.</p><p>В нашем ассортименте вы найдете широкий выбор классических штор &ldquo;Изящество&rdquo;, которые отличаются по цвету, размеру и стилю. Мы предлагаем как однотонные варианты, так и шторы с различными узорами и принтами. Это позволит вам выбрать именно те шторы, которые наилучшим образом подойдут для вашего интерьера.</p><p>Классические шторы &ldquo;Изящество&rdquo; просты в уходе и легко сочетаются с другими элементами декора. Они станут отличным выбором для тех, кто хочет обновить свой интерьер, добавив в него нотку элегантности и изысканности.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/tekstil/klassicheskie-shtory-izyaschestvo",
				"images" => array("1044_shtory-garmoniya-uyuta.webp", "1048_shtory-elegant.webp", "1043_shtory-garmoniya-uyuta.webp"),
				"param" => array("2" => array("12", "11", "10", "3"), "20" => "23", "17" => array("153", "152"), "16" => "148"),
				"price" => array(array("price" => "4490", "param" => array("17" => "153", "2" => "0"), "image_rel" => "1044_shtory-garmoniya-uyuta.webp"), array("price" => "3490", "param" => array("17" => "152", "2" => "0"), "image_rel" => "1043_shtory-garmoniya-uyuta.webp")),
				"new" => "1",
				"sort" => 69,
				'import_id' => '69',
			),
			array(
				"id" => 70,
				"cat_id" => array(48, 3),
				"brand_id" => 2,
				"article" => "95218",
				"name" => array("Шторы “Элегант”", "Elegant curtains"),
				"anons" => array("<p>Предлагаем вам стильные и функциональные шторы &ldquo;Элегант&rdquo;, которые станут прекрасным дополнением интерьера вашего дома или офиса.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Выполненные из качественного материала, они обеспечат комфорт и уют в помещении, а также защитят от яркого солнечного света и посторонних глаз. В нашем ассортименте представлены различные модели штор &ldquo;Элегант&rdquo;, отличающиеся дизайном, цветом и размером, поэтому вы без труда сможете подобрать подходящий вариант для вашего окна. С нашей продукцией ваш дом заиграет новыми красками, а гости обязательно оценят ваш вкус и стремление к комфорту.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/tekstil/shtory-elegant",
				"images" => array("1043_shtory-garmoniya-uyuta.webp", "1042_shtory-garmoniya-uyuta.webp", "1044_shtory-garmoniya-uyuta.webp", "1048_shtory-elegant.webp"),
				"param" => array("2" => array("133", "151", "10"), "20" => "26", "16" => "145", "17" => array("154", "152", "153")),
				"price" => array(array("price" => "4490", "param" => array("17" => "0", "2" => "133"), "image_rel" => "1044_shtory-garmoniya-uyuta.webp"), array("price" => "4490", "param" => array("17" => "0", "2" => "151"), "image_rel" => "1048_shtory-elegant.webp"), array("price" => "4490", "param" => array("17" => "0", "2" => "10"), "image_rel" => "1043_shtory-garmoniya-uyuta.webp")),
				"sort" => 70,
				'import_id' => '70',
			),
			array(
				"id" => 71,
				"cat_id" => array(48, 3),
				"brand_id" => 2,
				"article" => "71031",
				"name" => array("Шторы &quot;Гармония уюта&quot;", "Harmony of comfort"),
				"anons" => array("<p>Шторы &ldquo;Гармония уюта&rdquo; - идеальное решение для создания уютной и комфортной атмосферы в вашем доме.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Изготовленные из высококачественного материала, они обеспечивают отличную защиту от солнечных лучей и посторонних глаз, сохраняя при этом естественную вентиляцию помещения.</p><p>Наши шторы представлены в широком ассортименте дизайнов и расцветок, что позволяет подобрать идеальный вариант для любого интерьера. Благодаря разнообразию размеров и типов крепления, вы сможете легко выбрать шторы, которые идеально подойдут для вашего окна.</p><p>&ldquo;Гармония уюта&rdquo; не только стильно украсят ваше жилище, но и помогут создать атмосферу спокойствия и уюта. Сделайте свой дом еще более комфортным и привлекательным с нашими шторами!</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/dom-i-sad/tekstil/garmoniya-uyuta",
				"images" => array("1042_shtory-garmoniya-uyuta.webp", "1043_shtory-garmoniya-uyuta.webp", "1044_shtory-garmoniya-uyuta.webp"),
				"rel" => array(69, 68, 70),
				"param" => array("2" => array("151", "47", "38", "10", "2"), "16" => "146", "20" => "24", "17" => array("154", "153", "152")),
				"price" => array(array("price" => "5590", "param" => array("17" => "154", "2" => "0"), "image_rel" => "1043_shtory-garmoniya-uyuta.webp"), array("price" => "3590", "param" => array("17" => "153", "2" => "0"), "image_rel" => "1044_shtory-garmoniya-uyuta.webp"), array("price" => "2290", "param" => array("17" => "152", "2" => "0"), "image_rel" => "1042_shtory-garmoniya-uyuta.webp")),
				"sort" => 71,
				'import_id' => '71',
			),
			array(
				"id" => 72,
				"cat_id" => array(40, 1),
				"brand_id" => 10,
				"article" => "95161",
				"name" => array("Осенняя куртка для девочки “Принцесса осень”", "Autumn jacket for girls “Princess autumn”"),
				"anons" => array("<p>Яркая и красивая осенняя куртка &ldquo;Принцесса осень&rdquo; станет отличным выбором для вашей маленькой модницы.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Куртка изготовлена из водоотталкивающего материала и утеплена синтетическим наполнителем, что обеспечивает комфорт и тепло в прохладные осенние дни. Капюшон и эластичные манжеты защитят вашу малышку от ветра и непогоды. А декоративные элементы в виде пуговиц и нашивок с изображением осенних листьев сделают куртку еще более привлекательной для маленьких принцесс.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/detyam/osennyaya-kurtka-dlya-devochki-printsessa-osen",
				"images" => array("1004_osennyaya-kurtka-dlya-devochki-.webp"),
				"rel" => array(1, 2, 31),
				"param" => array("2" => "133", "8" => array("117", "116", "115", "114"), "10" => "122", "11" => "136"),
				"price" => array(array("price" => "6900", "param" => array("8" => "0", "2" => "133"))),
				"sort" => 72,
				'import_id' => '72',
			),
			array(
				"id" => 73,
				"cat_id" => array(6, 45, 1),
				"brand_id" => 13,
				"article" => "95177",
				"name" => array("Куртка Cosmos Coat", "Cosmos Coat Jacket"),
				"anons" => array("<p>Куртка Cosmos Coat - это стильная и практичная верхняя одежда, идеально подходящая для переменчивой погоды межсезонья.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Она изготовлена из качественного материала, который обладает водоотталкивающими свойствами и при этом хорошо пропускает воздух, обеспечивая комфорт при ношении.</p><p>Стильный дизайн куртки с воротником-стойкой и удобными карманами на молнии позволяет сочетать ее с различными элементами гардероба, а широкая цветовая гамма позволит выбрать модель, которая подойдет именно вам. Внутри куртки имеется мягкая подкладка из искусственного меха, которая обеспечивает дополнительное тепло и уют.</p><p>Cosmos Coat станет отличным выбором для тех, кто ищет универсальную и стильную верхнюю одежду на каждый день. Она прекрасно сочетается как с джинсами и кроссовками, так и с брюками и ботинками, позволяя создавать разнообразные образы в зависимости от настроения и ситуации.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/verkhnyaya-odezhda/kurtka-cosmos-coat",
				"images" => array("969_pukhovik-s-kapyushonom-galaxy-jacket.webp", "970_pukhovik-s-kapyushonom-galaxy-jacket.webp"),
				"param" => array("11" => "125", "10" => "119", "2" => "42", "8" => array("117", "116", "115")),
				"price" => array(array("price" => "5500", "param" => array("8" => "0", "2" => "0"))),
				"sort" => 73,
				'import_id' => '73',
			),
			array(
				"id" => 74,
				"cat_id" => array(6, 45, 1),
				"brand_id" => 9,
				"article" => "92471",
				"name" => array("Пуховик с капюшоном Galaxy Jacket", "Down jacket with a hood Galaxy Jacket"),
				"anons" => array("<p>Пуховик с капюшоном Galaxy Jacket - это стильный и функциональный элемент верхней одежды, предназначенный для холодных зимних дней.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Он изготовлен из качественных материалов, включая водонепроницаемый полиэстер и утеплитель из пуха, обеспечивающий превосходную теплоизоляцию.</p><p>Одной из особенностей этой модели является удобный капюшон с опушкой из искусственного меха, который надежно защищает голову и шею от холода и ветра. На фронтальной части пуховика расположены два удобных кармана на молнии, в которых можно хранить мелкие предметы или согреть руки в холодную погоду.</p><p>Внутри пуховика есть эластичные манжеты на рукавах, которые предотвращают попадание холодного воздуха под одежду. Кроме того, модель оснащена застежкой-молнией с ветрозащитным клапаном, обеспечивающей надежную защиту от холода и влаги.</p><p>Galaxy Jacket представлена в нескольких цветовых решениях, что позволяет подобрать модель, подходящую под любой вкус и стиль. Этот пуховик станет отличным дополнением зимнего гардероба и поможет сохранить тепло и комфорт даже в самые суровые морозы.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/verkhnyaya-odezhda/pukhovik-s-kapyushonom-galaxy-jacket",
				"images" => array("970_pukhovik-s-kapyushonom-galaxy-jacket.webp", "969_pukhovik-s-kapyushonom-galaxy-jacket.webp"),
				"param" => array("11" => "126", "10" => "9", "2" => "10", "8" => array("118", "117", "116", "115", "114")),
				"price" => array(array("price" => "12990", "param" => array("8" => "0", "2" => "0"))),
				"new" => "1",
				"sort" => 74,
				'import_id' => '74',
			),
			array(
				"id" => 75,
				"cat_id" => array(6, 46, 45, 44, 1),
				"brand_id" => 8,
				"article" => "95102",
				"name" => array("Пуховик Nebula Blazer", "Nebula Blazer Down Jacket"),
				"anons" => array("<p>Пуховик Nebula Blazer - это теплая и стильная модель верхней одежды, которая идеально подходит для холодной зимы.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Изготавливается из качественного полиэстера с наполнителем из пуха и пера, что обеспечивает превосходную теплоизоляцию и легкость.</p><p>Пуховик имеет длину чуть ниже бедер и удобный капюшон с меховой отделкой, который защищает голову от холода и ветра. Застёжка-молния с ветрозащитной планкой обеспечивает надежное закрытие и предотвращает проникновение холодного воздуха.</p><p>Модель оснащена четырьмя карманами: двумя прорезными на груди и двумя накладными боковыми, которые позволяют хранить мелкие предметы, такие как перчатки или ключи. Внутренние манжеты на рукавах предотвращают задувание холодного воздуха и обеспечивают дополнительный комфорт.</p><p>Nebula Blazer представлен в широкой цветовой гамме, что позволяет каждому выбрать модель, подходящую к его стилю и предпочтениям. Этот пуховик станет незаменимым элементом зимнего гардероба и обеспечит комфорт и уют даже в самые холодные дни.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/verkhnyaya-odezhda/pukhovik-nebula-blazer",
				"images" => array("967_pukhovik-nebula-blazer.webp", "968_pukhovik-nebula-blazer.webp"),
				"param" => array("11" => "128", "10" => "120", "2" => "42", "8" => array("118", "117", "116", "115", "114")),
				"price" => array(array("price" => "15990", "param" => array("8" => "0", "2" => "0"))),
				"sort" => 75,
				'import_id' => '75',
			),
			array(
				"id" => 76,
				"cat_id" => array(6, 44, 1),
				"brand_id" => 6,
				"article" => "95092",
				"name" => array("Плащ Stardust Cardigan", "Stardust Cardigan Raincoat"),
				"anons" => array("<p>Плащ Stardust Cardigan - это легкий и удобный вариант верхней одежды, который прекрасно подойдет для теплой весенней или осенней погоды.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Изготавливается из мягкого акрила, который приятен на ощупь и хорошо сохраняет тепло.</p><p>Длина плаща достигает ниже бедер, обеспечивая дополнительную защиту от ветра и холода. Воротник-стойка и манжеты на рукавах выполнены из эластичного трикотажа, что обеспечивает плотное прилегание к телу и сохраняет тепло.</p><p>Два накладных кармана на груди и один внутренний карман на молнии позволяют удобно хранить мелкие предметы и защищают их от потери. Застёжка на пуговицах придает плащу классический вид и обеспечивает надежную фиксацию.</p><p>Благодаря разнообразию цветов, плащ Stardust Cardigan станет отличным выбором как для мужчин, так и для женщин. Он прекрасно сочетается с различными видами одежды и обуви, создавая стильные и удобные образы.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/verkhnyaya-odezhda/plasch-stardust-cardigan",
				"images" => array("964_plasch-stardust-cardigan.webp", "965_plasch-stardust-cardigan.webp", "966_plasch-stardust-cardigan.webp"),
				"param" => array("11" => "126", "10" => "124", "2" => "19", "8" => array("116", "115", "114")),
				"price" => array(array("price" => "11000", "param" => array("2" => "0", "8" => "115")), array("price" => "11500", "param" => array("2" => "0", "8" => "116")), array("price" => "10500", "param" => array("2" => "0", "8" => "114"))),
				"hit" => "1",
				"new" => "1",
				"sort" => 76,
				'import_id' => '76',
			),
			array(
				"id" => 77,
				"cat_id" => array(6, 46, 45, 44, 1),
				"brand_id" => 4,
				"article" => "95082",
				"name" => array("Куртка Moonlight Trenchcoat", "Moonlight Trench coat jacket"),
				"anons" => array("<p>Куртка Moonlight Trenchcoat - это классическая модель верхней одежды, которая уже много лет остается актуальной и модной.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Она изготовлена из высококачественного хлопка с добавлением полиэстера для дополнительной прочности и износостойкости.</p><p>Куртка имеет длину до середины бедра, что обеспечивает комфорт и защиту от непогоды. Широкий отложной воротник и манжеты на рукавах выполнены из мягкого трикотажа, который хорошо сохраняет тепло и приятен на ощупь.</p><p>На куртке имеются два глубоких кармана по бокам и один внутренний карман для хранения мелких предметов. Застежка выполнена в виде ряда пуговиц, которые надежно фиксируют куртку и придают ей классический вид.</p><p>Представлена в нескольких цветовых вариантах, куртка Moonlight Trenchcoat станет отличным выбором для тех, кто ценит стиль и комфорт в одежде. Она идеально подойдет как для деловых встреч, так и для повседневной носки.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/verkhnyaya-odezhda/kurtka-moonlight-trenchcoat",
				"images" => array("960_plasch-solar-flare-parka.webp", "961_plasch-solar-flare-parka.webp"),
				"param" => array("11" => "125", "10" => "121", "2" => "11", "8" => array("118", "117", "116", "115")),
				"price" => array(array("price" => "8890", "param" => array("8" => "0", "2" => "0"))),
				"new" => "1",
				"sort" => 77,
				'import_id' => '77',
			),
			array(
				"id" => 78,
				"cat_id" => array(6, 46, 44, 1),
				"brand_id" => 2,
				"article" => "95072",
				"name" => array("Плащ Solar Flare Parka", "Solar Flare Parka Raincoat"),
				"anons" => array("<p>Плащ Solar Flare Parka - это стильная и функциональная верхняя одежда, которая идеально подойдет для прохладной погоды. Он изготовлен из прочного нейлона с водоотталкивающей пропиткой, что гарантирует защиту от дождя и ветра. Длина плаща достигает колен, что позволяет сохранять тепло и при этом выглядеть стильно.</p>", "<p>This product is a stylish and comfortable wardrobe item that will become an integral part of your image! Made of high-quality materials, it provides maximum comfort and durability.</p>"),
				"text" => array("<p>Капюшон плаща имеет регулируемую затяжку, что позволяет настроить его под индивидуальные особенности головы. Два боковых кармана на молнии обеспечивают удобное хранение мелких предметов, таких как ключи или телефон.</p><p>Застежка плаща выполнена в виде скрытой молнии, что придает ему элегантный вид и удобство в использовании. Подкладка из мягкого полиэстера обеспечивает дополнительный комфорт и тепло.</p><p>Плащ представлен в нескольких размерах и цветовых вариантах, что позволяет выбрать наиболее подходящий вариант для каждого покупателя. Он станет отличным дополнением к гардеробу и поможет создать стильный образ в любую погоду.</p>", "<p>The versatile design allows you to combine it with various elements of clothing, creating unique images. Do not miss the opportunity to purchase this product at a bargain price!<br>A versatile and stylish wardrobe element created from high-quality materials for maximum comfort and durability. Combine it with different looks to create a unique style. Various colors and sizes are available. Have time to buy at a bargain price!</p><p>Made from high-quality materials, these versatile clothes will provide you with comfort and durability, whether you're spending the day at the office or relaxing at a party. Thanks to its unique design, these clothes perfectly match any style, making your image unique. Do not miss your chance to update your wardrobe and buy this wonderful thing at an attractive price!</p><p>Versatile clothing made of high-quality and durable materials, ideal for any look. Comfort and style in one bottle!<br>Available sizes: S, M, L, XL (check the availability of sizes with the manager)<br>Colors: black, white, gray (other colors are available on request)<br>Materials: cotton, polyester (variations of materials are possible on request)</p>"),
				"rewrite" => "shop/odezhda-i-obuv/verkhnyaya-odezhda/plasch-solar-flare-parka",
				"images" => array("961_plasch-solar-flare-parka.webp", "960_plasch-solar-flare-parka.webp"),
				"rel" => array(75, 77, 76),
				"param" => array("8" => array("118", "117", "116", "115", "114"), "2" => "42", "10" => "121", "11" => "127"),
				"price" => array(array("price" => "5990", "param" => array("8" => "0", "2" => "0"))),
				"sort" => 78,
				'import_id' => '78',
			),
			array(
				"id" => 79,
				"cat_id" => array(17, 2),
				"brand_id" => 1,
				"article" => "95052",
				"name" => array("Matrix Game Controller", "Matrix Game Controller"),
				"anons" => array("<p>Matrix Game Controller - это бюджетная игровая консоль, идеально подходящая для начинающих геймеров или тех, кто играет изредка.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Она оснащена процессором Intel Atom и 2 ГБ оперативной памяти, которых достаточно для запуска простых игр на низких настройках графики. Matrix Game Controller имеет 10-дюймовый дисплей и жесткий диск объемом 320 ГБ, а работает под управлением Linux.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/igry-i-konsoli/matrix-game-controller",
				"images" => array("951_neo-console.webp", "949_neo-console.webp"),
				"param" => array("5" => "94", "4" => "5", "3" => "107"),
				"price" => array(array("price" => "15999")),
				"new" => "1",
				"sort" => 79,
				'import_id' => '79',
			),
			array(
				"id" => 80,
				"cat_id" => array(17, 2),
				"brand_id" => 14,
				"article" => "95062",
				"name" => array("Vortex Gaming Station", "Vortex Gaming Station"),
				"anons" => array("<p>Vortex Gaming Station - это мощная игровая консоль, предназначенная для хардкорных геймеров.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Она оснащена новейшим процессором AMD Ryzen Threadripper и видеокартой AMD Radeon RX 590, что позволяет ей легко справляться с любыми играми на максимальных настройках графики. 18-дюймовый дисплей Vortex Gaming Station имеет разрешение 4K, обеспечивая невероятно четкое и яркое изображение. Игровая консоль работает под управлением Ubuntu и имеет стильный дизайн.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/igry-i-konsoli/vortex-gaming-station",
				"images" => array("950_neo-console.webp", "951_neo-console.webp", "947_neo-console.webp"),
				"param" => array("4" => "109", "3" => "108", "5" => "97"),
				"price" => array(array("price" => "36590")),
				"sort" => 80,
				'import_id' => '80',
			),
			array(
				"id" => 81,
				"cat_id" => array(17, 2),
				"brand_id" => 5,
				"article" => "95032",
				"name" => array("Spectra Game Box", "Spectra Game Box"),
				"anons" => array("<p>Spectra Game Box - это игровая консоль для тех, кто хочет играть в игры на ходу.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Она оснащена процессором Intel Core i5 и 8 ГБ оперативной памяти, что позволяет ей запускать большинство современных игр на средних настройках графики. Консоль имеет жесткий диск на 1 ТБ и слот для SD-карт, что позволяет загружать и хранить большое количество игр. 14-дюймовый дисплей с разрешением Full HD обеспечивает качественное изображение во время игры. Операционная система Spectra Game Box - Windows 10.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/igry-i-konsoli/spectra-game-box",
				"images" => array("948_neo-console.webp", "949_neo-console.webp", "950_neo-console.webp"),
				"param" => array("3" => "91", "5" => "95", "4" => "20"),
				"price" => array(array("price" => "19990")),
				"new" => "1",
				"sort" => 81,
				'import_id' => '81',
			),
			array(
				"id" => 82,
				"cat_id" => array(17, 2),
				"brand_id" => 1,
				"article" => "95022",
				"name" => array("Neo Console", "Neo Console"),
				"anons" => array("<p>Neo Console - это новейшая игровая консоль, которая обеспечивает невероятный игровой опыт благодаря своей мощной графике и быстрому процессору.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Она оснащена 8-ядерным процессором AMD Ryzen и графическим адаптером NVIDIA GeForce RTX 2080, что позволяет играть в самые требовательные игры без каких-либо задержек или проблем. Консоль также имеет 16 ГБ оперативной памяти DDR5, что обеспечивает быструю работу операционной системы и игр.</p><p>Neo Console имеет жесткий диск емкостью 2 ТБ, на котором можно хранить большое количество игр и других файлов. Консоль оснащена 55-дюймовым 4K-телевизором с поддержкой технологии HDR, что обеспечивает потрясающее качество изображения во время игры. Neo Console работает под управлением операционной системы Ubuntu и имеет стильный и современный дизайн.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/igry-i-konsoli/neo-console",
				"images" => array("947_neo-console.webp", "948_neo-console.webp", "949_neo-console.webp", "950_neo-console.webp", "951_neo-console.webp"),
				"rel" => array(79, 81, 80),
				"param" => array("5" => "95", "4" => "20", "3" => "88"),
				"price" => array(array("price" => "25990")),
				"sort" => 82,
				'import_id' => '82',
			),
			array(
				"id" => 83,
				"cat_id" => array(16, 2),
				"brand_id" => 14,
				"article" => "15014",
				"name" => array("Spectrum PC", "Spectrum PC"),
				"anons" => array("<p>Spectrum PC - это бюджетный персональный компьютер, который подойдет для начинающих пользователей и тех, кто не нуждается в высокой производительности.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Он оснащен процессором Intel Celeron и 4 ГБ оперативной памяти, чего достаточно для работы с офисными приложениями и просмотра интернет страниц. Для хранения данных компьютер имеет жесткий диск объемом 500 ГБ. Монитор с диагональю 21,5 дюйма и разрешением 1920x1080 обеспечивает достаточно качественное изображение для работы и развлечений. Операционная система - Windows 10 Home.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/kompyutery/spectrum-pc",
				"images" => array("940_matrix-pc.webp", "937_matrix-pc.webp"),
				"param" => array("3" => "107", "4" => "20", "5" => "95"),
				"price" => array(array("price" => "59890")),
				"new" => "1",
				"action" => "1",
				"sort" => 83,
				'import_id' => '83',
			),
			array(
				"id" => 84,
				"cat_id" => array(16, 2),
				"brand_id" => 5,
				"article" => "85508",
				"name" => array("Vortex Computer", "Vortex Computer"),
				"anons" => array("<p>Vortex Computer - это компактный и мощный персональный компьютер, который идеально подойдет для дома и офиса.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Он оснащен процессором Intel Core i7 восьмого поколения и 16 ГБ оперативной памяти, что обеспечивает высокую производительность для любых задач. Компьютер имеет жесткий диск на 1 ТБ и SSD на 256 ГБ для быстрой загрузки операционной системы и приложений. 23,8-дюймовый монитор имеет разрешение Full HD и широкие углы обзора, что обеспечивает отличное качество изображения. Компьютер работает под управлением Windows 10 Home и имеет стильный корпус с подсветкой на передней панели.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/kompyutery/vortex-computer",
				"images" => array("939_matrix-pc.webp", "938_matrix-pc.webp", "940_matrix-pc.webp", "937_matrix-pc.webp"),
				"param" => array("5" => "96", "3" => "108", "4" => "20"),
				"price" => array(array("price" => "85990")),
				"sort" => 84,
				'import_id' => '84',
			),
			array(
				"id" => 85,
				"cat_id" => array(16, 2),
				"brand_id" => 5,
				"article" => "15536",
				"name" => array("Matrix PC", "Matrix PC"),
				"anons" => array("<p>Matrix PC - это высокопроизводительный персональный компьютер, который идеально подойдет для геймеров, дизайнеров и программистов.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Он оснащен новейшим процессором AMD Ryzen Threadripper, который обеспечивает непревзойденную производительность для многопоточных приложений. Matrix PC также имеет 64 ГБ оперативной памяти DDR4 и видеокарту NVIDIA GeForce RTX 2080 Ti, что позволяет запускать самые требовательные игры и программы без каких-либо проблем.</p><p>Компьютер оснащен двумя жесткими дисками: один на 2 ТБ для хранения игр и программ, а другой на 1 ТБ для операционной системы и других приложений. Большой 27-дюймовый монитор с разрешением 4K и поддержкой технологии G-Sync обеспечивает потрясающее качество изображения. Matrix PC работает под управлением операционной системы Windows 10 Pro и имеет стильный дизайн с подсветкой на передней панели.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/kompyutery/matrix-pc",
				"images" => array("937_matrix-pc.webp", "938_matrix-pc.webp", "939_matrix-pc.webp", "940_matrix-pc.webp"),
				"rel" => array(83, 84),
				"param" => array("4" => "5", "3" => "106", "5" => "97"),
				"price" => array(array("price" => "89990")),
				"hit" => "1",
				"sort" => 85,
				'import_id' => '85',
			),
			array(
				"id" => 86,
				"cat_id" => array(15, 2),
				"brand_id" => 14,
				"article" => "76193",
				"name" => array("Sirius PC", "Sirius PC"),
				"anons" => array("<p>Sirius PC - это ноутбук среднего класса, который идеально подходит для офисных задач и интернет-серфинга. Он оснащен процессором Intel Core i5 и 8 ГБ оперативной памяти, что достаточно для большинства повседневных задач.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Диагональ экрана ноутбука составляет 15,6 дюйма, а разрешение - 1920x1080 пикселей. Накопитель ноутбука имеет объем 256 ГБ, чего достаточно для хранения большого количества данных.</p><p>Sirius PC работает под управлением операционной системы Windows 10 и имеет встроенную видеокарту Intel HD Graphics. Батарея ноутбука обладает емкостью 50 Втч и обеспечивает до 6 часов автономной работы.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/noutbuki/sirius-pc",
				"images" => array("932_xenon-computer.webp", "931_xenon-computer.webp", "927_ampera-laptop.webp", "925_ampera-laptop.webp"),
				"param" => array("5" => "95", "4" => "5", "3" => "108"),
				"price" => array(array("price" => "28890")),
				"sort" => 86,
				'import_id' => '86',
			),
			array(
				"id" => 87,
				"cat_id" => array(15, 2),
				"brand_id" => 5,
				"article" => "12070",
				"name" => array("Xenon Computer", "Xenon Computer"),
				"anons" => array("<p>Xenon Computer - это мощный ноутбук, предназначенный для профессиональных задач, таких как редактирование видео, 3D-моделирование и игры.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Он оснащен процессором AMD Ryzen 7 и видеокартой NVIDIA GeForce GTX 1060, что позволяет ему легко справляться с самыми требовательными задачами.</p><p>Экран ноутбука имеет диагональ 17,3 дюйма и разрешение 1920x1080 пикселей, обеспечивая яркое и четкое изображение. Жесткий диск ноутбука имеет объем 1 ТБ, что позволяет хранить огромное количество данных.</p><p>Xenon Computer оснащен 16 ГБ оперативной памяти DDR4, что является оптимальным для большинства современных приложений. Батарея ноутбука имеет емкость 80 Втч, что обеспечивает до 8 часов автономной работы.</p><p>В целом, Xenon Computer является отличным выбором для профессионалов, которые нуждаются в мощном и надежном ноутбуке для своих рабочих задач.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/noutbuki/xenon-computer",
				"images" => array("931_xenon-computer.webp", "932_xenon-computer.webp"),
				"param" => array("5" => "96", "4" => "20", "3" => "107"),
				"price" => array(array("price" => "43890")),
				"sort" => 87,
				'import_id' => '87',
			),
			array(
				"id" => 88,
				"cat_id" => array(15, 2),
				"brand_id" => 1,
				"article" => "91044",
				"name" => array("Orion Notebook", "Orion Notebook"),
				"anons" => array("<p>Orion Notebook - это легкий и компактный ноутбук, который идеально подходит для путешествий и работы в дороге.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Он оснащен процессором Intel Core i3 и 4 ГБ оперативной памяти, что достаточно для выполнения большинства задач, таких как просмотр веб-страниц, работа с документами и просмотр видео.</p><p>11,6-дюймовый экран Orion Notebook имеет разрешение 1366x768 пикселей, что обеспечивает достаточно четкое изображение. Ноутбук оснащен встроенным накопителем на 32 ГБ и слотом для карт памяти, что позволяет хранить большой объем данных на самом ноутбуке.</p><p>Orion Notebook работает под управлением операционной системы Linux, что делает его более экономичным вариантом по сравнению с ноутбуками на Windows. Батарея ноутбука обеспечивает до 4 часов автономной работы, что достаточно для работы в течение всего дня.</p><p>Несмотря на свою компактность, Orion Notebook оснащен всеми необходимыми разъемами для подключения внешних устройств, такими как USB, HDMI и разъем для наушников. Этот ноутбук идеально подходит для тех, кто ищет недорогое и компактное устройство для работы и учебы в дороге.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/noutbuki/orion-notebook",
				"images" => array("927_ampera-laptop.webp", "926_ampera-laptop.webp", "925_ampera-laptop.webp"),
				"param" => array("5" => "97", "4" => "109", "3" => "108"),
				"price" => array(array("price" => "85990")),
				"sort" => 88,
				'import_id' => '88',
			),
			array(
				"id" => 89,
				"cat_id" => array(15, 2),
				"brand_id" => 14,
				"article" => "67854",
				"name" => array("Ampera Laptop", "Ampera Laptop"),
				"anons" => array("<p>Ampera Laptop - это современный ноутбук, который отличается высокой производительностью и стильным дизайном.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Он оснащен новейшим процессором Intel Core i5, который обеспечивает быстрое выполнение задач и плавную работу даже с самыми требовательными приложениями. Ноутбук имеет 8 ГБ оперативной памяти и SSD-накопитель объемом 256 ГБ, что позволяет хранить большое количество данных и быстро загружать систему.</p><p>15,6-дюймовый экран Ampera Laptop имеет разрешение Full HD, благодаря чему изображение выглядит четким и ярким. Антибликовое покрытие экрана обеспечивает комфортное использование ноутбука даже при ярком освещении.</p><p>Ноутбук оснащен встроенной видеокартой Intel HD Graphics, которая позволяет работать с графикой и играть в нетребовательные игры. Устройство также имеет все необходимые разъемы для подключения внешних устройств, такие как HDMI, USB и разъем для наушников.</p><p>Ampera Laptop работает под управлением операционной системы Windows 10, которая предоставляет пользователю широкий спектр возможностей и приложений. Батарея ноутбука обеспечивает до 5 часов автономной работы, что позволяет использовать его в дороге или на деловых встречах.</p><p>Таким образом, Ampera Laptop является идеальным выбором для тех, кто нуждается в надежном и производительном ноутбуке для работы и развлечений.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/noutbuki/ampera-laptop",
				"images" => array("925_ampera-laptop.webp", "926_ampera-laptop.webp", "927_ampera-laptop.webp"),
				"rel" => array(88, 86, 87),
				"param" => array("5" => "97", "4" => "109", "3" => "106"),
				"price" => array(array("price" => "238890")),
				"sort" => 89,
				'import_id' => '89',
			),
			array(
				"id" => 90,
				"cat_id" => array(10, 31, 30, 29, 28, 23, 27, 26, 25, 24, 19, 22, 21, 20, 18, 2),
				"brand_id" => 1,
				"article" => "Kl3566755",
				"name" => array("Телевизор с ИИ Orion OLS-75PU10", "TV with AI Orion OLS-75PU10"),
				"anons" => array("<p>ТВ с ИИ Orion OLS-75PU10 изменит Ваше представление о телевидении и позволит наслаждаться просмотром любимых программ и фильмов с максимальным комфортом и пользой.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Особенности товара:</p><ol><li>Машинное обучение: Наш телевизор с ИИ постоянно обучается и анализирует Ваши предпочтения, чтобы предложить наиболее подходящий для Вас контент. Со временем, он становится все более и более персонализированным.</li><li>Интеграция в Умный Дом: Телевизор легко интегрируется в систему Умный Дом, что позволяет управлять им с помощью голосовых команд или через мобильное приложение.</li><li>Высокое качество изображения: Телевизор оснащен современной матрицей с высоким разрешением и яркостью, что обеспечивает превосходное качество изображения.</li><li>Голосовой ассистент: Телевизор имеет встроенного голосового ассистента, который поможет Вам найти нужный контент, управлять другими устройствами в доме и ответит на Ваши вопросы.</li><li>Доступная цена: Несмотря на то, что наш товар является новинкой на рынке, мы предлагаем его по доступной цене, которая несомненно порадует Вас.</li></ol>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/televizory/televizor-s-iskusstvennym-intellektom",
				"images" => array("889_televizor-ampera-a55sv800.webp", "888_televizor-ampera-a55sv800.webp", "887_televizor-ampera-a55sv800.webp", "886_televizor-ampera-a55sv800.webp"),
				"rel" => array(92, 91),
				"param" => array("33" => "62", "34" => "69", "35" => "71", "36" => "76", "5" => "93", "4" => "5", "3" => "91"),
				"price" => array(array("price" => "79999")),
				"hit" => "1",
				"sort" => 90,
				'import_id' => '90',
			),
			array(
				"id" => 91,
				"cat_id" => array(10, 31, 30, 29, 28, 23, 27, 26, 25, 24, 19, 22, 21, 20, 18, 2),
				"brand_id" => 14,
				"article" => "A4566730",
				"name" => array("Televizor HTY-43HDRO6F-HC22 43", "Televizor HTY-43HDRO6F-HC22 43"),
				"anons" => array("<p>Телевизоры под управлением операционной системы Tizen предлагают пользователям простой и интуитивно понятный интерфейс, позволяющий легко находить и просматривать интересующий контент.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Одной из ключевых особенностей Tizen является возможность синхронизации с другими устройствами, поддерживающими эту операционную систему, что позволяет пользователям легко переключаться между различными источниками сигнала.</p><p>Телевизоры на Tizen поддерживают широкий спектр аудио и видео форматов, что делает их универсальными и подходящими для просмотра различных типов контента.</p><p>Tizen обеспечивает быстрый и стабильный доступ к интернету, позволяя пользователям смотреть онлайн-видео, слушать музыку и пользоваться другими интернет-сервисами прямо на телевизоре.</p><p>Наконец, телевизоры на Tizen имеют низкое энергопотребление, что делает их более экологичными и экономичными по сравнению с моделями на других операционных системах.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/televizory/televizor-hty-43hdro6f-hc22-43",
				"images" => array("888_televizor-ampera-a55sv800.webp", "887_televizor-ampera-a55sv800.webp", "886_televizor-ampera-a55sv800.webp", "889_televizor-ampera-a55sv800.webp"),
				"param" => array("33" => "62", "34" => "69", "35" => "71", "36" => "76"),
				"price" => array(array("price" => "49999")),
				"hit" => "1",
				"sort" => 91,
				'import_id' => '91',
			),
			array(
				"id" => 92,
				"cat_id" => array(10, 31, 30, 29, 28, 23, 27, 26, 25, 24, 19, 22, 21, 20, 18, 2),
				"brand_id" => 1,
				"article" => "R4566711",
				"name" => array("ScreenWave HTY-50FDX100T2 50", "ScreenWave HTY-50FDX100T2 50"),
				"anons" => array("<p>Телевизоры с WebOS также обладают высокой энергоэффективностью и могут работать с различными форматами аудио и видео.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Эта операционная система обеспечивает быстрый доступ к популярным приложениям и сервисам, таким как YouTube, Netflix и другим.</p><p>WebOS поддерживает функции голосового управления и позволяет управлять телевизором с помощью голосовых команд.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/televizory/screenwave-hty-50fdx100t2-50",
				"images" => array("887_televizor-ampera-a55sv800.webp", "886_televizor-ampera-a55sv800.webp", "888_televizor-ampera-a55sv800.webp", "889_televizor-ampera-a55sv800.webp"),
				"param" => array("3" => "88", "4" => "5", "5" => "95", "33" => "64", "34" => "67", "35" => "73", "36" => "74"),
				"price" => array(array("price" => "35999")),
				"hit" => "1",
				"sort" => 92,
				'import_id' => '92',
			),
			array(
				"id" => 93,
				"cat_id" => array(10, 31, 30, 29, 28, 23, 27, 26, 25, 24, 19, 22, 21, 20, 18, 2),
				"brand_id" => 5,
				"article" => "OP3566753",
				"name" => array("Телевизор Ampera A55SV800", "Ampera A55SV800 TV"),
				"anons" => array("<p>Ampera A55SV800 - это отличный выбор для тех, кто ищет качественный и функциональный телевизор. Он оснащен 55-дюймовым экраном, который обеспечивает яркое и четкое изображение с разрешением 4K. Благодаря широким углам обзора, вы сможете наслаждаться просмотром с любой точки комнаты.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Телевизор Ampera A55SV800 имеет встроенный Smart TV, который предоставляет доступ к огромному количеству контента в интернете. Вы сможете смотреть любимые фильмы, сериалы, спортивные трансляции и многое другое прямо на вашем телевизоре.</p><p>Кроме того, Ampera A55SV800 поддерживает Wi-Fi и Bluetooth, что позволяет подключать к нему различные устройства, такие как смартфоны, планшеты и ноутбуки. Это делает его идеальным выбором для домашнего кинотеатра или игровой зоны.</p><p>В целом, Ampera A55SV800 является отличным решением для тех, кто хочет получить максимум от своего телевизора. Он сочетает в себе высокое качество изображения, удобство Smart TV и возможность подключения различных устройств.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/televizory/televizor-ampera-a55sv800",
				"images" => array("886_televizor-ampera-a55sv800.webp", "887_televizor-ampera-a55sv800.webp", "888_televizor-ampera-a55sv800.webp", "889_televizor-ampera-a55sv800.webp"),
				"rel" => array(90, 91, 92),
				"param" => array("5" => "94", "4" => "20", "3" => "106", "33" => "62", "34" => "68", "35" => "72", "36" => "75"),
				"price" => array(array("price" => "25990")),
				"sort" => 93,
				'import_id' => '93',
			),
			array(
				"id" => 94,
				"cat_id" => array(14, 33, 2),
				"brand_id" => 1,
				"article" => "34710",
				"name" => array("CosmosSmart", "CosmosSmart"),
				"anons" => array("<p>CosmosSmart - это уникальный смартфон, который сочетает в себе стильный дизайн, высокую производительность и широкий функционал.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Он оснащен большим 6,7-дюймовым дисплеем с разрешением 1080x2400 пикселей, что обеспечивает яркие и насыщенные цвета. За быстродействие отвечает мощный процессор Qualcomm Snapdragon 855, а 8 ГБ оперативной памяти позволят без проблем работать с несколькими приложениями одновременно.</p><p>Основная камера состоит из трех модулей: 48 МП основного, 16 МП широкоугольного и 8 МП телеобъектива. Она позволяет снимать качественные фотографии и видео в любых условиях освещения. Фронтальная камера на 16 МП подойдет для ярких и детализированных селфи.</p><p>Смартфон работает на базе операционной системы Android 10, что гарантирует стабильную работу и наличие всех необходимых приложений. Встроенный аккумулятор емкостью 4500 мАч обеспечивает длительное время работы без подзарядки, а поддержка быстрой зарядки поможет быстро восстановить запас энергии.</p><p>CosmosSmart также оснащен всеми необходимыми интерфейсами, включая Wi-Fi 802.11ac, Bluetooth 5.0, GPS и NFC. Это делает его отличным выбором для тех, кому нужен универсальный смартфон для выполнения любых задач.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/smartfony/cosmossmart",
				"images" => array("905_galaxytouch.webp", "904_galaxytouch.webp", "903_galaxytouch.webp", "902_galaxytouch.webp"),
				"param" => array("1" => "85", "3" => "91", "4" => "5", "5" => "94"),
				"price" => array(array("price" => "12999")),
				"sort" => 94,
				'import_id' => '94',
			),
			array(
				"id" => 95,
				"cat_id" => array(14, 34, 33, 2),
				"brand_id" => 5,
				"article" => "171049",
				"name" => array("ElectraNote", "ElectraNote"),
				"anons" => array("<p>ElectraNote - это современный и стильный смартфон, который идеально подойдет для тех, кто ценит качество и функциональность.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Он оснащен мощным процессором, который обеспечивает быструю работу приложений и игр. Благодаря большому экрану, вы сможете наслаждаться яркими и четкими изображениями. Основная камера позволяет делать качественные снимки, а фронтальная камера подойдет для селфи и видеозвонков. ElectraNote также имеет встроенный аккумулятор, который обеспечивает длительное время работы без подзарядки. В целом, ElectraNote является отличным выбором для тех, кто ищет надежный и функциональный смартфон.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/smartfony/electranote",
				"images" => array("903_galaxytouch.webp", "902_galaxytouch.webp", "904_galaxytouch.webp", "905_galaxytouch.webp"),
				"param" => array("1" => "86", "3" => "89", "4" => "20", "5" => "95"),
				"price" => array(array("price" => "27999")),
				"sort" => 95,
				'import_id' => '95',
			),
			array(
				"id" => 96,
				"cat_id" => array(14, 34, 2),
				"brand_id" => 14,
				"article" => "s71041d",
				"name" => array("GalaxyTouch", "GalaxyTouch"),
				"anons" => array("<p>GalaxyTouch - это мощный и стильный смартфон, который обладает множеством функций и возможностей.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Он имеет большой 6,7-дюймовый дисплей с высоким разрешением, который обеспечивает яркие и четкие изображения. За быстродействие смартфона отвечает мощный процессор, а оперативная память объемом 8 ГБ позволяет без проблем работать с множеством приложений одновременно.</p><p>GalaxyTouch оснащен тройной основной камерой, которая позволяет снимать высококачественные фотографии и видео. Фронтальная камера предназначена для селфи и видеозвонков, ее разрешение составляет 32 мегапикселя.</p><p>Работает смартфон на базе операционной системы Android, что предоставляет доступ к широкому спектру приложений и функций. Встроенный аккумулятор обеспечивает длительное время автономной работы, а поддержка быстрой зарядки позволяет быстро восстановить заряд батареи.</p><p>Помимо этого, GalaxyTouch оснащен всеми необходимыми интерфейсами для беспроводной связи, включая Wi-Fi, Bluetooth и GPS. Также он имеет модуль NFC для бесконтактных платежей. В целом, GalaxyTouch - это отличный выбор для тех, кто ищет мощный, стильный и функциональный смартфон.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/smartfony/galaxytouch",
				"images" => array("902_galaxytouch.webp", "903_galaxytouch.webp", "904_galaxytouch.webp", "905_galaxytouch.webp"),
				"rel" => array(95, 94),
				"param" => array("1" => "87", "3" => "88", "4" => "20", "5" => "97"),
				"price" => array(array("price" => "120990")),
				"hit" => "1",
				"sort" => 96,
				'import_id' => '96',
			),
			array(
				"id" => 97,
				"cat_id" => array(35, 38, 37, 36),
				"brand_id" => 1,
				"article" => "2395632",
				"name" => array("Чехол для смартфона AuraPhone", "Yotaphone Smartphone Case"),
				"anons" => array("<p>Чехол для смартфона - это необходимый аксессуар для каждого владельца мобильного устройства.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Он обеспечивает защиту от повреждений, таких как царапины, сколы и трещины, а также предотвращает выскальзывание телефона из рук. Чехлы могут быть изготовлены из различных материалов, таких как силикон, пластик, кожа или текстиль, и иметь различные дизайны и цвета. Некоторые чехлы также имеют дополнительные функции, такие как подставка для просмотра видео или окошко для быстрого просмотра уведомлений. Выбор чехла зависит от личных предпочтений и стиля владельца смартфона.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/smartfony/aksessuary-dlya-smartfonov/chekhol-dlya-smartfona-auraphone",
				"images" => array("920_zaschitnoe-steklo-dlya-smartf.webp", "918_zaschitnoe-steklo-dlya-smartf.webp"),
				"param" => array("6" => "99", "7" => array("105", "104", "103", "102")),
				"price" => array(array("price" => "599", "param" => array("2" => "0"))),
				"sort" => 97,
				'import_id' => '97',
			),
			array(
				"id" => 98,
				"cat_id" => array(35, 38, 37, 36, 2),
				"brand_id" => 14,
				"article" => "4095631",
				"name" => array("Провод для смартфона CosmosSmart", "Cosmosmart Smartphone Wire"),
				"anons" => array("<p>Провод для смартфона представляет собой кабель, который используется для зарядки аккумулятора устройства и передачи данных между смартфоном и другими устройствами.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Он состоит из двух частей: одна часть подключается к смартфону, а другая - к источнику питания или другому устройству. Провода могут быть различной длины и иметь разные разъемы, чтобы соответствовать различным моделям смартфонов.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/smartfony/aksessuary-dlya-smartfonov/provod-dlya-smartfona-cosmossmart",
				"images" => array("919_zaschitnoe-steklo-dlya-smartf.webp", "918_zaschitnoe-steklo-dlya-smartf.webp"),
				"param" => array("6" => "98", "7" => array("104", "103", "102")),
				"price" => array(array("price" => "690", "param" => array("2" => "0"))),
				"sort" => 98,
				'import_id' => '98',
			),
			array(
				"id" => 99,
				"cat_id" => array(35, 38, 37, 36),
				"brand_id" => 5,
				"article" => "5495623",
				"name" => array("Защитное стекло для смартфона LunaPad", "Protective glass for the LunaPad smartphone"),
				"anons" => array("<p>Защитное стекло для смартфона - необходимый аксессуар, который помогает сохранить экран устройства в целости и сохранности.</p>", "<p>Introducing the innovative Technical Product X with advanced features to improve your workflow. Our latest development is Technical Product X, designed to make your work easier and more efficient.</p>"),
				"text" => array("<p>Оно представляет собой прозрачную пленку, которая клеится на экран смартфона и защищает его от царапин, трещин и других повреждений. Защитные стекла могут быть разной толщины и прочности, а также иметь различные покрытия, например, олеофобное или антибликовое. Важно выбрать стекло, которое подходит именно для вашей модели смартфона, чтобы обеспечить максимальную защиту и удобство использования.</p>", "<p>This advanced product has a number of unique features and capabilities that will help you save time and effort. With it, you can quickly and accurately complete tasks that previously required a lot of time and effort.</p><p>Key Features:<br>Upgraded Design: Thanks to the improved design, the Technical Product X provides a more comfortable and safe use.<br>Improved performance: Our new product offers improved technical characteristics that allow you to complete tasks faster and more accurately.<br>Easy to use: The intuitive interface and training materials make Technical Product X accessible to any user, regardless of the level of training.<br>Compatibility with other devices: Technical Product X is easily integrated with other devices and software, which allows it to be used in various work scenarios.<br>Durability and reliability: We guarantee the quality and durability of our product through the use of durable and reliable materials.</p>"),
				"rewrite" => "shop/elektronika/smartfony/aksessuary-dlya-smartfonov/zaschitnoe-steklo-dlya-smartfona-lunapad",
				"images" => array("918_zaschitnoe-steklo-dlya-smartf.webp", "919_zaschitnoe-steklo-dlya-smartf.webp", "920_zaschitnoe-steklo-dlya-smartf.webp"),
				"param" => array("6" => "101", "7" => array("105", "104", "103", "102")),
				"price" => array(array("price" => "999.3", "param" => array("2" => "0"))),
				"sort" => 99,
				'import_id' => '99',
			),
			array(
				"id" => 100,
				"cat_id" => array(12, 67),
				"brand_id" => 12,
				"article" => "25373",
				"name" => array("Универсальное чистящее средство “MultiClean”", "Universal cleaning agent “MultiClean”"),
				"anons" => array("<p>Универсальное чистящее средство &ldquo;MultiClean&rdquo; - это эффективный помощник в уборке вашего дома.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Оно легко удаляет грязь, жир и пятна с различных поверхностей, включая полы, стены, мебель и бытовую технику. &ldquo;MultiClean&rdquo; не оставляет разводов и не повреждает поверхности, делая их чистыми и блестящими. Средство обладает приятным ароматом, который быстро выветривается после уборки. Абсолютно безопасно для кожи рук и не вызывает раздражения.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/bytovaya-khimiya/universalnoe-chistyaschee-sredstvo-multiclean",
				"images" => array("1117_universalnoe-chistyaschee-sre.webp"),
				"param" => array("30" => "81", "32" => "204", "31" => "53"),
				"price" => array(array("price" => "390")),
				"sort" => 100,
				'import_id' => '100',
			),
			array(
				"id" => 101,
				"cat_id" => array(12, 66),
				"brand_id" => 12,
				"article" => "25373",
				"name" => array("Освежающий стиральный порошок “SparklingLaundry”", "Refreshing washing powder “SparklingLaundry”"),
				"anons" => array("<p>Освежающий стиральный порошок &ldquo;SparklingLaundry&rdquo; - это эффективное средство для стирки, которое подарит вашей одежде свежесть и приятный аромат.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Он быстро удаляет даже самые сложные загрязнения, сохраняя яркость и цвет тканей. &ldquo;SparklingLaundry&rdquo; подходит для стирки как белого, так и цветного белья, а его гипоаллергенная формула делает его безопасным для людей с чувствительной кожей. Благодаря экономичному расходу, одной упаковки порошка хватит на долгое время.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/bytovaya-khimiya/osvezhayuschiy-stiralnyy-poroshok-sparklinglaundry",
				"images" => array("1116_osvezhayuschiy-stiralnyy-poro.webp"),
				"param" => array("30" => "78", "32" => "59", "31" => "57"),
				"price" => array(array("price" => "2490")),
				"sort" => 101,
				'import_id' => '101',
			),
			array(
				"id" => 102,
				"cat_id" => array(12, 68),
				"brand_id" => 4,
				"article" => "25373",
				"name" => array("Освежитель воздуха “FreshWave”", "FreshWave Air Freshener"),
				"anons" => array("<p>Освежитель воздуха &ldquo;FreshWave&rdquo; подарит вам ощущение свежести и чистоты на длительное время.</p>", "<p>We present you this product, which is characterized by high quality, stylish design and ease of use. The main characteristics of the product include its advantages in relation to other products. Do not miss the opportunity to purchase this product at a special price, which is valid for a limited time.</p>"),
				"text" => array("<p>Он эффективно устраняет неприятные запахи и наполняет помещение приятным ароматом. &ldquo;FreshWave&rdquo; имеет удобный распылитель, который позволяет регулировать интенсивность аромата. Освежитель абсолютно безопасен для здоровья и не вызывает аллергических реакций.</p>", "<p>This product is characterized by high quality and stylish design. It is ideal for any interior and will be pleasing to the eye. Our product has a number of advantages, including: durability, ease of use and affordable price.<br>This product was created using advanced technology and high-quality materials. Thanks to the special production technology, our product has increased durability and resistance to external influences. Do not miss the opportunity to buy this product for a special promotion - only a limited number! Have time to order right now on our website or by phone.</p><p>Enjoy the quality of our product and have a good shopping!</p>"),
				"rewrite" => "shop/bytovaya-khimiya/osvezhitel-vozdukha-freshwave",
				"images" => array("1115_osvezhitel-vozdukha-freshwave.webp"),
				"rel" => 3,
				"param" => array("32" => "60", "31" => "53", "30" => "80"),
				"price" => array(array("price" => "890")),
				"sort" => 102,
				'import_id' => '102',
			),
		),
		'shop_order' => array(
			array(
				'status' => 3,
				'status_id' => 4,
				'summ' => 8238.97,
				'delivery_id' => 1,
				'payment_id' => 1,
				'user_id' => 2,
				'referer' => 'https://yandex.ru/search/?text=auraphone',
				"created" => '1703040254',
				'goods' => array(
					array(
						'good_id' => 54,
						'count_goods' => 1,
						'price' => 7999,
					),
				),
				'param' => array(
					1 => 'Михаил Волков',
					2 => '+79998886655',
					3 => 'mih@volkov.ru',
					4 => 'Москва, пр.Вернадского, д.27, кв.15',
					6 => '1',
				),
			),
			array(
				'status' => 3,
				'status_id' => 4,
				'summ' => 360570,
				'delivery_id' => 1,
				'payment_id' => 1,
				"created" => '1714061085',
				'goods' => array(
					array(
						'good_id' => 96,
						'count_goods' => 1,
						'price' => 120990,
					),
					array(
						'good_id' => 98,
						'count_goods' => 1,
						'price' => 690,
					),
					array(
						'good_id' => 89,
						'count_goods' => 1,
						'price' => 238890,
					),
				),
				'param' => array(
					1 => 'Андрей Серов',
					2 => '+7999000033',
					3 => 'dserov@yandex.ru',
					4 => 'Москва, Саратовская 9, кв. 81',
					6 => '1',
				),
			),
			array(
				'status' => 1,
				'status_id' => 2,
				'summ' => 12990,
				'delivery_id' => 1,
				'payment_id' => 1,
				"created" => '1713061085',
				'goods' => array(
					array(
						'good_id' => 74,
						'count_goods' => 1,
						'price' => 12990,
						'param' => array(
							2 => 10,
							8 => 114,
						)
					),
				),
				'param' => array(
					1 => 'Спиридонов Антон',
					2 => '+79998881100',
					3 => 'fetur@yandex.ru',
					4 => 'Москва, Энтузиастов 90, квартира 31',
					6 => '1',
				),
			),
			array(
				'status_id' => 1,
				'summ' => 32950,
				'delivery_id' => 1,
				'payment_id' => 1,
				'created' => 'now',
				'goods' => array(
					array(
						'good_id' => 78,
						'count_goods' => 1,
						'price' => 5990,
						'param' => array(
							2 => 42,
							8 => 114,
						)
					),
					array(
						'good_id' => 25,
						'count_goods' => 1,
						'price' => 8490,
						'param' => array(
							2 => 133,
							37 => 208,
						)
					),
					array(
						'good_id' => 19,
						'count_goods' => 1,
						'price' => 5490,
						'param' => array(
							2 => 42,
							8 => 114,
						)
					),
					array(
						'good_id' => 2,
						'count_goods' => 2,
						'price' => 6490,
						'param' => array(
							2 => 151,
							8 => 114,
						)
					),
				),
				'param' => array(
					1 => 'Елена Иванова',
					2 => '+78885557744',
					3 => 'lena@mysite.com',
					4 => 'Новосибирск, Федосеева 10, к15',
					6 => '1',
				),
			),
		),
		'shop_discount' => array(
			array(
				'id' => 2,
				'discount' => 15,
				'threshold_cumulative' => 10000,
				'text' => '15% от накопленных 10K',
				'variable' => 'order',
			),
			array(
				'id' => 3,
				'discount' => 10,
				'threshold' => 3000,
				'text' => '10% на заказы более 3K',
				'variable' => 'order',
			),
		),
		'shop_additional_cost' => array(
			array(
				'id' => 1,
				'name' => array('Подарочная упаковка'),
				'text' => array('Мы красиво оформляем Ваш заказ'),
				'percent' => 3,
			),
			array(
				'id' => 2,
				'name' => array('Расширенная гарантия'),
				'amount' => 1000,
			),
			array(
				'id' => 3,
				'name' => array('Непромокаемый чехол'),
				'price' => 800,
				'shop_rel' => '1',
				'cat_id' => array(14),
			),
		),
		'shop_additional_cost_rel' => array(
			array(
				'additional_cost_id' => 3,
				'element_id' => 54,
			),
		),
		'shop_order_additional_cost' => array(
			array(
				'order_id' => 1,
				'additional_cost_id' => 1,
				'summ' => '239.97',
			),
		),
		'shop_waitlist' => array(
			array(
				'good_id' => 50,
				'lang_id' => 1,
				'mail' => 'erema@ochen_zhdu_kita.ru',
				'param' => 'a:0:{}',
			),
		),
		'shop_wishlist' => array(
			array(
				'good_id' => 2,
				'session_id' => '6161nnvam6quvpfgdq7v4c7r23',
				'count' => 2,
				'param' => 'a:2:{i:8;i:114;i:2;i:151;}',
			),
			array(
				'good_id' => 1,
				'session_id' => '6161nnvam6quvpfgdq7v4c7r23',
				'count' => 1,
				'param' => 'a:2:{i:2;i:42;i:8;i:114;}',
			),
		),
		'shop_brand' => array(
			array(
				"id" => 1,
				"name" => array("AcmeTech", "AcmeTech"),
				"text" => array("<p>AcmeTech является одним из ведущих производителей высокотехнологичного оборудования, предлагая широкий ассортимент продукции для различных отраслей промышленности.</p>"),
				"rewrite" => "shop/acmetech",
				"images" => array("343_acmetech.webp"),
			),
			array(
				"id" => 2,
				"name" => array("BellaTextile", "BellaTextile"),
				"text" => array("<p>BellaTextile - это крупнейшая текстильная компания, которая производит высококачественные ткани и одежду, используя инновационные технологии и экологически чистые материалы.</p>"),
				"rewrite" => "shop/bellatextile",
				"images" => array("344_bellatextile.webp"),
			),
			array(
				"id" => 3,
				"name" => array("ColorGarden", "ColorGarden"),
				"text" => array("<p>ColorGarden - это компания, занимающаяся растениеводством и садоводством, которая предлагает широкий ассортимент цветов, деревьев и кустарников для ландшафтного дизайна.</p>"),
				"rewrite" => "shop/colorgarden",
				"images" => array("345_colorgarden.webp"),
			),
			array(
				"id" => 4,
				"name" => array("EcoBuild", "EcoBuild"),
				"text" => array("<p>EcoBuild предлагает комплексные решения для экологической архитектуры и строительства, помогая клиентам создавать устойчивые и красивые здания.</p>"),
				"rewrite" => "shop/ecobuild",
				"images" => array("346_ecobuild.webp"),
			),
			array(
				"id" => 5,
				"name" => array("ElectricVille", "ElectricVille"),
				"text" => array("<p>ElectricVille - это производитель электронных устройств, предлагающий широкий ассортимент продукции, включая бытовую технику, электронику для автомобилей и промышленные системы.</p>"),
				"rewrite" => "shop/electricville",
				"images" => array("347_electricville.webp"),
			),
			array(
				"id" => 6,
				"name" => array("FashionNova", "FashionNova"),
				"text" => array("<p>FashionNova - это модный бренд, который предлагает широкий ассортимент одежды, обуви и аксессуаров для женщин, мужчин и детей.</p>"),
				"rewrite" => "shop/fashionnova",
				"images" => array("348_fashionnova.webp"),
			),
			array(
				"id" => 7,
				"name" => array("FitnessRoute", "FitnessRoute"),
				"text" => array("<p>FitnessRoute - это производитель спортивных товаров и оборудования, предлагающий высококачественные товары для фитнеса, йоги, пилатеса и других видов спорта.</p>"),
				"rewrite" => "shop/fitnessroute",
				"images" => array("349_fitnessroute.webp"),
			),
			array(
				"id" => 8,
				"name" => array("FunTravel", "FunTravel"),
				"text" => array("<p>FunTravel является надежным партнером для путешественников, предоставляя высококачественные туристические товары по доступным ценам.</p>"),
				"rewrite" => "shop/funtravel",
				"images" => array("350_funtravel.webp"),
			),
			array(
				"id" => 9,
				"name" => array("HealthyCuisine", "HealthyCuisine"),
				"text" => array("<p>HealthyCuisine - это компания, которая предлагает здоровые продукты питания и кулинарные решения для людей, следящих за своим здоровьем и питанием.</p>"),
				"rewrite" => "shop/healthycuisine",
				"images" => array("351_healthycuisine.webp"),
			),
			array(
				"id" => 10,
				"name" => array("KidsJoy", "KidsJoy"),
				"text" => array("<p>KidsJoy - это популярный бренд детских игрушек и товаров, предлагающий широкий выбор продукции для развития и развлечения детей всех возрастов.</p>"),
				"rewrite" => "shop/kidsjoy",
				"images" => array("352_kidsjoy.webp"),
			),
			array(
				"id" => 11,
				"name" => array("LuxuryLodge", "LuxuryLodge"),
				"text" => array("<p>LuxuryLodge предлагает роскошные товары для дома, включая мебель, декор и текстиль, создавая уникальные интерьеры для самых взыскательных клиентов.</p>"),
				"rewrite" => "shop/luxurylodge",
				"images" => array("353_luxurylodge.webp"),
			),
			array(
				"id" => 12,
				"name" => array("MegaHome", "MegaHome"),
				"text" => array("<p>MegaHome - это магазин товаров для дома и интерьера, предлагающий широкий ассортимент товаров от ведущих производителей.</p>"),
				"rewrite" => "shop/megahome",
				"images" => array("354_megahome.webp"),
			),
			array(
				"id" => 13,
				"name" => array("NatureShelter", "NatureShelter"),
				"text" => array("<p>NatureShelter - это производитель товаров для активного отдыха на природе, предлагающий палатки, спальные мешки, рюкзаки и другое туристическое снаряжение.</p>"),
				"rewrite" => "shop/natureshelter",
				"images" => array("355_natureshelter.webp"),
			),
			array(
				"id" => 14,
				"name" => array("TechGuru", "TechGuru"),
				"text" => array("<p>TechGuru - это разработчик и производитель ИТ-решений и программного обеспечения для различных отраслей нашей жизни.</p>"),
				"rewrite" => "shop/techguru",
				"images" => array("356_techguru.webp"),
			),
		),
		'shop_set' => array(
			array(
				"element_id" => 56,
				"set_element_id" => 63,
			),
			array(
				"element_id" => 56,
				"set_element_id" => 62,
			),
			array(
				"element_id" => 56,
				"set_element_id" => 55,
			),
		),
	);

	/**
	 * Выполняет действия при установке модуля
	 *
	 * @return void
	 */
	protected function action()
	{
		if (! empty($_SESSION["install_name"]))
		{
			$this->config[] =
				array(
					"name" => "nameshop",
					"value" => $_SESSION["install_name"],
				);
		}
		elseif(defined('TIT'.$this->langs[0]))
		{
			$this->config[] =
				array(
					"name" => "nameshop",
					"value" => constant('TIT'.$this->langs[0]),
				);
		}
	}

	/**
	 * Выполняет действия при установке модуля после основной установки
	 *
	 * @return void
	 */
	public function action_post()
	{
		DB::query("UPDATE {shop_price} SET price_id=id");
		$this->diafan->_shop->price_calc();
	}

	/**
	 * Делает записи о связанных с ценой изображениях
	 *
	 * @param array $row информаци о текущем элементе
	 * @return void
	 */
	protected function action_shop_price_image_rel($row)
	{
		if(empty($this->cache["images"][$row["image_rel"]]))
		{
			return;
		}
		$image_id = $this->cache["images"][$row["image_rel"]];
		DB::query("INSERT INTO {shop_price_image_rel} (price_id, image_id) VALUES (%d, %d)", $row["id"], $image_id);
	}

	/**
	 * Действия при удалении модуля
	 *
	 * @return void
	 */
	protected function uninstall_action()
	{
		DB::query("DELETE FROM {rewrite} WHERE BINARY rewrite='shop/cart/done'");
		DB::query("DELETE FROM {site} WHERE [name]='Заказ оформлен'");
	}
}
